@@ -134,7 +134,7 @@ time {
   expressions+=("-e" "':x;/^\n*$/{\$d;N;bx;}'")
   # Adds a trailing newline if one doesn't already exist
   expressions+=("-e" "'\$a\'")
-  git_files -z | grep -zv 'googleapis.patch$' |
+  git_files -z | grep -zv '\.patch$' |
     grep -zv '\.gz$' |
     grep -zv '\.pb$' |
     grep -zv '\.png$' |
@@ -144,9 +144,14 @@ time {
 
 # Apply buildifier to fix the BUILD and .bzl formatting rules.
 #    https://github.com/bazelbuild/buildtools/tree/master/buildifier
-printf "%-50s" "Running buildifier:" >&2
+printf "%-50s" "Running buildifier (lint):" >&2
 time {
   git_files -z -- '*.BUILD' '*.bzl' '*.bazel' |
+    xargs -r -P "$(nproc)" -n 50 -0 buildifier --lint=fix
+}
+printf "%-50s" "Running buildifier (format):" >&2
+time {
+  git_files -z -- '*.BUILD' '*.bzl' '*.bazel' '*.bzlmod' |
     xargs -r -P "$(nproc)" -n 50 -0 buildifier -mode=fix
 }
 
@@ -206,6 +211,9 @@ time {
 
   mapfile -t libraries < <(features::libraries)
   for library in "${libraries[@]}" opentelemetry; do
+    if [[ "${library}" == "storage_grpc" ]]; then
+      continue
+    fi
     ci/generate-markdown/update-library-readme.sh "${library}"
   done
 }
