@@ -36,6 +36,7 @@ readonly ENABLED_FEATURES
 cmake "${cmake_args[@]}" \
   -DCMAKE_INSTALL_PREFIX="${INSTALL_PREFIX}" \
   -DCMAKE_INSTALL_MESSAGE=NEVER \
+  -DGOOGLE_CLOUD_CPP_ENABLE_CLANG_ABI_COMPAT_17=ON \
   -DBUILD_TESTING=OFF \
   -DGOOGLE_CLOUD_CPP_ENABLE_EXAMPLES=OFF \
   -DGOOGLE_CLOUD_CPP_ENABLE="${ENABLED_FEATURES}"
@@ -69,10 +70,12 @@ expected_dirs+=(
   # no RPC services in google/cloud/appengine/logging
   ./include/google/appengine/logging
   ./include/google/appengine/logging/v1
+  # no RPC services in google/cloud/backupdr/logging
+  ./include/google/cloud/backupdr/logging
+  ./include/google/cloud/backupdr/logging/v1
   # no RPC services in google/cloud/bigquery/logging
   ./include/google/cloud/bigquery/logging
   ./include/google/cloud/bigquery/logging/v1
-  ./include/google/cloud/bigquery/v2
   ./include/google/cloud/bigquery/v2/minimal
   ./include/google/cloud/bigquery/v2/minimal/internal
   ./include/google/cloud/bigquery/v2/minimal/mocks
@@ -94,6 +97,7 @@ expected_dirs+=(
   ./include/google/cloud/gkehub/v1/multiclusteringress
   ./include/google/cloud/grpc_utils
   ./include/google/cloud/internal
+  ./include/google/cloud/internal/win32
   # no RPC services in google/cloud/metastore/logging
   ./include/google/cloud/metastore/logging
   ./include/google/cloud/metastore/logging/v1
@@ -114,7 +118,11 @@ expected_dirs+=(
   # no gRPC services in google/cloud/secretmanager/logging
   ./include/google/cloud/secretmanager/logging
   ./include/google/cloud/secretmanager/logging/v1
+  # no RPC services in google/cloud/servicehealth/logging
+  ./include/google/cloud/servicehealth/logging
+  ./include/google/cloud/servicehealth/logging/v1
   ./include/google/cloud/spanner/mocks
+  ./include/google/cloud/storage/async
   ./include/google/cloud/storage/internal/async
   ./include/google/cloud/storage/internal/curl
   ./include/google/cloud/storage/internal/grpc
@@ -147,9 +155,9 @@ expected_dirs+=(
   ./lib64/cmake/google_cloud_cpp_mocks
   ./lib64/cmake/google_cloud_cpp_oauth2
   ./lib64/cmake/google_cloud_cpp_opentelemetry
-  ./lib64/cmake/google_cloud_cpp_pubsub_mocks
   ./lib64/cmake/google_cloud_cpp_rest_internal
   ./lib64/cmake/google_cloud_cpp_rest_protobuf_internal
+  ./lib64/cmake/google_cloud_cpp_storage_grpc
   ./lib64/cmake/google_cloud_cpp_storage_grpc_mocks
   ./lib64/pkgconfig
 )
@@ -200,11 +208,14 @@ cmake --log-level WARNING -GNinja -DCMAKE_PREFIX_PATH="${INSTALL_PREFIX}" \
 cmake --build "${out_dir}"
 env -C "${out_dir}" ctest "${ctest_args[@]}"
 
+io::log_h1 "Build and run quickstarts"
 # Tests the installed artifacts by building and running the quickstarts.
 # shellcheck disable=SC2046
 feature_list="$(printf "%s;" $(features::libraries))"
 # GCS+gRPC and OpenTelemetry also have quickstarts.
-feature_list="${feature_list}experimental-storage_grpc;opentelemetry"
+feature_list="${feature_list}opentelemetry"
+
+io::log_h2 "Most features"
 cmake -G Ninja \
   -S "${PROJECT_ROOT}/ci/verify_quickstart" \
   -B "${PROJECT_ROOT}/cmake-out/quickstart" \
@@ -212,6 +223,15 @@ cmake -G Ninja \
   "-DFEATURES=${feature_list}"
 cmake --build "${PROJECT_ROOT}/cmake-out/quickstart"
 
+io::log_h2 "GCS+gRPC"
+cmake -G Ninja \
+  -S "${PROJECT_ROOT}/ci/verify_quickstart" \
+  -B "${PROJECT_ROOT}/cmake-out/quickstart" \
+  "-DCMAKE_PREFIX_PATH=${INSTALL_PREFIX}" \
+  "-DFEATURES=storage_grpc"
+cmake --build "${PROJECT_ROOT}/cmake-out/quickstart"
+
+io::log_h2 "Delete installed artifacts and run compiled quickstarts"
 # Deletes all the installed artifacts, and installs only the runtime components
 # to verify that we can still execute the compiled quickstart programs.
 rm -rf "${INSTALL_PREFIX:?}"/{include,lib64}
