@@ -18,6 +18,7 @@ set -euo pipefail
 
 source "$(dirname "$0")/../../lib/init.sh"
 source module ci/cloudbuild/builds/lib/bazel.sh
+source module ci/cloudbuild/builds/lib/cloudcxxrc.sh
 source module ci/cloudbuild/builds/lib/integration.sh
 
 export CC=gcc
@@ -50,7 +51,7 @@ args+=("--instrument_test_targets")
 #     https://github.com/bazelbuild/bazel/issues/3236
 args+=("--sandbox_tmpfs_path=/tmp")
 io::log_h2 "Running coverage on non-integration tests."
-bazel coverage "${args[@]}" --test_tag_filters=-integration-test ...
+bazel coverage "${args[@]}" --test_tag_filters=-integration-test "${BAZEL_TARGETS[@]}"
 
 GOOGLE_CLOUD_CPP_SPANNER_SLOW_INTEGRATION_TESTS="instance"
 mapfile -t integration_args < <(integration::bazel_args)
@@ -91,7 +92,7 @@ time {
   mapfile -t coverage_dat < <(find "$(bazel info output_path)" -name "coverage.dat")
   io::log "Found ${#coverage_dat[@]} coverage.dat files"
   mapfile -t lcov_flags < <(printf -- "--add-tracefile=%s\n" "${coverage_dat[@]}")
-  lcov --quiet "${lcov_flags[@]}" --output-file "${MERGED_COVERAGE}"
+  lcov --quiet --ignore-errors negative "${lcov_flags[@]}" --output-file "${MERGED_COVERAGE}"
   ls -lh "${MERGED_COVERAGE}"
 }
 
