@@ -40,5 +40,5 @@ function ctest::has_no_tests() {
   local prefix="$2"
   shift 2
   local ctest_args=("$@")
-  ctest --test-dir "${dir}" --show-only -R "${prefix}" "${ctest_args[@]}" 2>&1 | grep -q 'Total Tests: 0'
+  env -C "${dir}" ctest --show-only -R "${prefix}" "${ctest_args[@]}" 2>&1 | grep -q 'Total Tests: 0'
 }
