@@ -18,6 +18,7 @@ set -euo pipefail
 
 source "$(dirname "$0")/../../lib/init.sh"
 source module ci/cloudbuild/builds/lib/bazel.sh
+source module ci/cloudbuild/builds/lib/cloudcxxrc.sh
 source module ci/cloudbuild/builds/lib/integration.sh
 source module ci/lib/io.sh
 
@@ -26,7 +27,7 @@ export CXX=clang++
 
 mapfile -t args < <(bazel::common_args)
 args+=("--config=msan")
-io::run bazel test "${args[@]}" --test_tag_filters=-integration-test,-no-msan ...
+io::run bazel test "${args[@]}" --test_tag_filters=-integration-test,-no-msan "${BAZEL_TARGETS[@]}"
 
 mapfile -t integration_args < <(integration::bazel_args)
 integration::bazel_with_emulators test "${args[@]}" "${integration_args[@]}"
