@@ -18,20 +18,21 @@ set -euo pipefail
 
 source "$(dirname "$0")/../../lib/init.sh"
 source module ci/cloudbuild/builds/lib/bazel.sh
+source module ci/cloudbuild/builds/lib/cloudcxxrc.sh
 source module ci/cloudbuild/builds/lib/integration.sh
 source module ci/lib/io.sh
 
 export CC=clang
 export CXX=clang++
 
-# With UBSAN builds we need to ignore the `quickstart` programs, see:
+# With UBSAN builds we need to ignore the `quickstart` and demo programs, see:
 #   https://github.com/bazelbuild/bazel/issues/11122
-deleted_packages="$(find google/cloud -type d -name quickstart | xargs printf ",%s")"
+deleted_packages="$(find google/cloud -type d -name quickstart -o -name demo | xargs printf ",%s")"
 
 mapfile -t args < <(bazel::common_args)
 args+=("--config=ubsan")
 args+=("--deleted_packages=${deleted_packages:1}")
-io::run bazel test "${args[@]}" --test_tag_filters=-integration-test ...
+io::run bazel test "${args[@]}" --test_tag_filters=-integration-test "${BAZEL_TARGETS[@]}"
 
 mapfile -t integration_args < <(integration::bazel_args)
 integration::bazel_with_emulators test "${args[@]}" "${integration_args[@]}"
