@@ -34,6 +34,8 @@ export GOOGLE_CLOUD_CPP_USER_PROJECT="${GOOGLE_CLOUD_PROJECT}"
 export GOOGLE_CLOUD_CPP_TEST_REGION="us-central1"
 # Some quickstart programs require a zone.
 export GOOGLE_CLOUD_CPP_TEST_ZONE="us-central1-a"
+# Some tests and quickstarts need to specify a location as "global".
+export GOOGLE_CLOUD_CPP_TEST_GLOBAL="global"
 # Some quickstart programs require an organization.
 export GOOGLE_CLOUD_CPP_TEST_ORGANIZATION="1006341795026"
 
@@ -65,6 +67,9 @@ export GOOGLE_CLOUD_CPP_BIGTABLE_TEST_QUICKSTART_TABLE="quickstart"
 # Cloud Storage configuration parameters
 # An existing bucket, used in small tests that do not change the bucket metadata.
 export GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME="gcs-grpc-team-cloud-cpp-testing-bucket"
+# Another preexisting bucket, created with folders enabled, for tests that do
+# not modify the bucket:
+export GOOGLE_CLOUD_CPP_STORAGE_TEST_FOLDER_BUCKET_NAME="cloud-cpp-testing-folder-bucket"
 # A bucket with a different location and/or storage class from
 # `cloud-cpp-testing-bucket`, some requests (object copy and rewrite) succeed
 # immediately with buckets in the same location, and we want to demonstrate we
