@@ -428,6 +428,7 @@ bool AppendIfLinebreak(std::ostream& os, MarkdownContext const& /*ctx*/,
 //     <xsd:element name="nonbreakablespace" type="docEmptyType" />
 //     <xsd:element name="iexcl" type="docEmptyType" />
 // ... other "symbols", such as currency, math formulas, accents, etc. ...
+//     <xsd:element name="para" type="docEmptyType" />
 // ... upper case greek letters ...
 // ... lower case greek letters ...
 //   </xsd:choice>
@@ -453,6 +454,7 @@ bool AppendIfDocTitleCmdGroup(std::ostream& os, MarkdownContext const& ctx,
   // Unexpected: nonbreakablespace
   // Unexpected: many many symbols
   if (AppendIfNDash(os, ctx, node)) return true;
+  if (AppendIfParagraph(os, ctx, node)) return true;
   return false;
 }
 
@@ -529,8 +531,14 @@ bool AppendIfParagraph(std::ostream& os, MarkdownContext const& ctx,
                        pugi::xml_node node) {
   if (std::string_view{node.name()} != "para") return false;
   os << ctx.paragraph_start << ctx.paragraph_indent;
+  auto nested = ctx;
   for (auto const child : node) {
-    if (AppendIfDocCmdGroup(os, ctx, child)) continue;
+    // After the first successful item we need to insert a blank line before
+    // each additional item.
+    if (AppendIfDocCmdGroup(os, nested, child)) {
+      nested.paragraph_start = "\n\n";
+      continue;
+    }
     UnknownChildType(__func__, child);
   }
   return true;
@@ -550,8 +558,6 @@ bool AppendIfParagraph(std::ostream& os, MarkdownContext const& ctx,
 bool AppendIfProgramListing(std::ostream& os, MarkdownContext const& ctx,
                             pugi::xml_node node) {
   if (std::string_view{node.name()} != "programlisting") return false;
-  // Start with a new paragraph, with the right level of indentation, and a new
-  // code fence.
   os << ctx.paragraph_start << ctx.paragraph_indent << "```cpp";
   for (auto const child : node) {
     if (AppendIfCodeline(os, ctx, child)) continue;
@@ -807,7 +813,7 @@ bool AppendIfListItem(std::ostream& os, MarkdownContext const& ctx,
   nested.paragraph_indent = ctx.paragraph_indent + ctx.item_prefix;
   for (auto const child : node) {
     if (AppendIfParagraph(os, nested, child)) {
-      // Subsequence paragraphs within the same list item require a blank line
+      // Subsequent paragraphs within the same list item require a blank line
       nested.paragraph_start = "\n\n";
       nested.paragraph_indent =
           ctx.paragraph_indent + std::string(ctx.item_prefix.size(), ' ');
