@@ -716,6 +716,44 @@ for (StatusOr<int> const& v : sr) {
   EXPECT_EQ(kExpected, os.str());
 }
 
+TEST(Doxygen2Markdown, ParagraphProgramListingAddsNewLine) {
+  auto constexpr kXml = R"xml(<?xml version="1.0" standalone="yes"?>
+    <doxygen version="1.9.1" xml:lang="en-US">
+    <detaileddescription  id='test-node'>
+      <para>
+        <simplesect kind="par">
+          <title>Error Handling</title>
+            <para>Description goes here.</para>
+        </simplesect>
+        <programlisting>
+          <codeline><highlight class="keyword">namespace<sp/></highlight><highlight class="normal">cbt<sp/>=<sp/><ref refid="namespacegoogle_1_1cloud_1_1bigtable" kindref="compound">google::cloud::bigtable</ref>;</highlight></codeline>
+          <codeline><highlight class="normal"></highlight><highlight class="keyword">namespace<sp/></highlight><highlight class="normal">btadmin<sp/>=<sp/>google::bigtable::admin::v2;</highlight></codeline>
+        </programlisting>
+      </para>
+    </detaileddescription>
+ </doxygen>)xml";
+
+  auto constexpr kExpected = R"md(
+
+###### Error Handling
+
+Description goes here.
+
+```cpp
+namespace cbt = google::cloud::bigtable;
+namespace btadmin = google::bigtable::admin::v2;
+```)md";
+
+  pugi::xml_document doc;
+  doc.load_string(kXml);
+  auto selected = doc.select_node("//*[@id='test-node']");
+  std::ostringstream os;
+  MarkdownContext mdctx;
+  mdctx.paragraph_start = "";
+  ASSERT_TRUE(AppendIfDetailedDescription(os, mdctx, selected.node()));
+  EXPECT_EQ(kExpected, os.str());
+}
+
 TEST(Doxygen2Markdown, ParagraphVerbatim) {
   auto constexpr kXml = R"xml(<?xml version="1.0" standalone="yes"?>
     <doxygen version="1.9.1" xml:lang="en-US">
@@ -868,6 +906,34 @@ Required. Parent resource name. The format of this value varies depending on the
   EXPECT_EQ(kExpected, os.str());
 }
 
+TEST(Doxygen2Markdown, ParagraphWithParagraph) {
+  auto constexpr kXml = R"xml(<?xml version="1.0" standalone="yes"?>
+    <doxygen version="1.9.1" xml:lang="en-US">
+    <para id='tested-node'>
+      <para>
+        <simplesect kind="warning">
+        <para>Something clever about the warning.</para>
+      </para>
+    </para>
+    </doxygen>)xml";
+  auto constexpr kExpected = R"md(
+
+
+
+
+
+<aside class="warning"><b>Warning:</b>
+Something clever about the warning.
+</aside>)md";
+  pugi::xml_document doc;
+  doc.load_string(kXml);
+  auto selected = doc.select_node("//*[@id='tested-node']");
+  ASSERT_TRUE(selected);
+  std::ostringstream os;
+  ASSERT_TRUE(AppendIfParagraph(os, {}, selected.node()));
+  EXPECT_EQ(kExpected, os.str());
+}
+
 TEST(Doxygen2Markdown, ItemizedListSimple) {
   pugi::xml_document doc;
   doc.load_string(R"xml(<?xml version="1.0" standalone="yes"?>
