@@ -42,10 +42,16 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/appengine/logging/v1:logging_cc_grpc" \
       "@com_google_googleapis//google/appengine/legacy:legacy_cc_grpc"
   )"
+  ["apphub"]="@com_google_googleapis//google/cloud/apphub/v1:apphub_cc_grpc"
   ["artifactregistry"]="@com_google_googleapis//google/devtools/artifactregistry/v1:artifactregistry_cc_grpc"
   ["asset"]="@com_google_googleapis//google/cloud/asset/v1:asset_cc_grpc"
   ["assuredworkloads"]="@com_google_googleapis//google/cloud/assuredworkloads/v1:assuredworkloads_cc_grpc"
   ["automl"]="@com_google_googleapis//google/cloud/automl/v1:automl_cc_grpc"
+  ["backupdr"]="$(
+    printf ",%s" \
+      "@com_google_googleapis//google/cloud/backupdr/v1:backupdr_cc_grpc" \
+      "@com_google_googleapis//google/cloud/backupdr/logging/v1:logging_cc_grpc"
+  )"
   ["baremetalsolution"]="@com_google_googleapis//google/cloud/baremetalsolution/v2:baremetalsolution_cc_grpc"
   ["batch"]="@com_google_googleapis//google/cloud/batch/v1:batch_cc_grpc"
   ["beyondcorp"]="$(
@@ -67,6 +73,7 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/cloud/bigquery/reservation/v1:reservation_cc_grpc" \
       "@com_google_googleapis//google/cloud/bigquery/storage/v1:storage_cc_grpc"
   )"
+  ["bigquerycontrol"]="@com_google_googleapis//google/cloud/bigquery/v2:bigquery_cc_proto"
   ["bigtable"]="$(
     printf ",%s" \
       "@com_google_googleapis//google/bigtable/v2:bigtable_cc_grpc" \
@@ -89,6 +96,8 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/devtools/cloudbuild/v1:cloudbuild_cc_grpc" \
       "@com_google_googleapis//google/devtools/cloudbuild/v2:cloudbuild_cc_grpc"
   )"
+  ["cloudcontrolspartner"]="@com_google_googleapis//google/cloud/cloudcontrolspartner/v1:cloudcontrolspartner_cc_grpc"
+  ["cloudquotas"]="@com_google_googleapis//google/api/cloudquotas/v1:cloudquotas_cc_grpc"
   ["commerce"]="@com_google_googleapis//google/cloud/commerce/consumer/procurement/v1:procurement_cc_grpc"
   ["common"]="@com_google_googleapis//google/cloud/common:common_cc_grpc"
   ["composer"]="@com_google_googleapis//google/cloud/orchestration/airflow/service/v1:service_cc_grpc"
@@ -119,6 +128,7 @@ declare -A -r LIBRARIES=(
   )"
   ["datastream"]="@com_google_googleapis//google/cloud/datastream/v1:datastream_cc_grpc"
   ["deploy"]="@com_google_googleapis//google/cloud/deploy/v1:deploy_cc_grpc"
+  ["developerconnect"]="@com_google_googleapis//google/cloud/developerconnect/v1:developerconnect_cc_grpc"
   ["dialogflow_es"]="@com_google_googleapis//google/cloud/dialogflow/v2:dialogflow_cc_grpc"
   ["dialogflow_cx"]="@com_google_googleapis//google/cloud/dialogflow/cx/v3:cx_cc_grpc"
   ["discoveryengine"]="@com_google_googleapis//google/cloud/discoveryengine/v1:discoveryengine_cc_grpc"
@@ -144,6 +154,7 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/cloud/gkebackup/logging/v1:logging_cc_grpc" \
       "@com_google_googleapis//google/cloud/gkebackup/v1:gkebackup_cc_grpc"
   )"
+  ["gkeconnect"]="@com_google_googleapis//google/cloud/gkeconnect/gateway/v1:gateway_cc_grpc"
   ["gkehub"]="$(
     printf ",%s" \
       "@com_google_googleapis//google/cloud/gkehub/v1:gkehub_cc_grpc" \
@@ -169,6 +180,7 @@ declare -A -r LIBRARIES=(
   ["logging_type"]="@com_google_googleapis//google/logging/type:type_cc_grpc"
   ["logging"]="@com_google_googleapis//google/logging/v2:logging_cc_grpc"
   ["managedidentities"]="@com_google_googleapis//google/cloud/managedidentities/v1:managedidentities_cc_grpc"
+  ["managedkafka"]="@com_google_googleapis//google/cloud/managedkafka/v1:managedkafka_cc_grpc"
   ["memcache"]="@com_google_googleapis//google/cloud/memcache/v1:memcache_cc_grpc"
   ["metastore"]="$(
     printf ",%s" \
@@ -211,7 +223,9 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/cloud/policytroubleshooter/iam/v3:iam_cc_grpc"
   )"
   ["privateca"]="@com_google_googleapis//google/cloud/security/privateca/v1:privateca_cc_grpc"
+  ["privilegedaccessmanager"]="@com_google_googleapis//google/cloud/privilegedaccessmanager/v1:privilegedaccessmanager_cc_grpc"
   ["profiler"]="@com_google_googleapis//google/devtools/cloudprofiler/v2:cloudprofiler_cc_grpc"
+  ["publicca"]="@com_google_googleapis//google/cloud/security/publicca/v1:publicca_cc_grpc"
   ["pubsub"]="@com_google_googleapis//google/pubsub/v1:pubsub_cc_grpc"
   ["pubsublite"]="@com_google_googleapis//google/cloud/pubsublite/v1:pubsublite_cc_grpc"
   ["rapidmigrationassessment"]="@com_google_googleapis//google/cloud/rapidmigrationassessment/v1:rapidmigrationassessment_cc_grpc"
@@ -237,13 +251,23 @@ declare -A -r LIBRARIES=(
       "@com_google_googleapis//google/cloud/secretmanager/logging/v1:logging_cc_grpc"
   )"
   ["securesourcemanager"]="@com_google_googleapis//google/cloud/securesourcemanager/v1:securesourcemanager_cc_grpc"
-  ["securitycenter"]="@com_google_googleapis//google/cloud/securitycenter/v1:securitycenter_cc_grpc"
+  ["securitycenter"]="$(
+    printf ",%s" \
+      "@com_google_googleapis//google/cloud/securitycenter/v1:securitycenter_cc_grpc" \
+      "@com_google_googleapis//google/cloud/securitycenter/v2:securitycenter_cc_grpc"
+  )"
+  ["securitycentermanagement"]="@com_google_googleapis//google/cloud/securitycentermanagement/v1:securitycentermanagement_cc_grpc"
   ["servicecontrol"]="$(
     printf ",%s" \
       "@com_google_googleapis//google/api/servicecontrol/v1:servicecontrol_cc_grpc" \
       "@com_google_googleapis//google/api/servicecontrol/v2:servicecontrol_cc_grpc"
   )"
   ["servicedirectory"]="@com_google_googleapis//google/cloud/servicedirectory/v1:servicedirectory_cc_grpc"
+  ["servicehealth"]="$(
+    printf ",%s" \
+      "@com_google_googleapis//google/cloud/servicehealth/v1:servicehealth_cc_grpc" \
+      "@com_google_googleapis//google/cloud/servicehealth/logging/v1:logging_cc_grpc"
+  )"
   ["servicemanagement"]="@com_google_googleapis//google/api/servicemanagement/v1:servicemanagement_cc_grpc"
   ["serviceusage"]="@com_google_googleapis//google/api/serviceusage/v1:serviceusage_cc_grpc"
   ["shell"]="@com_google_googleapis//google/cloud/shell/v1:shell_cc_grpc"
@@ -260,6 +284,7 @@ declare -A -r LIBRARIES=(
   )"
   ["sql"]="@com_google_googleapis//google/cloud/sql/v1:sql_cc_proto"
   ["storage"]="@com_google_googleapis//google/storage/v2:storage_cc_grpc"
+  ["storagecontrol"]="@com_google_googleapis//google/storage/control/v2:control_cc_grpc"
   ["storageinsights"]="@com_google_googleapis//google/cloud/storageinsights/v1:storageinsights_cc_grpc"
   ["storagetransfer"]="@com_google_googleapis//google/storagetransfer/v1:storagetransfer_cc_grpc"
   ["support"]="@com_google_googleapis//google/cloud/support/v2:support_cc_grpc"
