@@ -52,14 +52,14 @@ using ::testing::IsEmpty;
 using ::testing::Not;
 using ::testing::VariantWith;
 
-auto constexpr kErrorCode = static_cast<int>(StatusCode::kAborted);
+auto constexpr kErrorCode = "ABORTED";
 
 TEST(GoldenKitchenSinkTracingStubTest, GenerateAccessToken) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, GenerateAccessToken)
-      .WillOnce([](auto& context, auto const&) {
+      .WillOnce([](auto& context, auto const&, auto const&) {
         ValidatePropagator(context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         return internal::AbortedError("fail");
@@ -68,7 +68,7 @@ TEST(GoldenKitchenSinkTracingStubTest, GenerateAccessToken) {
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::GenerateAccessTokenRequest request;
-  auto result = under_test.GenerateAccessToken(context, request);
+  auto result = under_test.GenerateAccessToken(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -81,23 +81,24 @@ TEST(GoldenKitchenSinkTracingStubTest, GenerateAccessToken) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, GenerateIdToken) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, GenerateIdToken).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, GenerateIdToken)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::GenerateIdTokenRequest request;
-  auto result = under_test.GenerateIdToken(context, request);
+  auto result = under_test.GenerateIdToken(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -110,23 +111,24 @@ TEST(GoldenKitchenSinkTracingStubTest, GenerateIdToken) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, WriteLogEntries) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, WriteLogEntries).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, WriteLogEntries)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::WriteLogEntriesRequest request;
-  auto result = under_test.WriteLogEntries(context, request);
+  auto result = under_test.WriteLogEntries(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -139,23 +141,24 @@ TEST(GoldenKitchenSinkTracingStubTest, WriteLogEntries) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, ListLogs) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, ListLogs).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, ListLogs)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListLogsRequest request;
-  auto result = under_test.ListLogs(context, request);
+  auto result = under_test.ListLogs(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -167,7 +170,7 @@ TEST(GoldenKitchenSinkTracingStubTest, ListLogs) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, StreamingRead) {
@@ -199,7 +202,7 @@ TEST(GoldenKitchenSinkTracingStubTest, StreamingRead) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, ListServiceAccountKeys) {
@@ -207,7 +210,7 @@ TEST(GoldenKitchenSinkTracingStubTest, ListServiceAccountKeys) {
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, ListServiceAccountKeys)
-      .WillOnce([](auto& context, auto const&) {
+      .WillOnce([](auto& context, auto const&, auto const&) {
         ValidatePropagator(context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         return internal::AbortedError("fail");
@@ -216,7 +219,7 @@ TEST(GoldenKitchenSinkTracingStubTest, ListServiceAccountKeys) {
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListServiceAccountKeysRequest request;
-  auto result = under_test.ListServiceAccountKeys(context, request);
+  auto result = under_test.ListServiceAccountKeys(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -229,23 +232,24 @@ TEST(GoldenKitchenSinkTracingStubTest, ListServiceAccountKeys) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, DoNothing) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, DoNothing).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, DoNothing)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::protobuf::Empty request;
-  auto result = under_test.DoNothing(context, request);
+  auto result = under_test.DoNothing(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -258,7 +262,7 @@ TEST(GoldenKitchenSinkTracingStubTest, DoNothing) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, StreamingWrite) {
@@ -292,7 +296,7 @@ TEST(GoldenKitchenSinkTracingStubTest, StreamingWrite) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingRead) {
@@ -300,7 +304,7 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingRead) {
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, AsyncStreamingRead)
-      .WillOnce([](auto const&, auto context, auto const&) {
+      .WillOnce([](auto const&, auto context, auto, auto const&) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -312,7 +316,8 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingRead) {
   google::cloud::CompletionQueue cq;
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   auto stream = under_test.AsyncStreamingRead(
-      cq, std::make_shared<grpc::ClientContext>(), Request{});
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}), Request{});
   auto start = stream->Start().get();
   EXPECT_FALSE(start);
   auto finish = stream->Finish().get();
@@ -327,8 +332,7 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingRead) {
               "google.test.admin.database.v1.GoldenKitchenSink/StreamingRead"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingWrite) {
@@ -336,7 +340,7 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingWrite) {
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, AsyncStreamingWrite)
-      .WillOnce([](auto const&, auto context) {
+      .WillOnce([](auto const&, auto context, auto) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -349,7 +353,8 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingWrite) {
   google::cloud::CompletionQueue cq;
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   auto stream = under_test.AsyncStreamingWrite(
-      cq, std::make_shared<grpc::ClientContext>());
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}));
   auto start = stream->Start().get();
   EXPECT_FALSE(start);
   auto finish = stream->Finish().get();
@@ -364,8 +369,7 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingWrite) {
               "google.test.admin.database.v1.GoldenKitchenSink/StreamingWrite"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingReadWrite) {
@@ -373,7 +377,7 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingReadWrite) {
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, AsyncStreamingReadWrite)
-      .WillOnce([](auto const&, auto context) {
+      .WillOnce([](auto const&, auto context, auto) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -386,7 +390,8 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingReadWrite) {
   google::cloud::CompletionQueue cq;
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   auto stream = under_test.AsyncStreamingReadWrite(
-      cq, std::make_shared<grpc::ClientContext>());
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}));
   auto start = stream->Start().get();
   EXPECT_FALSE(start);
   auto finish = stream->Finish().get();
@@ -401,24 +406,24 @@ TEST(GoldenKitchenSinkTracingStubTest, AsyncStreamingReadWrite) {
                     "StreamingReadWrite"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, ExplicitRouting1) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, ExplicitRouting1).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, ExplicitRouting1)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ExplicitRoutingRequest request;
-  auto result = under_test.ExplicitRouting1(context, request);
+  auto result = under_test.ExplicitRouting1(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -431,23 +436,24 @@ TEST(GoldenKitchenSinkTracingStubTest, ExplicitRouting1) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenKitchenSinkTracingStubTest, ExplicitRouting2) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
-  EXPECT_CALL(*mock, ExplicitRouting2).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, ExplicitRouting2)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ExplicitRoutingRequest request;
-  auto result = under_test.ExplicitRouting2(context, request);
+  auto result = under_test.ExplicitRouting2(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -460,7 +466,7 @@ TEST(GoldenKitchenSinkTracingStubTest, ExplicitRouting2) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(MakeGoldenKitchenSinkTracingStub, OpenTelemetry) {
@@ -468,11 +474,14 @@ TEST(MakeGoldenKitchenSinkTracingStub, OpenTelemetry) {
 
   auto mock = std::make_shared<MockGoldenKitchenSinkStub>();
   EXPECT_CALL(*mock, DoNothing)
-      .WillOnce(Return(internal::AbortedError("fail")));
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = MakeGoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
-  auto result = under_test->DoNothing(context, {});
+  auto result = under_test->DoNothing(context, Options{}, {});
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -488,7 +497,7 @@ TEST(MakeGoldenKitchenSinkTracingStub, NoOpenTelemetry) {
 
   auto under_test = MakeGoldenKitchenSinkTracingStub(mock);
   grpc::ClientContext context;
-  auto result = under_test->DoNothing(context, {});
+  auto result = under_test->DoNothing(context, Options{}, {});
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 }
 
