@@ -48,6 +48,7 @@ using ::testing::AtLeast;
 using ::testing::Contains;
 using ::testing::ContainsRegex;
 using ::testing::ElementsAre;
+using ::testing::Eq;
 using ::testing::HasSubstr;
 using ::testing::Pair;
 using ::testing::Return;
@@ -88,7 +89,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabases) {
   EXPECT_CALL(*mock, ListDatabases)
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListDatabasesRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -103,7 +104,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabases) {
           })
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListDatabasesRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -118,7 +119,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabases) {
           })
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListDatabasesRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -186,7 +187,7 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseSuccess) {
     return make_ready_future(make_status_or(op));
   });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -218,13 +219,13 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseCancel) {
   AsyncSequencer<StatusOr<google::longrunning::Operation>> get;
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const&,
+      .WillRepeatedly([&](CompletionQueue&, auto, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -242,6 +243,43 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, CreateDatabaseStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::CreateDatabaseMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, CreateDatabase(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Database database;
+        database.set_name("test-database");
+        op.mutable_response()->PackFrom(database);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::CreateDatabaseRequest dbase;
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->CreateDatabase(NoAwaitTag{}, dbase);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto database = conn->CreateDatabase(*operation).get();
+  ASSERT_STATUS_OK(database);
+  EXPECT_EQ("test-database", database->name());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, GetDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
@@ -251,7 +289,7 @@ TEST(GoldenThingAdminConnectionTest, GetDatabaseSuccess) {
       .WillOnce(Return(Status(StatusCode::kDeadlineExceeded, "try-again")))
       .WillOnce(
           [&expected_name](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::GetDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.name());
@@ -318,7 +356,7 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlSuccess) {
     return make_ready_future(make_status_or(op));
   });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -354,13 +392,13 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlCancel) {
   AsyncSequencer<StatusOr<google::longrunning::Operation>> get;
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const&,
+      .WillRepeatedly([&](CompletionQueue&, auto, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -378,6 +416,47 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::UpdateDatabaseDdlMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, UpdateDatabaseDdl(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::UpdateDatabaseDdlMetadata metadata;
+        metadata.set_database("test-database");
+        op.mutable_metadata()->PackFrom(metadata);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
+  request.set_database(
+      "projects/test-project/instances/test-instance/databases/test-database");
+  *request.add_statements() =
+      "ALTER TABLE Albums ADD COLUMN MarketingBudget INT64";
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->UpdateDatabaseDdl(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto update = conn->UpdateDatabaseDdl(*operation).get();
+  ASSERT_STATUS_OK(update);
+  EXPECT_EQ("test-database", update->database());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, DropDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
@@ -386,7 +465,7 @@ TEST(GoldenThingAdminConnectionTest, DropDatabaseSuccess) {
   EXPECT_CALL(*mock, DropDatabase)
       .WillOnce(
           [&expected_name](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
@@ -440,7 +519,7 @@ TEST(GoldenThingAdminConnectionTest, GetDatabaseDdlSuccess) {
       "projects/test-project/instances/test-instance/databases/test-database";
   EXPECT_CALL(*mock, GetDatabaseDdl)
       .WillOnce(Return(Status(StatusCode::kDeadlineExceeded, "try-again")))
-      .WillOnce([&expected_name](grpc::ClientContext&,
+      .WillOnce([&expected_name](grpc::ClientContext&, Options const&,
                                  ::google::test::admin::database::v1::
                                      GetDatabaseDdlRequest const& request) {
         EXPECT_EQ(expected_name, request.database());
@@ -509,7 +588,7 @@ TEST(GoldenThingAdminConnectionTest, SetIamPolicySuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, SetIamPolicy)
       .WillOnce([&expected_name, &expected_policy](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     google::iam::v1::SetIamPolicyRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
         EXPECT_THAT(request.policy(), IsProtoEqual(expected_policy));
@@ -575,7 +654,7 @@ TEST(GoldenThingAdminConnectionTest, GetIamPolicySuccess) {
   std::string const expected_member = "user:foobar@example.com";
   EXPECT_CALL(*mock, GetIamPolicy)
       .WillOnce([&expected_name, &expected_role, &expected_member](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     google::iam::v1::GetIamPolicyRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
         google::iam::v1::Policy response;
@@ -637,7 +716,7 @@ TEST(GoldenThingAdminConnectionTest, TestIamPermissionsSuccess) {
   std::string const expected_permission = "spanner.databases.read";
   EXPECT_CALL(*mock, TestIamPermissions)
       .WillOnce([&expected_name, &expected_permission](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     google::iam::v1::TestIamPermissionsRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
         EXPECT_EQ(1, request.permissions_size());
@@ -700,7 +779,7 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupSuccess) {
     return make_ready_future(make_status_or(op));
   });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -735,13 +814,13 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupCancel) {
   AsyncSequencer<StatusOr<google::longrunning::Operation>> get;
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const&,
+      .WillRepeatedly([&](CompletionQueue&, auto, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -759,6 +838,46 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, CreateBackupStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::CreateBackupMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, CreateBackup(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Backup backup;
+        backup.set_name("test-backup");
+        op.mutable_response()->PackFrom(backup);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::CreateBackupRequest request;
+  request.set_parent("projects/test-project/instances/test-instance");
+  request.set_backup_id("test-backup");
+  request.mutable_backup()->set_name("test-backup");
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->CreateBackup(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto backup = conn->CreateBackup(*operation).get();
+  ASSERT_STATUS_OK(backup);
+  EXPECT_EQ("test-backup", backup->name());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, GetBackupSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
@@ -767,7 +886,7 @@ TEST(GoldenThingAdminConnectionTest, GetBackupSuccess) {
   EXPECT_CALL(*mock, GetBackup)
       .WillOnce(Return(Status(StatusCode::kDeadlineExceeded, "try-again")))
       .WillOnce([&expected_name](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     ::google::test::admin::database::v1::GetBackupRequest const&
                         request) {
         EXPECT_EQ(expected_name, request.name());
@@ -824,7 +943,7 @@ TEST(GoldenThingAdminConnectionTest, UpdateBackupSuccess) {
   EXPECT_CALL(*mock, UpdateBackup)
       .WillOnce(
           [&expected_name](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::UpdateBackupRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.backup().name());
@@ -882,7 +1001,7 @@ TEST(GoldenThingAdminConnectionTest, DeleteBackupSuccess) {
   EXPECT_CALL(*mock, DeleteBackup)
       .WillOnce(
           [&expected_name](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::DeleteBackupRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.name());
@@ -936,7 +1055,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackups) {
   EXPECT_CALL(*mock, ListBackups)
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListBackupsRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -949,7 +1068,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackups) {
           })
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListBackupsRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -962,7 +1081,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackups) {
           })
       .WillOnce(
           [&expected_parent](
-              grpc::ClientContext&,
+              grpc::ClientContext&, Options const&,
               ::google::test::admin::database::v1::ListBackupsRequest const&
                   request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -1028,7 +1147,7 @@ TEST(GoldenThingAdminConnectionTest, RestoreDatabaseSuccess) {
     return make_ready_future(make_status_or(op));
   });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -1065,13 +1184,13 @@ TEST(GoldenThingAdminConnectionTest, RestoreBackupCancel) {
   AsyncSequencer<StatusOr<google::longrunning::Operation>> get;
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const&,
+      .WillRepeatedly([&](CompletionQueue&, auto, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const&,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -1089,6 +1208,48 @@ TEST(GoldenThingAdminConnectionTest, RestoreBackupCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, RestoreDatabaseStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::RestoreDatabaseMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, RestoreDatabase(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Database database;
+        database.set_name("test-database");
+        op.mutable_response()->PackFrom(database);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::RestoreDatabaseRequest request;
+  request.set_parent("projects/test-project/instances/test-instance");
+  request.set_database_id(
+      "projects/test-project/instances/test-instance/databases/test-database");
+  request.set_backup(
+      "projects/test-project/instances/test-instance/backups/test-backup");
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->RestoreDatabase(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto database = conn->RestoreDatabase(*operation).get();
+  ASSERT_STATUS_OK(database);
+  EXPECT_EQ("test-database", database->name());
+}
+
 /// @test Verify that we can list database operations in multiple pages.
 TEST(GoldenThingAdminConnectionTest, ListDatabaseOperations) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
@@ -1096,7 +1257,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabaseOperations) {
       "projects/test-project/instances/test-instance";
   EXPECT_CALL(*mock, ListDatabaseOperations)
       .WillOnce(
-          [&expected_parent](grpc::ClientContext&,
+          [&expected_parent](grpc::ClientContext&, Options const&,
                              ::google::test::admin::database::v1::
                                  ListDatabaseOperationsRequest const& request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -1110,7 +1271,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabaseOperations) {
             return make_status_or(page);
           })
       .WillOnce(
-          [&expected_parent](grpc::ClientContext&,
+          [&expected_parent](grpc::ClientContext&, Options const&,
                              ::google::test::admin::database::v1::
                                  ListDatabaseOperationsRequest const& request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -1124,7 +1285,7 @@ TEST(GoldenThingAdminConnectionTest, ListDatabaseOperations) {
             return make_status_or(page);
           })
       .WillOnce(
-          [&expected_parent](grpc::ClientContext&,
+          [&expected_parent](grpc::ClientContext&, Options const&,
                              ::google::test::admin::database::v1::
                                  ListDatabaseOperationsRequest const& request) {
             EXPECT_EQ(expected_parent, request.parent());
@@ -1189,7 +1350,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackupOperations) {
       "projects/test-project/instances/test-instance";
   EXPECT_CALL(*mock, ListBackupOperations)
       .WillOnce([&expected_parent](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     ::google::test::admin::database::v1::
                         ListBackupOperationsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
@@ -1202,7 +1363,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackupOperations) {
         return make_status_or(page);
       })
       .WillOnce([&expected_parent](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     ::google::test::admin::database::v1::
                         ListBackupOperationsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
@@ -1214,7 +1375,7 @@ TEST(GoldenThingAdminConnectionTest, ListBackupOperations) {
         return make_status_or(page);
       })
       .WillOnce([&expected_parent](
-                    grpc::ClientContext&,
+                    grpc::ClientContext&, Options const&,
                     ::google::test::admin::database::v1::
                         ListBackupOperationsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
@@ -1274,7 +1435,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, AsyncGetDatabase)
       .WillOnce(
-          [](CompletionQueue&, auto,
+          [](CompletionQueue&, auto, auto,
              ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             google::test::admin::database::v1::Database db;
             db.set_name("test-database");
@@ -1296,7 +1457,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseTooManyFailures) {
   EXPECT_CALL(*mock, AsyncGetDatabase)
       .Times(AtLeast(2))
       .WillRepeatedly(
-          [](CompletionQueue&, auto,
+          [](CompletionQueue&, auto, auto,
              ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             return make_ready_future<
                 StatusOr<google::test::admin::database::v1::Database>>(
@@ -1330,7 +1491,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseCancel) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, AsyncGetDatabase)
       .WillOnce(
-          [&p](CompletionQueue&, auto,
+          [&p](CompletionQueue&, auto, auto,
                ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             return p.get_future();
           });
@@ -1357,7 +1518,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseSuccess) {
   EXPECT_CALL(*mock, AsyncDropDatabase)
       .WillOnce(
           [&expected_name](
-              CompletionQueue&, auto,
+              CompletionQueue&, auto, auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
@@ -1383,7 +1544,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseFailure) {
       .Times(AtLeast(2))
       .WillRepeatedly(
           [&expected_name](
-              CompletionQueue&, auto,
+              CompletionQueue&, auto, auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
@@ -1418,7 +1579,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseCancel) {
   EXPECT_CALL(*mock, AsyncDropDatabase)
       .WillOnce(
           [&p, &expected_name](
-              CompletionQueue&, auto,
+              CompletionQueue&, auto, auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
