@@ -70,7 +70,7 @@ TEST_F(MetadataDecoratorTest, ExplicitApiClientHeader) {
   // is sufficient.
   EXPECT_CALL(*mock_, GetDatabase)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::GetDatabaseRequest const&
                      request) {
             IsContextMDValid(context,
@@ -85,14 +85,14 @@ TEST_F(MetadataDecoratorTest, ExplicitApiClientHeader) {
   google::test::admin::database::v1::GetDatabaseRequest request;
   request.set_name(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.GetDatabase(context, request);
+  auto status = stub.GetDatabase(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, GetDatabase) {
   EXPECT_CALL(*mock_, GetDatabase)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::GetDatabaseRequest const&
                      request) {
             IsContextMDValid(context,
@@ -107,14 +107,14 @@ TEST_F(MetadataDecoratorTest, GetDatabase) {
   google::test::admin::database::v1::GetDatabaseRequest request;
   request.set_name(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.GetDatabase(context, request);
+  auto status = stub.GetDatabase(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, ListDatabases) {
   EXPECT_CALL(*mock_, ListDatabases)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::ListDatabasesRequest const&
                      request) {
             IsContextMDValid(context,
@@ -128,14 +128,14 @@ TEST_F(MetadataDecoratorTest, ListDatabases) {
   grpc::ClientContext context;
   google::test::admin::database::v1::ListDatabasesRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.ListDatabases(context, request);
+  auto status = stub.ListDatabases(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
-TEST_F(MetadataDecoratorTest, CreateDatabase) {
+TEST_F(MetadataDecoratorTest, AsyncCreateDatabase) {
   EXPECT_CALL(*mock_, AsyncCreateDatabase)
       .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context, Options const&,
+          [this](google::cloud::CompletionQueue&, auto context, auto,
                  google::test::admin::database::v1::CreateDatabaseRequest const&
                      request) {
             IsContextMDValid(*context,
@@ -149,16 +149,38 @@ TEST_F(MetadataDecoratorTest, CreateDatabase) {
   CompletionQueue cq;
   google::test::admin::database::v1::CreateDatabaseRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.AsyncCreateDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncCreateDatabase(cq, std::make_shared<grpc::ClientContext>(),
+                               internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
-TEST_F(MetadataDecoratorTest, UpdateDatabaseDdl) {
+TEST_F(MetadataDecoratorTest, CreateDatabase) {
+  EXPECT_CALL(*mock_, CreateDatabase)
+      .WillOnce(
+          [this](grpc::ClientContext& context, Options const&,
+                 google::test::admin::database::v1::CreateDatabaseRequest const&
+                     request) {
+            IsContextMDValid(context,
+                             "google.test.admin.database.v1."
+                             "GoldenThingAdmin.CreateDatabase",
+                             request);
+            return TransientError();
+          });
+
+  GoldenThingAdminMetadata stub(mock_, {});
+  grpc::ClientContext context;
+  google::test::admin::database::v1::CreateDatabaseRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.CreateDatabase(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
+TEST_F(MetadataDecoratorTest, AsyncUpdateDatabaseDdl) {
   EXPECT_CALL(*mock_, AsyncUpdateDatabaseDdl)
       .WillOnce(
           [this](
-              google::cloud::CompletionQueue&, auto context, Options const&,
+              google::cloud::CompletionQueue&, auto context, auto,
               google::test::admin::database::v1::UpdateDatabaseDdlRequest const&
                   request) {
             IsContextMDValid(*context,
@@ -173,15 +195,39 @@ TEST_F(MetadataDecoratorTest, UpdateDatabaseDdl) {
   google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
   request.set_database(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.AsyncUpdateDatabaseDdl(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncUpdateDatabaseDdl(cq, std::make_shared<grpc::ClientContext>(),
+                                  internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST_F(MetadataDecoratorTest, UpdateDatabaseDdl) {
+  EXPECT_CALL(*mock_, UpdateDatabaseDdl)
+      .WillOnce(
+          [this](
+              grpc::ClientContext& context, Options const&,
+              google::test::admin::database::v1::UpdateDatabaseDdlRequest const&
+                  request) {
+            IsContextMDValid(context,
+                             "google.test.admin.database.v1."
+                             "GoldenThingAdmin.UpdateDatabaseDdl",
+                             request);
+            return TransientError();
+          });
+
+  GoldenThingAdminMetadata stub(mock_, {});
+  grpc::ClientContext context;
+  google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
+  request.set_database(
+      "projects/my_project/instances/my_instance/databases/my_database");
+  auto status = stub.UpdateDatabaseDdl(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST_F(MetadataDecoratorTest, DropDatabase) {
   EXPECT_CALL(*mock_, DropDatabase)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::DropDatabaseRequest const&
                      request) {
             IsContextMDValid(
@@ -196,14 +242,14 @@ TEST_F(MetadataDecoratorTest, DropDatabase) {
   google::test::admin::database::v1::DropDatabaseRequest request;
   request.set_database(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.DropDatabase(context, request);
+  auto status = stub.DropDatabase(context, Options{}, request);
   EXPECT_EQ(TransientError(), status);
 }
 
 TEST_F(MetadataDecoratorTest, GetDatabaseDdl) {
   EXPECT_CALL(*mock_, GetDatabaseDdl)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::GetDatabaseDdlRequest const&
                      request) {
             IsContextMDValid(
@@ -218,13 +264,13 @@ TEST_F(MetadataDecoratorTest, GetDatabaseDdl) {
   google::test::admin::database::v1::GetDatabaseDdlRequest request;
   request.set_database(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.GetDatabaseDdl(context, request);
+  auto status = stub.GetDatabaseDdl(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, SetIamPolicy) {
   EXPECT_CALL(*mock_, SetIamPolicy)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::iam::v1::SetIamPolicyRequest const& request) {
         IsContextMDValid(
             context,
@@ -238,13 +284,13 @@ TEST_F(MetadataDecoratorTest, SetIamPolicy) {
   google::iam::v1::SetIamPolicyRequest request;
   request.set_resource(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.SetIamPolicy(context, request);
+  auto status = stub.SetIamPolicy(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, GetIamPolicy) {
   EXPECT_CALL(*mock_, GetIamPolicy)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::iam::v1::GetIamPolicyRequest const& request) {
         IsContextMDValid(
             context,
@@ -258,14 +304,14 @@ TEST_F(MetadataDecoratorTest, GetIamPolicy) {
   google::iam::v1::GetIamPolicyRequest request;
   request.set_resource(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.GetIamPolicy(context, request);
+  auto status = stub.GetIamPolicy(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, TestIamPermissions) {
   EXPECT_CALL(*mock_, TestIamPermissions)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::iam::v1::TestIamPermissionsRequest const& request) {
             IsContextMDValid(context,
                              "google.test.admin.database.v1."
@@ -279,14 +325,14 @@ TEST_F(MetadataDecoratorTest, TestIamPermissions) {
   google::iam::v1::TestIamPermissionsRequest request;
   request.set_resource(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.TestIamPermissions(context, request);
+  auto status = stub.TestIamPermissions(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
-TEST_F(MetadataDecoratorTest, CreateBackup) {
+TEST_F(MetadataDecoratorTest, AsyncCreateBackup) {
   EXPECT_CALL(*mock_, AsyncCreateBackup)
       .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context, Options const&,
+          [this](google::cloud::CompletionQueue&, auto context, auto,
                  google::test::admin::database::v1::CreateBackupRequest const&
                      request) {
             IsContextMDValid(
@@ -300,15 +346,37 @@ TEST_F(MetadataDecoratorTest, CreateBackup) {
   CompletionQueue cq;
   google::test::admin::database::v1::CreateBackupRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.AsyncCreateBackup(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncCreateBackup(cq, std::make_shared<grpc::ClientContext>(),
+                             internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST_F(MetadataDecoratorTest, CreateBackup) {
+  EXPECT_CALL(*mock_, CreateBackup)
+      .WillOnce(
+          [this](grpc::ClientContext& context, Options const&,
+                 google::test::admin::database::v1::CreateBackupRequest const&
+                     request) {
+            IsContextMDValid(context,
+                             "google.test.admin.database.v1."
+                             "GoldenThingAdmin.CreateBackup",
+                             request);
+            return TransientError();
+          });
+
+  GoldenThingAdminMetadata stub(mock_, {});
+  grpc::ClientContext context;
+  google::test::admin::database::v1::CreateBackupRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.CreateBackup(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST_F(MetadataDecoratorTest, GetBackup) {
   EXPECT_CALL(*mock_, GetBackup)
       .WillOnce([this](
-                    grpc::ClientContext& context,
+                    grpc::ClientContext& context, Options const&,
                     google::test::admin::database::v1::GetBackupRequest const&
                         request) {
         IsContextMDValid(
@@ -322,14 +390,14 @@ TEST_F(MetadataDecoratorTest, GetBackup) {
   google::test::admin::database::v1::GetBackupRequest request;
   request.set_name(
       "projects/my_project/instances/my_instance/backups/my_backup");
-  auto status = stub.GetBackup(context, request);
+  auto status = stub.GetBackup(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, UpdateBackup) {
   EXPECT_CALL(*mock_, UpdateBackup)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::UpdateBackupRequest const&
                      request) {
             IsContextMDValid(
@@ -344,14 +412,14 @@ TEST_F(MetadataDecoratorTest, UpdateBackup) {
   google::test::admin::database::v1::UpdateBackupRequest request;
   request.mutable_backup()->set_name(
       "projects/my_project/instances/my_instance/backups/my_backup");
-  auto status = stub.UpdateBackup(context, request);
+  auto status = stub.UpdateBackup(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, DeleteBackup) {
   EXPECT_CALL(*mock_, DeleteBackup)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::DeleteBackupRequest const&
                      request) {
             IsContextMDValid(
@@ -366,14 +434,14 @@ TEST_F(MetadataDecoratorTest, DeleteBackup) {
   google::test::admin::database::v1::DeleteBackupRequest request;
   request.set_name(
       "projects/my_project/instances/my_instance/backups/my_backup");
-  auto status = stub.DeleteBackup(context, request);
+  auto status = stub.DeleteBackup(context, Options{}, request);
   EXPECT_EQ(TransientError(), status);
 }
 
 TEST_F(MetadataDecoratorTest, ListBackups) {
   EXPECT_CALL(*mock_, ListBackups)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::test::admin::database::v1::ListBackupsRequest const&
                      request) {
             IsContextMDValid(
@@ -387,14 +455,13 @@ TEST_F(MetadataDecoratorTest, ListBackups) {
   grpc::ClientContext context;
   google::test::admin::database::v1::ListBackupsRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.ListBackups(context, request);
+  auto status = stub.ListBackups(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
-TEST_F(MetadataDecoratorTest, RestoreDatabase) {
+TEST_F(MetadataDecoratorTest, AsyncRestoreDatabase) {
   EXPECT_CALL(*mock_, AsyncRestoreDatabase)
-      .WillOnce([this](google::cloud::CompletionQueue&, auto context,
-                       Options const&,
+      .WillOnce([this](google::cloud::CompletionQueue&, auto context, auto,
                        google::test::admin::database::v1::
                            RestoreDatabaseRequest const& request) {
         IsContextMDValid(
@@ -408,14 +475,37 @@ TEST_F(MetadataDecoratorTest, RestoreDatabase) {
   CompletionQueue cq;
   google::test::admin::database::v1::RestoreDatabaseRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.AsyncRestoreDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncRestoreDatabase(cq, std::make_shared<grpc::ClientContext>(),
+                                internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST_F(MetadataDecoratorTest, RestoreDatabase) {
+  EXPECT_CALL(*mock_, RestoreDatabase)
+      .WillOnce(
+          [this](
+              grpc::ClientContext& context, Options const&,
+              google::test::admin::database::v1::RestoreDatabaseRequest const&
+                  request) {
+            IsContextMDValid(context,
+                             "google.test.admin.database.v1."
+                             "GoldenThingAdmin.RestoreDatabase",
+                             request);
+            return TransientError();
+          });
+
+  GoldenThingAdminMetadata stub(mock_, {});
+  grpc::ClientContext context;
+  google::test::admin::database::v1::RestoreDatabaseRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.RestoreDatabase(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST_F(MetadataDecoratorTest, ListDatabaseOperations) {
   EXPECT_CALL(*mock_, ListDatabaseOperations)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::test::admin::database::v1::
                            ListDatabaseOperationsRequest const& request) {
         IsContextMDValid(context,
@@ -429,13 +519,13 @@ TEST_F(MetadataDecoratorTest, ListDatabaseOperations) {
   grpc::ClientContext context;
   google::test::admin::database::v1::ListDatabaseOperationsRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.ListDatabaseOperations(context, request);
+  auto status = stub.ListDatabaseOperations(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, ListBackupOperations) {
   EXPECT_CALL(*mock_, ListBackupOperations)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::test::admin::database::v1::
                            ListBackupOperationsRequest const& request) {
         IsContextMDValid(context,
@@ -449,14 +539,14 @@ TEST_F(MetadataDecoratorTest, ListBackupOperations) {
   grpc::ClientContext context;
   google::test::admin::database::v1::ListBackupOperationsRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status = stub.ListBackupOperations(context, request);
+  auto status = stub.ListBackupOperations(context, Options{}, request);
   EXPECT_EQ(TransientError(), status.status());
 }
 
 TEST_F(MetadataDecoratorTest, AsyncGetDatabase) {
   EXPECT_CALL(*mock_, AsyncGetDatabase)
       .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context,
+          [this](google::cloud::CompletionQueue&, auto context, auto,
                  google::test::admin::database::v1::GetDatabaseRequest const&
                      request) {
             IsContextMDValid(*context,
@@ -473,15 +563,16 @@ TEST_F(MetadataDecoratorTest, AsyncGetDatabase) {
   google::test::admin::database::v1::GetDatabaseRequest request;
   request.set_name(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status = stub.AsyncGetDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), request);
+  auto status =
+      stub.AsyncGetDatabase(cq, std::make_shared<grpc::ClientContext>(),
+                            internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
 TEST_F(MetadataDecoratorTest, AsyncDropDatabase) {
   EXPECT_CALL(*mock_, AsyncDropDatabase)
       .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context,
+          [this](google::cloud::CompletionQueue&, auto context, auto,
                  google::test::admin::database::v1::DropDatabaseRequest const&
                      request) {
             IsContextMDValid(*context,
@@ -497,7 +588,8 @@ TEST_F(MetadataDecoratorTest, AsyncDropDatabase) {
   request.set_database(
       "projects/my_project/instances/my_instance/databases/my_database");
   auto status = stub.AsyncDropDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get());
 }
 
@@ -505,7 +597,7 @@ TEST_F(MetadataDecoratorTest, LongRunningWithoutRouting) {
   EXPECT_CALL(*mock_, AsyncLongRunningWithoutRouting)
       .WillOnce(
           [this](
-              google::cloud::CompletionQueue&, auto context, Options const&,
+              google::cloud::CompletionQueue&, auto context, auto,
               google::test::admin::database::v1::RestoreDatabaseRequest const&
                   request) {
             IsContextMDValid(*context,
@@ -520,34 +612,35 @@ TEST_F(MetadataDecoratorTest, LongRunningWithoutRouting) {
   google::test::admin::database::v1::RestoreDatabaseRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
   auto status = stub.AsyncLongRunningWithoutRouting(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
 TEST_F(MetadataDecoratorTest, GetOperation) {
   EXPECT_CALL(*mock_, AsyncGetOperation)
-      .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context, Options const&,
-                 google::longrunning::GetOperationRequest const& request) {
-            IsContextMDValid(*context,
-                             "google.longrunning.Operations.GetOperation",
-                             request);
-            return LongrunningTransientError();
-          });
+      .WillOnce([this](
+                    google::cloud::CompletionQueue&, auto context, auto,
+                    google::longrunning::GetOperationRequest const& request) {
+        IsContextMDValid(*context, "google.longrunning.Operations.GetOperation",
+                         request);
+        return LongrunningTransientError();
+      });
 
   GoldenThingAdminMetadata stub(mock_, {});
   CompletionQueue cq;
   google::longrunning::GetOperationRequest request;
   request.set_name("operations/my_operation");
-  auto status = stub.AsyncGetOperation(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncGetOperation(cq, std::make_shared<grpc::ClientContext>(),
+                             internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
 TEST_F(MetadataDecoratorTest, CancelOperation) {
   EXPECT_CALL(*mock_, AsyncCancelOperation)
       .WillOnce(
-          [this](google::cloud::CompletionQueue&, auto context, Options const&,
+          [this](google::cloud::CompletionQueue&, auto context, auto,
                  google::longrunning::CancelOperationRequest const& request) {
             IsContextMDValid(*context,
                              "google.longrunning.Operations.CancelOperation",
@@ -559,8 +652,9 @@ TEST_F(MetadataDecoratorTest, CancelOperation) {
   CompletionQueue cq;
   google::longrunning::CancelOperationRequest request;
   request.set_name("operations/my_operation");
-  auto status = stub.AsyncCancelOperation(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto status =
+      stub.AsyncCancelOperation(cq, std::make_shared<grpc::ClientContext>(),
+                                internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get());
 }
 
