@@ -183,7 +183,7 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncCreateDatabase)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    ::google::test::admin::database::v1::
                        CreateDatabaseRequest const&) {
         google::longrunning::Operation op;
@@ -193,7 +193,7 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseSuccess) {
       });
   EXPECT_CALL(*mock, AsyncGetOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -220,7 +220,7 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseCancel) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncCreateDatabase)
       .WillOnce([&](CompletionQueue&,
-                    std::unique_ptr<rest_internal::RestContext>, Options const&,
+                    std::unique_ptr<rest_internal::RestContext>, auto,
                     ::google::test::admin::database::v1::
                         CreateDatabaseRequest const&) {
         return make_ready_future(make_status_or(op));
@@ -230,15 +230,14 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseCancel) {
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
       .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<rest_internal::RestContext>,
-                          Options const&,
+                          std::unique_ptr<rest_internal::RestContext>, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -256,6 +255,43 @@ TEST(GoldenThingAdminConnectionTest, CreateDatabaseCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, CreateDatabaseStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::CreateDatabaseMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, CreateDatabase(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Database database;
+        database.set_name("test-database");
+        op.mutable_response()->PackFrom(database);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::CreateDatabaseRequest dbase;
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->CreateDatabase(NoAwaitTag{}, dbase);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto database = conn->CreateDatabase(*operation).get();
+  ASSERT_STATUS_OK(database);
+  EXPECT_EQ("test-database", database->name());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, GetDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
@@ -325,7 +361,7 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncUpdateDatabaseDdl)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    ::google::test::admin::database::v1::
                        UpdateDatabaseDdlRequest const&) {
         ::google::test::admin::database::v1::UpdateDatabaseDdlMetadata metadata;
@@ -337,7 +373,7 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlSuccess) {
       });
   EXPECT_CALL(*mock, AsyncGetOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -368,7 +404,7 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlCancel) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncUpdateDatabaseDdl)
       .WillOnce([&](CompletionQueue&,
-                    std::unique_ptr<rest_internal::RestContext>, Options const&,
+                    std::unique_ptr<rest_internal::RestContext>, auto,
                     ::google::test::admin::database::v1::
                         UpdateDatabaseDdlRequest const&) {
         return make_ready_future(make_status_or(op));
@@ -378,15 +414,14 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlCancel) {
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
       .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<rest_internal::RestContext>,
-                          Options const&,
+                          std::unique_ptr<rest_internal::RestContext>, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -404,6 +439,47 @@ TEST(GoldenThingAdminConnectionTest, UpdateDatabaseDdlCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminRestConnectionTest, UpdateDatabaseDdlStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::UpdateDatabaseDdlMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, UpdateDatabaseDdl(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::UpdateDatabaseDdlMetadata metadata;
+        metadata.set_database("test-database");
+        op.mutable_metadata()->PackFrom(metadata);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
+  request.set_database(
+      "projects/test-project/instances/test-instance/databases/test-database");
+  *request.add_statements() =
+      "ALTER TABLE Albums ADD COLUMN MarketingBudget INT64";
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->UpdateDatabaseDdl(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto update = conn->UpdateDatabaseDdl(*operation).get();
+  ASSERT_STATUS_OK(update);
+  EXPECT_EQ("test-database", update->database());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, DropDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
@@ -722,7 +798,7 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupSuccess) {
   EXPECT_CALL(*mock, AsyncCreateBackup)
       .WillOnce(
           [](CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-             Options const&,
+             auto,
              ::google::test::admin::database::v1::CreateBackupRequest const&) {
             google::longrunning::Operation op;
             op.set_name("test-operation-name");
@@ -731,7 +807,7 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupSuccess) {
           });
   EXPECT_CALL(*mock, AsyncGetOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -762,7 +838,7 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupCancel) {
   EXPECT_CALL(*mock, AsyncCreateBackup)
       .WillOnce(
           [&](CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-              Options const&,
+              auto,
               ::google::test::admin::database::v1::CreateBackupRequest const&) {
             return make_ready_future(make_status_or(op));
           });
@@ -771,15 +847,14 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupCancel) {
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
       .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<rest_internal::RestContext>,
-                          Options const&,
+                          std::unique_ptr<rest_internal::RestContext>, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -797,6 +872,46 @@ TEST(GoldenThingAdminConnectionTest, CreateBackupCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, CreateBackupStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::CreateBackupMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, CreateBackup(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Backup backup;
+        backup.set_name("test-backup");
+        op.mutable_response()->PackFrom(backup);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::CreateBackupRequest request;
+  request.set_parent("projects/test-project/instances/test-instance");
+  request.set_backup_id("test-backup");
+  request.mutable_backup()->set_name("test-backup");
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->CreateBackup(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto backup = conn->CreateBackup(*operation).get();
+  ASSERT_STATUS_OK(backup);
+  EXPECT_EQ("test-backup", backup->name());
+}
+
 /// @test Verify that the successful case works.
 TEST(GoldenThingAdminConnectionTest, GetBackupSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
@@ -1061,7 +1176,7 @@ TEST(GoldenThingAdminConnectionTest, RestoreDatabaseSuccess) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncRestoreDatabase)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    ::google::test::admin::database::v1::
                        RestoreDatabaseRequest const&) {
         google::longrunning::Operation op;
@@ -1071,7 +1186,7 @@ TEST(GoldenThingAdminConnectionTest, RestoreDatabaseSuccess) {
       });
   EXPECT_CALL(*mock, AsyncGetOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         google::longrunning::Operation op;
@@ -1109,15 +1224,14 @@ TEST(GoldenThingAdminConnectionTest, RestoreBackupCancel) {
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
       .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<rest_internal::RestContext>,
-                          Options const&,
+                          std::unique_ptr<rest_internal::RestContext>, auto,
                           google::longrunning::GetOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return get.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext>, Options const&,
+                   std::unique_ptr<rest_internal::RestContext>, auto,
                    google::longrunning::CancelOperationRequest const& r) {
         EXPECT_EQ("test-operation-name", r.name());
         return make_ready_future(Status{});
@@ -1135,6 +1249,48 @@ TEST(GoldenThingAdminConnectionTest, RestoreBackupCancel) {
   EXPECT_THAT(db, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(GoldenThingAdminConnectionTest, RestoreDatabaseStartAwait) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  google::longrunning::Operation expected_operation;
+  expected_operation.set_name("test-operation-name");
+  google::test::admin::database::v1::RestoreDatabaseMetadata metadata;
+  expected_operation.mutable_metadata()->PackFrom(metadata);
+
+  EXPECT_CALL(*mock, RestoreDatabase(_, _, _)).WillOnce([&] {
+    return make_status_or(expected_operation);
+  });
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([expected_operation](
+                    CompletionQueue&, auto, auto,
+                    google::longrunning::GetOperationRequest const& r) {
+        EXPECT_EQ(expected_operation.name(), r.name());
+        google::longrunning::Operation op;
+        op.set_name(r.name());
+        op.set_done(true);
+        ::google::test::admin::database::v1::Database database;
+        database.set_name("test-database");
+        op.mutable_response()->PackFrom(database);
+        return make_ready_future(make_status_or(op));
+      });
+
+  auto conn = CreateTestingConnection(std::move(mock));
+  ::google::test::admin::database::v1::RestoreDatabaseRequest request;
+  request.set_parent("projects/test-project/instances/test-instance");
+  request.set_database_id(
+      "projects/test-project/instances/test-instance/databases/test-database");
+  request.set_backup(
+      "projects/test-project/instances/test-instance/backups/test-backup");
+  google::cloud::internal::OptionsSpan span(
+      google::cloud::internal::MergeOptions(Options{}, conn->options()));
+  auto operation = conn->RestoreDatabase(NoAwaitTag{}, request);
+  ASSERT_STATUS_OK(operation);
+  EXPECT_THAT(operation->name(), Eq(expected_operation.name()));
+
+  auto database = conn->RestoreDatabase(*operation).get();
+  ASSERT_STATUS_OK(database);
+  EXPECT_EQ("test-database", database->name());
+}
+
 /// @test Verify that we can list database operations in multiple pages.
 TEST(GoldenThingAdminConnectionTest, ListDatabaseOperations) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
@@ -1321,7 +1477,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseSuccess) {
   EXPECT_CALL(*mock, AsyncGetDatabase)
       .WillOnce(
           [](CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-             Options const&,
+             auto,
              ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             google::test::admin::database::v1::Database db;
             db.set_name("test-database");
@@ -1344,7 +1500,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseTooManyFailures) {
       .Times(AtLeast(2))
       .WillRepeatedly(
           [](CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-             Options const&,
+             auto,
              ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             return make_ready_future<
                 StatusOr<google::test::admin::database::v1::Database>>(
@@ -1379,7 +1535,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncGetDatabaseCancel) {
   EXPECT_CALL(*mock, AsyncGetDatabase)
       .WillOnce(
           [&p](CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-               Options const&,
+               auto,
                ::google::test::admin::database::v1::GetDatabaseRequest const&) {
             return p.get_future();
           });
@@ -1410,7 +1566,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseSuccess) {
       .WillOnce(
           [&expected_name](
               CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-              Options const&,
+              auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
@@ -1437,7 +1593,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseFailure) {
       .WillRepeatedly(
           [&expected_name](
               CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-              Options const&,
+              auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
@@ -1477,7 +1633,7 @@ TEST(GoldenThingAdminConnectionTest, AsyncDropDatabaseCancel) {
       .WillOnce(
           [&p, &expected_name](
               CompletionQueue&, std::unique_ptr<rest_internal::RestContext>,
-              Options const&,
+              auto,
               ::google::test::admin::database::v1::DropDatabaseRequest const&
                   request) {
             EXPECT_EQ(expected_name, request.database());
