@@ -118,6 +118,27 @@ TEST(ThingAdminRestMetadataDecoratorTest, ExplicitApiClientHeader) {
   EXPECT_EQ(TransientError(), status.status());
 }
 
+TEST(ThingAdminRestMetadataDecoratorTest, ServiceApiVersionNotSpecified) {
+  // We use knowledge of the implementation to assert that testing a single RPC
+  // is sufficient.
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  EXPECT_CALL(*mock, GetDatabase)
+      .WillOnce(
+          [](rest_internal::RestContext& context, Options const&,
+             google::test::admin::database::v1::GetDatabaseRequest const&) {
+            EXPECT_THAT(context.GetHeader("x-goog-api-version"), IsEmpty());
+            return TransientError();
+          });
+
+  GoldenThingAdminRestMetadata stub(mock, "test-client-header");
+  rest_internal::RestContext context;
+  google::test::admin::database::v1::GetDatabaseRequest request;
+  request.set_name(
+      "projects/my_project/instances/my_instance/databases/my_database");
+  auto status = stub.GetDatabase(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST(ThingAdminRestMetadataDecoratorTest, GetDatabase) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, GetDatabase)
@@ -174,8 +195,7 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncCreateDatabase) {
   EXPECT_CALL(*mock, AsyncCreateDatabase)
       .WillOnce(
           [](CompletionQueue&,
-             std::unique_ptr<rest_internal::RestContext> context,
-             Options const&,
+             std::unique_ptr<rest_internal::RestContext> context, auto,
              google::test::admin::database::v1::CreateDatabaseRequest const&) {
             EXPECT_THAT(
                 context->GetHeader("x-goog-api-client"),
@@ -194,17 +214,42 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncCreateDatabase) {
   google::test::admin::database::v1::CreateDatabaseRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
   auto status = stub.AsyncCreateDatabase(
-      cq, std::move(context), Options{}.set<QuotaUserOption>("test-quota-user"),
+      cq, std::move(context),
+      internal::MakeImmutableOptions(
+          Options{}.set<QuotaUserOption>("test-quota-user")),
       request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST(ThingAdminRestMetadataDecoratorTest, CreateDatabase) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  EXPECT_CALL(*mock, CreateDatabase)
+      .WillOnce(
+          [](rest_internal::RestContext& context, Options const&,
+             google::test::admin::database::v1::CreateDatabaseRequest const&) {
+            EXPECT_THAT(
+                context.GetHeader("x-goog-api-client"),
+                Contains(google::cloud::internal::GeneratedLibClientHeader()));
+            EXPECT_THAT(context.GetHeader("x-goog-user-project"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-quota-user"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-server-timeout"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-request-params"), IsEmpty());
+            return TransientError();
+          });
+
+  GoldenThingAdminRestMetadata stub(mock);
+  rest_internal::RestContext context;
+  google::test::admin::database::v1::CreateDatabaseRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.CreateDatabase(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST(ThingAdminRestMetadataDecoratorTest, AsyncUpdateDatabaseDdl) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, AsyncUpdateDatabaseDdl)
       .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<rest_internal::RestContext> context,
-                   Options const&,
+                   std::unique_ptr<rest_internal::RestContext> context, auto,
                    google::test::admin::database::v1::
                        UpdateDatabaseDdlRequest const&) {
         EXPECT_THAT(
@@ -223,11 +268,36 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncUpdateDatabaseDdl) {
   google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
   request.set_database(
       "projects/my_project/instances/my_instance/databases/my_database");
-  auto status =
-      stub.AsyncUpdateDatabaseDdl(cq, std::move(context), Options{}, request);
+  auto status = stub.AsyncUpdateDatabaseDdl(
+      cq, std::move(context), internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST(ThingAdminRestMetadataDecoratorTest, UpdateDatabaseDdl) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  EXPECT_CALL(*mock, UpdateDatabaseDdl)
+      .WillOnce([](rest_internal::RestContext& context, Options const&,
+                   google::test::admin::database::v1::
+                       UpdateDatabaseDdlRequest const&) {
+        EXPECT_THAT(
+            context.GetHeader("x-goog-api-client"),
+            Contains(google::cloud::internal::GeneratedLibClientHeader()));
+        EXPECT_THAT(context.GetHeader("x-goog-user-project"), IsEmpty());
+        EXPECT_THAT(context.GetHeader("x-goog-quota-user"), IsEmpty());
+        EXPECT_THAT(context.GetHeader("x-server-timeout"), IsEmpty());
+        EXPECT_THAT(context.GetHeader("x-goog-request-params"), IsEmpty());
+        return TransientError();
+      });
+
+  GoldenThingAdminRestMetadata stub(mock);
+  rest_internal::RestContext context;
+  google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
+  request.set_database(
+      "projects/my_project/instances/my_instance/databases/my_database");
+  auto status = stub.UpdateDatabaseDdl(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST(ThingAdminRestMetadataDecoratorTest, DropDatabaseExplicitRoutingMatch) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, DropDatabase)
@@ -387,8 +457,7 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncCreateBackup) {
   EXPECT_CALL(*mock, AsyncCreateBackup)
       .WillOnce(
           [](CompletionQueue&,
-             std::unique_ptr<rest_internal::RestContext> context,
-             Options const&,
+             std::unique_ptr<rest_internal::RestContext> context, auto,
              google::test::admin::database::v1::CreateBackupRequest const&) {
             EXPECT_THAT(
                 context->GetHeader("x-goog-api-client"),
@@ -405,11 +474,35 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncCreateBackup) {
   auto context = std::make_unique<rest_internal::RestContext>();
   google::test::admin::database::v1::CreateBackupRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status =
-      stub.AsyncCreateBackup(cq, std::move(context), Options{}, request);
+  auto status = stub.AsyncCreateBackup(
+      cq, std::move(context), internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST(ThingAdminRestMetadataDecoratorTest, CreateBackup) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  EXPECT_CALL(*mock, CreateBackup)
+      .WillOnce(
+          [](rest_internal::RestContext& context, Options const&,
+             google::test::admin::database::v1::CreateBackupRequest const&) {
+            EXPECT_THAT(
+                context.GetHeader("x-goog-api-client"),
+                Contains(google::cloud::internal::GeneratedLibClientHeader()));
+            EXPECT_THAT(context.GetHeader("x-goog-user-project"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-quota-user"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-server-timeout"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-request-params"), IsEmpty());
+            return TransientError();
+          });
+
+  GoldenThingAdminRestMetadata stub(mock);
+  rest_internal::RestContext context;
+  google::test::admin::database::v1::CreateBackupRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.CreateBackup(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST(ThingAdminRestMetadataDecoratorTest, GetBackup) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, GetBackup)
@@ -513,8 +606,7 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncRestoreDatabase) {
   EXPECT_CALL(*mock, AsyncRestoreDatabase)
       .WillOnce(
           [](CompletionQueue&,
-             std::unique_ptr<rest_internal::RestContext> context,
-             Options const&,
+             std::unique_ptr<rest_internal::RestContext> context, auto,
              google::test::admin::database::v1::RestoreDatabaseRequest const&) {
             EXPECT_THAT(
                 context->GetHeader("x-goog-api-client"),
@@ -531,11 +623,35 @@ TEST(ThingAdminRestMetadataDecoratorTest, AsyncRestoreDatabase) {
   auto context = std::make_unique<rest_internal::RestContext>();
   google::test::admin::database::v1::RestoreDatabaseRequest request;
   request.set_parent("projects/my_project/instances/my_instance");
-  auto status =
-      stub.AsyncRestoreDatabase(cq, std::move(context), Options{}, request);
+  auto status = stub.AsyncRestoreDatabase(
+      cq, std::move(context), internal::MakeImmutableOptions({}), request);
   EXPECT_EQ(TransientError(), status.get().status());
 }
 
+TEST(ThingAdminRestMetadataDecoratorTest, RestoreDatabase) {
+  auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
+  EXPECT_CALL(*mock, RestoreDatabase)
+      .WillOnce(
+          [](rest_internal::RestContext& context, Options const&,
+             google::test::admin::database::v1::RestoreDatabaseRequest const&) {
+            EXPECT_THAT(
+                context.GetHeader("x-goog-api-client"),
+                Contains(google::cloud::internal::GeneratedLibClientHeader()));
+            EXPECT_THAT(context.GetHeader("x-goog-user-project"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-quota-user"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-server-timeout"), IsEmpty());
+            EXPECT_THAT(context.GetHeader("x-goog-request-params"), IsEmpty());
+            return TransientError();
+          });
+
+  GoldenThingAdminRestMetadata stub(mock);
+  rest_internal::RestContext context;
+  google::test::admin::database::v1::RestoreDatabaseRequest request;
+  request.set_parent("projects/my_project/instances/my_instance");
+  auto status = stub.RestoreDatabase(context, Options{}, request);
+  EXPECT_EQ(TransientError(), status.status());
+}
+
 TEST(ThingAdminRestMetadataDecoratorTest, ListDatabaseOperations) {
   auto mock = std::make_shared<MockGoldenThingAdminRestStub>();
   EXPECT_CALL(*mock, ListDatabaseOperations)
