@@ -46,12 +46,14 @@ using ::google::cloud::testing_util::SpanWithStatus;
 using ::google::cloud::testing_util::ThereIsAnActiveSpan;
 using ::google::test::admin::database::v1::Backup;
 using ::google::test::admin::database::v1::Database;
+using ::testing::_;
+using ::testing::A;
 using ::testing::AllOf;
 using ::testing::ElementsAre;
 using ::testing::IsEmpty;
 using ::testing::Not;
 
-auto constexpr kErrorCode = static_cast<int>(StatusCode::kAborted);
+auto constexpr kErrorCode = "ABORTED";
 
 TEST(GoldenThingAdminTracingConnectionTest, Options) {
   struct TestOption {
@@ -91,19 +93,23 @@ TEST(GoldenThingAdminTracingConnectionTest, ListDatabases) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::ListDatabases"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, CreateDatabase) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminConnection>();
-  EXPECT_CALL(*mock, CreateDatabase).WillOnce([] {
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    EXPECT_TRUE(OTelContextCaptured());
-    return make_ready_future<StatusOr<Database>>(
-        internal::AbortedError("fail"));
-  });
+  EXPECT_CALL(
+      *mock,
+      CreateDatabase(
+          A<google::test::admin::database::v1::CreateDatabaseRequest const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<Database>>(
+            internal::AbortedError("fail"));
+      });
 
   auto under_test = GoldenThingAdminTracingConnection(mock);
   google::test::admin::database::v1::CreateDatabaseRequest request;
@@ -118,7 +124,61 @@ TEST(GoldenThingAdminTracingConnectionTest, CreateDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::CreateDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
+}
+
+TEST(GoldenThingAdminTracingConnectionTest, CreateDatabaseStart) {
+  auto span_catcher = InstallSpanCatcher();
+
+  auto mock = std::make_shared<MockGoldenThingAdminConnection>();
+  EXPECT_CALL(*mock, CreateDatabase(A<NoAwaitTag>(), _)).WillOnce([] {
+    EXPECT_TRUE(ThereIsAnActiveSpan());
+    return StatusOr<google::longrunning::Operation>(
+        internal::AbortedError("fail"));
+  });
+
+  auto under_test = GoldenThingAdminTracingConnection(mock);
+  google::test::admin::database::v1::CreateDatabaseRequest request;
+  auto result = under_test.CreateDatabase(NoAwaitTag{}, request);
+  EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(
+      spans,
+      ElementsAre(AllOf(
+          SpanHasInstrumentationScope(), SpanKindIsClient(),
+          SpanNamed("golden_v1::GoldenThingAdminConnection::CreateDatabase"),
+          SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
+          SpanHasAttributes(
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
+}
+
+TEST(GoldenThingAdminTracingConnectionTest, CreateDatabaseAwait) {
+  auto span_catcher = InstallSpanCatcher();
+
+  auto mock = std::make_shared<MockGoldenThingAdminConnection>();
+  EXPECT_CALL(*mock, CreateDatabase(A<google::longrunning::Operation const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<Database>>(
+            internal::AbortedError("fail"));
+      });
+
+  auto under_test = GoldenThingAdminTracingConnection(mock);
+  google::longrunning::Operation operation;
+  auto result = under_test.CreateDatabase(operation).get();
+  EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(
+      spans,
+      ElementsAre(AllOf(
+          SpanHasInstrumentationScope(), SpanKindIsClient(),
+          SpanNamed("golden_v1::GoldenThingAdminConnection::CreateDatabase"),
+          SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
+          SpanHasAttributes(
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, GetDatabase) {
@@ -143,7 +203,7 @@ TEST(GoldenThingAdminTracingConnectionTest, GetDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::GetDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, UpdateDatabaseDdl) {
@@ -151,12 +211,15 @@ TEST(GoldenThingAdminTracingConnectionTest, UpdateDatabaseDdl) {
 
   auto mock = std::make_shared<MockGoldenThingAdminConnection>();
   using ::google::test::admin::database::v1::UpdateDatabaseDdlMetadata;
-  EXPECT_CALL(*mock, UpdateDatabaseDdl).WillOnce([] {
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    EXPECT_TRUE(OTelContextCaptured());
-    return make_ready_future<StatusOr<UpdateDatabaseDdlMetadata>>(
-        internal::AbortedError("fail"));
-  });
+  EXPECT_CALL(*mock,
+              UpdateDatabaseDdl(A<google::test::admin::database::v1::
+                                      UpdateDatabaseDdlRequest const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<UpdateDatabaseDdlMetadata>>(
+            internal::AbortedError("fail"));
+      });
 
   auto under_test = GoldenThingAdminTracingConnection(mock);
   google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
@@ -171,7 +234,7 @@ TEST(GoldenThingAdminTracingConnectionTest, UpdateDatabaseDdl) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::UpdateDatabaseDdl"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, DropDatabase) {
@@ -196,7 +259,7 @@ TEST(GoldenThingAdminTracingConnectionTest, DropDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::DropDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, GetDatabaseDdl) {
@@ -221,7 +284,7 @@ TEST(GoldenThingAdminTracingConnectionTest, GetDatabaseDdl) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::GetDatabaseDdl"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, SetIamPolicy) {
@@ -246,7 +309,7 @@ TEST(GoldenThingAdminTracingConnectionTest, SetIamPolicy) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::SetIamPolicy"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, GetIamPolicy) {
@@ -271,7 +334,7 @@ TEST(GoldenThingAdminTracingConnectionTest, GetIamPolicy) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::GetIamPolicy"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, TestIamPermissions) {
@@ -297,18 +360,23 @@ TEST(GoldenThingAdminTracingConnectionTest, TestIamPermissions) {
               "golden_v1::GoldenThingAdminConnection::TestIamPermissions"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, CreateBackup) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminConnection>();
-  EXPECT_CALL(*mock, CreateBackup).WillOnce([] {
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    EXPECT_TRUE(OTelContextCaptured());
-    return make_ready_future<StatusOr<Backup>>(internal::AbortedError("fail"));
-  });
+  EXPECT_CALL(
+      *mock,
+      CreateBackup(
+          A<google::test::admin::database::v1::CreateBackupRequest const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<Backup>>(
+            internal::AbortedError("fail"));
+      });
 
   auto under_test = GoldenThingAdminTracingConnection(mock);
   google::test::admin::database::v1::CreateBackupRequest request;
@@ -323,7 +391,7 @@ TEST(GoldenThingAdminTracingConnectionTest, CreateBackup) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::CreateBackup"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, GetBackup) {
@@ -348,7 +416,7 @@ TEST(GoldenThingAdminTracingConnectionTest, GetBackup) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::GetBackup"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, UpdateBackup) {
@@ -373,7 +441,7 @@ TEST(GoldenThingAdminTracingConnectionTest, UpdateBackup) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::UpdateBackup"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, DeleteBackup) {
@@ -398,7 +466,7 @@ TEST(GoldenThingAdminTracingConnectionTest, DeleteBackup) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::DeleteBackup"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, ListBackups) {
@@ -426,19 +494,21 @@ TEST(GoldenThingAdminTracingConnectionTest, ListBackups) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::ListBackups"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, RestoreDatabase) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminConnection>();
-  EXPECT_CALL(*mock, RestoreDatabase).WillOnce([] {
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    EXPECT_TRUE(OTelContextCaptured());
-    return make_ready_future<StatusOr<Database>>(
-        internal::AbortedError("fail"));
-  });
+  EXPECT_CALL(*mock, RestoreDatabase(A<google::test::admin::database::v1::
+                                           RestoreDatabaseRequest const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<Database>>(
+            internal::AbortedError("fail"));
+      });
 
   auto under_test = GoldenThingAdminTracingConnection(mock);
   google::test::admin::database::v1::RestoreDatabaseRequest request;
@@ -453,7 +523,7 @@ TEST(GoldenThingAdminTracingConnectionTest, RestoreDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::RestoreDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, ListDatabaseOperations) {
@@ -483,7 +553,7 @@ TEST(GoldenThingAdminTracingConnectionTest, ListDatabaseOperations) {
               "golden_v1::GoldenThingAdminConnection::ListDatabaseOperations"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, ListBackupOperations) {
@@ -513,19 +583,22 @@ TEST(GoldenThingAdminTracingConnectionTest, ListBackupOperations) {
               "golden_v1::GoldenThingAdminConnection::ListBackupOperations"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, LongRunningWithoutRouting) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminConnection>();
-  EXPECT_CALL(*mock, LongRunningWithoutRouting).WillOnce([] {
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    EXPECT_TRUE(OTelContextCaptured());
-    return make_ready_future<StatusOr<Database>>(
-        internal::AbortedError("fail"));
-  });
+  EXPECT_CALL(*mock,
+              LongRunningWithoutRouting(A<google::test::admin::database::v1::
+                                              RestoreDatabaseRequest const&>()))
+      .WillOnce([] {
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        EXPECT_TRUE(OTelContextCaptured());
+        return make_ready_future<StatusOr<Database>>(
+            internal::AbortedError("fail"));
+      });
 
   auto under_test = GoldenThingAdminTracingConnection(mock);
   google::test::admin::database::v1::RestoreDatabaseRequest request;
@@ -541,7 +614,7 @@ TEST(GoldenThingAdminTracingConnectionTest, LongRunningWithoutRouting) {
                     "LongRunningWithoutRouting"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, AsyncGetDatabase) {
@@ -568,7 +641,7 @@ TEST(GoldenThingAdminTracingConnectionTest, AsyncGetDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::AsyncGetDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingConnectionTest, AsyncDropDatabase) {
@@ -594,7 +667,7 @@ TEST(GoldenThingAdminTracingConnectionTest, AsyncDropDatabase) {
           SpanNamed("golden_v1::GoldenThingAdminConnection::AsyncDropDatabase"),
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(MakeGoldenThingAdminTracingConnection, TracingEnabled) {
