@@ -26,9 +26,6 @@ namespace golden_v1_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
-using ::google::cloud::testing_util::StatusIs;
-using ::testing::Return;
-
 #ifdef GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
 
 using ::google::cloud::testing_util::InstallSpanCatcher;
@@ -39,6 +36,7 @@ using ::google::cloud::testing_util::SpanHasInstrumentationScope;
 using ::google::cloud::testing_util::SpanKindIsClient;
 using ::google::cloud::testing_util::SpanNamed;
 using ::google::cloud::testing_util::SpanWithStatus;
+using ::google::cloud::testing_util::StatusIs;
 using ::google::cloud::testing_util::ThereIsAnActiveSpan;
 using ::google::cloud::testing_util::ValidatePropagator;
 using ::testing::_;
@@ -46,7 +44,7 @@ using ::testing::IsEmpty;
 using ::testing::Not;
 using ::testing::Unused;
 
-auto constexpr kErrorCode = static_cast<int>(StatusCode::kAborted);
+auto constexpr kErrorCode = "ABORTED";
 
 future<StatusOr<google::longrunning::Operation>> LongrunningError(
     Unused, std::shared_ptr<grpc::ClientContext> const& context, Unused,
@@ -62,16 +60,17 @@ TEST(GoldenThingAdminTracingStubTest, ListDatabases) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, ListDatabases).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, ListDatabases)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListDatabasesRequest request;
-  auto result = under_test.ListDatabases(context, request);
+  auto result = under_test.ListDatabases(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -84,7 +83,7 @@ TEST(GoldenThingAdminTracingStubTest, ListDatabases) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncCreateDatabase) {
@@ -97,7 +96,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCreateDatabase) {
   google::test::admin::database::v1::CreateDatabaseRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncCreateDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -110,23 +110,24 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCreateDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, GetDatabase) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, GetDatabase).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, GetDatabase)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::GetDatabaseRequest request;
-  auto result = under_test.GetDatabase(context, request);
+  auto result = under_test.GetDatabase(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -139,7 +140,7 @@ TEST(GoldenThingAdminTracingStubTest, GetDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncUpdateDatabaseDdl) {
@@ -152,7 +153,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncUpdateDatabaseDdl) {
   google::test::admin::database::v1::UpdateDatabaseDdlRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncUpdateDatabaseDdl(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -165,23 +167,24 @@ TEST(GoldenThingAdminTracingStubTest, AsyncUpdateDatabaseDdl) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, DropDatabase) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, DropDatabase).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, DropDatabase)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::DropDatabaseRequest request;
-  auto result = under_test.DropDatabase(context, request);
+  auto result = under_test.DropDatabase(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -194,23 +197,24 @@ TEST(GoldenThingAdminTracingStubTest, DropDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, GetDatabaseDdl) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, GetDatabaseDdl).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, GetDatabaseDdl)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::GetDatabaseDdlRequest request;
-  auto result = under_test.GetDatabaseDdl(context, request);
+  auto result = under_test.GetDatabaseDdl(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -223,23 +227,24 @@ TEST(GoldenThingAdminTracingStubTest, GetDatabaseDdl) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, SetIamPolicy) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, SetIamPolicy).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, SetIamPolicy)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::iam::v1::SetIamPolicyRequest request;
-  auto result = under_test.SetIamPolicy(context, request);
+  auto result = under_test.SetIamPolicy(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -252,23 +257,24 @@ TEST(GoldenThingAdminTracingStubTest, SetIamPolicy) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, GetIamPolicy) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, GetIamPolicy).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, GetIamPolicy)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::iam::v1::GetIamPolicyRequest request;
-  auto result = under_test.GetIamPolicy(context, request);
+  auto result = under_test.GetIamPolicy(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -281,7 +287,7 @@ TEST(GoldenThingAdminTracingStubTest, GetIamPolicy) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, TestIamPermissions) {
@@ -289,7 +295,7 @@ TEST(GoldenThingAdminTracingStubTest, TestIamPermissions) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, TestIamPermissions)
-      .WillOnce([](auto& context, auto const&) {
+      .WillOnce([](auto& context, auto const&, auto const&) {
         ValidatePropagator(context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         return internal::AbortedError("fail");
@@ -298,7 +304,7 @@ TEST(GoldenThingAdminTracingStubTest, TestIamPermissions) {
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::iam::v1::TestIamPermissionsRequest request;
-  auto result = under_test.TestIamPermissions(context, request);
+  auto result = under_test.TestIamPermissions(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -311,7 +317,7 @@ TEST(GoldenThingAdminTracingStubTest, TestIamPermissions) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncCreateBackup) {
@@ -323,8 +329,9 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCreateBackup) {
   auto under_test = GoldenThingAdminTracingStub(mock);
   google::test::admin::database::v1::CreateBackupRequest request;
   CompletionQueue cq;
-  auto result = under_test.AsyncCreateBackup(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto result =
+      under_test.AsyncCreateBackup(cq, std::make_shared<grpc::ClientContext>(),
+                                   internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -337,23 +344,24 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCreateBackup) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, GetBackup) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, GetBackup).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, GetBackup)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::GetBackupRequest request;
-  auto result = under_test.GetBackup(context, request);
+  auto result = under_test.GetBackup(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -365,23 +373,24 @@ TEST(GoldenThingAdminTracingStubTest, GetBackup) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, UpdateBackup) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, UpdateBackup).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, UpdateBackup)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::UpdateBackupRequest request;
-  auto result = under_test.UpdateBackup(context, request);
+  auto result = under_test.UpdateBackup(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -394,23 +403,24 @@ TEST(GoldenThingAdminTracingStubTest, UpdateBackup) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, DeleteBackup) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, DeleteBackup).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, DeleteBackup)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::DeleteBackupRequest request;
-  auto result = under_test.DeleteBackup(context, request);
+  auto result = under_test.DeleteBackup(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -423,23 +433,24 @@ TEST(GoldenThingAdminTracingStubTest, DeleteBackup) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, ListBackups) {
   auto span_catcher = InstallSpanCatcher();
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
-  EXPECT_CALL(*mock, ListBackups).WillOnce([](auto& context, auto const&) {
-    ValidatePropagator(context);
-    EXPECT_TRUE(ThereIsAnActiveSpan());
-    return internal::AbortedError("fail");
-  });
+  EXPECT_CALL(*mock, ListBackups)
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        EXPECT_TRUE(ThereIsAnActiveSpan());
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListBackupsRequest request;
-  auto result = under_test.ListBackups(context, request);
+  auto result = under_test.ListBackups(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -452,7 +463,7 @@ TEST(GoldenThingAdminTracingStubTest, ListBackups) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncRestoreDatabase) {
@@ -465,7 +476,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncRestoreDatabase) {
   google::test::admin::database::v1::RestoreDatabaseRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncRestoreDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -478,7 +490,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncRestoreDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, ListDatabaseOperations) {
@@ -486,7 +498,7 @@ TEST(GoldenThingAdminTracingStubTest, ListDatabaseOperations) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, ListDatabaseOperations)
-      .WillOnce([](auto& context, auto const&) {
+      .WillOnce([](auto& context, auto const&, auto const&) {
         ValidatePropagator(context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         return internal::AbortedError("fail");
@@ -495,7 +507,7 @@ TEST(GoldenThingAdminTracingStubTest, ListDatabaseOperations) {
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListDatabaseOperationsRequest request;
-  auto result = under_test.ListDatabaseOperations(context, request);
+  auto result = under_test.ListDatabaseOperations(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -508,7 +520,7 @@ TEST(GoldenThingAdminTracingStubTest, ListDatabaseOperations) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, ListBackupOperations) {
@@ -516,7 +528,7 @@ TEST(GoldenThingAdminTracingStubTest, ListBackupOperations) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, ListBackupOperations)
-      .WillOnce([](auto& context, auto const&) {
+      .WillOnce([](auto& context, auto const&, auto const&) {
         ValidatePropagator(context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         return internal::AbortedError("fail");
@@ -525,7 +537,7 @@ TEST(GoldenThingAdminTracingStubTest, ListBackupOperations) {
   auto under_test = GoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
   google::test::admin::database::v1::ListBackupOperationsRequest request;
-  auto result = under_test.ListBackupOperations(context, request);
+  auto result = under_test.ListBackupOperations(context, Options{}, request);
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -538,7 +550,7 @@ TEST(GoldenThingAdminTracingStubTest, ListBackupOperations) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncGetDatabase) {
@@ -546,7 +558,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncGetDatabase) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, AsyncGetDatabase)
-      .WillOnce([](auto const&, auto context, auto const&) {
+      .WillOnce([](auto const&, auto context, auto, auto const&) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -559,7 +571,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncGetDatabase) {
   google::test::admin::database::v1::GetDatabaseRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncGetDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -572,7 +585,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncGetDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncDropDatabase) {
@@ -580,7 +593,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncDropDatabase) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, AsyncDropDatabase)
-      .WillOnce([](auto const&, auto context, auto const&) {
+      .WillOnce([](auto const&, auto context, auto, auto const&) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -591,7 +604,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncDropDatabase) {
   google::test::admin::database::v1::DropDatabaseRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncDropDatabase(
-      cq, std::make_shared<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      google::cloud::internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -604,7 +618,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncDropDatabase) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncGetOperation) {
@@ -616,8 +630,9 @@ TEST(GoldenThingAdminTracingStubTest, AsyncGetOperation) {
   auto under_test = GoldenThingAdminTracingStub(mock);
   google::longrunning::GetOperationRequest request;
   CompletionQueue cq;
-  auto result = under_test.AsyncGetOperation(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+  auto result =
+      under_test.AsyncGetOperation(cq, std::make_shared<grpc::ClientContext>(),
+                                   internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -629,7 +644,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncGetOperation) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(GoldenThingAdminTracingStubTest, AsyncCancelOperation) {
@@ -637,7 +652,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCancelOperation) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](auto const&, auto context, Options const&, auto const&) {
+      .WillOnce([](auto const&, auto context, auto, auto const&) {
         ValidatePropagator(*context);
         EXPECT_TRUE(ThereIsAnActiveSpan());
         EXPECT_TRUE(OTelContextCaptured());
@@ -648,7 +663,8 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCancelOperation) {
   google::longrunning::CancelOperationRequest request;
   CompletionQueue cq;
   auto result = under_test.AsyncCancelOperation(
-      cq, std::make_shared<grpc::ClientContext>(), Options{}, request);
+      cq, std::make_shared<grpc::ClientContext>(),
+      internal::MakeImmutableOptions({}), request);
   EXPECT_THAT(result.get(), StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -660,7 +676,7 @@ TEST(GoldenThingAdminTracingStubTest, AsyncCancelOperation) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST(MakeGoldenThingAdminTracingStub, OpenTelemetry) {
@@ -668,11 +684,14 @@ TEST(MakeGoldenThingAdminTracingStub, OpenTelemetry) {
 
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, DropDatabase)
-      .WillOnce(Return(internal::AbortedError("fail")));
+      .WillOnce([](auto& context, auto const&, auto const&) {
+        ValidatePropagator(context);
+        return internal::AbortedError("fail");
+      });
 
   auto under_test = MakeGoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
-  auto result = under_test->DropDatabase(context, {});
+  auto result = under_test->DropDatabase(context, Options{}, {});
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 
   auto spans = span_catcher->GetSpans();
@@ -681,6 +700,9 @@ TEST(MakeGoldenThingAdminTracingStub, OpenTelemetry) {
 
 #else
 
+using ::google::cloud::testing_util::StatusIs;
+using ::testing::Return;
+
 TEST(MakeGoldenThingAdminTracingStub, NoOpenTelemetry) {
   auto mock = std::make_shared<MockGoldenThingAdminStub>();
   EXPECT_CALL(*mock, DropDatabase)
@@ -688,7 +710,7 @@ TEST(MakeGoldenThingAdminTracingStub, NoOpenTelemetry) {
 
   auto under_test = MakeGoldenThingAdminTracingStub(mock);
   grpc::ClientContext context;
-  auto result = under_test->DropDatabase(context, {});
+  auto result = under_test->DropDatabase(context, Options{}, {});
   EXPECT_THAT(result, StatusIs(StatusCode::kAborted));
 }
 
