@@ -36,7 +36,8 @@ class ClientGenerator : public ServiceCodeGenerator {
   ClientGenerator(google::protobuf::ServiceDescriptor const* service_descriptor,
                   VarsDictionary service_vars,
                   std::map<std::string, VarsDictionary> service_method_vars,
-                  google::protobuf::compiler::GeneratorContext* context);
+                  google::protobuf::compiler::GeneratorContext* context,
+                  std::vector<MixinMethod> const& mixin_methods);
 
   ~ClientGenerator() override = default;
 
@@ -49,6 +50,8 @@ class ClientGenerator : public ServiceCodeGenerator {
   Status GenerateHeader() override;
   Status GenerateCc() override;
 
+  bool HasIamPolicyExtension();
+
   // Descriptors for IAM policy producing method_signature extensions, if any.
   google::protobuf::MethodDescriptor const* get_iam_policy_extension_ = nullptr;
   google::protobuf::MethodDescriptor const* set_iam_policy_extension_ = nullptr;
