@@ -25,6 +25,7 @@ namespace generator_internal {
 namespace {
 
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::AllOf;
 using ::testing::Contains;
@@ -298,6 +299,14 @@ TEST(ProcessCommandLineArgs, ProcessOmitStubFactory) {
   EXPECT_THAT(*result, Contains(Pair("omit_stub_factory", "true")));
 }
 
+TEST(ProcessCommandLineArgs, ProcessOmitStreamingUpdater) {
+  auto result = ProcessCommandLineArgs(
+      "product_path=google/cloud/bigquery/storage/v1/"
+      ",omit_streaming_updater=true");
+  ASSERT_THAT(result,
+              IsOkAndHolds(Contains(Pair("omit_streaming_updater", "true"))));
+}
+
 TEST(ProcessCommandLineArgs, ProcessGenerateRoundRobinGenerator) {
   auto result = ProcessCommandLineArgs(
       "product_path=google/cloud/spanner/"
@@ -333,17 +342,6 @@ TEST(ProcessCommandLineArgs, ProcessArgForwardingProductPath) {
                                      "google/cloud/spanner/")));
 }
 
-TEST(ProcessCommandLineArgs, ProcessArgEmitRpc) {
-  auto result = ProcessCommandLineArgs(
-      "product_path=google/cloud/spanner/"
-      ",emit_rpc=Emitted1"
-      ",emit_rpc=Emitted2");
-  ASSERT_THAT(result, IsOk());
-  EXPECT_THAT(*result,
-              Contains(Pair("emitted_rpcs", AllOf(HasSubstr("Emitted1"),
-                                                  HasSubstr("Emitted2")))));
-}
-
 TEST(ProcessCommandLineArgs, ProcessServiceNameMapping) {
   auto result = ProcessCommandLineArgs(
       "product_path=google/cloud/pubsub/"
