@@ -15,11 +15,14 @@
 #include "generator/internal/descriptor_utils.h"
 #include "generator/generator_config.pb.h"
 #include "generator/internal/codegen_utils.h"
+#include "generator/internal/doxygen.h"
 #include "generator/internal/format_class_comments.h"
 #include "generator/internal/format_method_comments.h"
 #include "generator/internal/http_option_utils.h"
 #include "generator/internal/longrunning.h"
 #include "generator/internal/pagination.h"
+#include "generator/internal/predicate_utils.h"
+#include "generator/internal/request_id.h"
 #include "generator/internal/resolve_method_return.h"
 #include "generator/internal/routing.h"
 #include "generator/internal/scaffold_generator.h"
@@ -27,10 +30,13 @@
 #include "google/cloud/internal/absl_str_join_quiet.h"
 #include "google/cloud/internal/absl_str_replace_quiet.h"
 #include "google/cloud/internal/algorithm.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/strip.h"
 #include "absl/types/variant.h"
+#include <google/api/annotations.pb.h>
+#include <google/api/http.pb.h>
 #include <google/api/routing.pb.h>
 #include <google/longrunning/operations.pb.h>
 #include <google/protobuf/compiler/code_generator.h>
@@ -51,7 +57,6 @@ namespace {
 void SetMethodSignatureMethodVars(
     google::protobuf::ServiceDescriptor const& service,
     google::protobuf::MethodDescriptor const& method,
-    std::set<std::string> const& emitted_rpcs,
     std::set<std::string> const& omitted_rpcs, VarsDictionary& method_vars) {
   auto method_name = method.name();
   auto qualified_method_name = absl::StrCat(service.name(), ".", method_name);
@@ -127,19 +132,7 @@ void SetMethodSignatureMethodVars(
     };
     if (internal::ContainsIf(omitted_rpcs, any_match)) continue;
     if (field_deprecated) {
-      // RPCs with deprecated fields must be listed in either omitted_rpcs
-      // or emitted_rpcs. The former is used for newly-generated services,
-      // where we never want to support the deprecated field, and the
-      // latter for newly-deprecated fields, where we want to maintain
-      // backwards compatibility.
-      if (!internal::ContainsIf(emitted_rpcs, any_match)) {
-        GCP_LOG(FATAL) << "Deprecated RPC " << qualified_signature
-                       << " must be listed in either omitted_rpcs"
-                       << " or emitted_rpcs";
-      }
       method_vars["uses_deprecated_field"] = "true";
-      // TODO(#8486): Add a @deprecated Doxygen comment and the
-      // GOOGLE_CLOUD_CPP_DEPRECATED annotation to the generated RPC.
     }
     std::string key = "method_signature" + std::to_string(i);
     method_vars[key] = method_signature;
@@ -160,12 +153,11 @@ bool IsKnownIdempotentMethod(google::protobuf::MethodDescriptor const& m) {
 }
 
 std::string DefaultIdempotencyFromHttpOperation(
-    google::protobuf::MethodDescriptor const& method) {
+    google::protobuf::MethodDescriptor const& method,
+    absl::optional<google::api::HttpRule> http_rule) {
   if (IsKnownIdempotentMethod(method)) return "kIdempotent";
-  if (method.options().HasExtension(google::api::http)) {
-    google::api::HttpRule http_rule =
-        method.options().GetExtension(google::api::http);
-    switch (http_rule.pattern_case()) {
+  if (http_rule) {
+    switch (http_rule->pattern_case()) {
       case google::api::HttpRule::kGet:
       case google::api::HttpRule::kPut:
         return "kIdempotent";
@@ -198,120 +190,6 @@ struct ParameterCommentSubstitution {
   std::size_t uses = 0;
 };
 
-auto constexpr kDialogflowCXEnvironmentIdProto1 = R"""(
- list all environments for. Format: `projects/<Project
- ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment
- ID>`.)""";
-
-auto constexpr kDialogflowCXEnvironmentIdCpp1 = R"""(
- list all environments for. Format:
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>
- @endcode)""";
-
-auto constexpr kDialogflowCXEnvironmentIdProto2 = R"""(
- Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
- ID>/environments/<Environment ID>`.)""";
-
-auto constexpr kDialogflowCXEnvironmentIdCpp2 = R"""(
- Format:
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>
- @endcode)""";
-
-auto constexpr kDialogflowCXSessionIdProto = R"""(
- Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
- ID>/sessions/<Session ID>` or `projects/<Project ID>/locations/<Location
- ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>`.)""";
-
-auto constexpr kDialogflowCXSessionIdCpp = R"""(
- Format:
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/sessions/<Session ID>
- @endcode
-
- or
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>
- @endcode)""";
-
-auto constexpr kDialogflowCXEntityTypeIdProto = R"""(
- Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
- ID>/sessions/<Session ID>/entityTypes/<Entity Type ID>` or
- `projects/<Project ID>/locations/<Location ID>/agents/<Agent
- ID>/environments/<Environment ID>/sessions/<Session ID>/entityTypes/<Entity
- Type ID>`. If `Environment ID` is not specified, we assume default 'draft'
- environment.)""";
-
-auto constexpr kDialogflowCXEntityTypeIdCpp = R"""(
- Format:
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/sessions/<Session ID>/entityTypes/<Entity Type ID>
- @endcode
-
- or
-
- @code
- projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>/entityTypes/<Entity Type ID>
- @endcode
-
- If `Environment ID` is not specified, we assume the default 'draft'
- environment.)""";
-
-auto constexpr kDialogflowESSessionIdProto =
-    R"""( `projects/<Project ID>/agent/sessions/<Session ID>` or `projects/<Project
- ID>/agent/environments/<Environment ID>/users/<User ID>/sessions/<Session
- ID>`.)""";
-
-auto constexpr kDialogflowESSessionIdCpp = R"""(
- @code
- projects/<Project ID>/agent/sessions/<Session ID>
- @endcode
-
- or
-
- @code
- projects/<Project ID>/agent/environments/<Environment ID>/users/<User ID>/sessions/<Session ID>
- @endcode)""";
-
-auto constexpr kDialogflowESContextIdProto =
-    R"""( `projects/<Project ID>/agent/sessions/<Session ID>/contexts/<Context ID>`
- or `projects/<Project ID>/agent/environments/<Environment ID>/users/<User
- ID>/sessions/<Session ID>/contexts/<Context ID>`.)""";
-
-auto constexpr kDialogflowESContextIdCpp = R"""(
- @code
- projects/<Project ID>/agent/sessions/<Session ID>/contexts/<Context ID>
- @endcode
-
- or
-
- @code
- projects/<Project ID>/agent/environments/<Environment ID>/users/<User ID>/sessions/<Session ID>/contexts/<Context ID>`
- @endcode)""";
-
-auto constexpr kDialogflowESSessionEntityTypeDisplayNameProto =
-    R"""( `projects/<Project ID>/agent/sessions/<Session ID>/entityTypes/<Entity Type
- Display Name>` or `projects/<Project ID>/agent/environments/<Environment
- ID>/users/<User ID>/sessions/<Session ID>/entityTypes/<Entity Type Display
- Name>`.)""";
-
-auto constexpr kDialogflowESSessionEntityTypeDisplayNameCpp = R"""(
- @code
- projects/<Project ID>/agent/sessions/<Session ID>/entityTypes/<Entity Type Display Name>
- @endcode
-
- or
-
- @code
- projects/<Project ID>/agent/environments/<Environment ID>/users/<User ID>/sessions/<Session ID>/entityTypes/<Entity Type Display Name>
- @endcode)""";
-
 auto constexpr kLoggingConfigClientProto1 =
     R"""( The resource name of the link:
 
@@ -345,21 +223,43 @@ auto constexpr kLoggingConfigClientCpp2 =
    "folders/[FOLDER_ID]/locations/[LOCATION_ID]/buckets/[BUCKET_ID]")""";
 
 ParameterCommentSubstitution substitutions[] = {
-    // From dialogflow/cx/v3.
-    {kDialogflowCXEnvironmentIdProto1, kDialogflowCXEnvironmentIdCpp1},
-    {kDialogflowCXEnvironmentIdProto2, kDialogflowCXEnvironmentIdCpp2},
-    {kDialogflowCXSessionIdProto, kDialogflowCXSessionIdCpp},
-    {kDialogflowCXEntityTypeIdProto, kDialogflowCXEntityTypeIdCpp},
-    {"<Agent\n ID>", "<Agent ID>"},
-    {"<Location\n ID>", "<Location ID>"},
-    {"<Transition\n Route Group ID>", "<Transition Route Group ID>"},
-    {"<Agent ID>.", "<Agent ID>`."},  // missing close quote
+    // Unescaped elements in bigtable/admin/v2.
+    {" projects/<project>/instances/<instance>/"
+     "tables/<table>/authorizedViews/<authorized_view>",
+     " `projects/<project>/instances/<instance>/"
+     "tables/<table>/authorizedViews/<authorized_view>`"},
 
-    // From dialogflow/v2.
-    {kDialogflowESSessionIdProto, kDialogflowESSessionIdCpp},
-    {kDialogflowESContextIdProto, kDialogflowESContextIdCpp},
-    {kDialogflowESSessionEntityTypeDisplayNameProto,
-     kDialogflowESSessionEntityTypeDisplayNameCpp},
+    // There a number of comments (almost all from DialogFlow) that
+    // - use single code quotes
+    // - are broken up over multiple lines
+    // - contain variables in angle brackets
+    //
+    // Doxygen thinks these are unrecognized HTML tags. To work around this, we
+    // remove the line breaks, so the code quotes occupy a single line.
+    {"agents/\n <Agent ID>", "agents/<Agent ID>"},
+    {"<Agent\n ID>", "<Agent ID>"},
+    {"<Agent ID>/\n testCases/", "<Agent ID>/testCases/"},
+    {"<Agent ID>/\n environments/", "<Agent ID>/environments/"},
+    {"<Conversation\n ID>", "<Conversation ID>"},
+    {"<Conversation Model\n ID>", "<Conversation Model ID>"},
+    {"<Entity Type Display\n Name>", "<Entity Type Display Name>"},
+    {"<Entity Type\n Display Name>", "<Entity Type Display Name>"},
+    {"<Entity\n Type ID>", "<Entity Type ID>"},
+    {"<Environment\n ID>", "<Environment ID>"},
+    {"<Location\n ID>", "<Location ID>"},
+    {"<Project\n ID>", "<Project ID>"},
+    {"<Project\n Number>", "<Project Number>"},
+    {"<Session\n ID>", "<Session ID>"},
+    {"<Transition\n Route Group ID>", "<Transition Route Group ID>"},
+    {"<User\n ID>", "<User ID>"},
+    {"<User ID>/\n sessions/", "<User ID>/sessions/"},
+
+    // Missing closed quote in dialogflow
+    {"<Agent ID>.", "<Agent ID>`."},
+
+    // Extra closed quote in dialogflow
+    {R"""(`projects/<Project ID>/locations/<Location ID>`/generators/<Generator ID>`)""",
+     R"""(`projects/<Project ID>/locations/<Location ID>/generators/<Generator ID>`)"""},
 
     // From logging/v2.
     {kLoggingConfigClientProto1, kLoggingConfigClientCpp1},
@@ -374,6 +274,8 @@ ParameterCommentSubstitution substitutions[] = {
     {" <parent>/instanceConfigs/us-east1,",
      " `<parent>/instanceConfigs/us-east1`,"},
     {" <parent>/instanceConfigs/nam3.", " `<parent>/instanceConfigs/nam3`."},
+    {" projects/<project>/instances/<instance>/databases/<database>",
+     " `projects/<project>/instances/<instance>/databases/<database>`"},
 
     // Extra quotes in asset/v1.
     {R"""( "folders/12345")", or a )""", R"""( "folders/12345"), or a )"""},
@@ -381,13 +283,17 @@ ParameterCommentSubstitution substitutions[] = {
     // Doxygen gets confused by single quotes in code spans:
     //    https://www.doxygen.nl/manual/markdown.html#mddox_code_spans
     // The workaround is to double quote these:
-    {R"""(`{instance} = '-'`)""", R"""(``{instance} = '-'``)"""},
-    {R"""(`{cluster} = '-'`)""", R"""(``{cluster} = '-'``)"""},
+    {R"""(`{instance} = '-'`)""", R"""(`{instance} = "-"`)"""},
+    {R"""(`{cluster} = '-'`)""", R"""(`{cluster} = "-"`)"""},
     {R"""(`projects/<Project ID or '-'>`)""",
-     R"""(``projects/<Project ID or '-'>``)"""},
-    // In this case the single quote is just a mistake so we can remove it.
-    {R"""(`projects/{project}/locations/{location}'`)""",
-     R"""(`projects/{project}/locations/{location}`)"""},
+     R"""(`projects/<Project ID or "-">`)"""},
+    // These appear in google/api/servicemanagement/v1/servicemanager.proto
+    {R"""(`filter='status=SUCCESS'`)""", R"""(`filter="status=SUCCESS"`)"""},
+    {R"""(`filter='strategy=TrafficPercentStrategy'`)""",
+     R"""(`filter="strategy=TrafficPercentStrategy"`)"""},
+
+    // Spanner attempts to explain what a backtick is.
+    {R"""(backticks (`` ` ``).)""", R"""(backticks.)"""},
 
     // Further trim some initial paragraphs for long descriptions.
     {R"""( The included patch
@@ -407,19 +313,15 @@ ParameterCommentSubstitution substitutions[] = {
     {"fields. Some eligible fields are:", "fields."},
     {" The allowable fields to\n update are:", ""},
 
-    // These appear in google/api/servicemanagement/v1/servicemanager.proto
-    // Doxygen gets confused by single quotes in code spans:
-    //    https://www.doxygen.nl/manual/markdown.html#mddox_code_spans
-    // The workaround is to double quote these:
-    {"`filter='status=SUCCESS'`", "``filter='status=SUCCESS'``"},
-    {"`filter='strategy=TrafficPercentStrategy'`",
-     "``filter='strategy=TrafficPercentStrategy'``"},
-
     {"conversionWorkspace/123/mappingRules/"
      "rule123@c7cfa2a8c7cfa2a8c7cfa2a8c7cfa2a8",
      "`conversionWorkspace/123/mappingRules/"
      "rule123@c7cfa2a8c7cfa2a8c7cfa2a8c7cfa2a8`"},
 
+    // From google/cloud/gkemulticloud/v1/azure_service.proto
+    {" projects/<project-id>/locations/<region>/azureClusters/<cluster-id>",
+     " `projects/<project-id>/locations/<region>/azureClusters/<cluster-id>`"},
+
     // Some comments include multiple newlines in a row. We need to preserve
     // these because they are paragraph separators. When used in `@param`
     // commands we need to represent them as `@n` or they do would terminate the
@@ -558,6 +460,17 @@ std::string FormatAdditionalPbHeaderPaths(VarsDictionary& vars) {
   return absl::StrJoin(additional_pb_header_paths, ",");
 }
 
+std::string FormatMixinPbHeaderPaths(
+    std::vector<MixinMethod> const& mixin_methods, std::string const& suffix) {
+  std::unordered_set<std::string> mixin_header_paths;
+  for (auto const& mixin_method : mixin_methods) {
+    mixin_header_paths.insert(absl::StrCat(
+        absl::StripSuffix(mixin_method.method.get().file()->name(), ".proto"),
+        suffix));
+  }
+  return absl::StrJoin(mixin_header_paths, ",");
+}
+
 // If a service name mapping exists, return the new name.
 // Parses a command line argument in the form:
 // {"service_name_mappings": "service_a=new_service_a,service=new_service"}.
@@ -594,6 +507,54 @@ absl::optional<std::string> GetReplacementComment(VarsDictionary const& vars,
   return absl::nullopt;
 }
 
+VarsDictionary GetMethodVars(
+    google::protobuf::ServiceDescriptor const& service,
+    YAML::Node const& service_config,
+    google::protobuf::MethodDescriptor const& method,
+    absl::optional<google::api::HttpRule> const& http_rule,
+    std::string const& grpc_stub_name,
+    VarsDictionary const& idempotency_overrides,
+    std::set<std::string> const& omitted_rpcs) {
+  VarsDictionary method_vars;
+  method_vars["idempotency"] =
+      DefaultIdempotencyFromHttpOperation(method, http_rule);
+  if (!idempotency_overrides.empty()) {
+    auto iter = idempotency_overrides.find(
+        absl::StrCat(service.name(), ".", method.name()));
+    if (iter != idempotency_overrides.end()) {
+      method_vars["idempotency"] = iter->second;
+    }
+  }
+  method_vars["grpc_stub"] = grpc_stub_name + "_";
+  method_vars["method_name"] = method.name();
+  method_vars["method_name_snake"] = CamelCaseToSnakeCase(method.name());
+  method_vars["request_type"] =
+      ProtoNameToCppName(method.input_type()->full_name());
+  method_vars["response_message_type"] = method.output_type()->full_name();
+  method_vars["response_type"] =
+      ProtoNameToCppName(method.output_type()->full_name());
+  method_vars["return_type"] =
+      IsResponseTypeEmpty(method)
+          ? "Status"
+          : absl::StrFormat("StatusOr<%s>", method_vars.at("response_type"));
+  auto request_id_field_name = RequestIdFieldName(service_config, method);
+  if (!request_id_field_name.empty()) {
+    method_vars["request_id_field_name"] = std::move(request_id_field_name);
+  }
+  SetLongrunningOperationMethodVars(method, method_vars);
+  AssignPaginationMethodVars(method, method_vars);
+  SetMethodSignatureMethodVars(service, method, omitted_rpcs, method_vars);
+  HttpExtensionInfo parsed_http_info;
+  if (http_rule) {
+    parsed_http_info = ParseHttpExtension(*http_rule);
+  }
+  method_vars["request_resource"] =
+      FormatRequestResource(*method.input_type(), parsed_http_info);
+  SetHttpDerivedMethodVars(parsed_http_info, method, method_vars);
+  SetHttpQueryParameters(parsed_http_info, method, method_vars);
+  return method_vars;
+}
+
 }  // namespace
 
 std::string CppTypeToString(FieldDescriptor const* field) {
@@ -623,17 +584,6 @@ std::string CppTypeToString(FieldDescriptor const* field) {
   return field->cpp_type_name();
 }
 
-// TODO(#11545): relocate this function to a separate TU.
-std::string FormatDoxygenLink(
-    google::protobuf::Descriptor const& message_type) {
-  google::protobuf::SourceLocation loc;
-  message_type.GetSourceLocation(&loc);
-  std::string output_type_proto_file_name = message_type.file()->name();
-  return absl::StrCat(
-      "@googleapis_link{", ProtoNameToCppName(message_type.full_name()), ",",
-      output_type_proto_file_name, "#L", loc.start_line + 1, "}");
-}
-
 std::string FormatMethodCommentsMethodSignature(
     google::protobuf::MethodDescriptor const& method,
     std::string const& signature, bool is_discovery_document_proto) {
@@ -673,11 +623,14 @@ bool CheckParameterCommentSubstitutions() {
 
 VarsDictionary CreateServiceVars(
     google::protobuf::ServiceDescriptor const& descriptor,
-    std::vector<std::pair<std::string, std::string>> const& initial_values) {
+    std::vector<std::pair<std::string, std::string>> const& initial_values,
+    std::vector<MixinMethod> const& mixin_methods) {
   VarsDictionary vars(initial_values.begin(), initial_values.end());
   auto const& service_name = GetEffectiveServiceName(vars, descriptor.name());
   vars["product_options_page"] = OptionsGroup(vars["product_path"]);
   vars["additional_pb_header_paths"] = FormatAdditionalPbHeaderPaths(vars);
+  vars["api_version"] =
+      descriptor.options().GetExtension(google::api::api_version);
   vars["class_comment_block"] = FormatClassCommentsFromServiceComments(
       descriptor, service_name, GetReplacementComment(vars, service_name));
   vars["client_class_name"] = absl::StrCat(service_name, "Client");
@@ -804,6 +757,10 @@ VarsDictionary CreateServiceVars(
       absl::StripSuffix(descriptor.file()->name(), ".proto"), ".grpc.pb.h");
   vars["proto_header_path"] = absl::StrCat(
       absl::StripSuffix(descriptor.file()->name(), ".proto"), ".pb.h");
+  vars["mixin_proto_grpc_header_paths"] =
+      FormatMixinPbHeaderPaths(mixin_methods, ".grpc.pb.h");
+  vars["mixin_proto_header_paths"] =
+      FormatMixinPbHeaderPaths(mixin_methods, ".pb.h");
   vars["retry_policy_name"] = absl::StrCat(service_name, "RetryPolicy");
   vars["retry_traits_name"] = absl::StrCat(service_name, "RetryTraits");
   vars["retry_traits_header_path"] =
@@ -837,6 +794,12 @@ VarsDictionary CreateServiceVars(
   vars["service_authority_env_var"] =
       absl::StrCat(service_endpoint_env_var_prefix, "_AUTHORITY");
   vars["service_name"] = service_name;
+  vars["sources_cc_path"] =
+      absl::StrCat(vars["product_path"], "internal/",
+                   ServiceNameToFilePath(service_name), "_sources.cc");
+  vars["streaming_cc_path"] =
+      absl::StrCat(vars["product_path"], "internal/",
+                   ServiceNameToFilePath(service_name), "_streaming.cc");
   vars["stub_class_name"] = absl::StrCat(service_name, "Stub");
   vars["stub_cc_path"] =
       absl::StrCat(vars["product_path"], "internal/",
@@ -908,7 +871,8 @@ std::map<std::string, std::string> ParseIdempotencyOverrides(
 
 std::map<std::string, VarsDictionary> CreateMethodVars(
     google::protobuf::ServiceDescriptor const& service,
-    VarsDictionary const& vars) {
+    YAML::Node const& service_config, VarsDictionary const& vars,
+    std::vector<MixinMethod> const& mixin_methods) {
   auto split_arg = [&vars](std::string const& arg) -> std::set<std::string> {
     auto l = vars.find(arg);
     if (l == vars.end()) return {};
@@ -916,40 +880,24 @@ std::map<std::string, VarsDictionary> CreateMethodVars(
     for (auto& a : s) a = SafeReplaceAll(a, "@", ",");
     return {s.begin(), s.end()};
   };
-  auto const emitted_rpcs = split_arg("emitted_rpcs");
   auto const omitted_rpcs = split_arg("omitted_rpcs");
   auto const idempotency_overrides = ParseIdempotencyOverrides(vars);
   std::map<std::string, VarsDictionary> service_methods_vars;
   for (int i = 0; i < service.method_count(); i++) {
     auto const& method = *service.method(i);
-    VarsDictionary method_vars;
-    method_vars["method_return_doxygen_link"] =
-        FormatDoxygenLink(*method.output_type());
-    method_vars["idempotency"] = DefaultIdempotencyFromHttpOperation(method);
-    if (!idempotency_overrides.empty()) {
-      auto iter = idempotency_overrides.find(
-          absl::StrCat(service.name(), ".", method.name()));
-      if (iter != idempotency_overrides.end()) {
-        method_vars["idempotency"] = iter->second;
-      }
+    absl::optional<google::api::HttpRule> http_rule;
+    if (method.options().HasExtension(google::api::http)) {
+      http_rule = method.options().GetExtension(google::api::http);
     }
-    method_vars["method_name"] = method.name();
-    method_vars["method_name_snake"] = CamelCaseToSnakeCase(method.name());
-    method_vars["request_type"] =
-        ProtoNameToCppName(method.input_type()->full_name());
-    method_vars["response_message_type"] = method.output_type()->full_name();
-    method_vars["response_type"] =
-        ProtoNameToCppName(method.output_type()->full_name());
-    SetLongrunningOperationMethodVars(method, method_vars);
-    AssignPaginationMethodVars(method, method_vars);
-    SetMethodSignatureMethodVars(service, method, emitted_rpcs, omitted_rpcs,
-                                 method_vars);
-    auto parsed_http_info = ParseHttpExtension(method);
-    method_vars["request_resource"] =
-        FormatRequestResource(*method.input_type(), parsed_http_info);
-    SetHttpDerivedMethodVars(parsed_http_info, method, method_vars);
-    SetHttpQueryParameters(parsed_http_info, method, method_vars);
-    service_methods_vars[method.full_name()] = method_vars;
+    service_methods_vars[method.full_name()] =
+        GetMethodVars(service, service_config, method, http_rule, "grpc_stub",
+                      idempotency_overrides, omitted_rpcs);
+  }
+  for (auto const& mixin_method : mixin_methods) {
+    auto const& method = mixin_method.method.get();
+    service_methods_vars[method.full_name()] = GetMethodVars(
+        service, service_config, method, mixin_method.http_override,
+        mixin_method.grpc_stub_name, idempotency_overrides, omitted_rpcs);
   }
   return service_methods_vars;
 }
@@ -966,14 +914,15 @@ Status PrintMethod(google::protobuf::MethodDescriptor const& method,
   }
 
   if (matching_patterns.empty())
-    return Status(StatusCode::kNotFound,
-                  absl::StrCat(file, ":", line, ": no matching patterns for: ",
-                               method.full_name()));
+    return internal::NotFoundError(
+        absl::StrCat(file, ":", line,
+                     ": no matching patterns for: ", method.full_name()),
+        GCP_ERROR_INFO());
   if (matching_patterns.size() > 1)
-    return Status(
-        StatusCode::kInternal,
-        absl::StrCat(file, ":", line, ": more than one pattern found for: ",
-                     method.full_name()));
+    return internal::InternalError(
+        absl::StrCat(file, ":", line,
+                     ": more than one pattern found for: ", method.full_name()),
+        GCP_ERROR_INFO());
   for (auto const& f : matching_patterns[0].fragments()) {
     printer.Print(line, file, vars, f(method));
   }
