@@ -86,10 +86,24 @@ char const* const kAnnotationsProto =
     "  HttpRule http = 72295728;\n"
     "}\n";
 
+char const* const kClientProto =
+    "syntax = \"proto3\";\n"
+    "package google.api;\n"
+    "import \"google/protobuf/descriptor.proto\";\n"
+    "extend google.protobuf.MethodOptions {\n"
+    "  repeated string method_signature = 1051;\n"
+    "}\n"
+    "extend google.protobuf.ServiceOptions {\n"
+    "  string default_host = 1049;\n"
+    "  string oauth_scopes = 1050;\n"
+    "  string api_version = 525000001;\n"
+    "}\n";
+
 char const* const kFrobberServiceProto =
     "syntax = \"proto3\";\n"
     "package google.cloud.frobber.v1;\n"
     "import \"google/api/annotations.proto\";\n"
+    "import \"google/api/client.proto\";\n"
     "import \"google/api/http.proto\";\n"
     "// Leading comments about message Bar.\n"
     "message Bar {\n"
@@ -100,6 +114,7 @@ char const* const kFrobberServiceProto =
     "// Leading comments about service FrobberService.\n"
     "// $Delimiter escapes$ $\n"
     "service FrobberService {\n"
+    "  option (google.api.api_version) = \"test-api-version\";\n"
     "  // Leading comments about rpc Method0.\n"
     "  rpc Method0(Bar) returns (Empty) {\n"
     "    option (google.api.http) = {\n"
@@ -108,13 +123,54 @@ char const* const kFrobberServiceProto =
     "  }\n"
     "}\n";
 
+auto constexpr kMixinLocationProto = R"""(
+syntax = "proto3";
+package google.cloud.location;
+import "google/api/annotations.proto";
+import "google/api/http.proto";
+
+message Request {
+  string name = 1;
+}
+message Response {}
+
+service Locations {
+  rpc GetLocation(Request) returns (Response) {
+    option (google.api.http) = {
+      get: "ToBeOverwrittenPath"
+    };
+  }
+
+  rpc ListLocations(Request) returns (Response) {
+    option (google.api.http) = {
+      get: "ToBeOverwrittenPath"
+    };
+  }
+}
+)""";
+
+auto constexpr kGetLocationHttpRule = R"pb(
+  selector: "google.cloud.location.Locations.GetLocation"
+  get: "/v1/{name=projects/*/locations/*}"
+  body: ""
+)pb";
+
+auto constexpr kListLocationsHttpRule = R"pb(
+  selector: "google.cloud.location.Locations.ListLocations"
+  post: "/{name=projects/*/locations}"
+  body: "*"
+)pb";
+
 class CreateServiceVarsTest
     : public testing::TestWithParam<std::pair<std::string, std::string>> {
  public:
   CreateServiceVarsTest()
       : source_tree_(std::map<std::string, std::string>{
+            {std::string("google/api/client.proto"), kClientProto},
             {std::string("google/api/http.proto"), kHttpProto},
             {std::string("google/api/annotations.proto"), kAnnotationsProto},
+            {std::string("google/cloud/location/locations.proto"),
+             kMixinLocationProto},
             {std::string("google/cloud/frobber/v1/frobber.proto"),
              kFrobberServiceProto}}),
         source_tree_db_(&source_tree_),
@@ -141,10 +197,12 @@ class CreateServiceVarsTest
 };
 
 TEST_F(CreateServiceVarsTest, FilesParseSuccessfully) {
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/api/http.proto"));
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/api/annotations.proto"));
-  EXPECT_NE(nullptr,
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/api/http.proto"));
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/api/annotations.proto"));
+  ASSERT_NE(nullptr,
             pool_.FindFileByName("google/cloud/frobber/v1/frobber.proto"));
+  ASSERT_NE(nullptr,
+            pool_.FindFileByName("google/cloud/location/locations.proto"));
 }
 
 TEST_F(CreateServiceVarsTest, RetryStatusCodeExpressionNotFound) {
@@ -200,6 +258,38 @@ TEST_F(CreateServiceVarsTest, ForwardingHeaderPaths) {
                           "google/cloud/frobber/v1/frobber_options.h"))));
 }
 
+TEST_F(CreateServiceVarsTest, MixinProtoHeaderPaths) {
+  FileDescriptor const* file =
+      pool_.FindFileByName("google/cloud/frobber/v1/frobber.proto");
+  FileDescriptor const* mixin_file =
+      pool_.FindFileByName("google/cloud/location/locations.proto");
+
+  google::api::HttpRule http_rule0;
+  ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
+      kGetLocationHttpRule, &http_rule0));
+
+  google::api::HttpRule http_rule1;
+  ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
+      kListLocationsHttpRule, &http_rule1));
+
+  std::vector<MixinMethod> mixin_methods{
+      {"", "", *mixin_file->service(0)->method(0), http_rule0},
+      {"", "", *mixin_file->service(0)->method(1), http_rule1}};
+
+  service_vars_ = CreateServiceVars(
+      *file->service(0),
+      {std::make_pair("product_path", "google/cloud/frobber/"),
+       std::make_pair("additional_proto_files",
+                      "google/cloud/add1.proto,google/cloud/add2.proto")},
+      mixin_methods);
+
+  EXPECT_THAT(service_vars_,
+              AllOf(Contains(Pair("mixin_proto_grpc_header_paths",
+                                  "google/cloud/location/locations.grpc.pb.h")),
+                    Contains(Pair("mixin_proto_header_paths",
+                                  "google/cloud/location/locations.pb.h"))));
+}
+
 TEST_P(CreateServiceVarsTest, KeySetCorrectly) {
   FileDescriptor const* service_file_descriptor =
       pool_.FindFileByName("google/cloud/frobber/v1/frobber.proto");
@@ -219,6 +309,7 @@ INSTANTIATE_TEST_SUITE_P(
         std::make_pair("product_options_page", "google-cloud-frobber-options"),
         std::make_pair("additional_pb_header_paths",
                        "google/cloud/add1.pb.h,google/cloud/add2.pb.h"),
+        std::make_pair("api_version", "test-api-version"),
         std::make_pair("class_comment_block",
                        "///\n/// Leading comments about service "
                        "FrobberService.\n/// $Delimiter escapes$ $\n///"),
@@ -303,6 +394,10 @@ INSTANTIATE_TEST_SUITE_P(
         std::make_pair("service_endpoint_env_var",
                        "GOOGLE_CLOUD_CPP_FROBBER_SERVICE_ENDPOINT"),
         std::make_pair("service_name", "FrobberService"),
+        std::make_pair("sources_cc_path",
+                       "google/cloud/frobber/internal/frobber_sources.cc"),
+        std::make_pair("streaming_cc_path",
+                       "google/cloud/frobber/internal/frobber_streaming.cc"),
         std::make_pair("stub_class_name", "FrobberServiceStub"),
         std::make_pair("stub_cc_path",
                        "google/cloud/frobber/internal/frobber_stub.cc"),
@@ -436,6 +531,8 @@ INSTANTIATE_TEST_SUITE_P(
         std::make_pair("service_endpoint_env_var",
                        "GOOGLE_CLOUD_CPP_FROBBER_SERVICE_ENDPOINT"),
         std::make_pair("service_name", "NewFrobberService"),
+        std::make_pair("sources_cc_path",
+                       "google/cloud/frobber/internal/new_frobber_sources.cc"),
         std::make_pair("stub_class_name", "NewFrobberServiceStub"),
         std::make_pair("stub_cc_path",
                        "google/cloud/frobber/internal/new_frobber_stub.cc"),
@@ -576,6 +673,8 @@ INSTANTIATE_TEST_SUITE_P(
         std::make_pair("service_endpoint_env_var",
                        "GOOGLE_CLOUD_CPP_FROBBER_SERVICE_ENDPOINT"),
         std::make_pair("service_name", "FrobberService"),
+        std::make_pair("sources_cc_path",
+                       "google/cloud/frobber/internal/frobber_sources.cc"),
         std::make_pair("stub_class_name", "FrobberServiceStub"),
         std::make_pair("stub_cc_path",
                        "google/cloud/frobber/internal/frobber_stub.cc"),
@@ -610,18 +709,6 @@ char const* const kIamProto =
     "message TestIamPermissionsRequest {}\n"
     "message TestIamPermissionsResponse {}\n";
 
-char const* const kClientProto =
-    "syntax = \"proto3\";\n"
-    "package google.api;\n"
-    "import \"google/protobuf/descriptor.proto\";\n"
-    "extend google.protobuf.MethodOptions {\n"
-    "  repeated string method_signature = 1051;\n"
-    "}\n"
-    "extend google.protobuf.ServiceOptions {\n"
-    "  string default_host = 1049;\n"
-    "  string oauth_scopes = 1050;\n"
-    "}\n";
-
 char const* const kLongrunningOperationsProto =
     "syntax = \"proto3\";\n"
     "package google.longrunning;\n"
@@ -666,11 +753,32 @@ package google.protobuf;
 message Empty {}
 )""";
 
-char const* const kServiceProto =
+auto constexpr kFieldInfoProto = R"""(
+syntax = "proto3";
+package google.api;
+import "google/protobuf/descriptor.proto";
+
+extend google.protobuf.FieldOptions {
+  google.api.FieldInfo field_info = 291403980;
+}
+message FieldInfo {
+  enum Format {
+    FORMAT_UNSPECIFIED = 0;
+    UUID4 = 1;
+    IPV4 = 2;
+    IPV6 = 3;
+    IPV4_OR_IPV6 = 4;
+  }
+  Format format = 1;
+}
+)""";
+
+auto constexpr kServiceProto =
     "syntax = \"proto3\";\n"
     "package my.service.v1;\n"
     "import \"google/api/annotations.proto\";\n"
     "import \"google/api/client.proto\";\n"
+    "import \"google/api/field_info.proto\";\n"
     "import \"google/api/http.proto\";\n"
     "import \"google/iam/v1/fake_iam.proto\";\n"
     "import \"google/protobuf/well_known.proto\";\n"
@@ -724,7 +832,7 @@ char const* const kServiceProto =
     "}\n"
     "// Leading comments about service Service.\n"
     "service Service {\n"
-    "  // Leading comments about rpc Method0$.\n"
+    "  // Leading comments about rpc Method0.\n"
     "  rpc Method0(Bar) returns (google.protobuf.Empty) {\n"
     "  }\n"
     "  // Leading comments about rpc Method1.\n"
@@ -773,7 +881,7 @@ char const* const kServiceProto =
     "    option (google.api.method_signature) = \"name,swallow_types\";\n"
     "    option (google.api.method_signature) = \"\";\n"
     "  }\n"
-    "  // Leading comments about rpc $Method6.\n"
+    "  // Leading comments about rpc Method6.\n"
     "  rpc Method6(Foo) returns (google.protobuf.Empty) {\n"
     "    option (google.api.http) = {\n"
     "       get: \"/v1/{name=projects/*/instances/*/databases/*}\"\n"
@@ -842,7 +950,20 @@ char const* const kServiceProto =
     "       body: \"*\"\n"
     "    };\n"
     "  }\n"
-    "}\n";
+    // Continue in raw string format.
+    R"""(
+  rpc WithRequestId(WithRequestIdRequest) returns (google.protobuf.Empty) {}
+  rpc WithoutRequestId(WithoutRequestIdRequest) returns (google.protobuf.Empty) {}
+}
+
+message WithRequestIdRequest {
+  string field = 1 [ (google.api.field_info).format = UUID4 ];
+}
+
+message WithoutRequestIdRequest {
+  string field = 1;
+}
+)""";
 
 auto constexpr kExtendedOperationsProto = R"""(
 syntax = "proto3";
@@ -917,6 +1038,7 @@ class CreateMethodVarsTest
   CreateMethodVarsTest()
       : source_tree_(std::map<std::string, std::string>{
             {std::string("google/api/client.proto"), kClientProto},
+            {std::string("google/api/field_info.proto"), kFieldInfoProto},
             {std::string("google/api/http.proto"), kHttpProto},
             {std::string("google/api/annotations.proto"), kAnnotationsProto},
             {std::string("google/iam/v1/fake_iam.proto"), kIamProto},
@@ -927,6 +1049,8 @@ class CreateMethodVarsTest
             {std::string("test/test.proto"), kSourceLocationTestInput},
             {std::string("google/protobuf/well_known.proto"), kWellKnownProto},
             {std::string("google/foo/v1/service.proto"), kServiceProto},
+            {std::string("google/cloud/location/locations.proto"),
+             kMixinLocationProto},
             {std::string("google/foo/v1/http_service.proto"),
              kHttpServiceProto}}),
         source_tree_db_(&source_tree_),
@@ -954,13 +1078,15 @@ class CreateMethodVarsTest
 };
 
 TEST_F(CreateMethodVarsTest, FilesParseSuccessfully) {
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/api/client.proto"));
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/api/http.proto"));
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/api/annotations.proto"));
-  EXPECT_NE(nullptr,
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/api/client.proto"));
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/api/http.proto"));
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/api/annotations.proto"));
+  ASSERT_NE(nullptr,
             pool_.FindFileByName("google/longrunning/operation.proto"));
-  EXPECT_NE(nullptr, pool_.FindFileByName("test/test.proto"));
-  EXPECT_NE(nullptr, pool_.FindFileByName("google/foo/v1/service.proto"));
+  ASSERT_NE(nullptr, pool_.FindFileByName("test/test.proto"));
+  ASSERT_NE(nullptr, pool_.FindFileByName("google/foo/v1/service.proto"));
+  ASSERT_NE(nullptr,
+            pool_.FindFileByName("google/cloud/location/locations.proto"));
 }
 
 TEST_F(CreateMethodVarsTest, FormatMethodCommentsProtobufRequest) {
@@ -971,7 +1097,7 @@ TEST_F(CreateMethodVarsTest, FormatMethodCommentsProtobufRequest) {
       *service_file_descriptor->service(0)->method(0), false);
   EXPECT_EQ(actual, R"""(  // clang-format off
   ///
-  /// Leading comments about rpc Method0$$.
+  /// Leading comments about rpc Method0.
   ///
   /// @param request Unary RPCs, such as the one wrapped by this
   ///     function, receive a single `request` proto message which includes all
@@ -990,7 +1116,7 @@ TEST_F(CreateMethodVarsTest, FormatMethodCommentsProtobufRequest) {
   /// [`future`]: @ref google::cloud::future
   /// [`StatusOr`]: @ref google::cloud::StatusOr
   /// [`Status`]: @ref google::cloud::Status
-  /// [my.service.v1.Bar]: @googleapis_reference_link{google/foo/v1/service.proto#L18}
+  /// [my.service.v1.Bar]: @googleapis_reference_link{google/foo/v1/service.proto#L19}
   ///
   // clang-format on
 )""");
@@ -1033,7 +1159,7 @@ TEST_F(CreateMethodVarsTest,
   /// [`future`]: @ref google::cloud::future
   /// [`StatusOr`]: @ref google::cloud::StatusOr
   /// [`Status`]: @ref google::cloud::Status
-  /// [my.service.v1.Bar]: @googleapis_reference_link{google/foo/v1/service.proto#L18}
+  /// [my.service.v1.Bar]: @googleapis_reference_link{google/foo/v1/service.proto#L19}
   ///
   // clang-format on
 )""");
@@ -1090,7 +1216,7 @@ TEST_F(CreateMethodVarsTest, FormatMethodCommentsMethodSignature) {
       *service_file_descriptor->service(0)->method(6), "labels", false);
   EXPECT_EQ(actual, R"""(  // clang-format off
   ///
-  /// Leading comments about rpc $$Method6.
+  /// Leading comments about rpc Method6.
   ///
   /// @param labels  labels $$field comment.
   /// @param opts Optional. Override the class-level options, such as retry and
@@ -1104,7 +1230,7 @@ TEST_F(CreateMethodVarsTest, FormatMethodCommentsMethodSignature) {
   /// [`future`]: @ref google::cloud::future
   /// [`StatusOr`]: @ref google::cloud::StatusOr
   /// [`Status`]: @ref google::cloud::Status
-  /// [my.service.v1.Foo]: @googleapis_reference_link{google/foo/v1/service.proto#L10}
+  /// [my.service.v1.Foo]: @googleapis_reference_link{google/foo/v1/service.proto#L11}
   ///
   // clang-format on
 )""");
@@ -1119,7 +1245,8 @@ TEST_F(CreateMethodVarsTest, SkipMethodsWithDeprecatedFields) {
           "omitted_rpcs",
           absl::StrCat(SafeReplaceAll(kMethod6Deprecated1, ",", "@"), ",",
                        SafeReplaceAll(kMethod6Deprecated2, ",", "@")))});
-  vars_ = CreateMethodVars(*service_file_descriptor->service(0), service_vars_);
+  vars_ = CreateMethodVars(*service_file_descriptor->service(0), YAML::Node{},
+                           service_vars_);
   auto method_vars = vars_.find("my.service.v1.Service.Method6");
   ASSERT_NE(method_vars, vars_.end());
   EXPECT_THAT(method_vars->second, Not(Contains(Pair("method_signature0", _))));
@@ -1137,7 +1264,8 @@ TEST_F(CreateMethodVarsTest, SkipMethodOverloadsWithDuplicateSignatures) {
           "omitted_rpcs",
           absl::StrCat(SafeReplaceAll(kMethod6Deprecated1, ",", "@"), ",",
                        SafeReplaceAll(kMethod6Deprecated2, ",", "@")))});
-  vars_ = CreateMethodVars(*service_file_descriptor->service(0), service_vars_);
+  vars_ = CreateMethodVars(*service_file_descriptor->service(0), YAML::Node{},
+                           service_vars_);
   auto method_vars = vars_.find("my.service.v1.Service.Method10");
   ASSERT_NE(method_vars, vars_.end());
   EXPECT_THAT(method_vars->second, Contains(Pair("method_signature0", _)));
@@ -1146,6 +1274,124 @@ TEST_F(CreateMethodVarsTest, SkipMethodOverloadsWithDuplicateSignatures) {
   EXPECT_THAT(method_vars->second, Not(Contains(Pair("method_signature3", _))));
 }
 
+TEST_F(CreateMethodVarsTest, WithRequestId) {
+  auto constexpr kServiceConfigYaml = R"""(publishing:
+  method_settings:
+  - selector: my.service.v1.Service.WithRequestId
+    auto_populated_fields:
+    - field
+)""";
+  auto const service_config = YAML::Load(kServiceConfigYaml);
+  ASSERT_EQ(service_config.Type(), YAML::NodeType::Map);
+
+  FileDescriptor const* service_file_descriptor =
+      pool_.FindFileByName("google/foo/v1/service.proto");
+  service_vars_ = CreateServiceVars(
+      *service_file_descriptor->service(0),
+      {std::make_pair(
+          "omitted_rpcs",
+          absl::StrCat(SafeReplaceAll(kMethod6Deprecated1, ",", "@"), ",",
+                       SafeReplaceAll(kMethod6Deprecated2, ",", "@")))});
+  vars_ = CreateMethodVars(*service_file_descriptor->service(0), service_config,
+                           service_vars_);
+  auto mv0 = vars_.find("my.service.v1.Service.WithRequestId");
+  ASSERT_NE(mv0, vars_.end());
+  EXPECT_THAT(mv0->second, Contains(Pair("request_id_field_name", "field")));
+
+  auto mv1 = vars_.find("my.service.v1.Service.WithoutRequestId");
+  ASSERT_NE(mv1, vars_.end());
+  EXPECT_THAT(mv1->second, Not(Contains(Pair("request_id_field_name", _))));
+}
+
+TEST_F(CreateMethodVarsTest, CreateMixinMethodsVars) {
+  FileDescriptor const* service_file_descriptor =
+      pool_.FindFileByName("google/foo/v1/service.proto");
+  FileDescriptor const* mixin_file =
+      pool_.FindFileByName("google/cloud/location/locations.proto");
+
+  google::api::HttpRule http_rule0;
+  ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
+      kGetLocationHttpRule, &http_rule0));
+
+  google::api::HttpRule http_rule1;
+  ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
+      kListLocationsHttpRule, &http_rule1));
+
+  std::vector<MixinMethod> mixin_methods{
+      {"stub1", "", *mixin_file->service(0)->method(0), http_rule0},
+      {"stub2", "", *mixin_file->service(0)->method(1), http_rule1}};
+
+  service_vars_ = CreateServiceVars(
+      *service_file_descriptor->service(0),
+      {std::make_pair(
+          "omitted_rpcs",
+          absl::StrCat(SafeReplaceAll(kMethod6Deprecated1, ",", "@"), ",",
+                       SafeReplaceAll(kMethod6Deprecated2, ",", "@")))},
+      mixin_methods);
+  vars_ = CreateMethodVars(*service_file_descriptor->service(0), YAML::Node{},
+                           service_vars_, mixin_methods);
+
+  EXPECT_THAT(
+      vars_,
+      AllOf(
+          Contains(Pair(
+              "google.cloud.location.Locations.GetLocation",
+              AllOf(Contains(Pair("idempotency", "kIdempotent")),
+                    Contains(Pair("method_http_query_parameters", "")),
+                    Contains(Pair("method_http_verb", "Get")),
+                    Contains(Pair("grpc_stub", "stub1_")),
+                    Contains(Pair("method_name", "GetLocation")),
+                    Contains(Pair("method_name_snake", "get_location")),
+                    Contains(Pair("method_request_body", "")),
+                    Contains(
+                        Pair("method_request_params",
+                             "\"name=\", internal::UrlEncode(request.name())")),
+                    Contains(Pair("method_rest_path",
+                                  "absl::StrCat(\"/\", "
+                                  "rest_internal::DetermineApiVersion(\"v1\", "
+                                  "options), \"/\", request.name())")),
+                    Contains(Pair("method_rest_path_async",
+                                  "absl::StrCat(\"/\", "
+                                  "rest_internal::DetermineApiVersion(\"v1\", "
+                                  "*options), \"/\", request.name())")),
+                    Contains(Pair("request_resource", "request")),
+                    Contains(Pair("request_type",
+                                  "google::cloud::location::Request")),
+                    Contains(Pair("response_message_type",
+                                  "google.cloud.location.Response")),
+                    Contains(Pair("response_type",
+                                  "google::cloud::location::Response")),
+                    Contains(
+                        Pair("return_type",
+                             "StatusOr<google::cloud::location::Response>"))))),
+          Contains(Pair(
+              "google.cloud.location.Locations.ListLocations",
+              AllOf(Contains(Pair("idempotency", "kNonIdempotent")),
+                    Contains(Pair("method_http_query_parameters", "")),
+                    Contains(Pair("method_http_verb", "Post")),
+                    Contains(Pair("grpc_stub", "stub2_")),
+                    Contains(Pair("method_name", "ListLocations")),
+                    Contains(Pair("method_name_snake", "list_locations")),
+                    Contains(Pair("method_request_body", "*")),
+                    Contains(
+                        Pair("method_request_params",
+                             "\"name=\", internal::UrlEncode(request.name())")),
+                    Contains(Pair("method_rest_path",
+                                  "absl::StrCat(\"/\", request.name())")),
+                    Contains(Pair("method_rest_path_async",
+                                  "absl::StrCat(\"/\", request.name())")),
+                    Contains(Pair("request_resource", "request")),
+                    Contains(Pair("request_type",
+                                  "google::cloud::location::Request")),
+                    Contains(Pair("response_message_type",
+                                  "google.cloud.location.Response")),
+                    Contains(Pair("response_type",
+                                  "google::cloud::location::Response")),
+                    Contains(Pair(
+                        "return_type",
+                        "StatusOr<google::cloud::location::Response>")))))));
+}
+
 TEST_P(CreateMethodVarsTest, KeySetCorrectly) {
   FileDescriptor const* service_file_descriptor =
       pool_.FindFileByName("google/foo/v1/service.proto");
@@ -1155,7 +1401,8 @@ TEST_P(CreateMethodVarsTest, KeySetCorrectly) {
           "omitted_rpcs",
           absl::StrCat(SafeReplaceAll(kMethod6Deprecated1, ",", "@"), ",",
                        SafeReplaceAll(kMethod6Deprecated2, ",", "@")))});
-  vars_ = CreateMethodVars(*service_file_descriptor->service(0), service_vars_);
+  vars_ = CreateMethodVars(*service_file_descriptor->service(0), YAML::Node{},
+                           service_vars_);
   auto method_iter = vars_.find(GetParam().method);
   ASSERT_TRUE(method_iter != vars_.end());
   EXPECT_THAT(method_iter->second,
@@ -1170,21 +1417,18 @@ INSTANTIATE_TEST_SUITE_P(
                              "Method0"),
         MethodVarsTestValues("my.service.v1.Service.Method0",
                              "method_name_snake", "method0"),
+        MethodVarsTestValues("my.service.v1.Service.Method0", "grpc_stub",
+                             "grpc_stub_"),
         MethodVarsTestValues("my.service.v1.Service.Method0", "request_type",
                              "my::service::v1::Bar"),
         MethodVarsTestValues("my.service.v1.Service.Method0",
                              "response_message_type", "google.protobuf.Empty"),
         MethodVarsTestValues("my.service.v1.Service.Method0", "response_type",
                              "google::protobuf::Empty"),
+        MethodVarsTestValues("my.service.v1.Service.Method0", "return_type",
+                             "Status"),
         MethodVarsTestValues("my.service.v1.Service.Method0", "idempotency",
                              "kNonIdempotent"),
-        MethodVarsTestValues("my.service.v1.Service.Method0",
-                             "method_return_doxygen_link",
-                             "@googleapis_link{google::protobuf::Empty,google/"
-                             "protobuf/well_known.proto#L5}"),
-        MethodVarsTestValues("my.service.v1.Service.Method0",
-                             "method_http_query_parameters",
-                             "my.service.v1.Service.Method0"),
         // Method1
         MethodVarsTestValues("my.service.v1.Service.Method1", "method_name",
                              "Method1"),
@@ -1194,16 +1438,8 @@ INSTANTIATE_TEST_SUITE_P(
                              "my::service::v1::Bar"),
         MethodVarsTestValues("my.service.v1.Service.Method1", "response_type",
                              "my::service::v1::Bar"),
-        MethodVarsTestValues("my.service.v1.Service.Method1",
-                             "method_return_doxygen_link",
-                             "@googleapis_link{my::service::v1::Bar,google/"
-                             "foo/v1/service.proto#L18}"),
-        MethodVarsTestValues("my.service.v1.Service.Method1",
-                             "method_http_query_parameters", R"""(,
-      rest_internal::TrimEmptyQueryParameters({std::make_pair("number", std::to_string(request.number())),
-        std::make_pair("toggle", request.toggle() ? "1" : "0"),
-        std::make_pair("title", request.title()),
-        std::make_pair("parent", request.parent())}))"""),
+        MethodVarsTestValues("my.service.v1.Service.Method1", "return_type",
+                             "StatusOr<my::service::v1::Bar>"),
         // Method2
         MethodVarsTestValues("my.service.v1.Service.Method2",
                              "longrunning_metadata_type",
@@ -1225,13 +1461,9 @@ INSTANTIATE_TEST_SUITE_P(
         MethodVarsTestValues("my.service.v1.Service.Method2",
                              "method_longrunning_deduced_return_doxygen_link",
                              "@googleapis_link{my::service::v1::Bar,google/"
-                             "foo/v1/service.proto#L18}"),
+                             "foo/v1/service.proto#L19}"),
         MethodVarsTestValues("my.service.v1.Service.Method2",
-                             "method_http_query_parameters", R"""(,
-      rest_internal::TrimEmptyQueryParameters({std::make_pair("number", std::to_string(request.number())),
-        std::make_pair("name", request.name()),
-        std::make_pair("toggle", request.toggle() ? "1" : "0"),
-        std::make_pair("title", request.title())}))"""),
+                             "method_http_query_parameters", ""),
         // Method3
         MethodVarsTestValues("my.service.v1.Service.Method3",
                              "longrunning_metadata_type",
@@ -1327,7 +1559,7 @@ INSTANTIATE_TEST_SUITE_P(
         MethodVarsTestValues("my.service.v1.Service.Method7",
                              "method_longrunning_deduced_return_doxygen_link",
                              "@googleapis_link{my::service::v1::Bar,google/"
-                             "foo/v1/service.proto#L18}"),
+                             "foo/v1/service.proto#L19}"),
         // Method8
         MethodVarsTestValues("my.service.v1.Service.Method8",
                              "method_signature0",
@@ -1346,11 +1578,7 @@ INSTANTIATE_TEST_SUITE_P(
             R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/", request.namespace_().name()))"""),
         // Method9
         MethodVarsTestValues("my.service.v1.Service.Method9",
-                             "method_http_query_parameters",
-                             R"""(,
-      rest_internal::TrimEmptyQueryParameters({std::make_pair("page_size", std::to_string(request.page_size())),
-        std::make_pair("page_token", request.page_token()),
-        std::make_pair("name", request.name())}))"""),
+                             "method_http_query_parameters", ""),
         // Method11
         MethodVarsTestValues("my.service.v1.Service.Method11",
                              "request_resource", "request.foo_resource()"),
