@@ -13,7 +13,9 @@
 // limitations under the License.
 
 #include "generator/internal/discovery_file.h"
+#include "generator/internal/codegen_utils.h"
 #include "generator/testing/descriptor_pool_fixture.h"
+#include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 
@@ -22,6 +24,7 @@ namespace cloud {
 namespace generator_internal {
 namespace {
 
+using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::Eq;
 using ::testing::HasSubstr;
@@ -141,8 +144,9 @@ auto constexpr kGetRequestTypeJson = R"""({
 auto constexpr kOperationTypeJson = R"""({})""";
 
 TEST_F(DiscoveryFileTest, FormatFileWithImport) {
-  auto constexpr kExpectedProto = R"""(// Copyright 2023 Google LLC
-//
+  std::string const expected_proto_copyright =
+      absl::StrCat("// Copyright ", CurrentCopyrightYear(), " Google LLC");
+  auto constexpr kExpectedProto = R"""(//
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -242,6 +246,7 @@ message GetMyResourceRequest {
   DiscoveryTypeVertex get_response("MyResource", "other.package", {}, &pool());
   r.AddResponseType("Operation", &operation_type);
   r.AddResponseType("MyResource", &get_response);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   DiscoveryFile f(&r, "my_path", "my_relative_proto_path", "my.package.name",
                   r.GetRequestTypesList());
   f.AddImportPath("path/to/import.proto");
@@ -249,21 +254,24 @@ message GetMyResourceRequest {
   types.emplace("Foo",
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
-  DiscoveryDocumentProperties document_properties{"my/service",
-                                                  "https://default.host",
-                                                  "my_product",
-                                                  "v1",
-                                                  "19700101",
-                                                  "file:///my_url",
-                                                  {}};
+  DiscoveryDocumentProperties document_properties{
+      "my/service", "https://default.host",
+      "my_product", "v1",
+      "19700101",   "file:///my_url",
+      {},           CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   ASSERT_STATUS_OK(result);
-  EXPECT_THAT(os.str(), Eq(kExpectedProto));
+  std::string copyright_line;
+  std::getline(os, copyright_line);
+  EXPECT_THAT(copyright_line, Eq(expected_proto_copyright));
+  std::string rest(std::istreambuf_iterator<char>(os), {});
+  EXPECT_THAT(rest, Eq(kExpectedProto));
 }
 
 TEST_F(DiscoveryFileTest, FormatFileWithoutImports) {
-  auto constexpr kExpectedProto = R"""(// Copyright 2023 Google LLC
-//
+  std::string const expected_proto_copyright =
+      absl::StrCat("// Copyright ", CurrentCopyrightYear(), " Google LLC");
+  auto constexpr kExpectedProto = R"""(//
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -360,27 +368,31 @@ message GetMyResourceRequest {
   DiscoveryTypeVertex get_response("MyResource", "other.package", {}, &pool());
   r.AddResponseType("Operation", &operation_type);
   r.AddResponseType("MyResource", &get_response);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   DiscoveryFile f(&r, "my_path", "my_relative_proto_path", "my.package.name",
                   r.GetRequestTypesList());
   std::map<std::string, DiscoveryTypeVertex> types;
   types.emplace("Foo",
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
-  DiscoveryDocumentProperties document_properties{"my/service",
-                                                  "https://default.host",
-                                                  "my_product",
-                                                  "v1",
-                                                  "19700101",
-                                                  "file:///my_url",
-                                                  {}};
+  DiscoveryDocumentProperties document_properties{
+      "my/service", "https://default.host",
+      "my_product", "v1",
+      "19700101",   "file:///my_url",
+      {},           CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   ASSERT_STATUS_OK(result);
-  EXPECT_THAT(os.str(), Eq(kExpectedProto));
+  std::string copyright_line;
+  std::getline(os, copyright_line);
+  EXPECT_THAT(copyright_line, Eq(expected_proto_copyright));
+  std::string rest(std::istreambuf_iterator<char>(os), {});
+  EXPECT_THAT(rest, Eq(kExpectedProto));
 }
 
 TEST_F(DiscoveryFileTest, FormatFileNoResource) {
-  auto constexpr kExpectedProto = R"""(// Copyright 2023 Google LLC
-//
+  std::string const expected_proto_copyright =
+      absl::StrCat("// Copyright ", CurrentCopyrightYear(), " Google LLC");
+  auto constexpr kExpectedProto = R"""(//
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -443,10 +455,17 @@ message GetMyResourceRequest {
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
   DiscoveryDocumentProperties document_properties{
-      "", "", "my_product", "v1", "19700101", "file:///my_url", {}};
+      "",           "",
+      "my_product", "v1",
+      "19700101",   "file:///my_url",
+      {},           CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   ASSERT_STATUS_OK(result);
-  EXPECT_THAT(os.str(), Eq(kExpectedProto));
+  std::string copyright_line;
+  std::getline(os, copyright_line);
+  EXPECT_THAT(copyright_line, Eq(expected_proto_copyright));
+  std::string rest(std::istreambuf_iterator<char>(os), {});
+  EXPECT_THAT(rest, Eq(kExpectedProto));
 }
 
 TEST_F(DiscoveryFileTest, FormatFileNoTypes) {
@@ -462,8 +481,9 @@ TEST_F(DiscoveryFileTest, FormatFileNoTypes) {
   }
 })""";
 
-  auto constexpr kExpectedProto = R"""(// Copyright 2023 Google LLC
-//
+  std::string const expected_proto_copyright =
+      absl::StrCat("// Copyright ", CurrentCopyrightYear(), " Google LLC");
+  auto constexpr kExpectedProto = R"""(//
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -503,22 +523,25 @@ service MyResources {
   auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
   ASSERT_TRUE(resource_json.is_object());
   DiscoveryResource r("myResources", "", resource_json);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   DiscoveryFile f(&r, "my_path", "my_relative_proto_path", "my.package.name",
                   r.GetRequestTypesList());
   std::map<std::string, DiscoveryTypeVertex> types;
   types.emplace("Foo",
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
-  DiscoveryDocumentProperties document_properties{"my/service",
-                                                  "https://default.host",
-                                                  "my_product",
-                                                  "v1",
-                                                  "19700101",
-                                                  "file:///my_url",
-                                                  {}};
+  DiscoveryDocumentProperties document_properties{
+      "my/service", "https://default.host",
+      "my_product", "v1",
+      "19700101",   "file:///my_url",
+      {},           CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   ASSERT_STATUS_OK(result);
-  EXPECT_THAT(os.str(), Eq(kExpectedProto));
+  std::string copyright_line;
+  std::getline(os, copyright_line);
+  EXPECT_THAT(copyright_line, Eq(expected_proto_copyright));
+  std::string rest(std::istreambuf_iterator<char>(os), {});
+  EXPECT_THAT(rest, Eq(kExpectedProto));
 }
 
 TEST_F(DiscoveryFileTest, FormatFileResourceScopeError) {
@@ -565,6 +588,7 @@ TEST_F(DiscoveryFileTest, FormatFileResourceScopeError) {
                                        get_request_type_json, &pool());
   r.AddRequestType("DoFooRequest", &do_foo_request_type);
   r.AddRequestType("GetMyResourceRequest", &get_request_type);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   DiscoveryFile f(&r, "my_path", "my_relative_proto_path", "my.package.name",
                   r.GetRequestTypesList());
   f.AddImportPath("path/to/import.proto");
@@ -573,7 +597,7 @@ TEST_F(DiscoveryFileTest, FormatFileResourceScopeError) {
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
   DiscoveryDocumentProperties document_properties{
-      "", "", "my_product", "v1", "", "", {}};
+      "", "", "my_product", "v1", "", "", {}, CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   EXPECT_THAT(result,
               StatusIs(StatusCode::kInvalidArgument, HasSubstr("scope")));
@@ -624,6 +648,7 @@ TEST_F(DiscoveryFileTest, FormatFileTypeMissingError) {
   DiscoveryTypeVertex get_response("MyResource", "other.package", {}, &pool());
   r.AddResponseType("Operation", &operation_type);
   r.AddResponseType("MyResource", &get_response);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   DiscoveryFile f(&r, "my_path", "my_relative_proto_path", "my.package.name",
                   r.GetRequestTypesList());
   f.AddImportPath("path/to/import.proto");
@@ -632,7 +657,7 @@ TEST_F(DiscoveryFileTest, FormatFileTypeMissingError) {
                 DiscoveryTypeVertex{"Foo", "my.package.name", {}, &pool()});
   std::stringstream os;
   DiscoveryDocumentProperties document_properties{
-      "", "", "my_product", "v1", "", "", {}};
+      "", "", "my_product", "v1", "", "", {}, CurrentCopyrightYear()};
   auto result = f.FormatFile(document_properties, types, os);
   EXPECT_THAT(result, StatusIs(StatusCode::kInvalidArgument,
                                HasSubstr("neither $ref nor type")));
