@@ -22,9 +22,12 @@ namespace cloud {
 namespace generator_internal {
 namespace {
 
+using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::Eq;
 using ::testing::HasSubstr;
+using ::testing::IsEmpty;
 
 class DiscoveryResourceTest : public generator_testing::DescriptorPoolFixture {
 };
@@ -58,6 +61,74 @@ TEST_F(DiscoveryResourceTest, RequiresLROImport) {
   EXPECT_TRUE(r.RequiresLROImport());
 }
 
+TEST_F(DiscoveryResourceTest, GetServiceApiVersionEmpty) {
+  auto constexpr kResourceJson = R"""({
+  "methods": {
+    "emptyResponseMethod1": {
+    },
+    "emptyResponseMethod2": {
+    }
+  }
+})""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  DiscoveryResource r("myTests", "", resource_json);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
+  EXPECT_THAT(r.GetServiceApiVersion(), IsOkAndHolds(IsEmpty()));
+}
+
+TEST_F(DiscoveryResourceTest, GetServiceApiVersionSameVersion) {
+  auto constexpr kResourceJson = R"""({
+  "methods": {
+    "emptyResponseMethod1": {
+      "apiVersion": "test-api-version"
+    },
+    "emptyResponseMethod2": {
+      "apiVersion": "test-api-version"
+    }
+  }
+})""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  DiscoveryResource r("myTests", "", resource_json);
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
+  EXPECT_THAT(r.GetServiceApiVersion(), IsOkAndHolds("test-api-version"));
+}
+
+TEST_F(DiscoveryResourceTest, GetServiceApiVersionDifferentVersion) {
+  auto constexpr kResourceJson = R"""({
+  "methods": {
+    "emptyResponseMethod1": {
+      "apiVersion": "test-api-version"
+    },
+    "emptyResponseMethod2": {
+      "apiVersion": "other-test-api-version"
+    }
+  }
+})""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  DiscoveryResource r("myTests", "", resource_json);
+  auto service_api_version = r.SetServiceApiVersion();
+  EXPECT_THAT(
+      service_api_version,
+      StatusIs(
+          StatusCode::kInvalidArgument,
+          HasSubstr(
+              "resource contains methods with different apiVersion values")));
+}
+
+TEST_F(DiscoveryResourceTest, GetServiceApiVersionNoMethods) {
+  auto constexpr kResourceJson = R"""({})""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  DiscoveryResource r("myTests", "", resource_json);
+  auto service_api_version = r.SetServiceApiVersion();
+  EXPECT_THAT(service_api_version,
+              StatusIs(StatusCode::kInvalidArgument,
+                       HasSubstr("resource contains no methods")));
+}
+
 TEST_F(DiscoveryResourceTest, FormatUrlPath) {
   EXPECT_THAT(DiscoveryResource::FormatUrlPath("base/path/test"),
               Eq("base/path/test"));
@@ -74,6 +145,9 @@ TEST_F(DiscoveryResourceTest, FormatUrlPath) {
           "projects/{project}/zones/{zoneName}/myTests/{fooId}:method1"),
       Eq("projects/{project}/zones/{zone_name}/myTests/"
          "{foo_id}:method1"));
+  EXPECT_THAT(
+      DiscoveryResource::FormatUrlPath("projects/{+project}/zones/{+zoneName}"),
+      Eq("projects/{project}/zones/{zone_name}"));
 }
 
 TEST_F(DiscoveryResourceTest, FormatRpcOptionsGetRegion) {
@@ -142,6 +216,45 @@ TEST_F(DiscoveryResourceTest, FormatRpcOptionsPatchZoneNoUpdateMaskParam) {
   EXPECT_THAT(*options, Eq(kExpectedProto));
 }
 
+TEST_F(DiscoveryResourceTest, FormatRpcOptionsReplacePeriodsWithUnderscore) {
+  auto constexpr kTypeJson = R"""({
+  "request_resource_field_name": "my_request_resource"
+})""";
+  auto constexpr kMethodJson = R"""({
+  "path": "projects/{project}/zones/{zone}/myTests/{fooId}/method1",
+  "httpMethod": "PATCH",
+  "response": {
+    "$ref": "Operation"
+  },
+  "parameters":  {
+    "project.snakeCaseName": {},
+    "zone": {},
+    "fooId": {}
+  },
+  "parameterOrder": [
+    "project.snakeCaseName",
+    "zone",
+    "fooId"
+  ]
+})""";
+  auto constexpr kExpectedProto =
+      R"""(    option (google.api.http) = {
+      patch: "base/path/projects/{project}/zones/{zone}/myTests/{foo_id}/method1"
+      body: "my_request_resource"
+    };
+    option (google.api.method_signature) = "project_snake_case_name,zone,foo_id,my_request_resource";
+    option (google.cloud.operation_service) = "ZoneOperations";)""";
+  auto method_json = nlohmann::json::parse(kMethodJson, nullptr, false);
+  ASSERT_TRUE(method_json.is_object());
+  auto type_json = nlohmann::json::parse(kTypeJson, nullptr, false);
+  ASSERT_TRUE(type_json.is_object());
+  DiscoveryResource r("myTests", "", method_json);
+  DiscoveryTypeVertex t("myType", "", type_json, &pool());
+  auto options = r.FormatRpcOptions(method_json, "base/path", {}, &t);
+  ASSERT_STATUS_OK(options);
+  EXPECT_THAT(*options, Eq(kExpectedProto));
+}
+
 TEST_F(DiscoveryResourceTest, FormatRpcOptionsPatchZoneUpdateMaskParam) {
   auto constexpr kTypeJson = R"""({
   "request_resource_field_name": "my_request_resource"
@@ -574,6 +687,118 @@ TEST_F(DiscoveryResourceTest, JsonToProtobufService) {
   auto constexpr kGetRequestTypeJson = R"""({})""";
   auto constexpr kDoFooRequestTypeJson = R"""({
   "request_resource_field_name": "my_foo_resource"
+})""";
+  auto constexpr kResourceJson = R"""({
+    "methods": {
+      "get": {
+        "description": "Description for the get method.",
+        "scopes": [
+          "https://www.googleapis.com/auth/cloud-platform"
+        ],
+        "apiVersion": "test-api-version",
+        "path": "projects/{project}/regions/{region}/myResources/{foo}",
+        "httpMethod": "GET",
+        "parameters": {
+          "project": {
+            "type": "string"
+          }
+        },
+        "response": {
+          "$ref": "MyResource"
+        },
+        "parameterOrder": [
+          "project",
+          "region",
+          "foo"
+        ]
+      },
+      "doFoo": {
+        "scopes": [
+          "https://www.googleapis.com/auth/cloud-platform"
+        ],
+        "apiVersion": "test-api-version",
+        "path": "projects/{project}/zones/{zone}/myResources/{fooId}/doFoo",
+        "httpMethod": "POST",
+        "parameters": {
+          "project": {
+            "type": "string"
+          }
+        },
+        "response": {
+          "$ref": "Operation"
+        },
+        "parameterOrder": [
+          "project",
+          "zone",
+          "fooId"
+        ]
+      }
+    }
+})""";
+  auto constexpr kExpectedProto =
+      R"""(// Service for the myResources resource.
+// https://cloud.google.com/$product_name$/docs/reference/rest/$version$/myResources
+service MyResources {
+  option (google.api.default_host) = "https://my.endpoint.com";
+  option (google.api.api_version) = "test-api-version";
+  option (google.api.oauth_scopes) =
+    "https://www.googleapis.com/auth/cloud-platform";
+
+  // https://cloud.google.com/$product_name$/docs/reference/rest/$version$/myResources/doFoo
+  rpc DoFoo(DoFooRequest) returns (other.package.Operation) {
+    option (google.api.http) = {
+      post: "base/path/projects/{project}/zones/{zone}/myResources/{foo_id}/doFoo"
+      body: "my_foo_resource"
+    };
+    option (google.api.method_signature) = "project,zone,foo_id,my_foo_resource";
+    option (google.cloud.operation_service) = "ZoneOperations";
+  }
+
+  // Description for the get method.
+  // https://cloud.google.com/$product_name$/docs/reference/rest/$version$/myResources/get
+  rpc GetMyResource(GetMyResourceRequest) returns (MyResource) {
+    option (google.api.http) = {
+      get: "base/path/projects/{project}/regions/{region}/myResources/{foo}"
+    };
+    option (google.api.method_signature) = "project,region,foo";
+  }
+}
+)""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  auto operation_type_json =
+      nlohmann::json::parse(kOperationTypeJson, nullptr, false);
+  ASSERT_TRUE(operation_type_json.is_object());
+  auto get_request_type_json =
+      nlohmann::json::parse(kGetRequestTypeJson, nullptr, false);
+  ASSERT_TRUE(get_request_type_json.is_object());
+  auto do_foo_request_type_json =
+      nlohmann::json::parse(kDoFooRequestTypeJson, nullptr, false);
+  ASSERT_TRUE(do_foo_request_type_json.is_object());
+  DiscoveryResource r("myResources", "this.package", resource_json);
+  DiscoveryTypeVertex t("GetMyResourceRequest", "this.package",
+                        get_request_type_json, &pool());
+  DiscoveryTypeVertex t2("DoFooRequest", "this.package",
+                         do_foo_request_type_json, &pool());
+  DiscoveryTypeVertex response("MyResource", "this.package", {}, &pool());
+  r.AddRequestType("GetMyResourceRequest", &t);
+  r.AddRequestType("DoFooRequest", &t2);
+  r.AddResponseType("MyResource", &response);
+  DiscoveryTypeVertex t3("Operation", "other.package", operation_type_json,
+                         &pool());
+  r.AddResponseType("Operation", &t3);
+  DiscoveryDocumentProperties document_properties{
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
+  auto emitted_proto = r.JsonToProtobufService(document_properties);
+  EXPECT_THAT(emitted_proto, IsOkAndHolds(kExpectedProto));
+}
+
+TEST_F(DiscoveryResourceTest, JsonToProtobufServiceNoApiVersion) {
+  auto constexpr kOperationTypeJson = R"""({})""";
+  auto constexpr kGetRequestTypeJson = R"""({})""";
+  auto constexpr kDoFooRequestTypeJson = R"""({
+  "request_resource_field_name": "my_foo_resource"
 })""";
   auto constexpr kResourceJson = R"""({
     "methods": {
@@ -672,7 +897,8 @@ service MyResources {
                          &pool());
   r.AddResponseType("Operation", &t3);
   DiscoveryDocumentProperties document_properties{
-      "base/path", "https://my.endpoint.com", "", "", "", "", {}};
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   auto emitted_proto = r.JsonToProtobufService(document_properties);
   ASSERT_STATUS_OK(emitted_proto);
   EXPECT_THAT(*emitted_proto, Eq(kExpectedProto));
@@ -703,7 +929,8 @@ TEST_F(DiscoveryResourceTest, JsonToProtobufServiceMissingOAuthScopes) {
                         get_request_type_json, &pool());
   r.AddRequestType("GetMyResourcesRequest", &t);
   DiscoveryDocumentProperties document_properties{
-      "base/path", "https://my.endpoint.com", "", "", "", "", {}};
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   auto emitted_proto = r.JsonToProtobufService(document_properties);
   EXPECT_THAT(
       emitted_proto,
@@ -740,7 +967,8 @@ TEST_F(DiscoveryResourceTest, JsonToProtobufServiceMissingRequestType) {
   ASSERT_TRUE(resource_json.is_object());
   DiscoveryResource r("myResources", "this.package", resource_json);
   DiscoveryDocumentProperties document_properties{
-      "base/path", "https://my.endpoint.com", "", "", "", "", {}};
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   auto emitted_proto = r.JsonToProtobufService(document_properties);
   EXPECT_THAT(
       emitted_proto,
@@ -781,7 +1009,8 @@ service MyResources {
   ASSERT_TRUE(resource_json.is_object());
   DiscoveryResource r("myResources", "this.package", resource_json);
   DiscoveryDocumentProperties document_properties{
-      "base/path", "https://my.endpoint.com", "", "", "", "", {}};
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   auto emitted_proto = r.JsonToProtobufService(document_properties);
   ASSERT_STATUS_OK(emitted_proto);
   EXPECT_THAT(*emitted_proto, Eq(kExpectedProto));
@@ -814,7 +1043,8 @@ TEST_F(DiscoveryResourceTest, JsonToProtobufServiceErrorFormattingRpcOptions) {
                         get_request_type_json, &pool());
   r.AddRequestType("GetMyResourcesRequest", &t);
   DiscoveryDocumentProperties document_properties{
-      "base/path", "https://my.endpoint.com", "", "", "", "", {}};
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.SetServiceApiVersion(), IsOk());
   auto emitted_proto = r.JsonToProtobufService(document_properties);
   EXPECT_THAT(
       emitted_proto,
@@ -822,6 +1052,40 @@ TEST_F(DiscoveryResourceTest, JsonToProtobufServiceErrorFormattingRpcOptions) {
                HasSubstr("Method does not define httpMethod and/or path.")));
 }
 
+TEST_F(DiscoveryResourceTest, JsonToProtobufServiceCalledWithoutApiVersionSet) {
+  auto constexpr kGetRequestTypeJson = R"""({})""";
+  auto constexpr kResourceJson = R"""({
+    "methods": {
+      "get": {
+        "scopes": [
+          "https://www.googleapis.com/auth/cloud-platform"
+        ],
+        "path": "projects/{project}/regions/{region}/myResources/{foo}",
+        "parameterOrder": [
+          "project",
+          "region",
+          "foo"
+        ]
+      }
+    }
+})""";
+  auto resource_json = nlohmann::json::parse(kResourceJson, nullptr, false);
+  ASSERT_TRUE(resource_json.is_object());
+  auto get_request_type_json =
+      nlohmann::json::parse(kGetRequestTypeJson, nullptr, false);
+  ASSERT_TRUE(get_request_type_json.is_object());
+  DiscoveryResource r("myResources", "this.package", resource_json);
+  DiscoveryTypeVertex t("GetMyResourcesRequest", "this.package",
+                        get_request_type_json, &pool());
+  r.AddRequestType("GetMyResourcesRequest", &t);
+  DiscoveryDocumentProperties document_properties{
+      "base/path", "https://my.endpoint.com", "", "", "", "", {}, "2023"};
+  EXPECT_THAT(r.JsonToProtobufService(document_properties),
+              StatusIs(StatusCode::kInternal,
+                       HasSubstr("SetServiceApiVersion must be called before "
+                                 "JsonToProtobufService is called")));
+}
+
 }  // namespace
 }  // namespace generator_internal
 }  // namespace cloud
