@@ -83,7 +83,9 @@ void ApplyResourceLabelsToTypesHelper(std::string const& resource_name,
   type.AddNeededByResource(resource_name);
   auto deps = type.needs_type();
   for (auto* dep : deps) {
-    ApplyResourceLabelsToTypesHelper(resource_name, *dep);
+    if (!internal::Contains(dep->needed_by_resource(), resource_name)) {
+      ApplyResourceLabelsToTypesHelper(resource_name, *dep);
+    }
   }
 }
 
@@ -130,7 +132,8 @@ StatusOr<std::map<std::string, DiscoveryTypeVertex>> ExtractTypesFromSchema(
   }
 
   auto const& schemas = discovery_doc["schemas"];
-  bool schemas_all_type_object = true;
+  std::vector<std::string> recognized_types = {"object", "any"};
+  bool schemas_all_recognized_types = true;
   bool schemas_all_have_id = true;
   std::string id;
   for (auto const& s : schemas) {
@@ -142,9 +145,10 @@ StatusOr<std::map<std::string, DiscoveryTypeVertex>> ExtractTypesFromSchema(
     }
     id = s["id"];
     std::string type = s.value("type", "untyped");
-    if (type != "object") {
-      GCP_LOG(ERROR) << id << " not type:object; is instead " << type;
-      schemas_all_type_object = false;
+    if (!internal::Contains(recognized_types, type)) {
+      GCP_LOG(ERROR) << id << " type is not in `recognized_types`; is instead "
+                     << type;
+      schemas_all_recognized_types = false;
       continue;
     }
     types.emplace(id, DiscoveryTypeVertex{
@@ -160,23 +164,27 @@ StatusOr<std::map<std::string, DiscoveryTypeVertex>> ExtractTypesFromSchema(
         "Discovery Document contains schema without id field.");
   }
 
-  if (!schemas_all_type_object) {
+  if (!schemas_all_recognized_types) {
     return internal::InvalidArgumentError(
-        "Discovery Document contains non object schema.");
+        "Discovery Document contains unrecognized schema type.");
   }
 
   return types;
 }
 
-std::map<std::string, DiscoveryResource> ExtractResources(
+StatusOr<std::map<std::string, DiscoveryResource>> ExtractResources(
     DiscoveryDocumentProperties const& document_properties,
     nlohmann::json const& discovery_doc) {
-  if (!discovery_doc.contains("resources")) return {};
+  if (!discovery_doc.contains("resources") ||
+      discovery_doc.find("resources")->empty()) {
+    return internal::InvalidArgumentError(
+        "No resources found in Discovery Document.");
+  }
   std::map<std::string, DiscoveryResource> resources;
   auto const& resources_json = discovery_doc.find("resources");
   for (auto r = resources_json->begin(); r != resources_json->end(); ++r) {
     std::string resource_name = r.key();
-    resources.emplace(
+    auto iter = resources.emplace(
         resource_name,
         DiscoveryResource{resource_name,
                           absl::StrFormat(kResourcePackageNameFormat,
@@ -185,7 +193,12 @@ std::map<std::string, DiscoveryResource> ExtractResources(
                                           CamelCaseToSnakeCase(resource_name),
                                           document_properties.version),
                           r.value()});
+    auto verify_service_api_version = iter.first->second.SetServiceApiVersion();
+    if (!verify_service_api_version.ok()) {
+      return verify_service_api_version;
+    }
   }
+
   return resources;
 }
 
@@ -574,6 +587,7 @@ Status GenerateProtosFromDiscoveryDoc(
     std::string const& protobuf_proto_path,
     std::string const& googleapis_proto_path, std::string const& output_path,
     std::string const& export_output_path,
+    bool enable_parallel_write_for_discovery_protos,
     std::set<std::string> operation_services) {
   auto default_hostname = DefaultHostFromRootUrl(discovery_doc);
   if (!default_hostname) return std::move(default_hostname).status();
@@ -582,7 +596,7 @@ Status GenerateProtosFromDiscoveryDoc(
       discovery_doc.value("basePath", ""), *default_hostname,
       discovery_doc.value("name", ""),     discovery_doc.value("version", ""),
       discovery_doc.value("revision", ""), discovery_doc_url,
-      std::move(operation_services)};
+      std::move(operation_services),       CurrentCopyrightYear()};
 
   if (document_properties.base_path.empty() ||
       document_properties.default_hostname.empty() ||
@@ -618,64 +632,74 @@ Status GenerateProtosFromDiscoveryDoc(
   if (!types) return std::move(types).status();
 
   auto resources = ExtractResources(document_properties, discovery_doc);
-  if (resources.empty()) {
-    return internal::InvalidArgumentError(
-        "No resources found in Discovery Document.");
-  }
+  if (!resources.ok()) return std::move(resources).status();
 
   auto method_status =
-      ProcessMethodRequestsAndResponses(resources, *types, &descriptor_pool);
+      ProcessMethodRequestsAndResponses(*resources, *types, &descriptor_pool);
   if (!method_status.ok()) return method_status;
 
   EstablishTypeDependencies(*types);
-  ApplyResourceLabelsToTypes(resources);
+  ApplyResourceLabelsToTypes(*resources);
   auto files = AssignResourcesAndTypesToFiles(
-      resources, *types, document_properties, output_path, export_output_path);
+      *resources, *types, document_properties, output_path, export_output_path);
   if (!files) return std::move(files).status();
 
   // google::protobuf::DescriptorPool lazily initializes itself. Searching for
   // types by name will fail if the descriptor has not yet been created. By
   // finding all the files we intend to write, the DescriptorPool builds its
-  // collection of descriptors for that file and any it imports. We must perform
-  // this mutation of the DescriptorPool before we begin the threaded write
-  // process. Additionally, populating the DescriptorPool allows us to snapshot
-  // the existing proto files before we overwrite them in place.
+  // collection of descriptors for that file and any it imports. We must
+  // perform this mutation of the DescriptorPool before we begin the threaded
+  // write process. Additionally, populating the DescriptorPool allows us to
+  // snapshot the existing proto files before we overwrite them in place.
   for (auto const& f : files->first) {
     (void)descriptor_pool.FindFileByName(f.relative_proto_path());
   }
 
-  std::vector<std::future<google::cloud::Status>> tasks;
-  tasks.reserve(files->first.size());
-  for (auto f : files->first) {
-    tasks.push_back(std::async(
-        std::launch::async,
-        [](DiscoveryFile const& f,
-           DiscoveryDocumentProperties const& document_properties,
-           std::map<std::string, DiscoveryTypeVertex> const& types) {
-          return f.WriteFile(document_properties, types);
-        },
-        std::move(f), document_properties, *types));
-  }
-
-  for (auto f : files->second) {
-    tasks.push_back(std::async(
-        std::launch::async,
-        [](DiscoveryProtoExportFile const& f) { return f.WriteFile(); },
-        std::move(f)));
-  }
-
-  bool file_write_error = false;
-  for (auto& t : tasks) {
-    auto result = t.get();
-    if (!result.ok()) {
-      GCP_LOG(ERROR) << result;
-      file_write_error = true;
+  if (enable_parallel_write_for_discovery_protos) {
+    std::vector<std::future<google::cloud::Status>> tasks;
+    tasks.reserve(files->first.size());
+    for (auto f : files->first) {
+      tasks.push_back(std::async(
+          std::launch::async,
+          [](DiscoveryFile const& f,
+             DiscoveryDocumentProperties const& document_properties,
+             std::map<std::string, DiscoveryTypeVertex> const& types) {
+            return f.WriteFile(document_properties, types);
+          },
+          std::move(f), document_properties, *types));
     }
-  }
 
-  if (file_write_error) {
-    return internal::InternalError(
-        "Error encountered writing file. Check log for additional details.");
+    for (auto f : files->second) {
+      tasks.push_back(std::async(
+          std::launch::async,
+          [](DiscoveryProtoExportFile const& f) { return f.WriteFile(); },
+          std::move(f)));
+    }
+
+    bool file_write_error = false;
+    for (auto& t : tasks) {
+      auto result = t.get();
+      if (!result.ok()) {
+        GCP_LOG(ERROR) << result;
+        file_write_error = true;
+      }
+    }
+
+    if (file_write_error) {
+      return internal::InternalError(
+          "Error encountered writing file. Check log for additional "
+          "details.");
+    }
+  } else {
+    for (auto const& f : files->first) {
+      auto s = f.WriteFile(document_properties, *types);
+      if (!s.ok()) return s;
+    }
+
+    for (auto const& f : files->second) {
+      auto s = f.WriteFile();
+      if (!s.ok()) return s;
+    }
   }
 
   return {};
