@@ -82,6 +82,20 @@ TEST(DiscoveryTypeVertexTest, DetermineIntroducerOptionalString) {
   EXPECT_THAT(DiscoveryTypeVertex::DetermineIntroducer(json), Eq("optional "));
 }
 
+TEST(DiscoveryTypeVertexTest, FormatMessageDescriptionWithDollarSigns) {
+  auto constexpr kMessageWithDollarSignInDescription = R"""(
+        {
+          "description": "Example: `sample_table$20190123`",
+          "type": "string"
+        }
+  )""";
+  auto json = nlohmann::json::parse(kMessageWithDollarSignInDescription,
+                                    nullptr, false);
+  ASSERT_TRUE(json.is_object());
+  EXPECT_THAT(DiscoveryTypeVertex::FormatMessageDescription(json, 0),
+              Eq("// Example: `sample_table$$20190123`\n"));
+}
+
 TEST(DiscoveryTypeVertexTest, FormatFieldOptionsEmpty) {
   auto constexpr kOptionalEmptyFieldJson = R"""({})""";
   auto json = nlohmann::json::parse(kOptionalEmptyFieldJson, nullptr, false);
@@ -193,6 +207,8 @@ INSTANTIATE_TEST_SUITE_P(
                               false, true, true},
         DetermineTypesSuccess{"string", R"""({"type":"string"})""", "string",
                               true, false, false, false},
+        DetermineTypesSuccess{"any", R"""({"type":"any"})""",
+                              "google.protobuf.Any", true, false, false, false},
         DetermineTypesSuccess{"boolean", R"""({"type":"boolean"})""", "bool",
                               true, false, false, false},
         DetermineTypesSuccess{"integer_no_format", R"""({"type":"integer"})""",
@@ -234,6 +250,10 @@ INSTANTIATE_TEST_SUITE_P(
             "map_string",
             R"""({"type":"object","additionalProperties":{"type":"string"}})""",
             "string", true, true, false, false},
+        DetermineTypesSuccess{
+            "any_to_struct",
+            R"""({"type":"object","additionalProperties":{"type":"any"}})""",
+            "google.protobuf.Struct", true, true, false, false},
         DetermineTypesSuccess{
             "map_nested_message",
             R"""({"type":"object","additionalProperties":{"type":"object", "properties":{}}})""",
@@ -445,6 +465,34 @@ TEST_F(DiscoveryTypeVertexDescriptorTest, JsonToProtobufMapType) {
   EXPECT_THAT(*result, Eq(kExpectedProto));
 }
 
+TEST_F(DiscoveryTypeVertexDescriptorTest, JsonToProtobufBigQueryMapType) {
+  auto constexpr kSchemaJson = R"""({
+      "additionalProperties": {
+        "$ref": "JsonValue"
+      },
+      "description": "Represents a single JSON object.",
+      "id": "JsonObject",
+      "type": "object"
+})""";
+
+  auto constexpr kExpectedProto = R"""(// Represents a single JSON object.
+message JsonObject {
+  // Represents a single JSON object.
+  map<string, JsonValue> json_object = 1 [json_name="JsonObject"];
+}
+)""";
+
+  auto json = nlohmann::json::parse(kSchemaJson, nullptr, false);
+  ASSERT_TRUE(json.is_object());
+  DiscoveryTypeVertex t("JsonObject", "test.package", json, &pool());
+  std::map<std::string, DiscoveryTypeVertex> types;
+  types.emplace("JsonValue",
+                DiscoveryTypeVertex{"JsonValue", "test.package", {}, &pool()});
+  auto result = t.JsonToProtobufMessage(types, "test.package");
+  ASSERT_STATUS_OK(result);
+  EXPECT_THAT(*result, Eq(kExpectedProto));
+}
+
 TEST_F(DiscoveryTypeVertexDescriptorTest, JsonToProtobufArrayTypes) {
   auto constexpr kSchemaJson = R"""(
 {
