@@ -15,7 +15,7 @@
 #include "generator/internal/http_option_utils.h"
 #include "generator/testing/error_collectors.h"
 #include "generator/testing/fake_source_tree.h"
-#include "google/cloud/testing_util/status_matchers.h"
+#include <google/api/annotations.pb.h>
 #include <google/protobuf/descriptor.pb.h>
 #include <google/protobuf/descriptor_database.h>
 #include <google/protobuf/text_format.h>
@@ -27,15 +27,18 @@ namespace generator_internal {
 namespace {
 
 using ::google::cloud::generator_testing::FakeSourceTree;
-using ::google::cloud::testing_util::IsOkAndHolds;
-using ::google::cloud::testing_util::StatusIs;
 using ::google::protobuf::DescriptorPool;
 using ::google::protobuf::FileDescriptor;
 using ::google::protobuf::FileDescriptorProto;
 using ::google::protobuf::MethodDescriptor;
+using ::testing::AllOf;
+using ::testing::Contains;
+using ::testing::ElementsAre;
 using ::testing::Eq;
 using ::testing::HasSubstr;
-using ::testing::SizeIs;
+using ::testing::IsEmpty;
+using ::testing::Optional;
+using ::testing::Pair;
 
 char const* const kHttpProto =
     "syntax = \"proto3\";\n"
@@ -106,6 +109,42 @@ syntax = "proto3";
 package google.protobuf;
 // Leading comments about message Empty.
 message Empty {}
+message DoubleValue {
+  // The double value.
+  double value = 1;
+}
+message FloatValue {
+  // The float value.
+  float value = 1;
+}
+message Int64Value {
+  // The int64 value.
+  int64 value = 1;
+}
+message UInt64Value {
+  // The uint64 value.
+  uint64 value = 1;
+}
+message Int32Value {
+  // The int32 value.
+  int32 value = 1;
+}
+message UInt32Value {
+  // The uint32 value.
+  uint32 value = 1;
+}
+message BoolValue {
+  // The bool value.
+  bool value = 1;
+}
+message StringValue {
+  // The string value.
+  string value = 1;
+}
+message BytesValue {
+  // The bytes value.
+  bytes value = 1;
+}
 )""";
 
 char const* const kServiceProto =
@@ -206,7 +245,6 @@ char const* const kServiceProto =
     "  rpc Method3(PaginatedInput) returns (PaginatedOutput) {\n"
     "    option (google.api.http) = {\n"
     "       get: \"/v1/foo\"\n"
-    "       body: \"*\"\n"
     "    };\n"
     "  }\n"
     "  // Leading comments about rpc $Method4.\n"
@@ -250,7 +288,7 @@ char const* const kServiceProto =
     "  }\n"
     "}\n";
 
-char const* const kServiceProtoWithoutVersion =
+char const* const kServiceProtoWithoutVersionInPackage =
     "syntax = \"proto3\";\n"
     "package my.service;\n"
     "import \"google/api/annotations.proto\";\n"
@@ -275,6 +313,83 @@ char const* const kServiceProtoWithoutVersion =
     "  }\n"
     "}\n";
 
+char const* const kBigQueryServiceProto = R"""(
+syntax = "proto3";
+package my.package.v1;
+import "google/api/annotations.proto";
+import "google/api/client.proto";
+import "google/api/http.proto";
+import "google/protobuf/well_known.proto";
+
+service BigQueryLikeService {
+  // RPC to get the results of a query job.
+  rpc GetQueryResults(GetQueryResultsRequest)
+      returns (GetQueryResultsResponse) {
+    option (google.api.http) = {
+      get: "/bigquery/v2/projects/{project_id=*}/queries/{job_id=*}"
+    };
+  }
+}
+
+// Request object of GetQueryResults.
+message GetQueryResultsRequest {
+  // Required. Project ID of the query job.
+  string project_id = 1;
+
+  // Required. Job ID of the query job.
+  string job_id = 2;
+
+  // Zero-based index of the starting row.
+  google.protobuf.UInt64Value start_index = 3;
+
+  // Page token, returned by a previous call, to request the next page of
+  // results.
+  google.protobuf.StringValue page_token = 4;
+
+  // Maximum number of results to read.
+  google.protobuf.UInt32Value max_results = 5;
+
+  // The geographic location of the job.
+  google.protobuf.BoolValue include_location = 7;
+
+  // Double field.
+  google.protobuf.DoubleValue double_value = 8;
+
+  // Float field.
+  google.protobuf.FloatValue float_value = 9;
+
+  // Int32 field.
+  google.protobuf.Int32Value int32_value = 10;
+
+  // Int64 field.
+  google.protobuf.Int64Value int64_value = 11;
+
+  // Non supported message type that is not a query param.
+  google.protobuf.Empty non_supported_type = 12;
+}
+
+// Response object of GetQueryResults.
+message GetQueryResultsResponse {}
+)""";
+
+auto constexpr kServiceProtoWithoutVersionInUrl = R"""(
+syntax = "proto3";
+package my.package.v1;
+import "google/api/annotations.proto";
+import "google/api/http.proto";
+
+message Request {}
+message Response {}
+
+service Foo {
+  rpc Bar(Request) returns (Response) {
+    option (google.api.http) = {
+      get: "/no/version/id/in/url"
+    };
+  }
+}
+)""";
+
 struct MethodVarsTestValues {
   MethodVarsTestValues(std::string m, std::string k, std::string v)
       : method(std::move(m)),
@@ -297,8 +412,13 @@ class HttpOptionUtilsTest
              kLongrunningOperationsProto},
             {std::string("google/protobuf/well_known.proto"), kWellKnownProto},
             {std::string("google/foo/v1/service.proto"), kServiceProto},
-            {std::string("google/foo/v1/service_without_version.proto"),
-             kServiceProtoWithoutVersion}}),
+            {std::string(
+                 "google/foo/v1/service_without_version_in_package.proto"),
+             kServiceProtoWithoutVersionInPackage},
+            {std::string("google/foo/v1/big_query_service.proto"),
+             kBigQueryServiceProto},
+            {std::string("google/foo/v1/service_without_version_in_url.proto"),
+             kServiceProtoWithoutVersionInUrl}}),
         source_tree_db_(&source_tree_),
         merged_db_(&simple_db_, &source_tree_db_),
         pool_(&merged_db_, &collector_) {
@@ -337,19 +457,14 @@ TEST_F(HttpOptionUtilsTest, ParseHttpExtensionWithPrefixAndSuffix) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(2);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpExtensionInfo>(info));
-  auto extension_info = absl::get<HttpExtensionInfo>(info);
-  EXPECT_THAT(extension_info.url_path,
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path,
               Eq("/v1/{parent=projects/*/instances/*}/databases"));
-  ASSERT_THAT(extension_info.field_substitutions, SizeIs(1));
-  EXPECT_THAT(extension_info.field_substitutions[0].first, Eq("parent"));
-  EXPECT_THAT(extension_info.field_substitutions[0].second,
-              Eq("projects/*/instances/*"));
-  EXPECT_THAT(extension_info.body, Eq("*"));
-  EXPECT_THAT(extension_info.http_verb, Eq("Post"));
-  EXPECT_THAT(extension_info.path_prefix, Eq("/v1/"));
-  EXPECT_THAT(extension_info.path_suffix, Eq("/databases"));
+  EXPECT_THAT(info.field_substitutions,
+              ElementsAre(Pair("parent", "projects/*/instances/*")));
+  EXPECT_THAT(info.body, Eq("*"));
+  EXPECT_THAT(info.http_verb, Eq("Post"));
 }
 
 TEST_F(HttpOptionUtilsTest,
@@ -358,17 +473,13 @@ TEST_F(HttpOptionUtilsTest,
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(6);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpExtensionInfo>(info));
-  auto extension_info = absl::get<HttpExtensionInfo>(info);
-  EXPECT_THAT(extension_info.url_path, Eq("/v1/projects/{project}/databases"));
-  ASSERT_THAT(extension_info.field_substitutions, SizeIs(1));
-  EXPECT_THAT(extension_info.field_substitutions[0].first, Eq("project"));
-  EXPECT_THAT(extension_info.field_substitutions[0].second, Eq("project"));
-  EXPECT_THAT(extension_info.body, Eq("body"));
-  EXPECT_THAT(extension_info.http_verb, Eq("Post"));
-  EXPECT_THAT(extension_info.path_prefix, Eq("/v1/projects/"));
-  EXPECT_THAT(extension_info.path_suffix, Eq("/databases"));
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path, Eq("/v1/projects/{project}/databases"));
+  EXPECT_THAT(info.field_substitutions,
+              ElementsAre(Pair("project", "project")));
+  EXPECT_THAT(info.body, Eq("body"));
+  EXPECT_THAT(info.http_verb, Eq("Post"));
 }
 
 TEST_F(HttpOptionUtilsTest,
@@ -377,20 +488,15 @@ TEST_F(HttpOptionUtilsTest,
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(7);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpExtensionInfo>(info));
-  auto extension_info = absl::get<HttpExtensionInfo>(info);
-  EXPECT_THAT(extension_info.url_path,
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path,
               Eq("/v1/projects/{project}/instances/{instance}/databases"));
-  ASSERT_THAT(extension_info.field_substitutions, SizeIs(2));
-  EXPECT_THAT(extension_info.field_substitutions[0].first, Eq("project"));
-  EXPECT_THAT(extension_info.field_substitutions[0].second, Eq("project"));
-  EXPECT_THAT(extension_info.field_substitutions[1].first, Eq("instance"));
-  EXPECT_THAT(extension_info.field_substitutions[1].second, Eq("instance"));
-  EXPECT_THAT(extension_info.body, Eq("*"));
-  EXPECT_THAT(extension_info.http_verb, Eq("Post"));
-  EXPECT_THAT(extension_info.path_prefix, Eq("/v1/projects/"));
-  EXPECT_THAT(extension_info.path_suffix, Eq("/databases"));
+  EXPECT_THAT(
+      info.field_substitutions,
+      ElementsAre(Pair("project", "project"), Pair("instance", "instance")));
+  EXPECT_THAT(info.body, Eq("*"));
+  EXPECT_THAT(info.http_verb, Eq("Post"));
 }
 
 TEST_F(HttpOptionUtilsTest, ParseHttpExtensionWithOnlyPrefix) {
@@ -398,32 +504,26 @@ TEST_F(HttpOptionUtilsTest, ParseHttpExtensionWithOnlyPrefix) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(1);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpExtensionInfo>(info));
-  auto extension_info = absl::get<HttpExtensionInfo>(info);
-  EXPECT_THAT(extension_info.url_path,
-              Eq("/v1/{name=projects/*/instances/*/backups/*}"));
-  ASSERT_THAT(extension_info.field_substitutions, SizeIs(1));
-  EXPECT_THAT(extension_info.field_substitutions[0].first, Eq("name"));
-  EXPECT_THAT(extension_info.field_substitutions[0].second,
-              Eq("projects/*/instances/*/backups/*"));
-  EXPECT_THAT(extension_info.body, Eq(""));
-  EXPECT_THAT(extension_info.http_verb, Eq("Delete"));
-  EXPECT_THAT(extension_info.path_prefix, Eq("/v1/"));
-  EXPECT_THAT(extension_info.path_suffix, Eq(""));
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path, Eq("/v1/{name=projects/*/instances/*/backups/*}"));
+  EXPECT_THAT(info.field_substitutions,
+              ElementsAre(Pair("name", "projects/*/instances/*/backups/*")));
+  EXPECT_THAT(info.body, Eq(""));
+  EXPECT_THAT(info.http_verb, Eq("Delete"));
 }
 
-TEST_F(HttpOptionUtilsTest, ParseHttpExtensionSimpleInfo) {
+TEST_F(HttpOptionUtilsTest, ParseHttpExtensionSimple) {
   FileDescriptor const* service_file_descriptor =
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(3);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpSimpleInfo>(info));
-  auto extension_info = absl::get<HttpSimpleInfo>(info);
-  EXPECT_THAT(extension_info.url_path, Eq("/v1/foo"));
-  EXPECT_THAT(extension_info.body, Eq("*"));
-  EXPECT_THAT(extension_info.http_verb, Eq("Get"));
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path, Eq("/v1/foo"));
+  EXPECT_THAT(info.field_substitutions, IsEmpty());
+  EXPECT_THAT(info.body, Eq(""));
+  EXPECT_THAT(info.http_verb, Eq("Get"));
 }
 
 TEST_F(HttpOptionUtilsTest,
@@ -432,14 +532,31 @@ TEST_F(HttpOptionUtilsTest,
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(5);
-  auto info = ParseHttpExtension(*method);
-  ASSERT_TRUE(absl::holds_alternative<HttpExtensionInfo>(info));
-  auto extension_info = absl::get<HttpExtensionInfo>(info);
-  EXPECT_THAT(extension_info.url_path,
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path,
               Eq("/v1/projects/{project=project}/instances/{instance=instance}/"
                  "databases"));
-  EXPECT_THAT(extension_info.body, Eq("*"));
-  EXPECT_THAT(extension_info.http_verb, Eq("Post"));
+  EXPECT_THAT(info.body, Eq("*"));
+  EXPECT_THAT(info.http_verb, Eq("Post"));
+}
+
+TEST_F(HttpOptionUtilsTest, ParseHttpExtensionWithoutVersionInPackage) {
+  FileDescriptor const* service_file = pool_.FindFileByName(
+      "google/foo/v1/service_without_version_in_package.proto");
+  MethodDescriptor const* method = service_file->service(0)->method(0);
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path, Eq("/v1/simple"));
+}
+
+TEST_F(HttpOptionUtilsTest, ParseHttpExtensionWithoutVersionInUrl) {
+  FileDescriptor const* service_file = pool_.FindFileByName(
+      "google/foo/v1/service_without_version_in_url.proto");
+  MethodDescriptor const* method = service_file->service(0)->method(0);
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
+  EXPECT_THAT(info.url_path, Eq("/no/version/id/in/url"));
 }
 
 TEST_F(HttpOptionUtilsTest, SetHttpDerivedMethodVarsSimpleInfo) {
@@ -448,13 +565,16 @@ TEST_F(HttpOptionUtilsTest, SetHttpDerivedMethodVarsSimpleInfo) {
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(0);
   VarsDictionary vars;
-  SetHttpDerivedMethodVars(ParseHttpExtension(*method), *method, vars);
-  EXPECT_THAT(vars.at("method_request_params"),
-              Eq("my.package.v1.Service.Method0"));
+  SetHttpDerivedMethodVars(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_http_verb"), Eq("Delete"));
   EXPECT_THAT(
       vars.at("method_rest_path"),
-      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/simple"))"""));
+      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/", "simple"))"""));
+  EXPECT_THAT(
+      vars.at("method_rest_path_async"),
+      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", *options), "/", "simple"))"""));
 }
 
 TEST_F(HttpOptionUtilsTest, SetHttpDerivedMethodVarsExtensionInfoSingleParam) {
@@ -463,7 +583,9 @@ TEST_F(HttpOptionUtilsTest, SetHttpDerivedMethodVarsExtensionInfoSingleParam) {
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(2);
   VarsDictionary vars;
-  SetHttpDerivedMethodVars(ParseHttpExtension(*method), *method, vars);
+  SetHttpDerivedMethodVars(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_request_params"),
               Eq("\"parent=\", internal::UrlEncode(request.parent())"));
   EXPECT_THAT(vars.at("method_request_body"), Eq("*"));
@@ -471,6 +593,9 @@ TEST_F(HttpOptionUtilsTest, SetHttpDerivedMethodVarsExtensionInfoSingleParam) {
   EXPECT_THAT(
       vars.at("method_rest_path"),
       Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/", request.parent(), "/", "databases"))"""));
+  EXPECT_THAT(
+      vars.at("method_rest_path_async"),
+      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", *options), "/", request.parent(), "/", "databases"))"""));
 }
 
 TEST_F(HttpOptionUtilsTest,
@@ -480,7 +605,9 @@ TEST_F(HttpOptionUtilsTest,
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(7);
   VarsDictionary vars;
-  SetHttpDerivedMethodVars(ParseHttpExtension(*method), *method, vars);
+  SetHttpDerivedMethodVars(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_request_params"),
               Eq("\"project=\", internal::UrlEncode(request.project()), "
                  "\"&\",\"instance=\", "
@@ -490,6 +617,9 @@ TEST_F(HttpOptionUtilsTest,
   EXPECT_THAT(
       vars.at("method_rest_path"),
       Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/", "projects", "/", request.project(), "/", "instances", "/", request.instance(), "/", "databases"))"""));
+  EXPECT_THAT(
+      vars.at("method_rest_path_async"),
+      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", *options), "/", "projects", "/", request.project(), "/", "instances", "/", request.instance(), "/", "databases"))"""));
 }
 
 TEST_F(HttpOptionUtilsTest,
@@ -499,12 +629,17 @@ TEST_F(HttpOptionUtilsTest,
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(5);
   VarsDictionary vars;
-  SetHttpDerivedMethodVars(ParseHttpExtension(*method), *method, vars);
+  SetHttpDerivedMethodVars(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_request_body"), Eq("*"));
   EXPECT_THAT(vars.at("method_http_verb"), Eq("Post"));
   EXPECT_THAT(
       vars.at("method_rest_path"),
       Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", options), "/", "projects", "/", request.project(), "/", "instances", "/", request.instance(), "/", "databases"))"""));
+  EXPECT_THAT(
+      vars.at("method_rest_path_async"),
+      Eq(R"""(absl::StrCat("/", rest_internal::DetermineApiVersion("v1", *options), "/", "projects", "/", request.project(), "/", "instances", "/", request.instance(), "/", "databases"))"""));
 }
 
 TEST_F(HttpOptionUtilsTest, SetHttpQueryParametersNoParams) {
@@ -513,7 +648,9 @@ TEST_F(HttpOptionUtilsTest, SetHttpQueryParametersNoParams) {
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(5);
   VarsDictionary vars;
-  SetHttpQueryParameters(ParseHttpExtension(*method), *method, vars);
+  SetHttpQueryParameters(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_http_query_parameters"), Eq(""));
 }
 
@@ -523,7 +660,9 @@ TEST_F(HttpOptionUtilsTest, SetHttpQueryParametersGetWithParams) {
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(4);
   VarsDictionary vars;
-  SetHttpQueryParameters(ParseHttpExtension(*method), *method, vars);
+  SetHttpQueryParameters(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
   EXPECT_THAT(vars.at("method_http_query_parameters"), Eq(""));
 }
 
@@ -533,12 +672,34 @@ TEST_F(HttpOptionUtilsTest, SetHttpGetQueryParametersGetPaginated) {
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(3);
   VarsDictionary vars;
-  SetHttpQueryParameters(ParseHttpExtension(*method), *method, vars);
-  EXPECT_THAT(vars.at("method_http_query_parameters"), Eq(R"""(,
-      rest_internal::TrimEmptyQueryParameters({std::make_pair("page_size", std::to_string(request.page_size())),
-        std::make_pair("page_token", request.page_token()),
-        std::make_pair("name", request.name()),
-        std::make_pair("include_foo", request.include_foo() ? "1" : "0")}))"""));
+  SetHttpQueryParameters(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
+  EXPECT_THAT(vars,
+              Contains(Pair("method_http_query_parameters",
+                            AllOf(HasSubstr("TrimEmptyQueryParameters"),
+                                  HasSubstr("page_token"), HasSubstr("name"),
+                                  HasSubstr("include_foo")))));
+}
+
+TEST_F(HttpOptionUtilsTest,
+       SetHttpGetQueryParametersGetWellKnownTypesPaginated) {
+  FileDescriptor const* service_file_descriptor =
+      pool_.FindFileByName("google/foo/v1/big_query_service.proto");
+  MethodDescriptor const* method =
+      service_file_descriptor->service(0)->method(0);
+  VarsDictionary vars;
+  SetHttpQueryParameters(
+      ParseHttpExtension(method->options().GetExtension(google::api::http)),
+      *method, vars);
+  EXPECT_THAT(
+      vars,
+      Contains(Pair(
+          "method_http_query_parameters",
+          AllOf(HasSubstr("TrimEmptyQueryParameters"), HasSubstr("page_token"),
+                HasSubstr("max_results"), HasSubstr("include_location"),
+                HasSubstr("double_value"), HasSubstr("float_value"),
+                HasSubstr("int32_value"), HasSubstr("int64_value")))));
 }
 
 TEST_F(HttpOptionUtilsTest, HasHttpAnnotationRoutingHeaderSuccess) {
@@ -563,15 +724,15 @@ TEST_F(HttpOptionUtilsTest, HasRoutingHeaderWrongUrlFormat) {
   /// @cond
   auto constexpr kServiceText = R"pb(
     name: "google/foo/v1/service.proto"
-    package: "google.protobuf"
+    package: "google.foo.v1"
     message_type { name: "Bar" }
     message_type { name: "Empty" }
     service {
       name: "Service"
       method {
         name: "Method0"
-        input_type: "google.protobuf.Bar"
-        output_type: "google.protobuf.Empty"
+        input_type: "google.foo.v1.Bar"
+        output_type: "google.foo.v1.Empty"
         options {
           [google.api.http] { post: "/v2/entries:list" body: "*" }
         }
@@ -620,9 +781,8 @@ TEST_F(HttpOptionUtilsTest, FormatRequestResourceFailedParse) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(2);
-  auto result = FormatRequestResource(
-      *method->input_type(),
-      absl::variant<absl::monostate, HttpSimpleInfo, HttpExtensionInfo>());
+  auto result =
+      FormatRequestResource(*method->input_type(), HttpExtensionInfo{});
   EXPECT_THAT(result, Eq("request"));
 }
 
@@ -631,7 +791,8 @@ TEST_F(HttpOptionUtilsTest, FormatRequestResourceWholeMessage) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(2);
-  auto info = ParseHttpExtension(*method);
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
   auto result = FormatRequestResource(*method->input_type(), info);
   EXPECT_THAT(result, Eq("request"));
 }
@@ -641,7 +802,8 @@ TEST_F(HttpOptionUtilsTest, FormatRequestResourceMessageField) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(6);
-  auto info = ParseHttpExtension(*method);
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
   auto result = FormatRequestResource(*method->input_type(), info);
   EXPECT_THAT(result, Eq("request.body()"));
 }
@@ -651,29 +813,24 @@ TEST_F(HttpOptionUtilsTest, FormatRequestResourceAnnotatedRequestField) {
       pool_.FindFileByName("google/foo/v1/service.proto");
   MethodDescriptor const* method =
       service_file_descriptor->service(0)->method(8);
-  auto info = ParseHttpExtension(*method);
+  auto info =
+      ParseHttpExtension(method->options().GetExtension(google::api::http));
   auto result = FormatRequestResource(*method->input_type(), info);
   EXPECT_THAT(result, Eq("request.namespace_()"));
 }
 
-TEST_F(HttpOptionUtilsTest, FormatApiVersionFromPackageName) {
-  FileDescriptor const* service_file_descriptor =
-      pool_.FindFileByName("google/foo/v1/service.proto");
-  MethodDescriptor const* method =
-      service_file_descriptor->service(0)->method(8);
-  EXPECT_THAT(FormatApiVersionFromPackageName(*method), IsOkAndHolds(Eq("v1")));
+TEST_F(HttpOptionUtilsTest, FormatApiVersionFromUrlPattern) {
+  std::string url_pattern_v1 = "/v1/foo/bar";
+  EXPECT_THAT(FormatApiVersionFromUrlPattern(url_pattern_v1),
+              Optional(std::string("v1")));
+  std::string url_pattern_v2 = "/foo/v2/bar";
+  EXPECT_THAT(FormatApiVersionFromUrlPattern(url_pattern_v2),
+              Optional(std::string("v2")));
 }
 
-TEST_F(HttpOptionUtilsTest, FormatApiVersionFromPackageNameError) {
-  FileDescriptor const* service_file_descriptor =
-      pool_.FindFileByName("google/foo/v1/service_without_version.proto");
-  MethodDescriptor const* method =
-      service_file_descriptor->service(0)->method(0);
-  EXPECT_THAT(
-      FormatApiVersionFromPackageName(*method),
-      StatusIs(
-          StatusCode::kInvalidArgument,
-          HasSubstr("Unrecognized API version in package name: my.service")));
+TEST_F(HttpOptionUtilsTest, FormatApiVersionFromUrlPatternNonExist) {
+  std::string url_pattern = "/foo/bar";
+  EXPECT_THAT(FormatApiVersionFromUrlPattern(url_pattern), Eq(absl::nullopt));
 }
 
 }  // namespace
