@@ -38,7 +38,8 @@ class LoggingDecoratorGenerator : public StubGeneratorBase {
       google::protobuf::ServiceDescriptor const* service_descriptor,
       VarsDictionary service_vars,
       std::map<std::string, VarsDictionary> service_method_vars,
-      google::protobuf::compiler::GeneratorContext* context);
+      google::protobuf::compiler::GeneratorContext* context,
+      std::vector<MixinMethod> const& mixin_methods);
 
   ~LoggingDecoratorGenerator() override = default;
 
@@ -49,6 +50,7 @@ class LoggingDecoratorGenerator : public StubGeneratorBase {
   LoggingDecoratorGenerator& operator=(LoggingDecoratorGenerator&&) = default;
 
  private:
+  bool HasStreamingMethod();
   Status GenerateHeader() override;
   Status GenerateCc() override;
 };
