@@ -46,12 +46,20 @@ TEST(LongrunningTest, IsGRPCLongrunningOperation) {
     service {
       name: "Service"
       method {
-        name: "Lro"
+        name: "GrpcLro"
+        input_type: "google.longrunning.Bar"
+        output_type: "google.longrunning.Operation"
+        options {
+          [google.longrunning.operation_info] {}
+        }
+      }
+      method {
+        name: "NonLro1"
         input_type: "google.longrunning.Bar"
         output_type: "google.longrunning.Operation"
       }
       method {
-        name: "NonLro"
+        name: "NonLro2"
         input_type: "google.longrunning.Bar"
         output_type: "google.longrunning.Bar"
       }
@@ -61,14 +69,20 @@ TEST(LongrunningTest, IsGRPCLongrunningOperation) {
   ASSERT_TRUE(TextFormat::ParseFromString(kServiceText, &service_file));
   DescriptorPool pool;
   FileDescriptor const* service_file_descriptor = pool.BuildFile(service_file);
-  EXPECT_TRUE(
-      IsLongrunningOperation(*service_file_descriptor->service(0)->method(0)));
   EXPECT_TRUE(IsGRPCLongrunningOperation(
       *service_file_descriptor->service(0)->method(0)));
   EXPECT_FALSE(IsHttpLongrunningOperation(
       *service_file_descriptor->service(0)->method(0)));
+  EXPECT_FALSE(IsGRPCLongrunningOperation(
+      *service_file_descriptor->service(0)->method(1)));
+  EXPECT_FALSE(IsGRPCLongrunningOperation(
+      *service_file_descriptor->service(0)->method(2)));
+  EXPECT_TRUE(
+      IsLongrunningOperation(*service_file_descriptor->service(0)->method(0)));
   EXPECT_FALSE(
       IsLongrunningOperation(*service_file_descriptor->service(0)->method(1)));
+  EXPECT_FALSE(
+      IsLongrunningOperation(*service_file_descriptor->service(0)->method(2)));
 }
 
 TEST(LongrunningTest, IsLongrunningMetadataTypeUsedAsResponseEmptyResponse) {
@@ -535,6 +549,8 @@ TEST_F(LongrunningVarsTest,
       service_file_descriptor->service(0)->method(1);
   VarsDictionary vars;
   SetLongrunningOperationMethodVars(*method, vars);
+  EXPECT_THAT(vars, Contains(Pair("longrunning_operation_type",
+                                  "google::longrunning::Operation")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_metadata_type",
                                   "google::protobuf::Struct")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_response_type",
@@ -580,6 +596,8 @@ TEST_F(LongrunningVarsTest, SetLongrunningOperationMethodVarsBespokeLRO) {
   EXPECT_FALSE(IsGRPCLongrunningOperation(*method));
   EXPECT_TRUE(IsHttpLongrunningOperation(*method));
   SetLongrunningOperationMethodVars(*method, vars);
+  EXPECT_THAT(vars, Contains(Pair("longrunning_operation_type",
+                                  "my::service::v1::Operation")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_response_type",
                                   "my::service::v1::Operation")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_deduced_response_message_type",
@@ -637,17 +655,22 @@ TEST_F(LongrunningVarsTest, SetLongrunningOperationServiceVarsNonGRPCGlobal) {
   EXPECT_THAT(vars, Contains(Pair("longrunning_set_operation_fields", R"""(
       r.set_project(request.project());
       r.set_operation(op);
+)""")));
+  EXPECT_THAT(vars,
+              Contains(Pair("longrunning_await_set_operation_fields", R"""(
+      r.set_project(info.project);
+      r.set_operation(info.operation);
 )""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_get_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/global/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/global/operations/", request.operation()))""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_cancel_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/global/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/global/operations/", request.operation()))""")));
 }
 
 TEST_F(LongrunningVarsTest,
@@ -673,15 +696,19 @@ TEST_F(LongrunningVarsTest,
           "DeleteOperationRequest")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_set_operation_fields", R"""(
       r.set_operation(op);
+)""")));
+  EXPECT_THAT(vars,
+              Contains(Pair("longrunning_await_set_operation_fields", R"""(
+      r.set_operation(info.operation);
 )""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_get_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/locations/global/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/locations/global/operations/", request.operation()))""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_cancel_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/locations/global/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/locations/global/operations/", request.operation()))""")));
 }
 
 TEST_F(LongrunningVarsTest, SetLongrunningOperationServiceVarsNonGRPCRegion) {
@@ -706,19 +733,25 @@ TEST_F(LongrunningVarsTest, SetLongrunningOperationServiceVarsNonGRPCRegion) {
       r.set_project(request.project());
       r.set_region(request.region());
       r.set_operation(op);
+)""")));
+  EXPECT_THAT(vars,
+              Contains(Pair("longrunning_await_set_operation_fields", R"""(
+      r.set_project(info.project);
+      r.set_region(info.region);
+      r.set_operation(info.operation);
 )""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_get_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/regions/", request.region(),
-                             "/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/regions/", request.region(),
+                              "/operations/", request.operation()))""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_cancel_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/regions/", request.region(),
-                             "/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/regions/", request.region(),
+                              "/operations/", request.operation()))""")));
 }
 
 TEST_F(LongrunningVarsTest, SetLongrunningOperationServiceVarsNonGRPCZone) {
@@ -743,19 +776,25 @@ TEST_F(LongrunningVarsTest, SetLongrunningOperationServiceVarsNonGRPCZone) {
       r.set_project(request.project());
       r.set_zone(request.zone());
       r.set_operation(op);
+)""")));
+  EXPECT_THAT(vars,
+              Contains(Pair("longrunning_await_set_operation_fields", R"""(
+      r.set_project(info.project);
+      r.set_zone(info.zone);
+      r.set_operation(info.operation);
 )""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_get_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/zones/", request.zone(),
-                             "/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/zones/", request.zone(),
+                              "/operations/", request.operation()))""")));
   EXPECT_THAT(vars, Contains(Pair("longrunning_cancel_operation_path_rest",
                                   R"""(absl::StrCat("/compute/",
-                             rest_internal::DetermineApiVersion("v1", options),
-                             "/projects/", request.project(),
-                             "/zones/", request.zone(),
-                             "/operations/", request.operation()))""")));
+                              rest_internal::DetermineApiVersion("v1", *options),
+                              "/projects/", request.project(),
+                              "/zones/", request.zone(),
+                              "/operations/", request.operation()))""")));
 }
 
 }  // namespace
