@@ -106,10 +106,12 @@ MetadataDecoratorGenerator::MetadataDecoratorGenerator(
     google::protobuf::ServiceDescriptor const* service_descriptor,
     VarsDictionary service_vars,
     std::map<std::string, VarsDictionary> service_method_vars,
-    google::protobuf::compiler::GeneratorContext* context)
+    google::protobuf::compiler::GeneratorContext* context,
+    std::vector<MixinMethod> const& mixin_methods)
     : StubGeneratorBase("metadata_header_path", "metadata_cc_path",
                         service_descriptor, std::move(service_vars),
-                        std::move(service_method_vars), context) {}
+                        std::move(service_method_vars), context,
+                        mixin_methods) {}
 
 Status MetadataDecoratorGenerator::GenerateHeader() {
   HeaderPrint(CopyrightLicenseFileHeader());
@@ -177,18 +179,19 @@ Status MetadataDecoratorGenerator::GenerateCc() {
 
   // includes
   CcPrint("\n");
-  CcLocalIncludes({vars("metadata_header_path"),
-                   "google/cloud/internal/absl_str_cat_quiet.h",
-                   HasExplicitRoutingMethod()
-                       ? "google/cloud/internal/absl_str_join_quiet.h"
-                       : "",
-                   "google/cloud/internal/api_client_header.h",
-                   HasExplicitRoutingMethod()
-                       ? "google/cloud/internal/routing_matcher.h"
-                       : "",
-                   "google/cloud/common_options.h", "google/cloud/status_or.h",
-                   "google/cloud/internal/url_encode.h"});
-  CcSystemIncludes({vars("proto_grpc_header_path"), "memory", "utility"});
+  CcLocalIncludes(
+      {vars("metadata_header_path"),
+       "google/cloud/internal/absl_str_cat_quiet.h",
+       HasExplicitRoutingMethod()
+           ? "google/cloud/internal/absl_str_join_quiet.h"
+           : "",
+       "google/cloud/internal/api_client_header.h",
+       "google/cloud/grpc_options.h",
+       HasExplicitRoutingMethod() ? "google/cloud/internal/routing_matcher.h"
+                                  : "",
+       "google/cloud/status_or.h", "google/cloud/internal/url_encode.h"});
+  CcSystemIncludes({vars("proto_grpc_header_path"), "memory", "string",
+                    "utility", "vector"});
 
   auto result = CcOpenNamespaces(NamespaceType::kInternal);
   if (!result.ok()) return result;
@@ -236,9 +239,10 @@ std::unique_ptr<::google::cloud::AsyncStreamingReadWriteRpc<
       $response_type$>>
 $metadata_class_name$::Async$method_name$(
     google::cloud::CompletionQueue const& cq,
-    std::shared_ptr<grpc::ClientContext> context) {
-  SetMetadata(*context, internal::CurrentOptions());
-  return child_->Async$method_name$(cq, std::move(context));
+    std::shared_ptr<grpc::ClientContext> context,
+    google::cloud::internal::ImmutableOptions options) {
+  SetMetadata(*context, *options);
+  return child_->Async$method_name$(cq, std::move(context), std::move(options));
 }
 )""");
       continue;
@@ -249,15 +253,30 @@ future<StatusOr<google::longrunning::Operation>>
 $metadata_class_name$::Async$method_name$(
     google::cloud::CompletionQueue& cq,
     std::shared_ptr<grpc::ClientContext> context,
-    Options const& options,
+    google::cloud::internal::ImmutableOptions options,
     $request_type$ const& request) {
 )""");
       CcPrintMethod(method, __FILE__, __LINE__,
-                    SetMetadataText(method, kPointer, "options"));
+                    SetMetadataText(method, kPointer, "*options"));
       CcPrintMethod(method, __FILE__, __LINE__, R"""(
-  return child_->Async$method_name$(cq, std::move(context), options, request);
+  return child_->Async$method_name$(
+      cq, std::move(context), std::move(options), request);
 }
 )""");
+      CcPrintMethod(method, __FILE__, __LINE__, R"""(
+StatusOr<google::longrunning::Operation>
+$metadata_class_name$::$method_name$(
+    grpc::ClientContext& context,
+    Options options,
+    $request_type$ const& request) {
+)""");
+      CcPrintMethod(method, __FILE__, __LINE__,
+                    SetMetadataText(method, kReference, "options"));
+      CcPrintMethod(method, __FILE__, __LINE__, R"""(
+  return child_->$method_name$(context, options, request);
+}
+)""");
+
       continue;
     }
     if (IsStreamingRead(method)) {
@@ -276,27 +295,24 @@ $metadata_class_name$::$method_name$(
 )""");
       continue;
     }
-    CcPrintMethod(
-        method,
-        {MethodPattern(
-            {
-                {IsResponseTypeEmpty,
-                 // clang-format off
-    "\nStatus\n",
-    "\nStatusOr<$response_type$>\n"},
-   {"$metadata_class_name$::$method_name$(\n"
-    "    grpc::ClientContext& context,\n"
-    "    $request_type$ const& request) {\n"},
-    {SetMetadataText(method, kReference, "internal::CurrentOptions()")},
-   {"\n  return child_->$method_name$(context, request);\n"
-    "}\n",}
-                // clang-format on
-            },
-            And(IsNonStreaming, Not(IsLongrunningOperation)))},
-        __FILE__, __LINE__);
+    CcPrintMethod(method, __FILE__, __LINE__, "\n$return_type$");
+    CcPrintMethod(method, __FILE__, __LINE__, R"""(
+$metadata_class_name$::$method_name$(
+    grpc::ClientContext& context,
+    Options const& options,
+    $request_type$ const& request) {
+)""");
+    CcPrintMethod(method, __FILE__, __LINE__,
+                  SetMetadataText(method, kReference, "options"));
+    CcPrintMethod(method, __FILE__, __LINE__, R"""(
+  return child_->$method_name$(context, options, request);
+}
+)""");
   }
 
   for (auto const& method : async_methods()) {
+    // Nothing to do, these are always asynchronous.
+    if (IsBidirStreaming(method) || IsLongrunningOperation(method)) continue;
     if (IsStreamingRead(method)) {
       auto const definition = absl::StrCat(
           R"""(
@@ -305,11 +321,13 @@ std::unique_ptr<::google::cloud::internal::AsyncStreamingReadRpc<
 $metadata_class_name$::Async$method_name$(
     google::cloud::CompletionQueue const& cq,
     std::shared_ptr<grpc::ClientContext> context,
+    google::cloud::internal::ImmutableOptions options,
     $request_type$ const& request) {
 )""",
-          SetMetadataText(method, kPointer, "internal::CurrentOptions()"),
+          SetMetadataText(method, kPointer, "*options"),
           R"""(
-  return child_->Async$method_name$(cq, std::move(context), request);
+  return child_->Async$method_name$(
+      cq, std::move(context), std::move(options), request);
 }
 )""");
       CcPrintMethod(method, __FILE__, __LINE__, definition);
@@ -326,33 +344,30 @@ std::unique_ptr<::google::cloud::internal::AsyncStreamingWriteRpc<
     $request_type$, $response_type$>>
 $metadata_class_name$::Async$method_name$(
     google::cloud::CompletionQueue const& cq,
-    std::shared_ptr<grpc::ClientContext> context) {
-  SetMetadata(*context, internal::CurrentOptions());
-  return child_->Async$method_name$(cq, std::move(context));
+    std::shared_ptr<grpc::ClientContext> context,
+    google::cloud::internal::ImmutableOptions options) {
+  SetMetadata(*context, *options);
+  return child_->Async$method_name$(cq, std::move(context), std::move(options));
 }
 )""");
       CcPrintMethod(method, __FILE__, __LINE__, definition);
       continue;
     }
-    CcPrintMethod(
-        method,
-        {MethodPattern(
-            {{IsResponseTypeEmpty,
-              // clang-format off
-    "\nfuture<Status>\n",
-    "\nfuture<StatusOr<$response_type$>>\n"},
-    {R"""($metadata_class_name$::Async$method_name$(
-    google::cloud::CompletionQueue& cq,
-    std::shared_ptr<grpc::ClientContext> context,
-    $request_type$ const& request) {
-)"""},
-   {SetMetadataText(method, kPointer, "internal::CurrentOptions()")}, {R"""(
-  return child_->Async$method_name$(cq, std::move(context), request);
+    CcPrintMethod(method, __FILE__, __LINE__, "\nfuture<$return_type$>");
+    CcPrintMethod(method, __FILE__, __LINE__, R"""(
+$metadata_class_name$::Async$method_name$(
+      google::cloud::CompletionQueue& cq,
+      std::shared_ptr<grpc::ClientContext> context,
+      google::cloud::internal::ImmutableOptions options,
+      $request_type$ const& request) {
+)""");
+    CcPrintMethod(method, __FILE__, __LINE__,
+                  SetMetadataText(method, kPointer, "*options"));
+    CcPrintMethod(method, __FILE__, __LINE__, R"""(
+  return child_->Async$method_name$(
+      cq, std::move(context), std::move(options), request);
 }
-)"""}},
-            // clang-format on
-            And(IsNonStreaming, Not(IsLongrunningOperation)))},
-        __FILE__, __LINE__);
+)""");
   }
 
   // long running operation support methods
@@ -362,21 +377,23 @@ future<StatusOr<google::longrunning::Operation>>
 $metadata_class_name$::AsyncGetOperation(
     google::cloud::CompletionQueue& cq,
     std::shared_ptr<grpc::ClientContext> context,
-    Options const& options,
+    google::cloud::internal::ImmutableOptions options,
     google::longrunning::GetOperationRequest const& request) {
-  SetMetadata(*context, options,
+  SetMetadata(*context, *options,
               absl::StrCat("name=", internal::UrlEncode(request.name())));
-  return child_->AsyncGetOperation(cq, std::move(context), options, request);
+  return child_->AsyncGetOperation(
+      cq, std::move(context), std::move(options), request);
 }
 
 future<Status> $metadata_class_name$::AsyncCancelOperation(
     google::cloud::CompletionQueue& cq,
     std::shared_ptr<grpc::ClientContext> context,
-    Options const& options,
+    google::cloud::internal::ImmutableOptions options,
     google::longrunning::CancelOperationRequest const& request) {
-   SetMetadata(*context, options,
-               absl::StrCat("name=", internal::UrlEncode(request.name())));
-  return child_->AsyncCancelOperation(cq, std::move(context), options, request);
+  SetMetadata(*context, *options,
+              absl::StrCat("name=", internal::UrlEncode(request.name())));
+  return child_->AsyncCancelOperation(
+      cq, std::move(context), std::move(options), request);
 }
 )""");
   }
@@ -391,16 +408,14 @@ void $metadata_class_name$::SetMetadata(grpc::ClientContext& context,
 
 void $metadata_class_name$::SetMetadata(grpc::ClientContext& context,
                                         Options const& options) {
-  for (auto const& kv : fixed_metadata_) {
-    context.AddMetadata(kv.first, kv.second);
+)""");
+  if (HasApiVersion()) {
+    CcPrint(
+        R"""(  context.AddMetadata("x-goog-api-version", "$api_version$");
+)""");
   }
-  context.AddMetadata("x-goog-api-client", api_client_header_);
-  if (options.has<UserProjectOption>()) {
-    context.AddMetadata(
-        "x-goog-user-project", options.get<UserProjectOption>());
-  }
-  auto const& authority = options.get<AuthorityOption>();
-  if (!authority.empty()) context.set_authority(authority);
+  CcPrint(R"""(  google::cloud::internal::SetMetadata(
+      context, options, fixed_metadata_, api_client_header_);
 }
 )""");
 
