@@ -38,7 +38,8 @@ class MetadataDecoratorGenerator : public StubGeneratorBase {
       google::protobuf::ServiceDescriptor const* service_descriptor,
       VarsDictionary service_vars,
       std::map<std::string, VarsDictionary> service_method_vars,
-      google::protobuf::compiler::GeneratorContext* context);
+      google::protobuf::compiler::GeneratorContext* context,
+      std::vector<MixinMethod> const& mixin_methods);
 
   ~MetadataDecoratorGenerator() override = default;
 
