@@ -37,7 +37,8 @@ class OptionDefaultsGenerator : public ServiceCodeGenerator {
       google::protobuf::ServiceDescriptor const* service_descriptor,
       VarsDictionary service_vars,
       std::map<std::string, VarsDictionary> service_method_vars,
-      google::protobuf::compiler::GeneratorContext* context);
+      google::protobuf::compiler::GeneratorContext* context,
+      std::vector<MixinMethod> const& mixin_methods);
 
   ~OptionDefaultsGenerator() override = default;
 
