@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "generator/internal/scaffold_generator.h"
+#include "generator/internal/codegen_utils.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
 #include "google/cloud/internal/absl_str_replace_quiet.h"
 #include "google/cloud/internal/filesystem.h"
@@ -25,11 +26,6 @@
 #include <fstream>
 #include <iterator>
 #include <regex>
-#ifdef _WIN32
-#include <direct.h>
-#else
-#include <sys/stat.h>
-#endif  // _WIN32
 
 namespace google {
 namespace cloud {
@@ -64,6 +60,15 @@ ProductPath ParseProductPath(std::string const& product_path) {
   return make_result(std::prev(v.end()));
 }
 
+std::string FormatCloudServiceDocsLink(
+    std::map<std::string, std::string> const& vars) {
+  return absl::StrCat("[cloud-service-docs]: ",
+                      vars.find("documentation_uri") == vars.end()
+                          ? "https://cloud.google.com/$site_root$ [EDIT HERE]"
+                          : "$documentation_uri$",
+                      "\n");
+}
+
 }  // namespace
 
 auto constexpr kApiIndexFilename = "api-index-v1.json";
@@ -118,7 +123,7 @@ nlohmann::json LoadApiIndex(std::string const& googleapis_path) {
 }
 
 std::map<std::string, std::string> ScaffoldVars(
-    std::string const& googleapis_path, nlohmann::json const& index,
+    std::string const& yaml_root, nlohmann::json const& index,
     google::cloud::cpp::generator::ServiceConfiguration const& service,
     bool experimental) {
   std::map<std::string, std::string> vars;
@@ -130,8 +135,14 @@ std::map<std::string, std::string> ScaffoldVars(
     vars.emplace("title", api.value("title", ""));
     vars.emplace("description", api.value("description", ""));
     vars.emplace("directory", api.value("directory", ""));
+    vars.emplace("service_config_yaml_name",
+                 absl::StrCat(api.value("directory", ""), "/",
+                              api.value("configFile", "")));
     vars.emplace("nameInServiceConfig", api.value("nameInServiceConfig", ""));
-    vars.emplace("configFile", api.value("configFile", ""));
+  }
+  if (!service.override_service_config_yaml_name().empty()) {
+    vars.emplace("service_config_yaml_name",
+                 service.override_service_config_yaml_name());
   }
   auto const library = LibraryName(service.product_path());
   vars["copyright_year"] = service.initial_copyright_year();
@@ -153,18 +164,16 @@ std::map<std::string, std::string> ScaffoldVars(
                      "to change without notice.\n\nPlease,"
                    : "While this library is **GA**, please";
 
-  // Try to load the service config YAML file. On failure just return the
-  // existing vars.
-  auto const config_file = vars.find("configFile");
-  auto const directory = vars.find("directory");
-  if (config_file == vars.end() || directory == vars.end()) {
+  // Find out if the service config YAML is configured.
+  auto path = ServiceConfigYamlPath(yaml_root, vars);
+  if (path.empty()) {
     GCP_LOG(WARNING) << "Missing directory and/or YAML config file name for: "
                      << service.service_proto_path();
     return vars;
   }
 
-  auto const path =
-      googleapis_path + "/" + directory->second + "/" + config_file->second;
+  // Try to load the service config YAML file. On failure just return the
+  // existing vars.
   auto status = google::cloud::internal::status(path);
   if (!exists(status)) {
     GCP_LOG(WARNING) << "Cannot find YAML service config file (" << path
@@ -213,12 +222,11 @@ std::map<std::string, std::string> ScaffoldVars(
   return vars;
 }
 
-void MakeDirectory(std::string const& path) {
-#if _WIN32
-  _mkdir(path.c_str());
-#else
-  mkdir(path.c_str(), 0755);
-#endif  // _WIN32
+std::string ServiceConfigYamlPath(
+    std::string const& root, std::map<std::string, std::string> const& vars) {
+  auto const name = vars.find("service_config_yaml_name");
+  if (name == vars.end()) return {};
+  return absl::StrCat(root, "/", name->second);
 }
 
 void GenerateMetadata(
@@ -336,10 +344,12 @@ void GenerateScaffold(
 
 void GenerateReadme(std::ostream& os,
                     std::map<std::string, std::string> const& variables) {
-  auto constexpr kText = R"""(# $title$ C++ Client Library
+  auto constexpr kText1 = R"""(# $title$ C++ Client Library
 $construction$
 This directory contains an idiomatic C++ client library for the
-[$title$][cloud-service-docs], a service to $description$
+[$title$][cloud-service-docs].
+
+$description$
 
 $status$ note that the Google Cloud C++ client
 libraries do **not** follow [Semantic Versioning](https://semver.org/).
@@ -361,13 +371,18 @@ this library.
   client library
 * Detailed header comments in our [public `.h`][source-link] files
 
-[cloud-service-docs]: https://cloud.google.com/$site_root$
-[doxygen-link]: https://cloud.google.com/cpp/docs/reference/$library$/latest/
+)""";
+
+  auto constexpr kText2 =
+      R"""([doxygen-link]: https://cloud.google.com/cpp/docs/reference/$library$/latest/
 [source-link]: https://github.com/googleapis/google-cloud-cpp/tree/main/google/cloud/$library$
 )""";
   google::protobuf::io::OstreamOutputStream output(&os);
   google::protobuf::io::Printer printer(&output, '$');
-  printer.Print(variables, kText);
+  printer.Print(
+      variables,
+      absl::StrCat(kText1, FormatCloudServiceDocsLink(variables), kText2)
+          .c_str());
 }
 
 void GenerateBuild(std::ostream& os,
@@ -386,67 +401,23 @@ void GenerateBuild(std::ostream& os,
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+load("//bazel:gapic.bzl", "cc_gapic_library")
+
 package(default_visibility = ["//visibility:private"])
 
 licenses(["notice"])  # Apache 2.0
 
 service_dirs = ["$service_subdirectory$"]
 
-src_dirs = service_dirs + [d + "internal/" for d in service_dirs]
+googleapis_deps = [
+    "@com_google_googleapis//$directory$:$library$_cc_grpc",
+]
 
-filegroup(
-    name = "srcs",
-    srcs = glob([d + "*.cc" for d in src_dirs]),
+cc_gapic_library(
+    name = "$library$",
+    googleapis_deps = googleapis_deps,
+    service_dirs = service_dirs,
 )
-
-filegroup(
-    name = "hdrs",
-    srcs = glob([d + "*.h" for d in src_dirs]),
-)
-
-filegroup(
-    name = "public_hdrs",
-    srcs = glob([d + "*.h" for d in service_dirs]),
-    visibility = ["//:__pkg__"],
-)
-
-filegroup(
-    name = "mocks",
-    srcs = glob([d + "mocks/*.h" for d in service_dirs]),
-    visibility = ["//:__pkg__"],
-)
-
-cc_library(
-    name = "google_cloud_cpp_$library$",
-    srcs = [":srcs"],
-    hdrs = [":hdrs"],
-    visibility = ["//:__pkg__"],
-    deps = [
-        "//:common",
-        "//:grpc_utils",
-        "@com_google_googleapis//$directory$:$library$_cc_grpc",
-    ],
-)
-
-cc_library(
-    name = "google_cloud_cpp_$library$_mocks",
-    hdrs = [":mocks"],
-    visibility = ["//:__pkg__"],
-    deps = [
-        ":google_cloud_cpp_$library$",
-        "@com_google_googletest//:gtest",
-    ],
-)
-
-[cc_test(
-    name = sample.replace("/", "_").replace(".cc", ""),
-    srcs = [sample],
-    tags = ["integration-test"],
-    deps = [
-        "//:$library$",
-        "//google/cloud/testing_util:google_cloud_cpp_testing_private",
-    ],
-) for sample in glob([d + "samples/*.cc" for d in service_dirs])]
 )""";
   google::protobuf::io::OstreamOutputStream output(&os);
   google::protobuf::io::Printer printer(&output, '$');
@@ -473,9 +444,8 @@ void GenerateCMakeLists(std::ostream& os,
 
 include(GoogleCloudCppLibrary)
 
-set(GOOGLE_CLOUD_CPP_SERVICE_DIRS "$service_subdirectory$")
-
-google_cloud_cpp_add_ga_grpc_library($library$ "$title$"$experimental$)
+google_cloud_cpp_add_gapic_library($library$ "$title$"$experimental$
+    SERVICE_DIRS "$service_subdirectory$")
 
 if (BUILD_TESTING AND GOOGLE_CLOUD_CPP_ENABLE_CXX_EXCEPTIONS)
     add_executable($library$_quickstart "quickstart/quickstart.cc")
@@ -499,12 +469,13 @@ endif ()
 
 void GenerateDoxygenMainPage(
     std::ostream& os, std::map<std::string, std::string> const& variables) {
-  auto constexpr kText = R"""(/*!
+  auto constexpr kText1 = R"""(/*!
 
 @mainpage $title$ C++ Client Library
 
-An idiomatic C++ client library for the [$title$][cloud-service-docs], a service
-to $description$
+An idiomatic C++ client library for the [$title$][cloud-service-docs].
+
+$description$
 
 $status$ note that the Google Cloud C++ client libraries do **not** follow
 [Semantic Versioning](https://semver.org/).
@@ -536,13 +507,17 @@ which should give you a taste of the $title$ C++ client library API.
 - @ref $library$-env - describes environment variables that can configure the
   behavior of the library.
 
-[cloud-service-docs]: https://cloud.google.com/$site_root$
+)""";
 
+  auto constexpr kText2 = R"""(
 */
 )""";
   google::protobuf::io::OstreamOutputStream output(&os);
   google::protobuf::io::Printer printer(&output, '$');
-  printer.Print(variables, kText);
+  printer.Print(
+      variables,
+      absl::StrCat(kText1, FormatCloudServiceDocsLink(variables), kText2)
+          .c_str());
 }
 
 void GenerateDoxygenEnvironmentPage(
@@ -570,7 +545,7 @@ request and response.  Unless you have configured your own logging backend,
 you should also set `GOOGLE_CLOUD_CPP_ENABLE_CLOG` to produce any output on
 the program's console.
 
-@see google::cloud::TracingComponentsOption
+@see google::cloud::LoggingComponentsOption
 
 `GOOGLE_CLOUD_CPP_TRACING_OPTIONS=...`: modifies the behavior of gRPC tracing,
 including whether messages will be output on multiple lines, or whether
@@ -797,8 +772,7 @@ some experience as a C++ developer and that you have a working C++ toolchain
 ## Before you begin
 
 To run the quickstart examples you will need a working Google Cloud Platform
-(GCP) project. The [quickstart][quickstart-link] covers the necessary
-steps in detail.
+(GCP) project.
 
 ## Configuring authentication for the C++ Client Library
 
@@ -919,7 +893,6 @@ set GRPC_DEFAULT_SSL_ROOTS_FILE_PATH=%cd%\roots.pem
 ```
 
 [bazel-install]: https://docs.bazel.build/versions/main/install.html
-[quickstart-link]: https://cloud.google.com/$site_root$/docs/quickstart
 [grpc-roots-pem-bug]: https://github.com/grpc/grpc/issues/16571
 [choco-cmake-link]: https://chocolatey.org/packages/cmake
 [homebrew-cmake-link]: https://formulae.brew.sh/formula/cmake
