@@ -17,9 +17,11 @@
 #include "generator/internal/longrunning.h"
 #include "generator/internal/pagination.h"
 #include "generator/internal/printer.h"
+#include "generator/internal/request_id.h"
 #include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/internal/absl_str_replace_quiet.h"
 #include "google/cloud/internal/algorithm.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/strip.h"
@@ -57,12 +59,14 @@ ServiceCodeGenerator::ServiceCodeGenerator(
     google::protobuf::ServiceDescriptor const* service_descriptor,
     VarsDictionary service_vars,
     std::map<std::string, VarsDictionary> service_method_vars,
-    google::protobuf::compiler::GeneratorContext* context)
+    google::protobuf::compiler::GeneratorContext* context,
+    std::vector<MixinMethod> const& mixin_methods)
     : service_descriptor_(service_descriptor),
       service_vars_(std::move(service_vars)),
       service_method_vars_(std::move(service_method_vars)),
       header_(context, service_vars_[header_path_key]),
-      cc_(context, service_vars_[cc_path_key]) {
+      cc_(context, service_vars_[cc_path_key]),
+      mixin_methods_(mixin_methods) {
   assert(service_descriptor != nullptr);
   assert(context != nullptr);
   SetVars(service_vars_[header_path_key]);
@@ -77,11 +81,13 @@ ServiceCodeGenerator::ServiceCodeGenerator(
     google::protobuf::ServiceDescriptor const* service_descriptor,
     VarsDictionary service_vars,
     std::map<std::string, VarsDictionary> service_method_vars,
-    google::protobuf::compiler::GeneratorContext* context)
+    google::protobuf::compiler::GeneratorContext* context,
+    std::vector<MixinMethod> const& mixin_methods)
     : service_descriptor_(service_descriptor),
       service_vars_(std::move(service_vars)),
       service_method_vars_(std::move(service_method_vars)),
-      header_(context, service_vars_[header_path_key]) {
+      header_(context, service_vars_[header_path_key]),
+      mixin_methods_(mixin_methods) {
   assert(service_descriptor != nullptr);
   assert(context != nullptr);
   SetVars(service_vars_[header_path_key]);
@@ -200,6 +206,27 @@ bool ServiceCodeGenerator::HasGenerateGrpcTransport() const {
          generate_grpc_transport->second == "true";
 }
 
+bool ServiceCodeGenerator::HasRequestId() const {
+  return std::any_of(service_method_vars_.begin(), service_method_vars_.end(),
+                     [](auto const& kv) {
+                       return kv.second.find("request_id_field_name") !=
+                              kv.second.end();
+                     });
+}
+
+bool ServiceCodeGenerator::HasRequestId(
+    google::protobuf::MethodDescriptor const& method) const {
+  auto mv = service_method_vars_.find(method.full_name());
+  if (mv == service_method_vars_.end()) return false;
+  auto const& method_vars = mv->second;
+  return method_vars.find("request_id_field_name") != method_vars.end();
+}
+
+bool ServiceCodeGenerator::HasApiVersion() const {
+  auto iter = service_vars_.find("api_version");
+  return iter != service_vars_.end() && !iter->second.empty();
+}
+
 std::vector<std::string>
 ServiceCodeGenerator::MethodSignatureWellKnownProtobufTypeIncludes() const {
   std::vector<std::string> include_paths;
@@ -381,8 +408,8 @@ Status ServiceCodeGenerator::OpenNamespaces(
     std::string const& ns_documentation) {
   auto result = service_vars_.find(product_path_var);
   if (result == service_vars_.end()) {
-    return Status(StatusCode::kInternal,
-                  product_path_var + " not found in vars");
+    return internal::InternalError(product_path_var + " not found in vars",
+                                   GCP_ERROR_INFO());
   }
   namespace_ = Namespace(service_vars_[product_path_var], ns_type);
   p.Print(R"""(
@@ -432,7 +459,6 @@ void ServiceCodeGenerator::SetMethods() {
     for (auto& a : s) a = SafeReplaceAll(a, "@", ",");
     return {s.begin(), s.end()};
   };
-  auto const emitted_rpcs = split_arg("emitted_rpcs");
   auto const omitted_rpcs = split_arg("omitted_rpcs");
   auto const gen_async_rpcs = split_arg("gen_async_rpcs");
 
@@ -445,20 +471,6 @@ void ServiceCodeGenerator::SetMethods() {
       return v == method_name || v == qualified_method_name;
     };
     bool omit_rpc = internal::ContainsIf(omitted_rpcs, any_match);
-    if (!omit_rpc && method->options().deprecated()) {
-      // Deprecated RPCs must be listed in either omitted_rpcs or
-      // emitted_rpcs. The former is used for newly-generated services,
-      // where we never want to support the deprecated RPC, and the
-      // latter for newly-deprecated RPCs, where we want to maintain
-      // backwards compatibility.
-      if (!internal::ContainsIf(emitted_rpcs, any_match)) {
-        GCP_LOG(FATAL) << "Deprecated RPC " << qualified_method_name
-                       << " must be listed in either omitted_rpcs"
-                       << " or emitted_rpcs";
-      }
-      // TODO(#8486): Add a @deprecated Doxygen comment and the
-      // GOOGLE_CLOUD_CPP_DEPRECATED annotation to the generated RPC.
-    }
     if (!omit_rpc) methods_.emplace_back(*method);
     if (internal::ContainsIf(gen_async_rpcs, any_match)) {
       // We still generate the async API for omitted (and possibly
@@ -466,6 +478,10 @@ void ServiceCodeGenerator::SetMethods() {
       async_methods_.emplace_back(*method);
     }
   }
+
+  for (auto const& mixin_method : mixin_methods_) {
+    methods_.emplace_back(mixin_method.method.get());
+  }
 }
 
 std::string ServiceCodeGenerator::GetPbIncludeByTransport() const {
@@ -473,11 +489,24 @@ std::string ServiceCodeGenerator::GetPbIncludeByTransport() const {
   return vars("proto_header_path");
 }
 
+std::vector<std::string> ServiceCodeGenerator::GetMixinPbIncludeByTransport()
+    const {
+  std::string const& mixin_pb_header_paths =
+      HasGenerateGrpcTransport() ? vars("mixin_proto_grpc_header_paths")
+                                 : vars("mixin_proto_header_paths");
+
+  return absl::StrSplit(mixin_pb_header_paths, ',');
+}
+
 bool ServiceCodeGenerator::IsDiscoveryDocumentProto() const {
   auto iter = service_vars_.find("proto_file_source");
   return (iter != service_vars_.end() && iter->second == "discovery_document");
 }
 
+std::vector<MixinMethod> const& ServiceCodeGenerator::MixinMethods() const {
+  return mixin_methods_;
+}
+
 }  // namespace generator_internal
 }  // namespace cloud
 }  // namespace google
