@@ -17,9 +17,11 @@
 #include "generator/internal/codegen_utils.h"
 #include "generator/internal/descriptor_utils.h"
 #include "generator/internal/discovery_to_proto.h"
+#include "generator/internal/format_method_comments.h"
 #include "generator/internal/scaffold_generator.h"
 #include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
 #include "google/cloud/status_or.h"
 #include "absl/flags/flag.h"
@@ -55,13 +57,17 @@ ABSL_FLAG(std::string, scaffold, "",
 ABSL_FLAG(bool, experimental_scaffold, false,
           "Generate experimental library support files.");
 ABSL_FLAG(bool, update_ci, true, "Update the CI support files.");
-ABSL_FLAG(bool, check_parameter_comment_substitutions, false,
-          "Check that the built-in parameter comment substitutions applied.");
+ABSL_FLAG(bool, check_comment_substitutions, false,
+          "Check that the built-in comment substitutions applied.");
 ABSL_FLAG(bool, generate_discovery_protos, false,
           "Generate only .proto files, no C++ code.");
-
+ABSL_FLAG(bool, enable_parallel_write_for_discovery_protos, true,
+          "Enable parallelized file writing for discovery protos. This allows "
+          "for readable logs.");
 namespace {
 
+using ::google::cloud::generator_internal::CheckMethodCommentSubstitutions;
+using ::google::cloud::generator_internal::CheckParameterCommentSubstitutions;
 using ::google::cloud::generator_internal::GenerateMetadata;
 using ::google::cloud::generator_internal::GenerateScaffold;
 using ::google::cloud::generator_internal::LibraryName;
@@ -69,6 +75,7 @@ using ::google::cloud::generator_internal::LibraryPath;
 using ::google::cloud::generator_internal::LoadApiIndex;
 using ::google::cloud::generator_internal::SafeReplaceAll;
 using ::google::cloud::generator_internal::ScaffoldVars;
+using ::google::cloud::generator_internal::ServiceConfigYamlPath;
 
 struct CommandLineArgs {
   std::string config_file;
@@ -83,6 +90,7 @@ struct CommandLineArgs {
   bool experimental_scaffold;
   bool update_ci;
   bool generate_discovery_protos;
+  bool enable_parallel_write_for_discovery_protos;
 };
 
 google::cloud::StatusOr<google::cloud::cpp::generator::GeneratorConfiguration>
@@ -96,8 +104,8 @@ GetConfig(std::string const& filepath) {
       buffer.str(), &generator_config);
 
   if (parse_result) return generator_config;
-  return google::cloud::Status(google::cloud::StatusCode::kInvalidArgument,
-                               "Unable to parse textproto file.");
+  return google::cloud::internal::InvalidArgumentError(
+      "Unable to parse textproto file.", GCP_ERROR_INFO());
 }
 
 /**
@@ -166,6 +174,13 @@ int WriteInstallDirectories(
     }
     auto const lib = LibraryName(product_path);
     install_directories.push_back("./lib64/cmake/google_cloud_cpp_" + lib);
+    // Note that storage does not have a public-facing mocks library. Only
+    // GCS+gRPC does.
+    // TODO(#5782) - install mocks for compute
+    if (lib != "compute" && lib != "storage") {
+      install_directories.push_back("./lib64/cmake/google_cloud_cpp_" + lib +
+                                    "_mocks");
+    }
   }
   std::sort(install_directories.begin(), install_directories.end());
   auto end =
@@ -195,6 +210,7 @@ google::cloud::Status GenerateProtosForRestProducts(
             generator_args.protobuf_proto_path,
             generator_args.googleapis_proto_path, generator_args.output_path,
             generator_args.export_output_path,
+            generator_args.enable_parallel_write_for_discovery_protos,
             std::set<std::string>(p.operation_services().begin(),
                                   p.operation_services().end()));
     if (!status.ok()) return status;
@@ -207,12 +223,15 @@ std::vector<std::future<google::cloud::Status>> GenerateCodeFromProtos(
     CommandLineArgs const& generator_args,
     google::protobuf::RepeatedPtrField<
         google::cloud::cpp::generator::ServiceConfiguration> const& services) {
+  auto const yaml_root = generator_args.golden_proto_path.empty()
+                             ? generator_args.googleapis_proto_path
+                             : generator_args.golden_proto_path;
+
   std::vector<std::future<google::cloud::Status>> tasks;
   auto const api_index = LoadApiIndex(generator_args.googleapis_proto_path);
   for (auto const& service : services) {
-    auto scaffold_vars =
-        ScaffoldVars(generator_args.googleapis_proto_path, api_index, service,
-                     generator_args.experimental_scaffold);
+    auto scaffold_vars = ScaffoldVars(yaml_root, api_index, service,
+                                      generator_args.experimental_scaffold);
     auto const generate_scaffold =
         LibraryPath(service.product_path()) == generator_args.scaffold;
     if (generate_scaffold) {
@@ -247,10 +266,6 @@ std::vector<std::future<google::cloud::Status>> GenerateCodeFromProtos(
       args.emplace_back(absl::StrCat("--cpp_codegen_opt=omit_rpc=",
                                      SafeReplaceAll(omit_rpc, ",", "@")));
     }
-    for (auto const& emit_rpc : service.emitted_rpcs()) {
-      args.emplace_back(absl::StrCat("--cpp_codegen_opt=emit_rpc=",
-                                     SafeReplaceAll(emit_rpc, ",", "@")));
-    }
     if (service.backwards_compatibility_namespace_alias()) {
       args.emplace_back(
           "--cpp_codegen_opt=backwards_compatibility_namespace_alias=true");
@@ -267,6 +282,9 @@ std::vector<std::future<google::cloud::Status>> GenerateCodeFromProtos(
     if (service.omit_stub_factory()) {
       args.emplace_back("--cpp_codegen_opt=omit_stub_factory=true");
     }
+    if (service.omit_streaming_updater()) {
+      args.emplace_back("--cpp_codegen_opt=omit_streaming_updater=true");
+    }
     if (service.generate_round_robin_decorator()) {
       args.emplace_back(
           "--cpp_codegen_opt=generate_round_robin_decorator=true");
@@ -348,27 +366,35 @@ std::vector<std::future<google::cloud::Status>> GenerateCodeFromProtos(
                        "=", kv.second));
     }
 
+    auto path = ServiceConfigYamlPath(yaml_root, scaffold_vars);
+    if (!path.empty()) {
+      args.emplace_back(absl::StrCat("--cpp_codegen_opt=service_config_yaml=",
+                                     std::move(path)));
+    }
+
     GCP_LOG(INFO) << "Generating service code using: "
                   << absl::StrJoin(args, ";") << "\n";
 
-    tasks.push_back(std::async(std::launch::async, [args] {
-      google::protobuf::compiler::CommandLineInterface cli;
-      google::cloud::generator::Generator generator;
-      cli.RegisterGenerator("--cpp_codegen_out", "--cpp_codegen_opt",
-                            &generator, "Codegen C++ Generator");
-      std::vector<char const*> c_args;
-      c_args.reserve(args.size());
-      for (auto const& arg : args) {
-        c_args.push_back(arg.c_str());
-      }
+    tasks.push_back(std::async(
+        std::launch::async, [args, proto_file = service.service_proto_path()] {
+          google::protobuf::compiler::CommandLineInterface cli;
+          google::cloud::generator::Generator generator;
+          cli.RegisterGenerator("--cpp_codegen_out", "--cpp_codegen_opt",
+                                &generator, "Codegen C++ Generator");
+          std::vector<char const*> c_args;
+          c_args.reserve(args.size());
+          for (auto const& arg : args) {
+            c_args.push_back(arg.c_str());
+          }
 
-      if (cli.Run(static_cast<int>(c_args.size()), c_args.data()) != 0)
-        return google::cloud::Status(google::cloud::StatusCode::kInternal,
-                                     absl::StrCat("Generating service from ",
-                                                  c_args.back(), " failed."));
-
-      return google::cloud::Status{};
-    }));
+          if (cli.Run(static_cast<int>(c_args.size()), c_args.data()) != 0) {
+            return google::cloud::internal::InternalError(
+                absl::StrCat("Generating service from ", proto_file,
+                             " failed."),
+                GCP_ERROR_INFO());
+          }
+          return google::cloud::Status{};
+        }));
   }
   return tasks;
 }
@@ -407,18 +433,20 @@ int main(int argc, char** argv) {
         << "; you'll need to recompile everything for that to work";
   }
 
-  CommandLineArgs const args{absl::GetFlag(FLAGS_config_file),
-                             absl::GetFlag(FLAGS_protobuf_proto_path),
-                             absl::GetFlag(FLAGS_googleapis_proto_path),
-                             absl::GetFlag(FLAGS_golden_proto_path),
-                             absl::GetFlag(FLAGS_discovery_proto_path),
-                             absl::GetFlag(FLAGS_output_path),
-                             absl::GetFlag(FLAGS_export_output_path),
-                             absl::GetFlag(FLAGS_scaffold_templates_path),
-                             absl::GetFlag(FLAGS_scaffold),
-                             absl::GetFlag(FLAGS_experimental_scaffold),
-                             absl::GetFlag(FLAGS_update_ci),
-                             absl::GetFlag(FLAGS_generate_discovery_protos)};
+  CommandLineArgs const args{
+      absl::GetFlag(FLAGS_config_file),
+      absl::GetFlag(FLAGS_protobuf_proto_path),
+      absl::GetFlag(FLAGS_googleapis_proto_path),
+      absl::GetFlag(FLAGS_golden_proto_path),
+      absl::GetFlag(FLAGS_discovery_proto_path),
+      absl::GetFlag(FLAGS_output_path),
+      absl::GetFlag(FLAGS_export_output_path),
+      absl::GetFlag(FLAGS_scaffold_templates_path),
+      absl::GetFlag(FLAGS_scaffold),
+      absl::GetFlag(FLAGS_experimental_scaffold),
+      absl::GetFlag(FLAGS_update_ci),
+      absl::GetFlag(FLAGS_generate_discovery_protos),
+      absl::GetFlag(FLAGS_enable_parallel_write_for_discovery_protos)};
 
   GCP_LOG(INFO) << "proto_path = " << args.protobuf_proto_path << "\n";
   GCP_LOG(INFO) << "googleapis_path = " << args.googleapis_proto_path << "\n";
@@ -473,15 +501,17 @@ int main(int argc, char** argv) {
     }
   }
 
-  // If we were asked to check the parameter comment substitutions, and some
-  // went unused, emit a fatal error so that we might remove/fix them. The
+  // If we were asked to check the comment substitutions, and some went
+  // unused, emit a fatal error so that we might remove/fix them. The
   // substitutions should probably be part of the config file (rather than
   // being built in) so that the check could be unconditional (instead of
   // flag-based).
-  if (absl::GetFlag(FLAGS_check_parameter_comment_substitutions) &&
-      !google::cloud::generator_internal::
-          CheckParameterCommentSubstitutions()) {
-    GCP_LOG(FATAL) << "Remove unused parameter comment substitution(s)";
+  if (absl::GetFlag(FLAGS_check_comment_substitutions)) {
+    bool parameters_ok = CheckParameterCommentSubstitutions();
+    bool methods_ok = CheckMethodCommentSubstitutions();
+    if (!parameters_ok || !methods_ok) {
+      GCP_LOG(FATAL) << "Remove unused comment substitution(s)";
+    }
   }
 
   return rc;
