@@ -22,7 +22,7 @@ FakeSourceTree::FakeSourceTree(std::map<std::string, std::string> files)
     : files_(std::move(files)) {}
 
 void FakeSourceTree::Insert(std::string const& filename, std::string contents) {
-  files_.emplace(filename, std::move(contents));
+  files_[filename] = std::move(contents);
 }
 
 google::protobuf::io::ZeroCopyInputStream* FakeSourceTree::Open(
