@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/appengine/services_client.h"
+//! [all]
+#include "google/cloud/appengine/v1/services_client.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,18 +22,19 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace appengine = ::google::cloud::appengine;
+  namespace appengine = ::google::cloud::appengine_v1;
   auto client = appengine::ServicesClient(appengine::MakeServicesConnection());
 
   ::google::appengine::v1::ListServicesRequest request;
   request.set_parent(std::string{"apps/"} + argv[1]);
   for (auto r : client.ListServices(request)) {
-    if (!r) throw std::runtime_error(r.status().message());
+    if (!r) throw std::move(r).status();
     std::cout << r->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
