@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/artifactregistry/artifact_registry_client.h"
+//! [all]
+#include "google/cloud/artifactregistry/v1/artifact_registry_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
@@ -22,19 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace artifactregistry = ::google::cloud::artifactregistry;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace artifactregistry = ::google::cloud::artifactregistry_v1;
   auto client = artifactregistry::ArtifactRegistryClient(
       artifactregistry::MakeArtifactRegistryConnection());
 
-  auto const parent =
-      std::string{"projects/"} + argv[1] + "/locations/" + argv[2];
-  for (auto r : client.ListRepositories(parent)) {
-    if (!r) throw std::runtime_error(r.status().message());
+  for (auto r : client.ListRepositories(location.FullName())) {
+    if (!r) throw std::move(r).status();
     std::cout << r->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
