@@ -29,6 +29,17 @@ namespace cloud {
 namespace bigquery_v2_minimal_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
+// JobCreationMode Enum Codes
+JobCreationMode JobCreationMode::UnSpecified() {
+  return JobCreationMode{"JOB_CREATION_MODE_UNSPECIFIED"};
+}
+JobCreationMode JobCreationMode::Required() {
+  return JobCreationMode{"JOB_CREATION_REQUIRED"};
+}
+JobCreationMode JobCreationMode::Optional() {
+  return JobCreationMode{"JOB_CREATION_OPTIONAL"};
+}
+
 Projection Projection::Full() {
   Projection projection;
   projection.value = "FULL";
@@ -94,6 +105,14 @@ std::string InsertJobRequest::DebugString(absl::string_view name,
       .Build();
 }
 
+std::string JobCreationMode::DebugString(absl::string_view name,
+                                         TracingOptions const& options,
+                                         int indent) const {
+  return internal::DebugFormatter(name, options, indent)
+      .StringField("value", value)
+      .Build();
+}
+
 std::string Projection::DebugString(absl::string_view name,
                                     TracingOptions const& options,
                                     int indent) const {
@@ -117,14 +136,6 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(GetJobRequest const& r) {
   auto const& opts = internal::CurrentOptions();
 
   rest_internal::RestRequest request;
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid GetJobRequest: Project Id is empty", GCP_ERROR_INFO());
-  }
-  if (r.job_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid GetJobRequest: Job Id is empty", GCP_ERROR_INFO());
-  }
   // Builds GetJob request path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
 
@@ -149,11 +160,6 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(
     ListJobsRequest const& r) {
   rest_internal::RestRequest request;
   auto const& opts = internal::CurrentOptions();
-
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid ListJobsRequest: Project Id is empty", GCP_ERROR_INFO());
-  }
   // Builds GetJob request path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
 
@@ -213,33 +219,12 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(
   auto const& opts = internal::CurrentOptions();
 
   rest_internal::RestRequest request;
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid InsertJobRequest: Project Id is empty", GCP_ERROR_INFO());
-  }
-
   // Builds InsertJob request path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
   std::string path =
       absl::StrCat(endpoint, "/projects/", r.project_id(), "/jobs");
   request.SetPath(std::move(path));
 
-  // Validate request body is a valid json Job payload.
-  nlohmann::json json_payload;
-  to_json(json_payload, r.job());
-
-  if (!json_payload.is_object()) {
-    return internal::InvalidArgumentError(
-        "Invalid InsertJobRequest: Invalid json payload", GCP_ERROR_INFO());
-  }
-
-  auto const& job = json_payload.get<Job>();
-
-  if (job.configuration.query.query.empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid InsertJobRequest: Invalid Job object", GCP_ERROR_INFO());
-  }
-
   return request;
 }
 
@@ -248,15 +233,6 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(
   auto const& opts = internal::CurrentOptions();
 
   rest_internal::RestRequest request;
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid CancelJobRequest: Project Id is empty", GCP_ERROR_INFO());
-  }
-  if (r.job_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid CancelJobRequest: Job Id is empty", GCP_ERROR_INFO());
-  }
-
   // Builds CancelJob request path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
   std::string path = absl::StrCat(endpoint, "/projects/", r.project_id(),
@@ -344,34 +320,12 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(
   auto const& opts = internal::CurrentOptions();
   rest_internal::RestRequest request;
 
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid PostQueryRequest: Project Id is empty", GCP_ERROR_INFO());
-  }
-
   // Builds PostQueryRequest path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
   std::string path =
       absl::StrCat(endpoint, "/projects/", r.project_id(), "/queries");
   request.SetPath(std::move(path));
 
-  // Validate request body is a valid json QueryRequest payload.
-  nlohmann::json json_payload;
-  to_json(json_payload, r.query_request());
-
-  if (!json_payload.is_object()) {
-    return internal::InvalidArgumentError(
-        "Invalid PostQueryRequest: Invalid json payload", GCP_ERROR_INFO());
-  }
-
-  auto const& query_request = json_payload.get<QueryRequest>();
-
-  if (query_request.query().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid PostQueryRequest: Missing required query field",
-        GCP_ERROR_INFO());
-  }
-
   return request;
 }
 
@@ -414,6 +368,7 @@ std::string QueryRequest::DebugString(absl::string_view name,
       .Field("labels", labels())
       .SubMessage("default_dataset", default_dataset())
       .SubMessage("format_options", format_options())
+      .SubMessage("job_creation_mode", job_creation_mode())
       .Build();
 }
 
@@ -444,16 +399,6 @@ StatusOr<rest_internal::RestRequest> BuildRestRequest(
   auto const& opts = internal::CurrentOptions();
   rest_internal::RestRequest request;
 
-  if (r.project_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid GetQueryResultsRequest: Project Id is empty",
-        GCP_ERROR_INFO());
-  }
-  if (r.job_id().empty()) {
-    return internal::InvalidArgumentError(
-        "Invalid GetQueryResultsRequest: Job Id is empty", GCP_ERROR_INFO());
-  }
-
   // Builds GetQueryResultsRequest path based on endpoint provided.
   std::string endpoint = GetBaseEndpoint(opts);
   std::string path = absl::StrCat(endpoint, "/projects/", r.project_id(),
