@@ -33,9 +33,6 @@ using ::google::cloud::bigquery_v2_minimal_testing::
 using ::google::cloud::bigquery_v2_minimal_testing::MakeJob;
 using ::google::cloud::bigquery_v2_minimal_testing::MakePostQueryRequest;
 
-using ::google::cloud::testing_util::StatusIs;
-using ::testing::HasSubstr;
-
 auto static const kMinCreationTimeMs = 1111111111111;
 auto static const kMaxCreationTimeMs = 1111111111211;
 auto static const kMinTime = std::chrono::system_clock::from_time_t(0) +
@@ -117,28 +114,6 @@ TEST(GetJobRequestTest, SuccessWithEndpoint) {
   }
 }
 
-TEST(GetJobRequestTest, EmptyProjectId) {
-  GetJobRequest request("", "test-job-id");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Project Id is empty")));
-}
-
-TEST(GetJobRequestTest, EmptyJobId) {
-  GetJobRequest request("test-project-id", "");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Job Id is empty")));
-}
-
 TEST(GetJobRequestTest, OutputStream) {
   GetJobRequest request("test-project-id", "test-job-id");
   request.set_location("test-location");
@@ -175,17 +150,6 @@ TEST(ListJobsRequestTest, Success) {
   EXPECT_EQ(expected, *actual);
 }
 
-TEST(ListJobsRequestTest, EmptyProjectId) {
-  ListJobsRequest request("");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Project Id is empty")));
-}
-
 TEST(ListJobsRequestTest, OutputStream) {
   auto const request = GetListJobsRequest();
   std::string expected =
@@ -304,31 +268,6 @@ TEST(InsertJobRequestTest, Success) {
   EXPECT_EQ(expected, *actual);
 }
 
-TEST(InsertJobRequestTest, EmptyProjectId) {
-  InsertJobRequest request("", MakeJob());
-
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto actual = BuildRestRequest(request);
-  EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument,
-                               HasSubstr("Project Id is empty")));
-}
-
-TEST(InsertJobRequestTest, EmptyQuery) {
-  InsertJobRequest request;
-  request.set_project_id("1234");
-
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto actual = BuildRestRequest(request);
-  EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument,
-                               HasSubstr("Invalid Job object")));
-}
-
 TEST(InsertJobRequest, DebugString) {
   InsertJobRequest request("1234", MakeJob());
 
@@ -1028,28 +967,6 @@ TEST(CancelJobRequestTest, SuccessWithDifferentEndpoints) {
   }
 }
 
-TEST(CancelJobRequestTest, FailEmptyProjectId) {
-  CancelJobRequest request("", "test-job-id");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Project Id is empty")));
-}
-
-TEST(CancelJobRequestTest, FailEmptyJobId) {
-  CancelJobRequest request("test-project-id", "");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Job Id is empty")));
-}
-
 TEST(CancelJobRequestTest, DebugString) {
   CancelJobRequest request("test-project-id", "test-job-id");
   request.set_location("test-location");
@@ -1094,30 +1011,6 @@ TEST(PostQueryRequestTest, Success) {
   EXPECT_EQ(expected, *actual);
 }
 
-TEST(PostQueryRequestTest, EmptyProjectId) {
-  PostQueryRequest request("");
-
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto actual = BuildRestRequest(request);
-  EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument,
-                               HasSubstr("Project Id is empty")));
-}
-
-TEST(PostQueryRequestTest, MissingQuery) {
-  PostQueryRequest request("1234");
-
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto actual = BuildRestRequest(request);
-  EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument,
-                               HasSubstr("Missing required query field")));
-}
-
 TEST(PostQueryRequestTest, DebugString) {
   auto const& request = MakePostQueryRequest();
 
@@ -1135,7 +1028,8 @@ TEST(PostQueryRequestTest, DebugString) {
       R"( parameter_value { value: "query-parameter-value" } })"
       R"( labels { key: "lk1" value: "lv1" } labels { key: "lk2" value: "lv2" })"
       R"( default_dataset { project_id: "2" dataset_id: "1" })"
-      R"( format_options { use_int64_timestamp: true } } })");
+      R"( format_options { use_int64_timestamp: true })"
+      R"( job_creation_mode { value: "JOB_CREATION_MODE_UNSPECIFIED" } } })");
 
   EXPECT_EQ(
       request.DebugString(
@@ -1155,7 +1049,8 @@ TEST(PostQueryRequestTest, DebugString) {
       R"( parameter_value { value: "query-p...<truncated>..." } })"
       R"( labels { key: "lk1" value: "lv1" } labels { key: "lk2" value: "lv2" })"
       R"( default_dataset { project_id: "2" dataset_id: "1" })"
-      R"( format_options { use_int64_timestamp: true } } })");
+      R"( format_options { use_int64_timestamp: true })"
+      R"( job_creation_mode { value: "JOB_CRE...<truncated>..." } } })");
 
   EXPECT_EQ(request.DebugString("PostQueryRequest", TracingOptions{}.SetOptions(
                                                         "single_line_mode=F")),
@@ -1209,6 +1104,9 @@ TEST(PostQueryRequestTest, DebugString) {
     format_options {
       use_int64_timestamp: true
     }
+    job_creation_mode {
+      value: "JOB_CREATION_MODE_UNSPECIFIED"
+    }
   }
 })");
 }
@@ -1283,28 +1181,6 @@ TEST(GetQueryResultsRequestTest, SuccessWithoutQueryParameters) {
   EXPECT_EQ(expected, *actual);
 }
 
-TEST(GetQueryResultsRequestTest, EmptyProjectId) {
-  GetQueryResultsRequest request("", "test-job-id");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Project Id is empty")));
-}
-
-TEST(GetQueryResultsRequestTest, EmptyJobId) {
-  GetQueryResultsRequest request("test-project-id", "");
-  Options opts;
-  opts.set<EndpointOption>("bigquery.googleapis.com");
-  internal::OptionsSpan span(opts);
-
-  auto rest_request = BuildRestRequest(request);
-  EXPECT_THAT(rest_request, StatusIs(StatusCode::kInvalidArgument,
-                                     HasSubstr("Job Id is empty")));
-}
-
 TEST(GetQueryResultsRequestTest, DebugString) {
   auto request = MakeFullGetQueryResultsRequest();
 
