@@ -57,8 +57,8 @@ TEST(JobStatsTest, JobStatsToFromJson) {
       R"(,"referencedTables":[{"datasetId":"1","projectId":"2","tableId":"3"}],"schema":{"fields":[{)"
       R"("categories":{"names":[]},"collation":"")"
       R"(,"defaultValueExpression":"","description":"","fields":{"fields":[]})"
-      R"(,"maxLength":0,"mode":"fmode","name":"fname-1","policyTags":{"names":[]},"precision":0)"
-      R"(,"rangeElementType":{"type":""},"roundingMode":"","scale":0,"type":""}]})"
+      R"(,"maxLength":"0","mode":"fmode","name":"fname-1","policyTags":{"names":[]},"precision":"0")"
+      R"(,"rangeElementType":{"type":""},"roundingMode":"","scale":"0","type":""}]})"
       R"(,"searchStatistics":{"indexUnusedReasons":[{"baseTable":{"datasetId":"1","projectId":"2")"
       R"(,"tableId":"3"},"code":"BASE_TABLE_TOO_SMALL","indexName":"test-index","message":""}])"
       R"(,"indexUsageMode":"PARTIALLY_USED"},"statementType":"statement_type")"
