@@ -1373,9 +1373,9 @@ TEST(JobTest, JobToFromJson) {
       R"(,"referencedTables":[{"datasetId":"1","projectId":"2","tableId":"3"}])"
       R"(,"schema":{"fields":[{"categories":{"names":[]},"collation":"")"
       R"(,"defaultValueExpression":"","description":"","fields":{"fields":[]})"
-      R"(,"maxLength":0,"mode":"fmode","name":"fname-1","policyTags":{"names":[]})"
-      R"(,"precision":0,"rangeElementType":{"type":""},"roundingMode":"")"
-      R"(,"scale":0,"type":""}]},"searchStatistics":{"indexUnusedReasons":[{)"
+      R"(,"maxLength":"0","mode":"fmode","name":"fname-1","policyTags":{"names":[]})"
+      R"(,"precision":"0","rangeElementType":{"type":""},"roundingMode":"")"
+      R"(,"scale":"0","type":""}]},"searchStatistics":{"indexUnusedReasons":[{)"
       R"("baseTable":{"datasetId":"1","projectId":"2","tableId":"3"})"
       R"(,"code":"BASE_TABLE_TOO_SMALL","indexName":"test-index","message":""}])"
       R"(,"indexUsageMode":"PARTIALLY_USED"},"statementType":"statement_type")"
@@ -1496,9 +1496,9 @@ TEST(JobTest, ListFormatJobToFromJson) {
       R"(,"referencedTables":[{"datasetId":"1","projectId":"2","tableId":"3"}])"
       R"(,"schema":{"fields":[{"categories":{"names":[]},"collation":"")"
       R"(,"defaultValueExpression":"","description":"","fields":{"fields":[]})"
-      R"(,"maxLength":0,"mode":"fmode","name":"fname-1","policyTags":{"names":[]})"
-      R"(,"precision":0,"rangeElementType":{"type":""},"roundingMode":"")"
-      R"(,"scale":0,"type":""}]},"searchStatistics":{"indexUnusedReasons":[{)"
+      R"(,"maxLength":"0","mode":"fmode","name":"fname-1","policyTags":{"names":[]})"
+      R"(,"precision":"0","rangeElementType":{"type":""},"roundingMode":"")"
+      R"(,"scale":"0","type":""}]},"searchStatistics":{"indexUnusedReasons":[{)"
       R"("baseTable":{"datasetId":"1","projectId":"2","tableId":"3"})"
       R"(,"code":"BASE_TABLE_TOO_SMALL","indexName":"test-index","message":""}])"
       R"(,"indexUsageMode":"PARTIALLY_USED"},"statementType":"statement_type")"
