@@ -17,6 +17,8 @@
 set -euo pipefail
 
 source "$(dirname "$0")/../../../../ci/lib/init.sh"
+source module /ci/cloudbuild/builds/lib/cloudcxxrc.sh
+source module ci/lib/io.sh
 
 if [[ $# -lt 1 ]]; then
   echo "Usage: $(basename "$0") <bazel-program> [bazel-test-args]"
@@ -29,10 +31,28 @@ BAZEL_VERB="$1"
 shift
 bazel_test_args=("$@")
 
+if bazel::has_no_tests "//google/cloud/bigtable/..." "${BAZEL_TARGETS[@]}"; then
+  exit 0
+fi
+
 # Configure run_emulators_utils.sh to find the instance admin emulator.
 BAZEL_BIN_DIR="$("${BAZEL_BIN}" info bazel-bin)"
 readonly BAZEL_BIN_DIR
-export CBT_INSTANCE_ADMIN_EMULATOR_CMD="${BAZEL_BIN_DIR}/google/cloud/bigtable/tests/instance_admin_emulator"
+
+CBT_INSTANCE_ADMIN_EMULATOR_START=(
+  "${BAZEL_BIN}"
+  run
+  "${bazel_test_args[@]}"
+  --
+  //google/cloud/bigtable/tests:instance_admin_emulator
+)
+# We need to build `instance_admin_emulator` without coverage, so it can start
+# up quickly.
+if [[ "${BAZEL_VERB}" == "coverage" ]]; then
+  io::run "${BAZEL_BIN}" build "${bazel_test_args[@]}" \
+    //google/cloud/bigtable/tests:instance_admin_emulator
+fi
+
 source module /google/cloud/bigtable/tools/run_emulator_utils.sh
 
 # These can only run against production
