@@ -19,7 +19,7 @@
 #include "google/cloud/bigtable/testing/random_names.h"
 #include "google/cloud/internal/getenv.h"
 #include "google/cloud/internal/random.h"
-#include "google/cloud/testing_util/crash_handler.h"
+#include "google/cloud/log.h"
 #include <sstream>
 
 namespace {
@@ -197,7 +197,7 @@ void AsyncReadRow(google::cloud::bigtable::Table table,
             [row_key](future<StatusOr<std::pair<bool, cbt::Row>>> row_future) {
               // Read a row, this returns a tuple (bool, row)
               auto tuple = row_future.get();
-              if (!tuple) throw std::runtime_error(tuple.status().message());
+              if (!tuple) throw std::move(tuple).status();
               if (!tuple->first) {
                 std::cout << "Row " << row_key << " not found\n";
                 return;
@@ -244,7 +244,7 @@ void AsyncCheckAndMutate(google::cloud::bigtable::Table table,
     branch_future
         .then([](future<StatusOr<cbt::MutationBranch>> f) {
           auto response = f.get();
-          if (!response) throw std::runtime_error(response.status().message());
+          if (!response) throw std::move(response).status();
           if (*response == cbt::MutationBranch::kPredicateMatched) {
             std::cout << "The predicate was matched\n";
           } else {
@@ -270,7 +270,7 @@ void AsyncSampleRows(google::cloud::bigtable::Table table,
     samples_future
         .then([](future<StatusOr<std::vector<cbt::RowKeySample>>> f) {
           auto samples = f.get();
-          if (!samples) throw std::runtime_error(samples.status().message());
+          if (!samples) throw std::move(samples).status();
           for (auto const& sample : *samples) {
             std::cout << "key=" << sample.row_key << " - "
                       << sample.offset_bytes << "\n";
@@ -290,7 +290,7 @@ void AsyncReadModifyWrite(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table, std::string const& row_key) {
     future<StatusOr<cbt::Row>> row_future = table.AsyncReadModifyWriteRow(
-        std::move(row_key),
+        row_key,
         cbt::ReadModifyWriteRule::AppendValue("fam", "list", ";element"));
 
     row_future
@@ -342,17 +342,18 @@ void RunAll(std::vector<std::string> const& argv) {
   // Create a table to run the tests on
   std::cout << "\nCreating table to run the examples (" << table_id << ")"
             << std::endl;
-  google::bigtable::admin::v2::GcRule gc;
-  gc.set_max_num_versions(10);
   google::bigtable::admin::v2::Table t;
   auto& families = *t.mutable_column_families();
-  *families["fam"].mutable_gc_rule() = std::move(gc);
+  families["fam"].mutable_gc_rule()->set_max_num_versions(10);
   auto schema = admin.CreateTable(cbt::InstanceName(project_id, instance_id),
                                   table_id, std::move(t));
-  if (!schema) throw std::runtime_error(schema.status().message());
+  if (!schema) throw std::move(schema).status();
 
-  cbt::Table table(cbt::MakeDataClient(project_id, instance_id), table_id,
-                   cbt::AlwaysRetryMutationPolicy());
+  using ::google::cloud::Options;
+  cbt::Table table(cbt::MakeDataConnection(),
+                   cbt::TableResource(project_id, instance_id, table_id),
+                   Options{}.set<cbt::IdempotentMutationPolicyOption>(
+                       cbt::AlwaysRetryMutationPolicy().clone()));
 
   std::cout << "\nRunning the AsyncApply() example" << std::endl;
   AsyncApply(table, {"row-0001"});
@@ -387,14 +388,12 @@ void RunAll(std::vector<std::string> const& argv) {
   std::cout << "\nRunning the AsyncReadModifyWrite() example" << std::endl;
   AsyncReadModifyWrite(table, {"read-modify-write-row-key"});
 
-  (void)admin.DeleteTable(table_id);
+  (void)admin.DeleteTable(table.table_name());
 }
 
 }  // anonymous namespace
 
-int main(int argc, char* argv[]) {
-  google::cloud::testing_util::InstallCrashHandler(argv[0]);
-
+int main(int argc, char* argv[]) try {
   using ::google::cloud::bigtable::examples::MakeCommandEntry;
   google::cloud::bigtable::examples::Example example({
       MakeCommandEntry("async-apply", {"<row-key>"}, AsyncApply),
@@ -411,4 +410,8 @@ int main(int argc, char* argv[]) {
       {"auto", RunAll},
   });
   return example.Run(argc, argv);
+} catch (std::exception const& ex) {
+  std::cerr << ex.what() << "\n";
+  ::google::cloud::LogSink::Instance().Flush();
+  return 1;
 }
