@@ -1,4 +1,4 @@
-// Copyright 2017 Google Inc.
+// Copyright 2017 Google LLC
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -26,8 +26,17 @@ namespace {
 namespace btproto = ::google::bigtable::v2;
 
 using ::google::cloud::testing_util::IsProtoEqual;
-using ::google::cloud::testing_util::chrono_literals::operator"" _ms;
-using ::google::cloud::testing_util::chrono_literals::operator"" _us;
+using ::google::cloud::testing_util::chrono_literals::operator"" _ms;  // NOLINT
+using ::google::cloud::testing_util::chrono_literals::operator"" _us;  // NOLINT
+
+TEST(FiltersTest, Equality) {
+  auto f1 = Filter::PassAllFilter();
+  auto f2 = Filter::Latest(10);
+  EXPECT_NE(f1, f2);
+
+  f2 = f1;
+  EXPECT_EQ(f1, f2);
+}
 
 TEST(FiltersTest, PassAllFilter) {
   auto proto = Filter::PassAllFilter().as_proto();
