@@ -1,4 +1,4 @@
-// Copyright 2018 Google Inc.
+// Copyright 2018 Google LLC
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -21,12 +21,16 @@ namespace bigtable {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
+#include "google/cloud/internal/disable_deprecation_warnings.inc"
+
 TEST(InstanceAdminClientTest, CreateDefaultClient) {
   auto admin_client = CreateDefaultInstanceAdminClient("test-project", {});
   ASSERT_TRUE(admin_client);
   EXPECT_EQ("test-project", admin_client->project());
 }
 
+#include "google/cloud/internal/diagnostics_pop.inc"
+
 TEST(InstanceAdminClientTest, MakeClient) {
   auto admin_client = MakeInstanceAdminClient("test-project");
   ASSERT_TRUE(admin_client);
