@@ -15,6 +15,7 @@
 #include "google/cloud/bigtable/internal/bulk_mutator.h"
 #include "google/cloud/bigtable/rpc_retry_policy.h"
 #include "google/cloud/bigtable/table.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
 #include <numeric>
 
@@ -25,8 +26,6 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
 namespace btproto = ::google::bigtable::v2;
 
-using ::google::cloud::Idempotency;
-
 BulkMutatorState::BulkMutatorState(
     std::string const& app_profile_id, std::string const& table_name,
     bigtable::IdempotentMutationPolicy& idempotent_policy,
@@ -98,6 +97,9 @@ void BulkMutatorState::OnRead(
     auto const index = static_cast<std::size_t>(entry.index());
     auto& annotation = annotations_[index];
     annotation.has_mutation_result = true;
+    // Note that we do not need to heed `RetryInfo` for the status of individual
+    // entries. The server only ever includes `RetryInfo` as the final status of
+    // the stream.
     auto status = MakeStatusFromRpcError(entry.status());
     // Successful responses are not even recorded, this class only reports
     // the failures.  The data for successful responses is discarded, because
@@ -123,8 +125,12 @@ void BulkMutatorState::OnRead(
   }
 }
 
-void BulkMutatorState::OnFinish(google::cloud::Status finish_status) {
+void BulkMutatorState::OnFinish(Status finish_status,
+                                bool enable_server_retries) {
   last_status_ = std::move(finish_status);
+  bool const retryable =
+      enable_server_retries &&
+      google::cloud::internal::GetRetryInfo(last_status_).has_value();
 
   int index = 0;
   for (auto& annotation : annotations_) {
@@ -134,19 +140,19 @@ void BulkMutatorState::OnFinish(google::cloud::Status finish_status) {
     }
     // If there are any mutations with unknown state, they need to be handled.
     auto& original = *mutations_.mutable_entries(index);
-    if (annotation.idempotency == Idempotency::kIdempotent) {
+    if (retryable || annotation.idempotency == Idempotency::kIdempotent) {
       // If the mutation was retryable, move it to the pending mutations to try
       // again, along with their index.
       pending_mutations_.add_entries()->Swap(&original);
       pending_annotations_.push_back(std::move(annotation));
     } else {
       if (last_status_.ok()) {
-        google::cloud::Status status(
-            google::cloud::StatusCode::kInternal,
+        auto status = internal::InternalError(
             "The server never sent a confirmation for this mutation but the "
             "stream didn't fail either. This is most likely a bug, please "
             "report it at "
-            "https://github.com/googleapis/google-cloud-cpp/issues/new");
+            "https://github.com/googleapis/google-cloud-cpp/issues/new",
+            GCP_ERROR_INFO());
         failures_.emplace_back(std::move(status), annotation.original_index);
       } else {
         failures_.emplace_back(last_status_, annotation.original_index);
@@ -168,12 +174,12 @@ std::vector<bigtable::FailedMutation> BulkMutatorState::OnRetryDone() && {
     } else if (!last_status_.ok()) {
       result.emplace_back(last_status_, annotation.original_index);
     } else {
-      google::cloud::Status status(
-          google::cloud::StatusCode::kInternal,
+      auto status = internal::InternalError(
           "The server never sent a confirmation for this mutation but the "
           "stream didn't fail either. This is most likely a bug, please "
           "report it at "
-          "https://github.com/googleapis/google-cloud-cpp/issues/new");
+          "https://github.com/googleapis/google-cloud-cpp/issues/new",
+          GCP_ERROR_INFO());
       result.emplace_back(std::move(status), annotation.original_index);
     }
   }
@@ -204,25 +210,29 @@ grpc::Status BulkMutator::MakeOneRequest(bigtable::DataClient& client,
 }
 
 Status BulkMutator::MakeOneRequest(BigtableStub& stub,
-                                   MutateRowsLimiter& limiter) {
+                                   MutateRowsLimiter& limiter,
+                                   Options const& options) {
   // Send the request to the server.
   auto const& mutations = state_.BeforeStart();
 
   // Configure the context
-  auto const& options = google::cloud::internal::CurrentOptions();
   auto context = std::make_shared<grpc::ClientContext>();
   google::cloud::internal::ConfigureContext(*context, options);
+  retry_context_.PreCall(*context);
+  bool enable_server_retries = options.get<EnableServerRetriesOption>();
 
   struct UnpackVariant {
     BulkMutatorState& state;
     MutateRowsLimiter& limiter;
+    bool enable_server_retries;
+
     bool operator()(btproto::MutateRowsResponse r) {
       limiter.Update(r);
       state.OnRead(std::move(r));
       return true;
     }
     bool operator()(Status s) {
-      state.OnFinish(std::move(s));
+      state.OnFinish(std::move(s), enable_server_retries);
       return false;
     }
   };
@@ -231,9 +241,11 @@ Status BulkMutator::MakeOneRequest(BigtableStub& stub,
   limiter.Acquire();
 
   // Read the stream of responses.
-  auto stream = stub.MutateRows(std::move(context), options, mutations);
-  while (absl::visit(UnpackVariant{state_, limiter}, stream->Read())) {
+  auto stream = stub.MutateRows(context, options, mutations);
+  while (absl::visit(UnpackVariant{state_, limiter, enable_server_retries},
+                     stream->Read())) {
   }
+  retry_context_.PostCall(*context);
   return state_.last_status();
 }
 
