@@ -735,6 +735,18 @@ TEST(RowRangeTest, IntersectEndingAtEndingAt) {
   EXPECT_EQ(R::EndingAt("k"), std::get<1>(tuple));
 }
 
+TEST(RowRangeTest, SanitizesInput) {
+  google::bigtable::v2::RowRange proto;
+  proto.set_start_key_open("");
+  proto.set_end_key_open("");
+  auto rr = RowRange(proto);
+  EXPECT_FALSE(rr.IsEmpty());
+
+  proto.set_end_key_closed("");
+  rr = RowRange(std::move(proto));
+  EXPECT_FALSE(rr.IsEmpty());
+}
+
 }  // namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace bigtable
