@@ -20,6 +20,7 @@
 #include "google/cloud/bigtable/internal/legacy_row_reader.h"
 #include "google/cloud/bigtable/internal/unary_client_utils.h"
 #include "google/cloud/internal/async_retry_unary_rpc.h"
+#include "google/cloud/internal/make_status.h"
 #include <thread>
 #include <type_traits>
 
@@ -54,9 +55,10 @@ Status Table::Apply(SingleRowMutation mut, Options opts) {
     return connection_->Apply(table_name_, std::move(mut));
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Per-operation options only apply to `Table`s constructed "
-                  "with a `DataConnection`.");
+    return google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO());
   }
 
   // Copy the policies in effect for this operation.  Many policy classes change
@@ -106,10 +108,10 @@ future<Status> Table::AsyncApply(SingleRowMutation mut, Options opts) {
     return connection_->AsyncApply(table_name_, std::move(mut));
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return make_ready_future(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."));
+    return make_ready_future(google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO()));
   }
 
   google::bigtable::v2::MutateRowRequest request;
@@ -156,9 +158,10 @@ std::vector<FailedMutation> Table::BulkApply(BulkMutation mut, Options opts) {
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return bigtable_internal::MakeFailedMutations(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."),
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()),
         mut.size());
   }
 
@@ -196,9 +199,10 @@ future<std::vector<FailedMutation>> Table::AsyncBulkApply(BulkMutation mut,
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return make_ready_future(bigtable_internal::MakeFailedMutations(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."),
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()),
         mut.size()));
   }
 
@@ -225,9 +229,10 @@ RowReader Table::ReadRows(RowSet row_set, std::int64_t rows_limit,
   if (!google::cloud::internal::IsEmpty(opts)) {
     return MakeRowReader(
         std::make_shared<bigtable_internal::StatusOnlyRowReader>(
-            Status(StatusCode::kInvalidArgument,
-                   "Per-operation options only apply to `Table`s constructed "
-                   "with a `DataConnection`.")));
+            google::cloud::internal::InvalidArgumentError(
+                "Per-operation options only apply to `Table`s constructed "
+                "with a `DataConnection`.",
+                GCP_ERROR_INFO())));
   }
 
   auto impl = std::make_shared<bigtable_internal::LegacyRowReader>(
@@ -246,9 +251,10 @@ StatusOr<std::pair<bool, Row>> Table::ReadRow(std::string row_key,
                                 std::move(filter));
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Per-operation options only apply to `Table`s constructed "
-                  "with a `DataConnection`.");
+    return google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO());
   }
 
   RowSet row_set(std::move(row_key));
@@ -265,9 +271,9 @@ StatusOr<std::pair<bool, Row>> Table::ReadRow(std::string row_key,
   }
   auto result = std::make_pair(true, std::move(**it));
   if (++it != reader.end()) {
-    return Status(
-        StatusCode::kInternal,
-        "internal error - RowReader returned more than one row in ReadRow()");
+    return google::cloud::internal::InternalError(
+        "internal error - RowReader returned more than one row in ReadRow(, "
+        "GCP_ERROR_INFO())");
   }
   return result;
 }
@@ -282,9 +288,10 @@ StatusOr<MutationBranch> Table::CheckAndMutateRow(
         std::move(true_mutations), std::move(false_mutations));
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Per-operation options only apply to `Table`s constructed "
-                  "with a `DataConnection`.");
+    return google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO());
   }
 
   grpc::Status status;
@@ -325,9 +332,10 @@ future<StatusOr<MutationBranch>> Table::AsyncCheckAndMutateRow(
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return make_ready_future<StatusOr<MutationBranch>>(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."));
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()));
   }
 
   btproto::CheckAndMutateRowRequest request;
@@ -382,9 +390,10 @@ StatusOr<std::vector<bigtable::RowKeySample>> Table::SampleRows(Options opts) {
     return connection_->SampleRows(table_name_);
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Per-operation options only apply to `Table`s constructed "
-                  "with a `DataConnection`.");
+    return google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO());
   }
 
   // Copy the policies in effect for this operation.
@@ -435,9 +444,10 @@ future<StatusOr<std::vector<bigtable::RowKeySample>>> Table::AsyncSampleRows(
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return make_ready_future<StatusOr<std::vector<RowKeySample>>>(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."));
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()));
   }
 
   auto cq = background_threads_->cq();
@@ -456,9 +466,10 @@ StatusOr<Row> Table::ReadModifyWriteRowImpl(
     return connection_->ReadModifyWriteRow(std::move(request));
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Per-operation options only apply to `Table`s constructed "
-                  "with a `DataConnection`.");
+    return google::cloud::internal::InvalidArgumentError(
+        "Per-operation options only apply to `Table`s constructed "
+        "with a `DataConnection`.",
+        GCP_ERROR_INFO());
   }
 
   grpc::Status status;
@@ -484,9 +495,10 @@ future<StatusOr<Row>> Table::AsyncReadModifyWriteRowImpl(
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return make_ready_future<StatusOr<Row>>(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."));
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()));
   }
 
   auto cq = background_threads_->cq();
@@ -522,9 +534,10 @@ future<StatusOr<std::pair<bool, Row>>> Table::AsyncReadRow(std::string row_key,
   }
   if (!google::cloud::internal::IsEmpty(opts)) {
     return make_ready_future<StatusOr<std::pair<bool, Row>>>(
-        Status(StatusCode::kInvalidArgument,
-               "Per-operation options only apply to `Table`s constructed "
-               "with a `DataConnection`."));
+        google::cloud::internal::InvalidArgumentError(
+            "Per-operation options only apply to `Table`s constructed "
+            "with a `DataConnection`.",
+            GCP_ERROR_INFO()));
   }
 
   class AsyncReadRowHandler {
