@@ -13,7 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/bigtable/internal/prefix_range_end.h"
-#include "absl/memory/memory.h"
+#include "absl/strings/match.h"
 #include <google/bigtable/admin/v2/bigtable_instance_admin.grpc.pb.h>
 #include <google/longrunning/operations.grpc.pb.h>
 #include <google/protobuf/text_format.h>
@@ -63,7 +63,7 @@ class InstanceAdminEmulator final
       stored_instance.set_state(btadmin::Instance::READY);
       response->set_name("create-instance/" + name);
       response->set_done(true);
-      auto contents = absl::make_unique<google::protobuf::Any>();
+      auto contents = std::make_unique<google::protobuf::Any>();
       contents->PackFrom(stored_instance);
       response->set_allocated_response(contents.release());
 
@@ -106,7 +106,7 @@ class InstanceAdminEmulator final
 
     std::string prefix = request->parent() + "/instances/";
     for (auto const& kv : instances_) {
-      if (0 == kv.first.find(prefix)) {
+      if (absl::StartsWith(kv.first, prefix)) {
         *response->add_instances() = kv.second;
       }
     }
@@ -169,7 +169,7 @@ class InstanceAdminEmulator final
     }
     response->set_name("update-instance/" + name);
     response->set_done(true);
-    auto contents = absl::make_unique<google::protobuf::Any>();
+    auto contents = std::make_unique<google::protobuf::Any>();
     contents->PackFrom(stored_instance);
     response->set_allocated_response(contents.release());
     return grpc::Status::OK;
@@ -224,7 +224,7 @@ class InstanceAdminEmulator final
       stored_cluster.set_state(btadmin::Cluster::READY);
       response->set_name("create-cluster/" + name);
       response->set_done(true);
-      auto contents = absl::make_unique<google::protobuf::Any>();
+      auto contents = std::make_unique<google::protobuf::Any>();
       contents->PackFrom(stored_cluster);
       response->set_allocated_response(contents.release());
       return grpc::Status::OK;
@@ -306,7 +306,7 @@ class InstanceAdminEmulator final
     stored_cluster.CopyFrom(*request);
     response->set_name("update-cluster/" + name);
     response->set_done(true);
-    auto contents = absl::make_unique<google::protobuf::Any>();
+    auto contents = std::make_unique<google::protobuf::Any>();
     contents->PackFrom(stored_cluster);
     response->set_allocated_response(contents.release());
     return grpc::Status::OK;
@@ -385,7 +385,7 @@ class InstanceAdminEmulator final
 
     auto const& parent = request->parent();
     for (auto const& kv : app_profiles_) {
-      if (0 == kv.first.find(parent)) {
+      if (absl::StartsWith(kv.first, parent)) {
         *response->add_app_profiles() = kv.second;
       }
     }
@@ -426,7 +426,7 @@ class InstanceAdminEmulator final
     }
     response->set_name("update-app-profile/" + name);
     response->set_done(true);
-    auto contents = absl::make_unique<google::protobuf::Any>();
+    auto contents = std::make_unique<google::protobuf::Any>();
     contents->PackFrom(stored_app_profile);
     response->set_allocated_response(contents.release());
     return grpc::Status::OK;
