@@ -43,9 +43,11 @@ namespace {
 
 using ::google::cloud::testing_util::FakeCompletionQueueImpl;
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::StatusIs;
 using ::testing::Contains;
 using ::testing::HasSubstr;
 using ::testing::Not;
+using ::testing::Pair;
 using ::testing::StrictMock;
 
 using Request = google::protobuf::Duration;
@@ -107,7 +109,7 @@ TEST(CompletionQueueTest, TimerCancel) {
     }
   };
   std::vector<std::thread> workers;
-  std::generate_n(std::back_inserter(runners), 8,
+  std::generate_n(std::back_inserter(workers), 8,
                   [&] { return std::thread(worker, cq); });
 
   for (auto& t : workers) t.join();
@@ -404,7 +406,11 @@ TEST(CompletionQueueTest, MakeRpcsAfterShutdown) {
           },
           r1, std::make_unique<grpc::ClientContext>())
           .then([](future<StatusOr<Response>> f) {
-            EXPECT_EQ(StatusCode::kCancelled, f.get().status().code());
+            auto status = f.get().status();
+            EXPECT_THAT(status, StatusIs(StatusCode::kCancelled));
+            auto const& metadata = status.error_info().metadata();
+            EXPECT_THAT(metadata,
+                        Contains(Pair("gl-cpp.error.origin", "client")));
           });
 
   Request r2;
@@ -421,6 +427,8 @@ TEST(CompletionQueueTest, MakeRpcsAfterShutdown) {
       },
       [](Status const& status) {
         EXPECT_EQ(StatusCode::kCancelled, status.code());
+        auto const& metadata = status.error_info().metadata();
+        EXPECT_THAT(metadata, Contains(Pair("gl-cpp.error.origin", "client")));
       });
 
   mock_cq->SimulateCompletion(true);
@@ -682,9 +690,8 @@ TEST_P(RunAsyncTest, Torture) {
     }
   };
   std::vector<std::thread> workers(kThreads);
-  std::generate(workers.begin(), workers.end(), [&worker, &cq] {
-    return std::thread{worker, cq};
-  });
+  std::generate(workers.begin(), workers.end(),
+                [&worker, &cq] { return std::thread{worker, cq}; });
   for (auto& w : workers) w.join();
   wait();
   cq.Shutdown();
