@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/composer/environments_client.h"
+//! [all]
+#include "google/cloud/composer/v1/environments_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
@@ -22,19 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace composer = ::google::cloud::composer;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace composer = ::google::cloud::composer_v1;
   auto client =
       composer::EnvironmentsClient(composer::MakeEnvironmentsConnection());
 
-  auto const parent =
-      std::string("projects/") + argv[0] + "/locations/" + argv[1];
-  for (auto e : client.ListEnvironments(parent)) {
-    if (!e) throw std::runtime_error(e.status().message());
+  for (auto e : client.ListEnvironments(location.FullName())) {
+    if (!e) throw std::move(e).status();
     std::cout << e->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
