@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/container/cluster_manager_client.h"
+//! [all]
+#include "google/cloud/container/v1/cluster_manager_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
@@ -22,20 +23,21 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace container = ::google::cloud::container;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace container = ::google::cloud::container_v1;
   auto client = container::ClusterManagerClient(
       container::MakeClusterManagerConnection());
 
-  auto const location =
-      std::string{"projects/"} + argv[1] + "/locations/" + argv[2];
-  auto response = client.ListClusters(location);
-  if (!response) throw std::runtime_error(response.status().message());
-  for (auto c : response->clusters()) {
+  auto response = client.ListClusters(location.FullName());
+  if (!response) throw std::move(response).status();
+  for (auto const& c : response->clusters()) {
     std::cout << c.DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
