@@ -12,10 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/containeranalysis/grafeas_client.h"
+//! [all]
+#include "google/cloud/containeranalysis/v1/grafeas_client.h"
 #include "google/cloud/project.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -23,18 +23,19 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace containeranalysis = ::google::cloud::containeranalysis;
+  namespace containeranalysis = ::google::cloud::containeranalysis_v1;
   auto client = containeranalysis::GrafeasClient(
       containeranalysis::MakeGrafeasConnection());
 
   auto const project = google::cloud::Project(argv[1]);
   for (auto n : client.ListNotes(project.FullName(), /*filter=*/"")) {
-    if (!n) throw std::runtime_error(n.status().message());
+    if (!n) throw std::move(n).status();
     std::cout << n->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
