@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/filestore/cloud_filestore_manager_client.h"
+//! [all]
+#include "google/cloud/filestore/v1/cloud_filestore_manager_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,18 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace filestore = ::google::cloud::filestore;
+  auto const location = google::cloud::Location(argv[1], "-");
+
+  namespace filestore = ::google::cloud::filestore_v1;
   auto client = filestore::CloudFilestoreManagerClient(
       filestore::MakeCloudFilestoreManagerConnection());
 
-  auto const parent = std::string{"projects/"} + argv[1] + "/locations/-";
-  for (auto i : client.ListInstances(parent)) {
-    if (!i) throw std::runtime_error(i.status().message());
+  for (auto i : client.ListInstances(location.FullName())) {
+    if (!i) throw std::move(i).status();
     std::cout << i->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
