@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/functions/cloud_functions_client.h"
+//! [all]
+#include "google/cloud/functions/v2/function_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,21 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace functions = ::google::cloud::functions;
-  auto client = functions::CloudFunctionsServiceClient(
-      functions::MakeCloudFunctionsServiceConnection());
+  auto const location = google::cloud::Location(argv[1], "-");
 
-  auto project_id = std::string(argv[1]);
-  auto request = google::cloud::functions::v1::ListFunctionsRequest{};
-  request.set_parent("projects/" + project_id + "locations/-");
+  namespace functions = ::google::cloud::functions_v2;
+  auto client = functions::FunctionServiceClient(
+      functions::MakeFunctionServiceConnection());
 
-  for (auto r : client.ListFunctions(std::move(request))) {
-    if (!r) throw std::runtime_error(r.status().message());
-    std::cout << r->DebugString() << "\n";
+  for (auto f : client.ListFunctions(location.FullName())) {
+    if (!f) throw std::move(f).status();
+    std::cout << f->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
