@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/grpc_options.h"
+#include "google/cloud/common_options.h"
 #include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
 #include "google/cloud/internal/background_threads_impl.h"
@@ -22,6 +23,33 @@ namespace cloud {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 
+void SetMetadata(grpc::ClientContext& context, Options const& options,
+                 std::multimap<std::string, std::string> const& fixed_metadata,
+                 std::string const& api_client_header) {
+  for (auto const& kv : fixed_metadata) {
+    context.AddMetadata(kv.first, kv.second);
+  }
+  context.AddMetadata("x-goog-api-client", api_client_header);
+  if (options.has<UserProjectOption>()) {
+    context.AddMetadata("x-goog-user-project",
+                        options.get<UserProjectOption>());
+  }
+  auto const& authority = options.get<AuthorityOption>();
+  if (!authority.empty()) context.set_authority(authority);
+  for (auto const& h : options.get<CustomHeadersOption>()) {
+    context.AddMetadata(h.first, h.second);
+  }
+  if (options.has<UserIpOption>() && !options.has<QuotaUserOption>()) {
+    context.AddMetadata("x-goog-user-ip", options.get<UserIpOption>());
+  }
+  if (options.has<QuotaUserOption>()) {
+    context.AddMetadata("x-goog-quota-user", options.get<QuotaUserOption>());
+  }
+  if (options.has<FieldMaskOption>()) {
+    context.AddMetadata("x-goog-fieldmask", options.get<FieldMaskOption>());
+  }
+}
+
 void ConfigureContext(grpc::ClientContext& context, Options const& opts) {
   if (opts.has<GrpcSetupOption>()) {
     opts.get<GrpcSetupOption>()(context);
