@@ -16,7 +16,9 @@
 #include "google/cloud/common_options.h"
 #include "google/cloud/internal/background_threads_impl.h"
 #include "google/cloud/testing_util/scoped_log.h"
+#include "google/cloud/testing_util/validate_metadata.h"
 #include <gmock/gmock.h>
+#include <grpcpp/grpcpp.h>
 #include <string>
 #include <utility>
 
@@ -26,11 +28,14 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
 namespace {
 
+using ::google::cloud::testing_util::ValidateMetadataFixture;
 using ::testing::Contains;
 using ::testing::ContainsRegex;
 using ::testing::IsEmpty;
 using ::testing::IsNull;
 using ::testing::NotNull;
+using ::testing::Pair;
+using ::testing::UnorderedElementsAre;
 using ms = std::chrono::milliseconds;
 using ThreadPool = internal::AutomaticallyCreatedBackgroundThreads;
 
@@ -308,6 +313,100 @@ TEST(GrpcOptionList, Unexpected) {
       Contains(ContainsRegex("caller: Unexpected option.+UnexpectedOption")));
 }
 
+TEST(GrpcSetMetadata, Base) {
+  grpc::ClientContext context;
+  internal::SetMetadata(context, Options{}, {}, "api-client-header");
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata, UnorderedElementsAre(
+                            Pair("x-goog-api-client", "api-client-header")));
+}
+
+TEST(GrpcSetMetadata, Full) {
+  grpc::ClientContext context;
+  internal::SetMetadata(
+      context,
+      Options{}
+          .set<UserProjectOption>("user-project")
+          .set<QuotaUserOption>("quota-user")
+          .set<AuthorityOption>("authority.googleapis.com")
+          .set<CustomHeadersOption>(
+              {{"custom-header-1", "v1"}, {"custom-header-2", "v2"}}),
+      {{"fixed-header-1", "v1"}, {"fixed-header-2", "v2"}},
+      "api-client-header");
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata,
+              UnorderedElementsAre(
+                  Pair("x-goog-user-project", "user-project"),
+                  Pair("x-goog-quota-user", "quota-user"),
+                  Pair("fixed-header-1", "v1"), Pair("fixed-header-2", "v2"),
+                  Pair("custom-header-1", "v1"), Pair("custom-header-2", "v2"),
+                  Pair("x-goog-api-client", "api-client-header")));
+}
+
+TEST(GrpcSetMetadata, Authority) {
+  grpc::ClientContext context;
+  internal::SetMetadata(
+      context, Options{}.set<AuthorityOption>("authority.googleapis.com"), {},
+      "api-client-header");
+  ValidateMetadataFixture fixture;
+  auto const authority = fixture.GetAuthority(context);
+  if (!authority.has_value()) GTEST_SKIP() << "cannot retrieve authority";
+  EXPECT_EQ(authority, "authority.googleapis.com");
+}
+
+TEST(GrpcSetMetadata, ConfigureUserIp) {
+  grpc::ClientContext context;
+  internal::SetMetadata(context, Options{}.set<UserIpOption>("1234"), {},
+                        "api-client-header");
+
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata, UnorderedElementsAre(
+                            Pair("x-goog-user-ip", "1234"),
+                            Pair("x-goog-api-client", "api-client-header")));
+}
+
+TEST(GrpcSetMetadata, ConfigureQuotaUser) {
+  grpc::ClientContext context;
+  internal::SetMetadata(context, Options{}.set<QuotaUserOption>("my-user"), {},
+                        "api-client-header");
+
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata, UnorderedElementsAre(
+                            Pair("x-goog-quota-user", "my-user"),
+                            Pair("x-goog-api-client", "api-client-header")));
+}
+
+TEST(GrpcSetMetadata, QuotaUserOverridesUserIp) {
+  grpc::ClientContext context;
+  internal::SetMetadata(
+      context,
+      Options{}.set<QuotaUserOption>("my-user").set<UserIpOption>("1234"), {},
+      "api-client-header");
+
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata, UnorderedElementsAre(
+                            Pair("x-goog-quota-user", "my-user"),
+                            Pair("x-goog-api-client", "api-client-header")));
+}
+
+TEST(GrpcClientContext, FieldMask) {
+  grpc::ClientContext context;
+  internal::SetMetadata(context,
+                        Options{}.set<FieldMaskOption>("items.name,token"), {},
+                        "api-client-header");
+
+  ValidateMetadataFixture fixture;
+  auto const metadata = fixture.GetMetadata(context);
+  EXPECT_THAT(metadata, UnorderedElementsAre(
+                            Pair("x-goog-fieldmask", "items.name,token"),
+                            Pair("x-goog-api-client", "api-client-header")));
+}
+
 TEST(GrpcClientContext, Configure) {
   auto setup = [](grpc::ClientContext& context) {
     // This might not be the most useful setting, but it is the most easily
