@@ -30,6 +30,7 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 namespace {
 
+using ::google::cloud::internal::ImmutableOptions;
 using ::google::cloud::testing_util::AsyncSequencer;
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::IsProtoEqual;
@@ -49,15 +50,16 @@ class MockStub {
  public:
   MOCK_METHOD(future<StatusOr<Operation>>, AsyncCreateResponse,
               (CompletionQueue & cq, std::shared_ptr<grpc::ClientContext>,
-               Options const&, Request const&),
+               ImmutableOptions, Request const&),
               ());
   MOCK_METHOD(future<StatusOr<Operation>>, AsyncGetOperation,
               (CompletionQueue & cq, std::shared_ptr<grpc::ClientContext>,
-               Options const&, google::longrunning::GetOperationRequest const&),
+               ImmutableOptions,
+               google::longrunning::GetOperationRequest const&),
               ());
   MOCK_METHOD(future<Status>, AsyncCancelOperation,
               (CompletionQueue & cq, std::shared_ptr<grpc::ClientContext>,
-               Options const&,
+               ImmutableOptions,
                google::longrunning::CancelOperationRequest const&),
               ());
 };
@@ -87,8 +89,8 @@ std::unique_ptr<BackoffPolicy> TestBackoffPolicy() {
 
 using StartOperation =
     std::function<future<StatusOr<google::longrunning::Operation>>(
-        CompletionQueue&, std::shared_ptr<grpc::ClientContext>, Options const&,
-        Request const&)>;
+        CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
+        ImmutableOptions, Request const&)>;
 
 using StartOperationImplicit =
     std::function<future<StatusOr<google::longrunning::Operation>>(
@@ -96,47 +98,30 @@ using StartOperationImplicit =
         Request const&)>;
 
 StartOperation MakeStart(std::shared_ptr<MockStub> const& m) {
-  return [m](CompletionQueue& cq, auto context, Options const& options,
-             Request const& request) {
-    return m->AsyncCreateResponse(cq, std::move(context), options, request);
-  };
-}
-
-AsyncPollLongRunningOperation MakePoll(std::shared_ptr<MockStub> const& m) {
-  return [m](CompletionQueue& cq, auto context, Options const& options,
-             google::longrunning::GetOperationRequest const& request) {
-    return m->AsyncGetOperation(cq, std::move(context), options, request);
-  };
-}
-
-AsyncCancelLongRunningOperation MakeCancel(std::shared_ptr<MockStub> const& m) {
-  return [m](CompletionQueue& cq, auto context, Options const& options,
-             google::longrunning::CancelOperationRequest const& request) {
-    return m->AsyncCancelOperation(cq, std::move(context), options, request);
-  };
-}
-
-StartOperationImplicit MakeStartImplicit(std::shared_ptr<MockStub> const& m) {
-  return [m](CompletionQueue& cq, auto context, Request const& request) {
-    return m->AsyncCreateResponse(cq, std::move(context), CurrentOptions(),
+  return [m](CompletionQueue& cq, auto context,
+             // NOLINTNEXTLINE(performance-unnecessary-value-param)
+             ImmutableOptions options, Request const& request) {
+    return m->AsyncCreateResponse(cq, std::move(context), std::move(options),
                                   request);
   };
 }
 
-AsyncPollLongRunningOperationImplicitOptions MakePollImplicit(
-    std::shared_ptr<MockStub> const& m) {
+AsyncPollLongRunningOperation MakePoll(std::shared_ptr<MockStub> const& m) {
   return [m](CompletionQueue& cq, auto context,
+             // NOLINTNEXTLINE(performance-unnecessary-value-param)
+             ImmutableOptions options,
              google::longrunning::GetOperationRequest const& request) {
-    return m->AsyncGetOperation(cq, std::move(context), CurrentOptions(),
+    return m->AsyncGetOperation(cq, std::move(context), std::move(options),
                                 request);
   };
 }
 
-AsyncCancelLongRunningOperationImplicitOptions MakeCancelImplicit(
-    std::shared_ptr<MockStub> const& m) {
+AsyncCancelLongRunningOperation MakeCancel(std::shared_ptr<MockStub> const& m) {
   return [m](CompletionQueue& cq, auto context,
+             // NOLINTNEXTLINE(performance-unnecessary-value-param)
+             ImmutableOptions options,
              google::longrunning::CancelOperationRequest const& request) {
-    return m->AsyncCancelOperation(cq, std::move(context), CurrentOptions(),
+    return m->AsyncCancelOperation(cq, std::move(context), std::move(options),
                                    request);
   };
 }
@@ -145,116 +130,6 @@ std::string CurrentTestName() {
   return testing::UnitTest::GetInstance()->current_test_info()->name();
 }
 
-TEST(AsyncLongRunningTest, RequestPollThenSuccessMetadataImplicitOptions) {
-  Response expected;
-  expected.set_seconds(123456);
-  google::longrunning::Operation starting_op;
-  starting_op.set_name("test-op-name");
-  google::longrunning::Operation done_op = starting_op;
-  done_op.set_done(true);
-  done_op.mutable_metadata()->PackFrom(expected);
-
-  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
-  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
-      .WillOnce([](std::chrono::nanoseconds) {
-        return make_ready_future(
-            make_status_or(std::chrono::system_clock::now()));
-      });
-  CompletionQueue cq(mock_cq);
-
-  auto mock = std::make_shared<MockStub>();
-  EXPECT_CALL(*mock, AsyncCreateResponse)
-      .WillOnce(
-          [&](CompletionQueue&, auto, Options const& options, Request const&) {
-            EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-            EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-            return make_ready_future(make_status_or(starting_op));
-          });
-  EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
-                    google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-        return make_ready_future(make_status_or(done_op));
-      });
-  auto policy = std::make_unique<MockPollingPolicy>();
-  EXPECT_CALL(*policy, clone()).Times(0);
-  EXPECT_CALL(*policy, OnFailure).Times(0);
-  EXPECT_CALL(*policy, WaitPeriod)
-      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  Request request;
-  request.set_seconds(123456);
-  request.set_nanos(456789);
-
-  OptionsSpan span(Options{}.set<StringOption>(CurrentTestName()));
-  auto actual =
-      AsyncLongRunningOperation<Response>(
-          cq, std::move(request), MakeStartImplicit(mock),
-          MakePollImplicit(mock), MakeCancelImplicit(mock),
-          &ExtractLongRunningResultMetadata<Response>, TestRetryPolicy(),
-          TestBackoffPolicy(), Idempotency::kIdempotent, std::move(policy),
-          "test-function")
-          .get();
-  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
-  ASSERT_THAT(actual, IsOk());
-  EXPECT_THAT(*actual, IsProtoEqual(expected));
-}
-
-TEST(AsyncLongRunningTest, RequestPollThenSuccessResponseImplicitOptions) {
-  Response expected;
-  expected.set_seconds(123456);
-  google::longrunning::Operation starting_op;
-  starting_op.set_name("test-op-name");
-  google::longrunning::Operation done_op = starting_op;
-  done_op.set_done(true);
-  done_op.mutable_response()->PackFrom(expected);
-
-  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
-  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
-      .WillOnce([](std::chrono::nanoseconds) {
-        return make_ready_future(
-            make_status_or(std::chrono::system_clock::now()));
-      });
-  CompletionQueue cq(mock_cq);
-
-  auto mock = std::make_shared<MockStub>();
-  EXPECT_CALL(*mock, AsyncCreateResponse)
-      .WillOnce(
-          [&](CompletionQueue&, auto, Options const& options, Request const&) {
-            EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-            EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-            return make_ready_future(make_status_or(starting_op));
-          });
-  EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
-                    google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-        return make_ready_future(make_status_or(done_op));
-      });
-  auto policy = std::make_unique<MockPollingPolicy>();
-  EXPECT_CALL(*policy, clone()).Times(0);
-  EXPECT_CALL(*policy, OnFailure).Times(0);
-  EXPECT_CALL(*policy, WaitPeriod)
-      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  Request request;
-  request.set_seconds(123456);
-  request.set_nanos(456789);
-
-  OptionsSpan span(Options{}.set<StringOption>(CurrentTestName()));
-  auto actual =
-      AsyncLongRunningOperation<Response>(
-          cq, std::move(request), MakeStartImplicit(mock),
-          MakePollImplicit(mock), MakeCancelImplicit(mock),
-          &ExtractLongRunningResultResponse<Response>, TestRetryPolicy(),
-          TestBackoffPolicy(), Idempotency::kIdempotent, std::move(policy),
-          "test-function")
-          .get();
-  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
-  ASSERT_THAT(actual, IsOk());
-  EXPECT_THAT(*actual, IsProtoEqual(expected));
-}
-
 TEST(AsyncLongRunningTest, RequestPollThenSuccessMetadata) {
   Response expected;
   expected.set_seconds(123456);
@@ -274,15 +149,15 @@ TEST(AsyncLongRunningTest, RequestPollThenSuccessMetadata) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncCreateResponse)
-      .WillOnce(
-          [&](CompletionQueue&, auto, Options const& options, Request const&) {
-            EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-            return make_ready_future(make_status_or(starting_op));
-          });
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    Request const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(starting_op));
+      });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(done_op));
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -327,15 +202,15 @@ TEST(AsyncLongRunningTest, RequestPollThenSuccessResponse) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncCreateResponse)
-      .WillOnce(
-          [&](CompletionQueue&, auto, Options const& options, Request const&) {
-            EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-            return make_ready_future(make_status_or(starting_op));
-          });
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    Request const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(starting_op));
+      });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(done_op));
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -377,27 +252,27 @@ TEST(AsyncLongRunningTest, RequestPollThenCancel) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncCreateResponse)
-      .WillOnce(
-          [&](CompletionQueue&, auto, Options const& options, Request const&) {
-            EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-            return make_ready_future(make_status_or(starting_op));
-          });
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    Request const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(starting_op));
+      });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       })
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(StatusOr<google::longrunning::Operation>(
             Status{StatusCode::kCancelled, "cancelled"}));
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::CancelOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(Status{});
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -433,6 +308,151 @@ TEST(AsyncLongRunningTest, RequestPollThenCancel) {
   EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled));
 }
 
+TEST(AsyncLongRunningTest, AwaitPollThenSuccessMetadata) {
+  Response expected;
+  expected.set_seconds(123456);
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+  google::longrunning::Operation done_op = starting_op;
+  done_op.set_done(true);
+  done_op.mutable_metadata()->PackFrom(expected);
+
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
+      .WillOnce([](std::chrono::nanoseconds) {
+        return make_ready_future(
+            make_status_or(std::chrono::system_clock::now()));
+      });
+  CompletionQueue cq(mock_cq);
+
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(done_op));
+      });
+  auto polling_policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*polling_policy, clone()).Times(0);
+  EXPECT_CALL(*polling_policy, OnFailure).Times(0);
+  EXPECT_CALL(*polling_policy, WaitPeriod)
+      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto actual = AsyncAwaitLongRunningOperation<Response>(
+                    cq, current, starting_op, MakePoll(mock), MakeCancel(mock),
+                    &ExtractLongRunningResultMetadata<Response>,
+                    std::move(polling_policy), "test-function")
+                    .get();
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  ASSERT_THAT(actual, IsOk());
+  EXPECT_THAT(*actual, IsProtoEqual(expected));
+}
+
+TEST(AsyncLongRunningTest, AwaitPollThenSuccessResponse) {
+  Response expected;
+  expected.set_seconds(123456);
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+  google::longrunning::Operation done_op = starting_op;
+  done_op.set_done(true);
+  done_op.mutable_response()->PackFrom(expected);
+
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
+      .WillOnce([](std::chrono::nanoseconds) {
+        return make_ready_future(
+            make_status_or(std::chrono::system_clock::now()));
+      });
+  CompletionQueue cq(mock_cq);
+
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(done_op));
+      });
+  auto polling_policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*polling_policy, clone()).Times(0);
+  EXPECT_CALL(*polling_policy, OnFailure).Times(0);
+  EXPECT_CALL(*polling_policy, WaitPeriod)
+      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto actual = AsyncAwaitLongRunningOperation<Response>(
+                    cq, current, starting_op, MakePoll(mock), MakeCancel(mock),
+                    &ExtractLongRunningResultResponse<Response>,
+                    std::move(polling_policy), "test-function")
+                    .get();
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  ASSERT_THAT(actual, IsOk());
+  EXPECT_THAT(*actual, IsProtoEqual(expected));
+}
+
+TEST(AsyncLongRunningTest, AwaitPollThenCancel) {
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  AsyncSequencer<void> timer;
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
+      .WillRepeatedly([&timer](std::chrono::nanoseconds) {
+        return timer.PushBack().then([](future<void>) {
+          return make_status_or(std::chrono::system_clock::now());
+        });
+      });
+  CompletionQueue cq(mock_cq);
+
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(make_status_or(starting_op));
+      })
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(StatusOr<google::longrunning::Operation>(
+            Status{StatusCode::kCancelled, "cancelled"}));
+      });
+  EXPECT_CALL(*mock, AsyncCancelOperation)
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
+                    google::longrunning::CancelOperationRequest const&) {
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
+        return make_ready_future(Status{});
+      });
+  auto polling_policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*polling_policy, clone()).Times(0);
+  EXPECT_CALL(*polling_policy, OnFailure)
+      .WillRepeatedly([](Status const& status) {
+        return status.code() != StatusCode::kCancelled;
+      });
+  EXPECT_CALL(*polling_policy, WaitPeriod)
+      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto pending = AsyncAwaitLongRunningOperation<Response>(
+      cq, current, starting_op, MakePoll(mock), MakeCancel(mock),
+      &ExtractLongRunningResultMetadata<Response>, std::move(polling_policy),
+      "test-function");
+
+  // Wait until the polling loop is backing off for a second time.
+  timer.PopFront().set_value();
+  auto t = timer.PopFront();
+  {
+    // cancel the long running operation
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
+  // release timer
+  t.set_value();
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
+  EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled));
+}
+
 }  // namespace
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
