@@ -16,6 +16,7 @@
 #include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/call_context.h"
 #include "google/cloud/internal/grpc_opentelemetry.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
 #include <algorithm>
 #include <mutex>
@@ -79,7 +80,7 @@ class AsyncPollingLoopImpl
     auto w = WeakFromThis();
     auto context = std::make_shared<grpc::ClientContext>();
     ConfigurePollContext(*context, *options_);
-    cancel_(cq_, std::move(context), *options_, request)
+    cancel_(cq_, std::move(context), options_, request)
         .then([w](future<Status> f) {
           if (auto self = w.lock()) self->OnCancel(f.get());
         });
@@ -126,7 +127,7 @@ class AsyncPollingLoopImpl
     auto self = shared_from_this();
     auto context = std::make_shared<grpc::ClientContext>();
     ConfigurePollContext(*context, *options_);
-    poll_(cq_, std::move(context), *options_, request)
+    poll_(cq_, std::move(context), options_, request)
         .then([self](future<StatusOr<Operation>> g) {
           self->OnPoll(std::move(g));
         });
@@ -147,9 +148,10 @@ class AsyncPollingLoopImpl
         // an accurate status to the user, otherwise they will have no idea
         // how to react. But for now, we leave the operation running. It
         // may eventually complete.
-        return promise_.set_value(Status(
-            StatusCode::kDeadlineExceeded,
-            location_ + "() - polling loop terminated by polling policy"));
+        return promise_.set_value(internal::DeadlineExceededError(
+            location_ + "() - polling loop terminated by "
+                        "polling policy",
+            GCP_ERROR_INFO()));
       }
       // This could be a transient error if the policy is exhausted.
       return promise_.set_value(std::move(op).status());
@@ -186,31 +188,6 @@ future<StatusOr<Operation>> AsyncPollingLoop(
   return loop->Start(std::move(op));
 }
 
-future<StatusOr<google::longrunning::Operation>> AsyncPollingLoop(
-    google::cloud::CompletionQueue cq, future<StatusOr<Operation>> op,
-    AsyncPollLongRunningOperationImplicitOptions poll,
-    AsyncCancelLongRunningOperationImplicitOptions cancel,
-    std::unique_ptr<PollingPolicy> polling_policy, std::string location) {
-  auto poll_wrapper =
-      [poll = std::move(poll)](
-          CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
-          Options const&,
-          google::longrunning::GetOperationRequest const& request) {
-        return poll(cq, std::move(context), request);
-      };
-  auto cancel_wrapper =
-      [cancel = std::move(cancel)](
-          CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
-          Options const&,
-          google::longrunning::CancelOperationRequest const& request) {
-        return cancel(cq, std::move(context), request);
-      };
-  return AsyncPollingLoop(std::move(cq), internal::SaveCurrentOptions(),
-                          std::move(op), std::move(poll_wrapper),
-                          std::move(cancel_wrapper), std::move(polling_policy),
-                          std::move(location));
-}
-
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace cloud
