@@ -53,12 +53,13 @@ class MockStub {
   MOCK_METHOD(future<StatusOr<google::longrunning::Operation>>,
               AsyncGetOperation,
               (CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
-               Options const&, google::longrunning::GetOperationRequest const&),
+               ImmutableOptions,
+               google::longrunning::GetOperationRequest const&),
               ());
 
   MOCK_METHOD(future<Status>, AsyncCancelOperation,
               (CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
-               Options const&,
+               ImmutableOptions,
                google::longrunning::CancelOperationRequest const&),
               ());
 };
@@ -71,17 +72,23 @@ class MockPollingPolicy : public PollingPolicy {
 };
 
 AsyncPollLongRunningOperation MakePoll(std::shared_ptr<MockStub> const& mock) {
-  return [mock](CompletionQueue& cq, auto context, Options const& options,
+  return [mock](CompletionQueue& cq, auto context,
+                // NOLINTNEXTLINE(performance-unnecessary-value-param)
+                ImmutableOptions options,
                 google::longrunning::GetOperationRequest const& request) {
-    return mock->AsyncGetOperation(cq, std::move(context), options, request);
+    return mock->AsyncGetOperation(cq, std::move(context), std::move(options),
+                                   request);
   };
 }
 
 AsyncCancelLongRunningOperation MakeCancel(
     std::shared_ptr<MockStub> const& mock) {
-  return [mock](CompletionQueue& cq, auto context, Options const& options,
+  return [mock](CompletionQueue& cq, auto context,
+                // NOLINTNEXTLINE(performance-unnecessary-value-param)
+                ImmutableOptions options,
                 google::longrunning::CancelOperationRequest const& request) {
-    return mock->AsyncCancelOperation(cq, std::move(context), options, request);
+    return mock->AsyncCancelOperation(cq, std::move(context),
+                                      std::move(options), request);
   };
 }
 
@@ -130,17 +137,17 @@ TEST(AsyncPollingLoopTest, ImmediateCancel) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const& options,
+      .WillOnce([](CompletionQueue&, auto, ImmutableOptions const& options,
                    google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(Status{
                 StatusCode::kCancelled, "test-function: operation cancelled"}));
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const& options,
+      .WillOnce([](CompletionQueue&, auto, ImmutableOptions const& options,
                    google::longrunning::CancelOperationRequest const& request) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         EXPECT_EQ(request.name(), "test-op-name");
         return make_ready_future(Status{});
       });
@@ -169,70 +176,6 @@ TEST(AsyncPollingLoopTest, ImmediateCancel) {
                                      HasSubstr("operation cancelled"))));
 }
 
-TEST(AsyncPollingLoopTest, ImmediateCancelImplicitOptions) {
-  google::longrunning::Operation starting_op;
-  starting_op.set_name("test-op-name");
-
-  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
-  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
-      .WillOnce([](std::chrono::nanoseconds) {
-        return make_ready_future(
-            make_status_or(std::chrono::system_clock::now()));
-      });
-  CompletionQueue cq(mock_cq);
-
-  auto mock = std::make_shared<MockStub>();
-  EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const& options,
-                   google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-        return make_ready_future(
-            StatusOr<google::longrunning::Operation>(Status{
-                StatusCode::kCancelled, "test-function: operation cancelled"}));
-      });
-  EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto, Options const& options,
-                   google::longrunning::CancelOperationRequest const& request) {
-        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
-        EXPECT_EQ(request.name(), "test-op-name");
-        return make_ready_future(Status{});
-      });
-  auto policy = std::make_unique<MockPollingPolicy>();
-  EXPECT_CALL(*policy, clone()).Times(0);
-  EXPECT_CALL(*policy, OnFailure).WillOnce([](Status const& status) {
-    EXPECT_THAT(status, StatusIs(StatusCode::kCancelled));
-    return false;
-  });
-  EXPECT_CALL(*policy, WaitPeriod)
-      .WillOnce(Return(std::chrono::milliseconds(1)));
-
-  promise<StatusOr<google::longrunning::Operation>> p;
-  OptionsSpan span(Options{}.set<StringOption>(CurrentTestName()));
-  auto pending = AsyncPollingLoop(
-      cq, p.get_future(),
-      [mock](auto& cq, auto context, auto const& request) {
-        return mock->AsyncGetOperation(cq, std::move(context), CurrentOptions(),
-                                       request);
-      },
-      [mock](auto& cq, auto context, auto const& request) {
-        return mock->AsyncCancelOperation(cq, std::move(context),
-                                          CurrentOptions(), request);
-      },
-      std::move(policy), "test-function");
-  {
-    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
-    pending.cancel();
-  }
-  p.set_value(starting_op);
-  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
-  auto actual = pending.get();
-  EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
-                               AllOf(HasSubstr("test-function"),
-                                     HasSubstr("operation cancelled"))));
-}
-
 TEST(AsyncPollingLoopTest, PollThenSuccess) {
   Response response;
   response.set_seconds(123456);
@@ -254,9 +197,9 @@ TEST(AsyncPollingLoopTest, PollThenSuccess) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(expected));
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -322,25 +265,25 @@ TEST(AsyncPollingLoopTest, PollThenEventualSuccess) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       })
-      .WillOnce([](CompletionQueue&, auto, Options const& options,
+      .WillOnce([](CompletionQueue&, auto, ImmutableOptions const& options,
                    google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(StatusOr<google::longrunning::Operation>(
             Status(StatusCode::kUnavailable, "try-again")));
       })
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       })
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(expected));
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -379,9 +322,10 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicy) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(2))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
+      .WillRepeatedly([&](CompletionQueue&, auto,
+                          ImmutableOptions const& options,
                           google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -426,9 +370,10 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicyWithFailure) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(2))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
+      .WillRepeatedly([&](CompletionQueue&, auto,
+                          ImmutableOptions const& options,
                           google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(StatusOr<google::longrunning::Operation>(
             Status{StatusCode::kUnavailable, "try-again"}));
       });
@@ -473,9 +418,10 @@ TEST(AsyncPollingLoopTest, PollLifetime) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(4)
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
+      .WillRepeatedly([&](CompletionQueue&, auto,
+                          ImmutableOptions const& options,
                           google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -518,15 +464,16 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringTimer) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
+      .WillRepeatedly([&](CompletionQueue&, auto,
+                          ImmutableOptions const& options,
                           google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::CancelOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(Status{});
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -579,15 +526,16 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringPoll) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
+      .WillRepeatedly([&](CompletionQueue&, auto,
+                          ImmutableOptions const& options,
                           google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const& options,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const& options,
                     google::longrunning::CancelOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         return make_ready_future(Status{});
       });
   auto policy = std::make_unique<MockPollingPolicy>();
@@ -640,9 +588,10 @@ TEST(AsyncPollingLoopTest, ConfigurePollContext) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, auto context, Options const& options,
+      .WillOnce([](CompletionQueue&, auto context,
+                   ImmutableOptions const& options,
                    google::longrunning::GetOperationRequest const&) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         // Ensure that our options have taken affect on the ClientContext before
         // we start using it.
         EXPECT_EQ(GRPC_COMPRESS_DEFLATE, context->compression_algorithm());
@@ -651,9 +600,10 @@ TEST(AsyncPollingLoopTest, ConfigurePollContext) {
                 StatusCode::kCancelled, "test-function: operation cancelled"}));
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, auto context, Options const& options,
+      .WillOnce([](CompletionQueue&, auto context,
+                   ImmutableOptions const& options,
                    google::longrunning::CancelOperationRequest const& request) {
-        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options->get<StringOption>(), CurrentTestName());
         // Ensure that our options have taken affect on the ClientContext before
         // we start using it.
         EXPECT_EQ(GRPC_COMPRESS_DEFLATE, context->compression_algorithm());
@@ -762,7 +712,7 @@ TEST(AsyncPollingLoopTest, SpanActiveDuringCancel) {
     return get_sequencer.PushBack();
   });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, auto, Options const&,
+      .WillOnce([&](CompletionQueue&, auto, ImmutableOptions const&,
                     google::longrunning::CancelOperationRequest const&) {
         EXPECT_THAT(span, IsActive());
         return make_ready_future(Status{});
