@@ -35,6 +35,7 @@ namespace {
 
 using ::google::cloud::testing_util::AsyncSequencer;
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::MockBackoffPolicy;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::Contains;
@@ -118,34 +119,14 @@ class AsyncRetryLoopCancelTest : public ::testing::Test {
   AsyncSequencer<Status> sequencer_;
 };
 
-TEST(AsyncRetryLoopTest, SuccessWithImplicitOptions) {
-  EXPECT_EQ(CurrentOptions().get<TestOption>(), "");
-  OptionsSpan span(Options{}.set<TestOption>("Success"));
-  EXPECT_EQ(CurrentOptions().get<TestOption>(), "Success");
-  AutomaticallyCreatedBackgroundThreads background;
-  auto pending = AsyncRetryLoop(
-      TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
-      background.cq(),
-      [](google::cloud::CompletionQueue&, auto,
-         int request) -> future<StatusOr<int>> {
-        EXPECT_EQ(CurrentOptions().get<TestOption>(), "Success");
-        return make_ready_future(StatusOr<int>(2 * request));
-      },
-      42, "error message");
-  OptionsSpan overlay(Options{}.set<TestOption>("uh-oh"));
-  StatusOr<int> actual = pending.get();
-  ASSERT_THAT(actual.status(), IsOk());
-  EXPECT_EQ(84, *actual);
-}
-
 TEST(AsyncRetryLoopTest, SuccessWithExplicitOptions) {
   AutomaticallyCreatedBackgroundThreads background;
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
       background.cq(),
-      [](google::cloud::CompletionQueue&, auto, Options const& options,
+      [](google::cloud::CompletionQueue&, auto, ImmutableOptions const& options,
          int request) -> future<StatusOr<int>> {
-        EXPECT_EQ(options.get<TestOption>(), "Success");
+        EXPECT_EQ(options->get<TestOption>(), "Success");
         return make_ready_future(StatusOr<int>(2 * request));
       },
       MakeImmutableOptions(Options{}.set<TestOption>("Success")),
@@ -162,9 +143,9 @@ TEST(AsyncRetryLoopTest, TransientThenSuccess) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
       background.cq(),
-      [&](google::cloud::CompletionQueue&, auto, Options const& options,
-          int request) {
-        EXPECT_EQ(options.get<TestOption>(), "TransientThenSuccess");
+      [&](google::cloud::CompletionQueue&, auto,
+          ImmutableOptions const& options, int request) {
+        EXPECT_EQ(options->get<TestOption>(), "TransientThenSuccess");
         if (++counter < 3) {
           return make_ready_future(
               StatusOr<int>(Status(StatusCode::kUnavailable, "try again")));
@@ -185,8 +166,9 @@ TEST(AsyncRetryLoopTest, ReturnJustStatus) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
       background.cq(),
-      [&](google::cloud::CompletionQueue&, auto, Options const& options, int) {
-        EXPECT_EQ(options.get<TestOption>(), "ReturnJustStatus");
+      [&](google::cloud::CompletionQueue&, auto,
+          ImmutableOptions const& options, int) {
+        EXPECT_EQ(options->get<TestOption>(), "ReturnJustStatus");
         if (++counter <= 3) {
           return make_ready_future(
               Status(StatusCode::kResourceExhausted, "slow-down"));
@@ -200,13 +182,10 @@ TEST(AsyncRetryLoopTest, ReturnJustStatus) {
   ASSERT_THAT(actual, IsOk());
 }
 
-class RetryPolicyWithSetup {
+class RetryPolicyWithSetup : public RetryPolicy {
  public:
-  virtual ~RetryPolicyWithSetup() = default;
-  virtual bool OnFailure(Status const&) = 0;
+  ~RetryPolicyWithSetup() override = default;
   virtual void Setup(grpc::ClientContext&) const = 0;
-  virtual bool IsExhausted() const = 0;
-  virtual bool IsPermanentFailure(Status const&) const = 0;
 };
 
 class MockRetryPolicy : public RetryPolicyWithSetup {
@@ -229,9 +208,9 @@ TEST(AsyncRetryLoopTest, UsesBackoffPolicy) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), std::move(mock), Idempotency::kIdempotent,
       background.cq(),
-      [&](google::cloud::CompletionQueue&, auto, Options const& options,
-          int request) {
-        EXPECT_EQ(options.get<TestOption>(), "UsesBackoffPolicy");
+      [&](google::cloud::CompletionQueue&, auto,
+          ImmutableOptions const& options, int request) {
+        EXPECT_EQ(options->get<TestOption>(), "UsesBackoffPolicy");
         if (++counter <= 3) {
           return make_ready_future(
               StatusOr<int>(Status(StatusCode::kUnavailable, "try again")));
@@ -251,8 +230,9 @@ TEST(AsyncRetryLoopTest, TransientFailureNonIdempotent) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kNonIdempotent,
       background.cq(),
-      [](google::cloud::CompletionQueue&, auto, Options const& options, int) {
-        EXPECT_EQ(options.get<TestOption>(), "TransientFailureNonIdempotent");
+      [](google::cloud::CompletionQueue&, auto, ImmutableOptions const& options,
+         int) {
+        EXPECT_EQ(options->get<TestOption>(), "TransientFailureNonIdempotent");
         return make_ready_future(StatusOr<int>(
             Status(StatusCode::kUnavailable, "test-message-try-again")));
       },
@@ -280,8 +260,9 @@ TEST(AsyncRetryLoopTest, PermanentFailureIdempotent) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
       background.cq(),
-      [](google::cloud::CompletionQueue&, auto, Options const& options, int) {
-        EXPECT_EQ(options.get<TestOption>(), "PermanentFailureIdempotent");
+      [](google::cloud::CompletionQueue&, auto, ImmutableOptions const& options,
+         int) {
+        EXPECT_EQ(options->get<TestOption>(), "PermanentFailureIdempotent");
         return make_ready_future(StatusOr<int>(
             Status(StatusCode::kPermissionDenied, "test-message-uh-oh")));
       },
@@ -306,8 +287,9 @@ TEST(AsyncRetryLoopTest, TooManyTransientFailuresIdempotent) {
   auto pending = AsyncRetryLoop(
       TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kIdempotent,
       background.cq(),
-      [](google::cloud::CompletionQueue&, auto, Options const& options, int) {
-        EXPECT_EQ(options.get<TestOption>(),
+      [](google::cloud::CompletionQueue&, auto, ImmutableOptions const& options,
+         int) {
+        EXPECT_EQ(options->get<TestOption>(),
                   "TooManyTransientFailuresIdempotent");
         return make_ready_future(StatusOr<int>(
             Status(StatusCode::kUnavailable, "test-message-try-again")));
@@ -336,8 +318,9 @@ TEST(AsyncRetryLoopTest, ExhaustedDuringBackoff) {
       LimitedErrorCountRetryPolicy<TestRetryablePolicy>(0).clone(),
       ExponentialBackoffPolicy(ms(0), ms(0), 2.0).clone(),
       Idempotency::kIdempotent, background.cq(),
-      [](google::cloud::CompletionQueue&, auto, Options const& options, int) {
-        EXPECT_EQ(options.get<TestOption>(), "ExhaustedDuringBackoff");
+      [](google::cloud::CompletionQueue&, auto, ImmutableOptions const& options,
+         int) {
+        EXPECT_EQ(options->get<TestOption>(), "ExhaustedDuringBackoff");
         return make_ready_future(StatusOr<int>(
             Status(StatusCode::kUnavailable, "test-message-try-again")));
       },
@@ -381,6 +364,28 @@ TEST(AsyncRetryLoopTest, ExhaustedBeforeStart) {
               Contains(Pair("gcloud-cpp.retry.function", "test-location")));
 }
 
+TEST(AsyncRetryLoopTest, HeedsRetryInfo) {
+  MockFunction<future<StatusOr<int>>(CompletionQueue&,
+                                     std::shared_ptr<grpc::ClientContext>,
+                                     ImmutableOptions, int)>
+      mock;
+  EXPECT_CALL(mock, Call)
+      .WillOnce([] {
+        auto status = ResourceExhaustedError("try again");
+        SetRetryInfo(status, RetryInfo{std::chrono::seconds(0)});
+        return make_ready_future<StatusOr<int>>(status);
+      })
+      .WillOnce(Return(make_ready_future<StatusOr<int>>(5)));
+
+  AutomaticallyCreatedBackgroundThreads background;
+  auto pending = AsyncRetryLoop(
+      TestRetryPolicy(), TestBackoffPolicy(), Idempotency::kNonIdempotent,
+      background.cq(), mock.AsStdFunction(),
+      MakeImmutableOptions(Options{}.set<EnableServerRetriesOption>(true)), 42,
+      "test-location");
+  EXPECT_THAT(pending.get(), IsOkAndHolds(5));
+}
+
 TEST(AsyncRetryLoopTest, SetsTimeout) {
   auto mock = std::make_unique<MockRetryPolicy>();
   EXPECT_CALL(*mock, OnFailure)
@@ -403,7 +408,7 @@ TEST(AsyncRetryLoopTest, SetsTimeout) {
   auto pending = AsyncRetryLoop(
       std::unique_ptr<RetryPolicyWithSetup>(std::move(mock)),
       TestBackoffPolicy(), Idempotency::kIdempotent, background.cq(),
-      [&](google::cloud::CompletionQueue&, auto, Options const&,
+      [&](google::cloud::CompletionQueue&, auto, ImmutableOptions const&,
           int /*request*/) {
         EXPECT_EQ(CurrentOptions().get<TestOption>(), "SetsTimeout");
         return make_ready_future(
