@@ -25,7 +25,7 @@ std::string GceMetadataScheme() { return "http"; }
 
 std::string GceMetadataHostname() {
   return google::cloud::internal::GetEnv(GceMetadataHostnameEnvVar())
-      .value_or("metadata.google.internal.");
+      .value_or("metadata.google.internal");
 }
 
 }  // namespace internal
