@@ -169,13 +169,12 @@ CurlImpl::CurlImpl(CurlHandle handle,
                    std::shared_ptr<CurlHandleFactory> factory,
                    Options const& options)
     : factory_(std::move(factory)),
-      request_headers_(nullptr, &curl_slist_free_all),
       handle_(std::move(handle)),
       multi_(factory_->CreateMultiHandle()) {
   CurlInitializeOnce(options);
 
   logging_enabled_ = google::cloud::internal::Contains(
-      options.get<TracingComponentsOption>(), "http");
+      options.get<LoggingComponentsOption>(), "http");
   follow_location_ = options.get<CurlFollowLocationOption>();
 
   socket_options_.recv_buffer_size_ =
@@ -361,8 +360,8 @@ Status CurlImpl::MakeRequest(HttpMethod method, RestContext& context,
   }
 
   if (method == HttpMethod::kPost) {
-    WriteVector writev{std::move(request)};
-    curl_off_t const size = writev.size();
+    writev_ = WriteVector{std::move(request)};
+    curl_off_t const size = writev_.size();
     status = handle_.SetOption(CURLOPT_POSTFIELDS, nullptr);
     if (!status.ok()) return OnTransferError(context, std::move(status));
     status = handle_.SetOption(CURLOPT_POST, 1L);
@@ -371,26 +370,26 @@ Status CurlImpl::MakeRequest(HttpMethod method, RestContext& context,
     if (!status.ok()) return OnTransferError(context, std::move(status));
     status = handle_.SetOption(CURLOPT_READFUNCTION, &ReadFunction);
     if (!status.ok()) return OnTransferError(context, std::move(status));
-    status = handle_.SetOption(CURLOPT_READDATA, &writev);
+    status = handle_.SetOption(CURLOPT_READDATA, &writev_);
     if (!status.ok()) return OnTransferError(context, std::move(status));
     status = handle_.SetOption(CURLOPT_SEEKFUNCTION, &SeekFunction);
     if (!status.ok()) return OnTransferError(context, std::move(status));
-    status = handle_.SetOption(CURLOPT_SEEKDATA, &writev);
+    status = handle_.SetOption(CURLOPT_SEEKDATA, &writev_);
     if (!status.ok()) return OnTransferError(context, std::move(status));
     SetHeader("Expect:");
     return MakeRequestImpl(context);
   }
 
   if (method == HttpMethod::kPut || method == HttpMethod::kPatch) {
-    WriteVector writev{std::move(request)};
-    curl_off_t const size = writev.size();
+    writev_ = WriteVector{std::move(request)};
+    curl_off_t const size = writev_.size();
     status = handle_.SetOption(CURLOPT_READFUNCTION, &ReadFunction);
     if (!status.ok()) return OnTransferError(context, std::move(status));
-    status = handle_.SetOption(CURLOPT_READDATA, &writev);
+    status = handle_.SetOption(CURLOPT_READDATA, &writev_);
     if (!status.ok()) return OnTransferError(context, std::move(status));
     status = handle_.SetOption(CURLOPT_SEEKFUNCTION, &SeekFunction);
     if (!status.ok()) return OnTransferError(context, std::move(status));
-    status = handle_.SetOption(CURLOPT_SEEKDATA, &writev);
+    status = handle_.SetOption(CURLOPT_SEEKDATA, &writev_);
     if (!status.ok()) return OnTransferError(context, std::move(status));
     status = handle_.SetOption(CURLOPT_UPLOAD, 1L);
     if (!status.ok()) return OnTransferError(context, std::move(status));
