@@ -20,8 +20,10 @@
 #include "google/cloud/log.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_split.h"
+#ifndef _WIN32
 #include <openssl/crypto.h>
 #include <openssl/opensslv.h>
+#endif
 #include <algorithm>
 #include <cctype>
 #include <csignal>
@@ -40,6 +42,9 @@ namespace {
 // LibreSSL calls itself OpenSSL > 2.0, but it really is based on SSL 1.0.2
 // and requires locks.
 #define GOOGLE_CLOUD_CPP_SSL_REQUIRES_LOCKS 1
+#elif defined(_WIN32)
+// We don't use OpenSSL on Windows.
+#define GOOGLE_CLOUD_CPP_SSL_REQUIRES_LOCKS 0
 #elif OPENSSL_VERSION_NUMBER < 0x10100000L  // Older than version 1.1.0
 // Before 1.1.0 OpenSSL requires locks to be used by multiple threads.
 #define GOOGLE_CLOUD_CPP_SSL_REQUIRES_LOCKS 1
@@ -182,6 +187,27 @@ std::string CleanupDebugData(char const* data, std::size_t size) {
   return text;
 }
 
+std::string DebugCensored(absl::string_view msg, absl::string_view payload) {
+  // We want to truncate the portion of the payload following this ": Bearer" to
+  // at most 32 characters, skipping everything else until any newline.
+  auto const bearer = absl::string_view{": Bearer "};
+  auto const limit = bearer.size() + 32;
+  auto const bearer_pos = payload.find(bearer);
+  if (bearer_pos == std::string::npos) return absl::StrCat(msg, payload);
+
+  auto const nl_pos = payload.find('\n', bearer_pos);
+  auto const prefix = payload.substr(0, bearer_pos);
+  auto trailer = absl::string_view{};
+  auto body = payload.substr(bearer_pos);
+  if (nl_pos != std::string::npos) {
+    trailer = payload.substr(nl_pos);
+    body = payload.substr(bearer_pos, nl_pos - bearer_pos);
+  }
+  auto const* marker = body.size() > limit ? "...<truncated>..." : "";
+  body = absl::ClippedSubstr(std::move(body), 0, limit);
+  return absl::StrCat(msg, prefix, body, marker, trailer);
+}
+
 }  // namespace
 
 std::string CurlSslLibraryId() {
@@ -226,7 +252,7 @@ bool SslLockingCallbacksInstalled() {
 }
 
 CurlPtr MakeCurlPtr() {
-  auto handle = CurlPtr(curl_easy_init(), &curl_easy_cleanup);
+  auto handle = CurlPtr(curl_easy_init());
   // We get better performance using a slightly larger buffer (128KiB) than the
   // default buffer size set by libcurl (16KiB).  We ignore errors because
   // failing to set this parameter just affects performance by a small amount.
@@ -256,7 +282,7 @@ std::size_t CurlAppendHeaderData(CurlReceivedHeaders& received_headers,
 }
 
 std::string DebugInfo(char const* data, std::size_t size) {
-  return absl::StrCat("== curl(Info): ", absl::string_view{data, size});
+  return DebugCensored("== curl(Info): ", absl::string_view{data, size});
 }
 
 std::string DebugRecvHeader(char const* data, std::size_t size) {
@@ -265,27 +291,7 @@ std::string DebugRecvHeader(char const* data, std::size_t size) {
 
 std::string DebugSendHeader(char const* data, std::size_t size) {
   // libcurl delivers multiple headers in a single payload, separated by '\n'.
-  auto const payload = absl::string_view{data, size};
-  // We want to truncate the portion of the payload following this ": Bearer" to
-  // at most 32 characters, skipping everything else until any newline.
-  auto const bearer = absl::string_view{": Bearer "};
-  auto const limit = bearer.size() + 32;
-  auto const bearer_pos = payload.find(bearer);
-  if (bearer_pos != std::string::npos) {
-    auto const nl_pos = payload.find('\n', bearer_pos);
-    auto const prefix = payload.substr(0, bearer_pos);
-    auto trailer = absl::string_view{};
-    auto body = payload.substr(bearer_pos);
-    if (nl_pos != std::string::npos) {
-      trailer = payload.substr(nl_pos);
-      body = payload.substr(bearer_pos, nl_pos - bearer_pos);
-    }
-    auto const* marker = body.size() > limit ? "...<truncated>..." : "";
-    body = absl::ClippedSubstr(std::move(body), 0, limit);
-    return absl::StrCat(">> curl(Send Header): ", prefix, body, marker,
-                        trailer);
-  }
-  return absl::StrCat(">> curl(Send Header): ", payload);
+  return DebugCensored(">> curl(Send Header): ", absl::string_view{data, size});
 }
 
 std::string DebugInData(char const* data, std::size_t size) {
