@@ -31,18 +31,16 @@ TEST(CurlWrappers, VersionToCurlCode) {
     std::string version;
     std::int64_t expected;
   } cases[] = {
-    {"", CURL_HTTP_VERSION_NONE},
-    {"default", CURL_HTTP_VERSION_NONE},
-    {"1.0", CURL_HTTP_VERSION_1_0},
-    {"1.1", CURL_HTTP_VERSION_1_1},
+      {"", CURL_HTTP_VERSION_NONE},     {"default", CURL_HTTP_VERSION_NONE},
+      {"1.0", CURL_HTTP_VERSION_1_0},   {"1.1", CURL_HTTP_VERSION_1_1},
 #if CURL_AT_LEAST_VERSION(7, 33, 0)
-    {"2.0", CURL_HTTP_VERSION_2_0},
+      {"2.0", CURL_HTTP_VERSION_2_0},
 #endif  // CURL >= 7.33.0
 #if CURL_AT_LEAST_VERSION(7, 47, 0)
-    {"2TLS", CURL_HTTP_VERSION_2TLS},
+      {"2TLS", CURL_HTTP_VERSION_2TLS},
 #endif  // CURL >= 7.47.0
 #if CURL_AT_LEAST_VERSION(7, 66, 0)
-    {"3", CURL_HTTP_VERSION_3},
+      {"3", CURL_HTTP_VERSION_3},
 #endif  // CURL >= 7.66.0
   };
   for (auto const& test : cases) {
@@ -133,6 +131,31 @@ header2: value2
   }
 }
 
+TEST(CurlWrappers, DebugInfo) {
+  struct TestCasse {
+    std::string input;
+    std::string expected;
+  } cases[] = {
+      {R"""(no-marker-no-nl)""", R"""(== curl(Info): no-marker-no-nl)"""},
+      {R"""(no-marker-w-nl
+)""",
+       R"""(== curl(Info): no-marker-w-nl
+)"""},
+
+      {R"""([HTTP/2] [1] [authorization: Bearer 012345678901234567890123456789)""",
+       R"""(== curl(Info): [HTTP/2] [1] [authorization: Bearer 012345678901234567890123456789)"""},
+      {R"""([HTTP/2] [1] [authorization: Bearer 01234567890123456789012345678912)""",
+       R"""(== curl(Info): [HTTP/2] [1] [authorization: Bearer 01234567890123456789012345678912)"""},
+
+      {R"""([HTTP/2] [1] [authorization: Bearer 012345678901234567890123456789123456)""",
+       R"""(== curl(Info): [HTTP/2] [1] [authorization: Bearer 01234567890123456789012345678912...<truncated>...)"""},
+  };
+
+  for (auto const& test : cases) {
+    EXPECT_EQ(test.expected, DebugInfo(test.input.data(), test.input.size()));
+  }
+}
+
 TEST(CurlWrappers, CurlInitializeOptions) {
   auto defaults = CurlInitializeOptions({});
   EXPECT_TRUE(defaults.get<EnableCurlSslLockingOption>());
