@@ -21,17 +21,15 @@ namespace internal {
 
 GrpcAccessTokenAuthentication::GrpcAccessTokenAuthentication(
     AccessToken const& access_token, Options const& opts)
-    : credentials_(grpc::AccessTokenCredentials(access_token.token)),
-      use_insecure_channel_(opts.get<UseInsecureChannelOption>()) {
+    : credentials_(grpc::AccessTokenCredentials(access_token.token)) {
   auto cainfo = LoadCAInfo(opts);
   if (cainfo) ssl_options_.pem_root_certs = std::move(*cainfo);
 }
 
 std::shared_ptr<grpc::Channel> GrpcAccessTokenAuthentication::CreateChannel(
     std::string const& endpoint, grpc::ChannelArguments const& arguments) {
-  auto credentials = use_insecure_channel_ ? grpc::InsecureChannelCredentials()
-                                           : grpc::SslCredentials(ssl_options_);
-  return grpc::CreateCustomChannel(endpoint, credentials, arguments);
+  return grpc::CreateCustomChannel(endpoint, grpc::SslCredentials(ssl_options_),
+                                   arguments);
 }
 
 bool GrpcAccessTokenAuthentication::RequiresConfigureContext() const {
