@@ -24,6 +24,7 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include "google/cloud/testing_util/validate_metadata.h"
 #include "google/cloud/testing_util/validate_propagator.h"
+#include "google/cloud/universe_domain_options.h"
 #include <google/iam/credentials/v1/iamcredentials.grpc.pb.h>
 #include <gmock/gmock.h>
 
@@ -73,7 +74,7 @@ TEST_F(MinimalIamCredentialsStubTest, AsyncGenerateAccessTokenLogging) {
       });
   auto stub = DecorateMinimalIamCredentialsStub(
       mock, Options{}
-                .set<TracingComponentsOption>({"auth"})
+                .set<LoggingComponentsOption>({"auth"})
                 .set<GrpcTracingOptionsOption>(
                     TracingOptions{}.SetOptions("single_line_mode")));
   GenerateAccessTokenRequest request;
@@ -98,7 +99,7 @@ TEST_F(MinimalIamCredentialsStubTest, AsyncGenerateAccessTokenNoLogging) {
         return make_ready_future(make_status_or(response));
       });
   auto stub = DecorateMinimalIamCredentialsStub(
-      mock, Options{}.set<TracingComponentsOption>({}));
+      mock, Options{}.set<LoggingComponentsOption>({}));
   GenerateAccessTokenRequest request;
   request.set_name("projects/-/serviceAccounts/test-only-sa@not-valid");
   CompletionQueue cq;
@@ -117,7 +118,7 @@ TEST_F(MinimalIamCredentialsStubTest, SignBlobLogging) {
   EXPECT_CALL(*mock, SignBlob).WillOnce(Return(expected));
   auto stub = DecorateMinimalIamCredentialsStub(
       mock, Options{}
-                .set<TracingComponentsOption>({"auth"})
+                .set<LoggingComponentsOption>({"auth"})
                 .set<GrpcTracingOptionsOption>(
                     TracingOptions{}.SetOptions("single_line_mode")));
   SignBlobRequest request;
@@ -136,7 +137,7 @@ TEST_F(MinimalIamCredentialsStubTest, SignBlobNoLogging) {
   expected.set_signed_blob("test-only-signed");
   EXPECT_CALL(*mock, SignBlob).WillOnce(Return(expected));
   auto stub = DecorateMinimalIamCredentialsStub(
-      mock, Options{}.set<TracingComponentsOption>({}));
+      mock, Options{}.set<LoggingComponentsOption>({}));
   SignBlobRequest request;
   request.set_name("projects/-/serviceAccounts/test-only-sa@not-valid");
   grpc::ClientContext context;
@@ -176,7 +177,7 @@ TEST_F(MinimalIamCredentialsStubTest, AsyncGenerateAccessTokenMetadata) {
       });
 
   auto stub = DecorateMinimalIamCredentialsStub(
-      mock, Options{}.set<TracingComponentsOption>({}));
+      mock, Options{}.set<LoggingComponentsOption>({}));
   GenerateAccessTokenRequest request;
   request.set_name("projects/-/serviceAccounts/test-only-sa@not-valid");
   CompletionQueue cq;
@@ -206,7 +207,7 @@ TEST_F(MinimalIamCredentialsStubTest, LoggingComponentNames) {
     auto mock = std::make_shared<MockMinimalIamCredentialsStub>();
     EXPECT_CALL(*mock, SignBlob).WillOnce(Return(SignBlobResponse{}));
     auto stub = DecorateMinimalIamCredentialsStub(
-        mock, Options{}.set<TracingComponentsOption>(c.components));
+        mock, Options{}.set<LoggingComponentsOption>(c.components));
     grpc::ClientContext context;
     (void)stub->SignBlob(context, SignBlobRequest{});
     auto const lines = log_.ExtractLines();
@@ -233,7 +234,7 @@ using ::google::cloud::testing_util::ValidatePropagator;
 using ::testing::_;
 using ::testing::IsEmpty;
 
-auto constexpr kErrorCode = static_cast<int>(StatusCode::kAborted);
+auto constexpr kErrorCode = "ABORTED";
 
 TEST_F(MinimalIamCredentialsStubTest, AsyncGenerateAccessTokenNoTracing) {
   auto span_catcher = testing_util::InstallSpanCatcher();
@@ -290,7 +291,7 @@ TEST_F(MinimalIamCredentialsStubTest, AsyncGenerateAccessTokenTracing) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 
 TEST_F(MinimalIamCredentialsStubTest, SignBlobNoTracing) {
@@ -339,10 +340,30 @@ TEST_F(MinimalIamCredentialsStubTest, SignBlobTracing) {
           SpanWithStatus(opentelemetry::trace::StatusCode::kError, "fail"),
           SpanHasAttributes(
               OTelAttribute<std::string>("grpc.peer", _),
-              OTelAttribute<int>("gl-cpp.status_code", kErrorCode)))));
+              OTelAttribute<std::string>("gl-cpp.status_code", kErrorCode)))));
 }
 #endif  // GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
 
+TEST(MakeMinimalIamCredentialsOptions, Default) {
+  auto o = MakeMinimalIamCredentialsOptions(
+      Options{}.set<EndpointOption>("storage.googleapis.com."));
+  EXPECT_EQ(o.get<EndpointOption>(), "iamcredentials.googleapis.com");
+}
+
+TEST(MakeMinimalIamCredentialsOptions, WithoutUniverseDomain) {
+  auto o = MakeMinimalIamCredentialsOptions(
+      Options{}.set<EndpointOption>("storage.googleapis.com."));
+  EXPECT_EQ(o.get<EndpointOption>(), "iamcredentials.googleapis.com");
+}
+
+TEST(MakeMinimalIamCredentialsOptions, WithUniverseDomain) {
+  auto o = MakeMinimalIamCredentialsOptions(
+      Options{}
+          .set<EndpointOption>("storage.googleapis.com.")
+          .set<internal::UniverseDomainOption>("my-ud.net"));
+  EXPECT_EQ(o.get<EndpointOption>(), "iamcredentials.my-ud.net");
+}
+
 }  // namespace
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
