@@ -14,11 +14,15 @@
 
 #include "google/cloud/internal/oauth2_compute_engine_credentials.h"
 #include "google/cloud/internal/absl_str_cat_quiet.h"
+#include "google/cloud/internal/backoff_policy.h"
 #include "google/cloud/internal/compute_engine_util.h"
+#include "google/cloud/internal/make_status.h"
+#include "google/cloud/internal/retry_policy_impl.h"
 #include "google/cloud/testing_util/mock_http_payload.h"
 #include "google/cloud/testing_util/mock_rest_client.h"
 #include "google/cloud/testing_util/mock_rest_response.h"
 #include "google/cloud/testing_util/status_matchers.h"
+#include "google/cloud/universe_domain_options.h"
 #include <gmock/gmock.h>
 #include <chrono>
 
@@ -33,6 +37,7 @@ using ::google::cloud::rest_internal::RestContext;
 using ::google::cloud::rest_internal::RestRequest;
 using ::google::cloud::rest_internal::RestResponse;
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::MakeMockHttpPayloadSuccess;
 using ::google::cloud::testing_util::MockRestClient;
 using ::google::cloud::testing_util::MockRestResponse;
@@ -201,21 +206,23 @@ TEST(ComputeEngineCredentialsTest, ParseMetadataServerResponse) {
     std::string payload;
     ServiceAccountMetadata expected;
   } cases[] = {
-      {R"js({"email": "foo@bar.baz", "scopes": ["scope1", "scope2"]})js",
-       ServiceAccountMetadata{{"scope1", "scope2"}, "foo@bar.baz"}},
+      {R"js({"email": "foo@bar.baz", "scopes": ["scope1", "scope2"], "universe_domain": "test-ud.net"})js",
+       ServiceAccountMetadata{
+           {"scope1", "scope2"}, "foo@bar.baz", "test-ud.net"}},
       {R"js({"email": "foo@bar.baz", "scopes": "scope1\nscope2\n"})js",
-       ServiceAccountMetadata{{"scope1", "scope2"}, "foo@bar.baz"}},
+       ServiceAccountMetadata{
+           {"scope1", "scope2"}, "foo@bar.baz", "googleapis.com"}},
       // Ignore invalid formats
-      {R"js({"email": ["1", "2"], "scopes": ["scope1", "scope2"]})js",
-       ServiceAccountMetadata{{"scope1", "scope2"}, ""}},
-      {R"js({"email": "foo@bar", "scopes": {"foo": "bar"}})js",
-       ServiceAccountMetadata{{}, "foo@bar"}},
+      {R"js({"email": ["1", "2"], "scopes": ["scope1", "scope2"], "universe_domain": true})js",
+       ServiceAccountMetadata{{"scope1", "scope2"}, "", {}}},
+      {R"js({"email": "foo@bar", "scopes": {"foo": "bar"}, "universe_domain": 42})js",
+       ServiceAccountMetadata{{}, "foo@bar", {}}},
       // Ignore missing fields
       {R"js({"scopes": ["scope1", "scope2"]})js",
-       ServiceAccountMetadata{{"scope1", "scope2"}, ""}},
+       ServiceAccountMetadata{{"scope1", "scope2"}, "", "googleapis.com"}},
       {R"js({"email": "foo@bar.baz"})js",
-       ServiceAccountMetadata{{}, "foo@bar.baz"}},
-      {R"js({})js", ServiceAccountMetadata{{}, ""}},
+       ServiceAccountMetadata{{}, "foo@bar.baz", "googleapis.com"}},
+      {R"js({})js", ServiceAccountMetadata{{}, "", "googleapis.com"}},
   };
 
   for (auto const& test : cases) {
@@ -233,6 +240,7 @@ TEST(ComputeEngineCredentialsTest, ParseMetadataServerResponse) {
     EXPECT_EQ(metadata.email, test.expected.email);
     EXPECT_THAT(metadata.scopes,
                 UnorderedElementsAreArray(test.expected.scopes));
+    EXPECT_EQ(metadata.universe_domain, test.expected.universe_domain);
   }
 }
 
@@ -410,6 +418,201 @@ TEST(ComputeEngineCredentialsTest, AccountEmail) {
   EXPECT_EQ(credentials.service_account_email(), refreshed_email);
 }
 
+auto expected_universe_domain_request = []() {
+  auto const expected_path = absl::StrCat(
+      internal::GceMetadataScheme(), "://", internal::GceMetadataHostname(),
+      "/computeMetadata/v1/universe/universe_domain");
+  return AllOf(
+      Property(&RestRequest::path, expected_path),
+      Property(&RestRequest::headers,
+               Contains(Pair("metadata-flavor", Contains("Google")))),
+      Property(&RestRequest::parameters, Contains(Pair("recursive", "true"))));
+};
+
+TEST(ComputeEngineCredentialsTest, UniverseDomainSuccess) {
+  auto const universe_domain_resp = std::string{R"""(my-ud.net)"""};
+
+  auto client = std::make_unique<MockRestClient>();
+  EXPECT_CALL(*client, Get(_, expected_universe_domain_request()))
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        return internal::UnavailableError("Transient Error");
+      })
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        auto response = std::make_unique<MockRestResponse>();
+        EXPECT_CALL(*response, StatusCode)
+            .WillRepeatedly(Return(HttpStatusCode::kOk));
+        EXPECT_CALL(std::move(*response), ExtractPayload).WillOnce([&] {
+          return MakeMockHttpPayloadSuccess(universe_domain_resp);
+        });
+        return std::unique_ptr<RestResponse>(std::move(response));
+      });
+
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call).WillOnce(Return(ByMove(std::move(client))));
+  ComputeEngineCredentials credentials(Options{},
+                                       client_factory.AsStdFunction());
+  EXPECT_THAT(credentials.universe_domain(), IsOkAndHolds("my-ud.net"));
+}
+
+TEST(ComputeEngineCredentialsTest, UniverseDomainPermanentFailure) {
+  auto client = std::make_unique<MockRestClient>();
+  EXPECT_CALL(*client, Get(_, expected_universe_domain_request()))
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        return internal::UnavailableError("Transient Error");
+      })
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        return internal::NotFoundError("Permanent Error");
+      });
+
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call).WillOnce(Return(ByMove(std::move(client))));
+  ComputeEngineCredentials credentials(Options{},
+                                       client_factory.AsStdFunction());
+  EXPECT_THAT(credentials.universe_domain(), StatusIs(StatusCode::kNotFound));
+}
+
+TEST(ComputeEngineCredentialsTest, UniverseDomainMDSResourceNotFound) {
+  auto client = std::make_unique<MockRestClient>();
+  EXPECT_CALL(*client, Get(_, expected_universe_domain_request()))
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        return internal::UnavailableError("Transient Error");
+      })
+      .WillOnce([&](RestContext&, RestRequest const&) {
+        auto response = std::make_unique<MockRestResponse>();
+        EXPECT_CALL(*response, StatusCode)
+            .WillRepeatedly(Return(HttpStatusCode::kNotFound));
+        return std::unique_ptr<RestResponse>(std::move(response));
+      });
+
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call).WillOnce(Return(ByMove(std::move(client))));
+  ComputeEngineCredentials credentials(Options{},
+                                       client_factory.AsStdFunction());
+  EXPECT_THAT(credentials.universe_domain(), IsOkAndHolds("googleapis.com"));
+}
+
+struct TestUniverseDomainRetryTraits {
+  static bool IsPermanentFailure(Status const& status) {
+    return !status.ok() && status.code() != StatusCode::kUnavailable;
+  }
+};
+
+class TestUniverseDomainRetryPolicy
+    : public internal::UniverseDomainRetryPolicy {
+ public:
+  ~TestUniverseDomainRetryPolicy() override = default;
+
+  explicit TestUniverseDomainRetryPolicy(int maximum_failures)
+      : impl_(maximum_failures) {}
+  TestUniverseDomainRetryPolicy(TestUniverseDomainRetryPolicy&& rhs) noexcept
+      : TestUniverseDomainRetryPolicy(rhs.maximum_failures()) {}
+  TestUniverseDomainRetryPolicy(
+      TestUniverseDomainRetryPolicy const& rhs) noexcept
+      : TestUniverseDomainRetryPolicy(rhs.maximum_failures()) {}
+
+  bool OnFailure(Status const& status) override {
+    return impl_.OnFailure(status);
+  }
+  bool IsExhausted() const override { return impl_.IsExhausted(); }
+  bool IsPermanentFailure(Status const& status) const override {
+    return impl_.IsPermanentFailure(status);
+  }
+  int maximum_failures() const { return impl_.maximum_failures(); }
+  std::unique_ptr<internal::UniverseDomainRetryPolicy> clone() const override {
+    return std::make_unique<TestUniverseDomainRetryPolicy>(maximum_failures());
+  }
+
+ private:
+  internal::LimitedErrorCountRetryPolicy<TestUniverseDomainRetryTraits> impl_;
+};
+
+TEST(ComputeEngineCredentialsTest,
+     UniverseDomainCredentialsOptionsCustomRetryPolicy) {
+  auto client = std::make_unique<MockRestClient>();
+  EXPECT_CALL(*client, Get(_, expected_universe_domain_request()))
+      .Times(3)
+      .WillRepeatedly([&](RestContext&, RestRequest const&) {
+        return internal::UnavailableError("Transient Error");
+      });
+
+  auto credentials_options =
+      Options{}.set<internal::UniverseDomainRetryPolicyOption>(
+          std::make_unique<TestUniverseDomainRetryPolicy>(2)->clone());
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call).WillOnce(Return(ByMove(std::move(client))));
+  ComputeEngineCredentials credentials(credentials_options,
+                                       client_factory.AsStdFunction());
+  EXPECT_THAT(credentials.universe_domain(),
+              StatusIs(StatusCode::kUnavailable));
+}
+
+TEST(ComputeEngineCredentialsTest, UniverseDomainCallOptionsCustomRetryPolicy) {
+  auto client = std::make_unique<MockRestClient>();
+  EXPECT_CALL(*client, Get(_, expected_universe_domain_request()))
+      .Times(4)
+      .WillRepeatedly([&](RestContext&, RestRequest const&) {
+        return internal::UnavailableError("Transient Error");
+      });
+
+  auto call_options = Options{}.set<internal::UniverseDomainRetryPolicyOption>(
+      std::make_unique<TestUniverseDomainRetryPolicy>(3)->clone());
+  auto credentials_options =
+      Options{}.set<internal::UniverseDomainRetryPolicyOption>(
+          std::make_unique<TestUniverseDomainRetryPolicy>(2)->clone());
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call).WillOnce(Return(ByMove(std::move(client))));
+  ComputeEngineCredentials credentials(credentials_options,
+                                       client_factory.AsStdFunction());
+  EXPECT_THAT(credentials.universe_domain(call_options),
+              StatusIs(StatusCode::kUnavailable));
+}
+
+TEST(ComputeEngineCredentialsTest, ProjectIdSuccess) {
+  auto expected_request = []() {
+    auto const expected_path = absl::StrCat(
+        internal::GceMetadataScheme(), "://", internal::GceMetadataHostname(),
+        "/computeMetadata/v1/project/project-id");
+    return AllOf(
+        Property(&RestRequest::path, expected_path),
+        Property(&RestRequest::headers,
+                 Contains(Pair("metadata-flavor", Contains("Google")))));
+  };
+
+  auto const expected = std::string{"test-only-project-id"};
+
+  MockHttpClientFactory client_factory;
+  EXPECT_CALL(client_factory, Call)
+      .WillOnce([&] {
+        auto client = std::make_unique<MockRestClient>();
+        EXPECT_CALL(*client, Get(_, expected_request())).WillOnce([] {
+          return internal::UnavailableError("Transient Error");
+        });
+        return client;
+      })
+      .WillOnce([&] {
+        auto client = std::make_unique<MockRestClient>();
+        EXPECT_CALL(*client, Get(_, expected_request())).WillOnce([&] {
+          auto response = std::make_unique<MockRestResponse>();
+          EXPECT_CALL(*response, StatusCode)
+              .WillRepeatedly(Return(HttpStatusCode::kOk));
+          EXPECT_CALL(std::move(*response), ExtractPayload).WillOnce([&] {
+            return MakeMockHttpPayloadSuccess(expected);
+          });
+          return std::unique_ptr<RestResponse>(std::move(response));
+        });
+        return client;
+      });
+  ComputeEngineCredentials credentials(Options{},
+                                       client_factory.AsStdFunction());
+  // The first attempt fails, no retry policies for project id, so the error
+  // should be returned to the caller.
+  EXPECT_THAT(credentials.project_id(), StatusIs(StatusCode::kUnavailable));
+  // The error is not cached, a second request may succeed.
+  EXPECT_THAT(credentials.project_id(), IsOkAndHolds(expected));
+  // Verify the value is cached and further lookups do not create requests.
+  EXPECT_THAT(credentials.project_id(), IsOkAndHolds(expected));
+}
+
 }  // namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace oauth2_internal
