@@ -15,23 +15,19 @@
 #include "google/cloud/internal/populate_grpc_options.h"
 #include "google/cloud/credentials.h"
 #include "google/cloud/grpc_options.h"
-#include "google/cloud/internal/getenv.h"
+#include "google/cloud/internal/credentials_impl.h"
+#include "google/cloud/internal/make_status.h"
+#include "google/cloud/internal/populate_common_options.h"
 
 namespace google {
 namespace cloud {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 
-Options PopulateGrpcOptions(Options opts, std::string const& emulator_env_var) {
-  if (!emulator_env_var.empty()) {
-    auto e = GetEnv(emulator_env_var.c_str());
-    if (e && !e->empty()) {
-      opts.set<GrpcCredentialOption>(grpc::InsecureChannelCredentials());
-    }
-  }
+Options PopulateGrpcOptions(Options opts) {
   if (!opts.has<GrpcCredentialOption>() &&
       !opts.has<UnifiedCredentialsOption>()) {
-    opts.set<GrpcCredentialOption>(grpc::GoogleDefaultCredentials());
+    opts.set<UnifiedCredentialsOption>(MakeGoogleDefaultCredentials());
   }
   if (!opts.has<GrpcTracingOptionsOption>()) {
     opts.set<GrpcTracingOptionsOption>(DefaultTracingOptions());
@@ -39,12 +35,6 @@ Options PopulateGrpcOptions(Options opts, std::string const& emulator_env_var) {
   return opts;
 }
 
-TracingOptions DefaultTracingOptions() {
-  auto tracing_options = GetEnv("GOOGLE_CLOUD_CPP_TRACING_OPTIONS");
-  if (!tracing_options) return {};
-  return TracingOptions{}.SetOptions(*tracing_options);
-}
-
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace cloud
