@@ -44,7 +44,7 @@ class MockStreamingWriteRpc
   MOCK_METHOD(bool, Write, (RequestType const&, grpc::WriteOptions),
               (override));
   MOCK_METHOD(StatusOr<ResponseType>, Close, (), (override));
-  MOCK_METHOD(StreamingRpcMetadata, GetRequestMetadata, (), (const, override));
+  MOCK_METHOD(RpcMetadata, GetRequestMetadata, (), (const, override));
 };
 
 class StreamingWriteRpcLoggingTest : public ::testing::Test {
@@ -115,10 +115,10 @@ TEST_F(StreamingWriteRpcLoggingTest, CloseWithError) {
 TEST_F(StreamingWriteRpcLoggingTest, GetRequestMetadata) {
   auto mock = std::make_unique<MockStream>();
   EXPECT_CALL(*mock, GetRequestMetadata).WillOnce([] {
-    return StreamingRpcMetadata({{":test-only", "value"}});
+    return RpcMetadata{{{":test-only", "value"}}, {}};
   });
   TestedStream stream(std::move(mock), TracingOptions{}, "test-id");
-  EXPECT_THAT(stream.GetRequestMetadata(),
+  EXPECT_THAT(stream.GetRequestMetadata().headers,
               Contains(Pair(":test-only", "value")));
   EXPECT_THAT(log_.ExtractLines(),
               Contains(AllOf(HasSubstr("GetRequestMetadata(test-id)"),
