@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/networkmanagement/reachability_client.h"
+//! [all]
+#include "google/cloud/networkmanagement/v1/reachability_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,19 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace networkmanagement = ::google::cloud::networkmanagement;
+  auto const location = google::cloud::Location(argv[1], "global");
+
+  namespace networkmanagement = ::google::cloud::networkmanagement_v1;
   auto client = networkmanagement::ReachabilityServiceClient(
       networkmanagement::MakeReachabilityServiceConnection());
 
-  auto const parent = std::string{"projects/"} + argv[1] + "/locations/global";
-
-  for (auto t : client.ListConnectivityTests(parent)) {
-    if (!t) throw std::runtime_error(t.status().message());
+  for (auto t : client.ListConnectivityTests(location.FullName())) {
+    if (!t) throw std::move(t).status();
     std::cout << t->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
