@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/opentelemetry/internal/recordable.h"
+#include "google/cloud/internal/random.h"
 #include "google/cloud/internal/time_utils.h"
 #include "google/cloud/version.h"
 #include "absl/time/clock.h"
@@ -117,8 +118,8 @@ Matcher<v2::AttributeValue const&> AttributeValue(
 }
 
 Matcher<v2::Span::Attributes const&> Attributes(
-    Matcher<protobuf::Map<std::string, v2::AttributeValue> const&> const&
-        attribute_map_matcher,
+    Matcher<google::protobuf::Map<
+        std::string, v2::AttributeValue> const&> const& attribute_map_matcher,
     int dropped_attributes_count = 0) {
   return AllOf(
       ResultOf(
@@ -181,6 +182,14 @@ Matcher<v2::Span::Link const&> Link(
           attributes_matcher));
 }
 
+auto TestGenerator() {
+  // Initialize using the googletest seed.
+  auto generator = internal::DefaultPRNG(GTEST_FLAG_GET(random_seed));
+  return [g = std::move(generator)](int size) mutable {
+    return std::uniform_int_distribution<int>(0, size - 1)(g);
+  };
+}
+
 TEST(SetTruncatableString, LessThanLimit) {
   v2::TruncatableString proto;
   SetTruncatableString(proto, "value", 1000);
@@ -365,7 +374,7 @@ TEST(Recordable, SetResourceCopiesResourceAttributes) {
       {"vector<uint8>", MakeCompositeAttribute<std::uint8_t>(5)},
   });
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetResource(resource);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(
@@ -414,7 +423,7 @@ TEST(Recordable, AddEvent) {
   auto expected_time = absl::FromChrono(now);
   auto event_attributes = KVIterable({{"key1", "value1"}, {"key2", "value2"}});
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.AddEvent("test-event", now, event_attributes);
   auto proto = std::move(rec).as_proto();
   EXPECT_EQ(proto.time_events().dropped_annotations_count(), 0);
@@ -432,7 +441,7 @@ TEST(Recordable, TruncatesEventName) {
   std::string const expected(kAnnotationDescriptionStringLimit, 'A');
   auto now = std::chrono::system_clock::now();
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.AddEvent(name, now, KVIterable({}));
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.time_events().time_event(),
@@ -442,13 +451,18 @@ TEST(Recordable, TruncatesEventName) {
 TEST(Recordable, DropsNewEventAtLimit) {
   auto now = std::chrono::system_clock::now();
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   for (std::size_t i = 0; i != kSpanAnnotationLimit + 10; ++i) {
     rec.AddEvent("event" + std::to_string(i), now, KVIterable({}));
   }
   auto proto = std::move(rec).as_proto();
   EXPECT_EQ(proto.time_events().dropped_annotations_count(), 10);
-  EXPECT_THAT(proto.time_events().time_event(), SizeIs(kSpanAnnotationLimit));
+  auto event_matcher = [](int i) {
+    return TimeEvent(_, Annotation("event" + std::to_string(i), _));
+  };
+  EXPECT_THAT(proto.time_events().time_event(),
+              AllOf(SizeIs(kSpanAnnotationLimit), Contains(event_matcher(0)),
+                    Contains(event_matcher(kSpanAnnotationLimit + 9))));
 }
 
 TEST(Recordable, DropsNewEventAttributeAtLimit) {
@@ -461,7 +475,7 @@ TEST(Recordable, DropsNewEventAttributeAtLimit) {
   auto event_attributes =
       KVIterable(std::move(too_many_attributes), &iteration_count);
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.AddEvent("test-event", now, event_attributes);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.time_events().time_event(),
@@ -486,7 +500,7 @@ TEST(Recordable, AddLink) {
 
   auto link_attributes = KVIterable({{"key1", "value1"}, {"key2", "value2"}});
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.AddLink(span_context, link_attributes);
   auto proto = std::move(rec).as_proto();
   EXPECT_EQ(proto.links().dropped_links_count(), 0);
@@ -499,7 +513,7 @@ TEST(Recordable, AddLink) {
 }
 
 TEST(Recordable, DropsNewLinkAtLimit) {
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   for (std::size_t i = 0; i != kSpanLinkLimit + 1; ++i) {
     rec.AddLink({false, false}, KVIterable({}));
   }
@@ -517,7 +531,7 @@ TEST(Recordable, DropsNewLinkAttributeAtLimit) {
   auto link_attributes =
       KVIterable(std::move(too_many_attributes), &iteration_count);
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.AddLink({false, false}, link_attributes);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.links().link(),
@@ -543,7 +557,7 @@ TEST(Recordable, SetStatus) {
                               grpc::StatusCode::OK, ""},
                              {opentelemetry::trace::StatusCode::kError, "fail",
                               grpc::StatusCode::UNKNOWN, "fail"}}) {
-    auto rec = Recordable(Project(kProjectId));
+    auto rec = Recordable(Project(kProjectId), TestGenerator());
     rec.SetStatus(test.code, test.desc);
     auto proto = std::move(rec).as_proto();
     EXPECT_EQ(proto.status().code(), test.expected_code);
@@ -552,7 +566,7 @@ TEST(Recordable, SetStatus) {
 }
 
 TEST(Recordable, SetName) {
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetName("name");
   auto proto = std::move(rec).as_proto();
   EXPECT_EQ(proto.display_name().value(), "name");
@@ -562,7 +576,7 @@ TEST(Recordable, SetNameTruncates) {
   std::string const name(kDisplayNameStringLimit + 1, 'A');
   std::string const expected(kDisplayNameStringLimit, 'A');
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetName(name);
   auto proto = std::move(rec).as_proto();
   EXPECT_EQ(proto.display_name().value(), expected);
@@ -586,7 +600,7 @@ TEST(Recordable, SetSpanKind) {
             google::devtools::cloudtrace::v2::Span::PRODUCER},
            {opentelemetry::trace::SpanKind::kConsumer,
             google::devtools::cloudtrace::v2::Span::CONSUMER}}) {
-    auto rec = Recordable(Project(kProjectId));
+    auto rec = Recordable(Project(kProjectId), TestGenerator());
     rec.SetSpanKind(test.input);
     auto proto = std::move(rec).as_proto();
     EXPECT_EQ(proto.span_kind(), test.expected);
@@ -608,7 +622,7 @@ TEST(Recordable, SetIdentity) {
 
   opentelemetry::trace::SpanContext span_context(trace_id, span_id, {}, false);
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetIdentity(span_context, parent_span_id);
   auto proto = std::move(rec).as_proto();
 
@@ -633,7 +647,7 @@ TEST(Recordable, InvalidParentSpanIsOmitted) {
   opentelemetry::trace::SpanId const invalid_parent_span_id;
   EXPECT_FALSE(invalid_parent_span_id.IsValid());
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetIdentity(span_context, invalid_parent_span_id);
   auto proto = std::move(rec).as_proto();
 
@@ -641,7 +655,7 @@ TEST(Recordable, InvalidParentSpanIsOmitted) {
 }
 
 TEST(Recordable, SetAttribute) {
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetAttribute("key", "value");
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.attributes(),
@@ -649,7 +663,7 @@ TEST(Recordable, SetAttribute) {
 }
 
 TEST(Recordable, SetAttributeRespectsLimit) {
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   for (std::size_t i = 0; i != kSpanAttributeLimit + 1; ++i) {
     rec.SetAttribute("key" + std::to_string(i), "value");
   }
@@ -666,7 +680,7 @@ TEST(Recordable, SetResourceMapsMonitoredResources) {
       {sc::kCloudAvailabilityZone, "us-central1-a"},
   });
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetResource(resource);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(
@@ -682,7 +696,7 @@ TEST(Recordable, SetStartTime) {
   auto start = std::chrono::system_clock::now();
   auto expected = absl::FromChrono(start);
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetStartTime(start);
   auto proto = std::move(rec).as_proto();
   auto actual = internal::ToAbslTime(proto.start_time());
@@ -694,7 +708,7 @@ TEST(Recordable, SetDuration) {
   auto duration = std::chrono::nanoseconds(12345);
   auto expected = absl::FromChrono(start) + absl::FromChrono(duration);
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetStartTime(start);
   rec.SetDuration(duration);
   auto proto = std::move(rec).as_proto();
@@ -707,7 +721,7 @@ TEST(Recordable, SetInstrumentationScope) {
       opentelemetry::sdk::instrumentationscope::InstrumentationScope::Create(
           "test-name", "test-version");
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetInstrumentationScope(*scope);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.attributes(),
@@ -721,7 +735,7 @@ TEST(Recordable, SetInstrumentationScopeOmitsEmptyVersion) {
       opentelemetry::sdk::instrumentationscope::InstrumentationScope::Create(
           "test-name");
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetInstrumentationScope(*scope);
   auto proto = std::move(rec).as_proto();
   EXPECT_THAT(proto.attributes(),
@@ -739,7 +753,7 @@ TEST(Recordable, ConstantAttributesOnlyOnRootSpan) {
   auto resource =
       opentelemetry::sdk::resource::Resource::Create({{"key", "value"}});
 
-  auto rec = Recordable(Project(kProjectId));
+  auto rec = Recordable(Project(kProjectId), TestGenerator());
   rec.SetInstrumentationScope(*scope);
   rec.SetIdentity(opentelemetry::trace::SpanContext::GetInvalid(), parent);
   rec.SetResource(resource);
