@@ -12,14 +12,15 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include "google/cloud/pubsub/admin/subscription_admin_client.h"
+#include "google/cloud/pubsub/admin/topic_admin_client.h"
 #include "google/cloud/pubsub/publisher.h"
 #include "google/cloud/pubsub/subscriber.h"
-#include "google/cloud/pubsub/subscription_admin_client.h"
 #include "google/cloud/pubsub/testing/random_names.h"
-#include "google/cloud/pubsub/topic_admin_client.h"
 #include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/format_time_point.h"
 #include "google/cloud/internal/getenv.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/random.h"
 #include "google/cloud/options.h"
 #include "google/cloud/testing_util/command_line_parsing.h"
@@ -37,6 +38,7 @@
 
 namespace {
 namespace pubsub = ::google::cloud::pubsub;
+namespace pubsub_admin = ::google::cloud::pubsub_admin;
 using ::google::cloud::future;
 using ::google::cloud::Status;
 using ::google::cloud::StatusOr;
@@ -152,7 +154,7 @@ void PublisherTask(Config const& config, ExperimentFlowControl& flow_control,
                    int task);
 
 std::vector<pubsub::Subscription> CreateSubscriptions(
-    pubsub::SubscriptionAdminClient subscription_admin,
+    pubsub_admin::SubscriptionAdminClient subscription_admin,
     google::cloud::internal::DefaultPRNG& generator, Config const& config,
     pubsub::Topic const& topic);
 
@@ -179,9 +181,10 @@ int main(int argc, char* argv[]) {
   }
   if (config->show_help) return 0;
 
-  pubsub::TopicAdminClient topic_admin(pubsub::MakeTopicAdminConnection());
-  pubsub::SubscriptionAdminClient subscription_admin(
-      pubsub::MakeSubscriptionAdminConnection());
+  pubsub_admin::TopicAdminClient topic_admin(
+      pubsub_admin::MakeTopicAdminConnection());
+  pubsub_admin::SubscriptionAdminClient subscription_admin(
+      pubsub_admin::MakeSubscriptionAdminConnection());
 
   auto generator = google::cloud::internal::MakeDefaultPRNG();
 
@@ -193,13 +196,13 @@ int main(int argc, char* argv[]) {
   if (config->topic_id.empty()) {
     config->topic_id = google::cloud::pubsub_testing::RandomTopicId(generator);
     auto topic = pubsub::Topic(config->project_id, config->topic_id);
-    auto create = topic_admin.CreateTopic(pubsub::TopicBuilder{topic});
+    auto create = topic_admin.CreateTopic(topic.FullName());
     if (!create) {
       std::cout << "CreateTopic() failed: " << create.status() << "\n";
       return 1;
     }
     delete_topic = [topic_admin, topic]() mutable {
-      (void)topic_admin.DeleteTopic(topic);
+      (void)topic_admin.DeleteTopic(topic.FullName());
     };
   }
 
@@ -290,7 +293,7 @@ int main(int argc, char* argv[]) {
   cleanup.Defer(delete_topic);
   for (auto const& sub : subscriptions) {
     cleanup.Defer([subscription_admin, sub]() mutable {
-      (void)subscription_admin.DeleteSubscription(sub);
+      (void)subscription_admin.DeleteSubscription(sub.FullName());
     });
   }
 
@@ -465,7 +468,7 @@ void PublisherTask(Config const& config, ExperimentFlowControl& flow_control,
 }
 
 std::vector<pubsub::Subscription> CreateSubscriptions(
-    pubsub::SubscriptionAdminClient subscription_admin,
+    pubsub_admin::SubscriptionAdminClient subscription_admin,
     google::cloud::internal::DefaultPRNG& generator, Config const& config,
     pubsub::Topic const& topic) {
   std::vector<pubsub::Subscription> subscriptions;
@@ -473,7 +476,10 @@ std::vector<pubsub::Subscription> CreateSubscriptions(
     auto sub = pubsub::Subscription(
         config.project_id,
         google::cloud::pubsub_testing::RandomSubscriptionId(generator));
-    auto create = subscription_admin.CreateSubscription(topic, sub);
+    google::pubsub::v1::Subscription request;
+    request.set_name(sub.FullName());
+    request.set_topic(topic.FullName());
+    auto create = subscription_admin.CreateSubscription(request);
     if (!create) continue;
     subscriptions.push_back(std::move(sub));
   }
@@ -551,33 +557,37 @@ google::cloud::StatusOr<Config> ParseArgsImpl(std::vector<std::string> args,
   }
 
   if (options.project_id.empty()) {
-    return google::cloud::Status(google::cloud::StatusCode::kInvalidArgument,
-                                 "missing or empty --project-id option");
+    return google::cloud::internal::InvalidArgumentError(
+        "missing or empty --project-id option");
   }
 
   return options;
 }
 
 google::cloud::StatusOr<Config> SelfTest(std::string const& cmd) {
-  auto error = [](std::string m) {
-    return google::cloud::Status(google::cloud::StatusCode::kUnknown,
-                                 std::move(m));
+  auto error = [](std::string m,
+                  google::cloud::internal::ErrorInfoBuilder info) {
+    return google::cloud::internal::UnknownError(std::move(m), std::move(info));
   };
   for (auto const& var : {"GOOGLE_CLOUD_PROJECT"}) {
     auto const value = GetEnv(var).value_or("");
     if (!value.empty()) continue;
     std::ostringstream os;
     os << "The environment variable " << var << " is not set or empty";
-    return error(std::move(os).str());
+    return error(std::move(os).str(), GCP_ERROR_INFO());
   }
   auto config = ParseArgsImpl({cmd, "--help"}, kDescription);
-  if (!config || !config->show_help) return error("--help parsing");
+  if (!config || !config->show_help) {
+    return error("--help parsing", GCP_ERROR_INFO());
+  }
   config = ParseArgsImpl({cmd, "--description", "--help"}, kDescription);
-  if (!config || !config->show_help) return error("--description parsing");
+  if (!config || !config->show_help) {
+    return error("--description parsing", GCP_ERROR_INFO());
+  }
   config = ParseArgsImpl({cmd, "--project-id="}, kDescription);
-  if (config) return error("--project-id validation");
+  if (config) return error("--project-id validation", GCP_ERROR_INFO());
   config = ParseArgsImpl({cmd, "--topic-id=test-topic"}, kDescription);
-  if (!config) return error("--topic-id");
+  if (!config) return error("--topic-id", GCP_ERROR_INFO());
 
   return ParseArgsImpl(
       {
