@@ -17,6 +17,7 @@
 set -euo pipefail
 
 source "$(dirname "$0")/../../../../ci/lib/init.sh"
+source module /ci/cloudbuild/builds/lib/ctest.sh
 source module /ci/etc/integration-tests-config.sh
 source module /google/cloud/pubsub/ci/lib/pubsub_emulator.sh
 
@@ -33,6 +34,10 @@ readonly BINARY_DIR
 shift
 ctest_args=("$@")
 
+if ctest::has_no_tests "${BINARY_DIR}" "^pubsub_" "${ctest_args[@]}"; then
+  exit 0
+fi
+
 cd "${BINARY_DIR}"
 # Start the emulator and arranges to kill it
 pubsub_emulator::start
