@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/pubsub/internal/batching_publisher_connection.h"
+#include "google/cloud/internal/make_status.h"
 
 namespace google {
 namespace cloud {
@@ -33,8 +34,8 @@ struct Batch {
     }
     if (static_cast<std::size_t>(response->message_ids_size()) !=
         waiters.size()) {
-      SatisfyAllWaiters(
-          Status(StatusCode::kUnknown, "mismatched message id count"));
+      SatisfyAllWaiters(internal::UnknownError("mismatched message id count",
+                                               GCP_ERROR_INFO()));
       return;
     }
     int idx = 0;
@@ -54,7 +55,7 @@ BatchingPublisherConnection::~BatchingPublisherConnection() {
 
 future<StatusOr<std::string>> BatchingPublisherConnection::Publish(
     PublishParams p) {
-  batch_->AddMessage(p.message);
+  sink_->AddMessage(p.message);
   auto const bytes = MessageSize(p.message);
   std::unique_lock<std::mutex> lk(mu_);
   do {
@@ -198,10 +199,7 @@ void BatchingPublisherConnection::FlushImpl(std::unique_lock<std::mutex> lk) {
   batch.weak = shared_from_this();
   request.set_topic(topic_full_name_);
 
-  auto handler = batch_->Flush();
-  sink_->AsyncPublish(std::move(request))
-      .then(std::move(batch))
-      .then(std::move(handler));
+  sink_->AsyncPublish(std::move(request)).then(std::move(batch));
 }
 
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
