@@ -15,7 +15,6 @@
 #include "google/cloud/pubsub/internal/batching_publisher_connection.h"
 #include "google/cloud/pubsub/internal/defaults.h"
 #include "google/cloud/pubsub/testing/mock_batch_sink.h"
-#include "google/cloud/pubsub/testing/mock_message_batch.h"
 #include "google/cloud/future.h"
 #include "google/cloud/internal/random.h"
 #include "google/cloud/testing_util/async_sequencer.h"
@@ -36,12 +35,12 @@ namespace {
 using ::google::cloud::testing_util::AsyncSequencer;
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::StatusIs;
+using ::testing::_;
 using ::testing::AnyOf;
 using ::testing::AtLeast;
 using ::testing::Contains;
 using ::testing::ElementsAre;
 using ::testing::HasSubstr;
-using ::testing::NiceMock;
 
 google::pubsub::v1::PublishResponse MakeResponse(
     google::pubsub::v1::PublishRequest const& request) {
@@ -64,12 +63,10 @@ std::vector<std::string> MessagesData(
 
 TEST(BatchingPublisherConnectionTest, FastDestructor) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   AsyncSequencer<void> async;
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   // This test will never get a chance to flush its message.
   EXPECT_CALL(*mock, AsyncPublish).Times(0);
 
@@ -84,7 +81,7 @@ TEST(BatchingPublisherConnectionTest, FastDestructor) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq(), mock_batch);
+      ordering_key, mock, background.cq());
 
   // Publishing a message starts the batch timer.
   auto pending = publisher->Publish(
@@ -101,12 +98,10 @@ TEST(BatchingPublisherConnectionTest, FastDestructor) {
 
 TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   AsyncSequencer<void> async;
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .Times(AtLeast(1))
       .WillRepeatedly([&](google::pubsub::v1::PublishRequest const& request) {
@@ -134,7 +129,7 @@ TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq(), mock_batch);
+      ordering_key, mock, background.cq());
 
   // We expect the responses to be satisfied in the context of the completion
   // queue threads. This is an important property, the processing of any
@@ -171,11 +166,9 @@ TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -201,7 +194,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
               .set<pubsub::MaxBatchMessagesOption>(2)
               .set<pubsub::MaxBatchBytesOption>(kMaxBytes)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq(), mock_batch);
+      ordering_key, mock, background.cq());
   auto r0 =
       publisher
           ->Publish({pubsub::MessageBuilder{}.SetData("test-data-0").Build()})
@@ -226,11 +219,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -259,7 +250,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxBatchBytesOption>(max_bytes)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq(), mock_batch);
+      ordering_key, mock, background.cq());
   auto r0 = publisher->Publish({m0}).then([](future<StatusOr<std::string>> f) {
     auto r = f.get();
     ASSERT_STATUS_OK(r);
@@ -277,9 +268,6 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kSinglePayload = 128;
@@ -291,6 +279,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
   // publisher should first flush the existing batch and then send the full
   // message.
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -320,7 +309,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(100)
               .set<pubsub::MaxBatchBytesOption>(kBatchLimit)),
-      ordering_key, mock, cq, mock_batch);
+      ordering_key, mock, cq);
   std::vector<future<Status>> results;
   for (int i = 0; i != 3; ++i) {
     results.push_back(
@@ -348,9 +337,6 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kSinglePayload = 128;
@@ -369,6 +355,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
       };
 
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -401,7 +388,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(100)
               .set<pubsub::MaxBatchBytesOption>(kBatchLimit)),
-      ordering_key, mock, cq, mock_batch);
+      ordering_key, mock, cq);
   std::vector<future<Status>> results;
   auto publish_single = [&] {
     results.push_back(
@@ -433,9 +420,6 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchTorture) {
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kMaxMessages = 20;
@@ -453,6 +437,7 @@ TEST(BatchingPublisherConnectionTest, BatchTorture) {
       };
 
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillRepeatedly([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -471,7 +456,7 @@ TEST(BatchingPublisherConnectionTest, BatchTorture) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(kMaxMessages)
               .set<pubsub::MaxBatchBytesOption>(kMaxPayload)),
-      ordering_key, mock, background.cq(), mock_batch);
+      ordering_key, mock, background.cq());
 
   auto worker = [&](int iterations) {
     auto gen = google::cloud::internal::DefaultPRNG(std::random_device{}());
@@ -491,20 +476,17 @@ TEST(BatchingPublisherConnectionTest, BatchTorture) {
     for (auto& r : results) EXPECT_STATUS_OK(r.get());
   };
   std::vector<std::thread> workers(4);
-  std::generate(workers.begin(), workers.end(), [&] {
-    return std::thread{worker, 1000};
-  });
+  std::generate(workers.begin(), workers.end(),
+                [&] { return std::thread{worker, 1000}; });
   publisher->Flush({});
   for (auto& w : workers) w.join();
 }
 
 TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -526,7 +508,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(std::chrono::milliseconds(5))),
-      /*ordering_key=*/{}, mock, cq, mock_batch);
+      /*ordering_key=*/{}, mock, cq);
   auto r0 =
       publisher
           ->Publish({pubsub::MessageBuilder{}.SetData("test-data-0").Build()})
@@ -557,11 +539,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
 
 TEST(BatchingPublisherConnectionTest, BatchByFlush) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const& request) {
         EXPECT_EQ(topic.FullName(), request.topic());
@@ -594,7 +574,7 @@ TEST(BatchingPublisherConnectionTest, BatchByFlush) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(std::chrono::milliseconds(5))),
-      ordering_key, mock, cq, mock_batch);
+      ordering_key, mock, cq);
 
   std::vector<future<void>> results;
   for (auto i : {0, 1}) {
@@ -637,12 +617,10 @@ TEST(BatchingPublisherConnectionTest, BatchByFlush) {
 
 TEST(BatchingPublisherConnectionTest, HandleError) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto const error_status = Status(StatusCode::kPermissionDenied, "uh-oh");
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillRepeatedly([&](google::pubsub::v1::PublishRequest const&) {
         return make_ready_future(
@@ -654,7 +632,7 @@ TEST(BatchingPublisherConnectionTest, HandleError) {
   auto publisher = BatchingPublisherConnection::Create(
       topic,
       DefaultPublisherOptions(Options{}.set<pubsub::MaxBatchMessagesOption>(2)),
-      ordering_key, mock, bg.cq(), mock_batch);
+      ordering_key, mock, bg.cq());
   auto r0 = publisher->Publish(
       {pubsub::MessageBuilder{}.SetData("test-data-0").Build()});
   auto r1 = publisher->Publish(
@@ -668,11 +646,9 @@ TEST(BatchingPublisherConnectionTest, HandleError) {
 
 TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillRepeatedly([&](google::pubsub::v1::PublishRequest const&) {
         google::pubsub::v1::PublishResponse response;
@@ -683,7 +659,7 @@ TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
   auto publisher = BatchingPublisherConnection::Create(
       topic,
       DefaultPublisherOptions(Options{}.set<pubsub::MaxBatchMessagesOption>(2)),
-      "test-ordering-key", mock, background.cq(), mock_batch);
+      "test-ordering-key", mock, background.cq());
   auto r0 = publisher->Publish(
       {pubsub::MessageBuilder{}.SetData("test-data-0").Build()});
   auto r1 = publisher->Publish(
@@ -697,14 +673,13 @@ TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
 
 TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
+
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto const error_status = Status(StatusCode::kPermissionDenied, "uh-oh");
 
   AsyncSequencer<void> async;
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([&](google::pubsub::v1::PublishRequest const&) {
         return async.PushBack().then([error_status](future<void>) {
@@ -720,7 +695,7 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
       topic,
       DefaultPublisherOptions(
           Options{}.set<pubsub::MaxBatchMessagesOption>(kBatchSize)),
-      ordering_key, mock, cq, mock_batch);
+      ordering_key, mock, cq);
   std::vector<future<StatusOr<std::string>>> results;
   // Create a full batch (by message count) and a partial batch.
   for (int i = 0; i != kBatchSize + kBatchSize / 2; ++i) {
@@ -748,15 +723,13 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
 
 TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingResume) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
-  auto mock_batch =
-      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
-  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
   auto const ordering_key = std::string{"test-key"};
 
   auto const error_status = Status(StatusCode::kPermissionDenied, "uh-oh");
 
   AsyncSequencer<void> async;
+  EXPECT_CALL(*mock, AddMessage(_)).Times(AtLeast(1));
   {
     ::testing::InSequence sequence;
     EXPECT_CALL(*mock, AsyncPublish)
@@ -784,7 +757,7 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingResume) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(kBatchSize)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, cq, mock_batch);
+      ordering_key, mock, cq);
   std::vector<future<StatusOr<std::string>>> results;
   // Create a full batch (by size).
   for (int i = 0; i != kBatchSize; ++i) {
