@@ -13,19 +13,13 @@
 // limitations under the License.
 
 #include "google/cloud/pubsub/internal/create_channel.h"
+#include "google/cloud/grpc_options.h"
 
 namespace google {
 namespace cloud {
 namespace pubsub_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
-std::shared_ptr<grpc::Channel> CreateChannel(Options const& opts,
-                                             int channel_id) {
-  return grpc::CreateCustomChannel(opts.get<EndpointOption>(),
-                                   opts.get<GrpcCredentialOption>(),
-                                   MakeChannelArguments(opts, channel_id));
-}
-
 grpc::ChannelArguments MakeChannelArguments(Options const& opts,
                                             int channel_id) {
   auto args = internal::MakeChannelArguments(opts);
