@@ -16,6 +16,7 @@
 #include "google/cloud/pubsub/options.h"
 #include "google/cloud/common_options.h"
 #include "google/cloud/connection_options.h"
+#include "google/cloud/credentials.h"
 #include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/user_agent_prefix.h"
 #include "google/cloud/testing_util/scoped_environment.h"
@@ -61,12 +62,11 @@ TEST(OptionsTest, UnsetEmulatorEnv) {
 TEST(OptionsTest, CommonDefaults) {
   auto opts = DefaultCommonOptions(Options{});
   EXPECT_EQ("pubsub.googleapis.com", opts.get<EndpointOption>());
-  EXPECT_EQ(typeid(grpc::GoogleDefaultCredentials()),
-            typeid(opts.get<GrpcCredentialOption>()));
+  EXPECT_TRUE(opts.has<UnifiedCredentialsOption>());
   EXPECT_EQ(static_cast<int>(DefaultThreadCount()),
             opts.get<GrpcNumChannelsOption>());
   EXPECT_EQ(internal::DefaultTracingComponents(),
-            opts.get<TracingComponentsOption>());
+            opts.get<LoggingComponentsOption>());
   EXPECT_EQ(internal::DefaultTracingOptions(),
             opts.get<GrpcTracingOptionsOption>());
   EXPECT_TRUE(opts.has<pubsub::RetryPolicyOption>());
@@ -94,7 +94,7 @@ TEST(OptionsTest, UserSetCommonOptions) {
           .set<GrpcCredentialOption>(grpc::InsecureChannelCredentials())
           .set<GrpcTracingOptionsOption>(
               TracingOptions{}.SetOptions("single_line_mode=F"))
-          .set<TracingComponentsOption>({"test-component"})
+          .set<LoggingComponentsOption>({"test-component"})
           .set<GrpcNumChannelsOption>(3)
           .set<GrpcBackgroundThreadPoolSizeOption>(5)
           .set<GrpcChannelArgumentsNativeOption>(channel_args)
@@ -104,7 +104,7 @@ TEST(OptionsTest, UserSetCommonOptions) {
   EXPECT_EQ(typeid(grpc::InsecureChannelCredentials()),
             typeid(opts.get<GrpcCredentialOption>()));
   EXPECT_FALSE(opts.get<GrpcTracingOptionsOption>().single_line_mode());
-  EXPECT_THAT(opts.get<TracingComponentsOption>(), Contains("test-component"));
+  EXPECT_THAT(opts.get<LoggingComponentsOption>(), Contains("test-component"));
   EXPECT_EQ(3U, opts.get<GrpcNumChannelsOption>());
   EXPECT_EQ(5U, opts.get<GrpcBackgroundThreadPoolSizeOption>());
 
@@ -260,10 +260,10 @@ TEST(OptionsTest, DefaultSubscriberOnly) {
   // Ensure that we do not set common options
   auto opts = DefaultSubscriberOptionsOnly(Options{});
   EXPECT_FALSE(opts.has<GrpcCredentialOption>());
+  EXPECT_FALSE(opts.has<UnifiedCredentialsOption>());
   EXPECT_FALSE(opts.has<EndpointOption>());
-  EXPECT_FALSE(opts.has<GrpcCredentialOption>());
   EXPECT_FALSE(opts.has<GrpcNumChannelsOption>());
-  EXPECT_FALSE(opts.has<TracingComponentsOption>());
+  EXPECT_FALSE(opts.has<LoggingComponentsOption>());
   EXPECT_FALSE(opts.has<GrpcTracingOptionsOption>());
   EXPECT_FALSE(opts.has<pubsub::BackoffPolicyOption>());
   EXPECT_FALSE(opts.has<GrpcBackgroundThreadPoolSizeOption>());
@@ -271,11 +271,10 @@ TEST(OptionsTest, DefaultSubscriberOnly) {
 
   // Ensure that we do set common options
   opts = DefaultSubscriberOptions(Options{});
-  EXPECT_TRUE(opts.has<GrpcCredentialOption>());
+  EXPECT_TRUE(opts.has<UnifiedCredentialsOption>());
   EXPECT_TRUE(opts.has<EndpointOption>());
-  EXPECT_TRUE(opts.has<GrpcCredentialOption>());
   EXPECT_TRUE(opts.has<GrpcNumChannelsOption>());
-  EXPECT_TRUE(opts.has<TracingComponentsOption>());
+  EXPECT_TRUE(opts.has<LoggingComponentsOption>());
   EXPECT_TRUE(opts.has<GrpcTracingOptionsOption>());
   EXPECT_TRUE(opts.has<GrpcBackgroundThreadPoolSizeOption>());
   EXPECT_TRUE(opts.has<UserAgentProductsOption>());
@@ -285,21 +284,20 @@ TEST(OptionsTest, DefaultPublisherOnly) {
   // Ensure that we do not set common options
   auto opts = DefaultPublisherOptionsOnly(Options{});
   EXPECT_FALSE(opts.has<GrpcCredentialOption>());
+  EXPECT_FALSE(opts.has<UnifiedCredentialsOption>());
   EXPECT_FALSE(opts.has<EndpointOption>());
-  EXPECT_FALSE(opts.has<GrpcCredentialOption>());
   EXPECT_FALSE(opts.has<GrpcNumChannelsOption>());
-  EXPECT_FALSE(opts.has<TracingComponentsOption>());
+  EXPECT_FALSE(opts.has<LoggingComponentsOption>());
   EXPECT_FALSE(opts.has<GrpcTracingOptionsOption>());
   EXPECT_FALSE(opts.has<GrpcBackgroundThreadPoolSizeOption>());
   EXPECT_FALSE(opts.has<UserAgentProductsOption>());
 
   // Ensure that we do set common options
   opts = DefaultPublisherOptions(Options{});
-  EXPECT_TRUE(opts.has<GrpcCredentialOption>());
+  EXPECT_TRUE(opts.has<UnifiedCredentialsOption>());
   EXPECT_TRUE(opts.has<EndpointOption>());
-  EXPECT_TRUE(opts.has<GrpcCredentialOption>());
   EXPECT_TRUE(opts.has<GrpcNumChannelsOption>());
-  EXPECT_TRUE(opts.has<TracingComponentsOption>());
+  EXPECT_TRUE(opts.has<LoggingComponentsOption>());
   EXPECT_TRUE(opts.has<GrpcTracingOptionsOption>());
   EXPECT_TRUE(opts.has<GrpcBackgroundThreadPoolSizeOption>());
   EXPECT_TRUE(opts.has<UserAgentProductsOption>());
