@@ -17,8 +17,8 @@
 #include "google/cloud/pubsub/internal/message_propagator.h"
 #include "google/cloud/pubsub/internal/message_carrier.h"
 #include "google/cloud/pubsub/message.h"
-#include "opentelemetry/context/propagation/text_map_propagator.h"
-#include "opentelemetry/context/runtime_context.h"
+#include <opentelemetry/context/propagation/text_map_propagator.h>
+#include <opentelemetry/context/runtime_context.h>
 
 namespace google {
 namespace cloud {
@@ -33,6 +33,14 @@ void InjectTraceContext(
   propagator.Inject(carrier, current);
 }
 
+opentelemetry::context::Context ExtractTraceContext(
+    pubsub::Message& message,
+    opentelemetry::context::propagation::TextMapPropagator& propagator) {
+  auto current = opentelemetry::context::RuntimeContext::GetCurrent();
+  MessageCarrier carrier(message);
+  return propagator.Extract(carrier, current);
+}
+
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace pubsub_internal
 }  // namespace cloud
