@@ -19,6 +19,7 @@
 #include "google/cloud/pubsub/options.h"
 #include "google/cloud/pubsub/testing/mock_batch_sink.h"
 #include "google/cloud/pubsub/topic.h"
+#include "google/cloud/common_options.h"
 #include "google/cloud/internal/opentelemetry.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
 #include "google/cloud/testing_util/opentelemetry_matchers.h"
@@ -43,9 +44,8 @@ using ::google::cloud::testing_util::SpanEventAttributesAre;
 using ::google::cloud::testing_util::SpanHasAttributes;
 using ::google::cloud::testing_util::SpanHasEvents;
 using ::google::cloud::testing_util::SpanHasInstrumentationScope;
+using ::google::cloud::testing_util::SpanIsRoot;
 using ::google::cloud::testing_util::SpanKindIsClient;
-using ::google::cloud::testing_util::SpanKindIsProducer;
-using ::google::cloud::testing_util::SpanLinkAttributesAre;
 using ::google::cloud::testing_util::SpanLinksSizeIs;
 using ::google::cloud::testing_util::SpanNamed;
 using ::google::cloud::testing_util::ThereIsAnActiveSpan;
@@ -56,6 +56,7 @@ using ::testing::Contains;
 namespace {
 
 auto constexpr kDefaultMaxLinks = 128;
+auto constexpr kDefaultEndpoint = "endpoint";
 
 void EndSpans(std::vector<opentelemetry::nostd::shared_ptr<
                   opentelemetry::trace::Span>> const& spans) {
@@ -95,17 +96,21 @@ void AddMessages(
 auto MakeTestOptions(size_t max_otel_link_count = kDefaultMaxLinks) {
   Options options;
   options.set<pubsub::MaxOtelLinkCountOption>(max_otel_link_count);
+  options.set<EndpointOption>(kDefaultEndpoint);
   return options;
 }
 
-std::shared_ptr<BatchSink> MakeTestBatchSink(std::shared_ptr<BatchSink> mock) {
-  return MakeTracingBatchSink(std::move(mock), MakeTestOptions());
-}
-
 pubsub::Topic TestTopic() {
   return pubsub::Topic("test-project", "test-topic");
 }
 
+std::shared_ptr<BatchSink> MakeTestBatchSink(std::shared_ptr<BatchSink> mock,
+                                             Options options = {}) {
+  return MakeTracingBatchSink(
+      TestTopic(), std::move(mock),
+      internal::MergeOptions(std::move(options), MakeTestOptions()));
+}
+
 google::pubsub::v1::PublishRequest MakeRequest(int n) {
   google::pubsub::v1::PublishRequest request;
   request.set_topic(TestTopic().FullName());
@@ -164,16 +169,13 @@ TEST(TracingBatchSink, AsyncPublish) {
   EXPECT_THAT(response, IsOk());
 
   auto spans = span_catcher->GetSpans();
-  EXPECT_THAT(
-      spans,
-      Contains(AllOf(
-          SpanHasInstrumentationScope(), SpanKindIsProducer(),
-          SpanNamed("publish"),
-          SpanHasAttributes(
-              OTelAttribute<std::int64_t>(sc::kMessagingBatchMessageCount, 1)),
-          SpanHasLinks(AllOf(LinkHasSpanContext(message_span->GetContext()),
-                             SpanLinkAttributesAre(OTelAttribute<int64_t>(
-                                 "messaging.gcp_pubsub.message.link", 0)))))));
+  EXPECT_THAT(spans,
+              Contains(AllOf(SpanHasInstrumentationScope(), SpanKindIsClient(),
+                             SpanNamed("test-topic publish"),
+                             SpanHasAttributes(OTelAttribute<std::int64_t>(
+                                 sc::kMessagingBatchMessageCount, 1)),
+                             SpanHasLinks(AllOf(LinkHasSpanContext(
+                                 message_span->GetContext()))))));
 }
 
 TEST(TracingBatchSink, PublishSpanHasAttributes) {
@@ -196,20 +198,64 @@ TEST(TracingBatchSink, PublishSpanHasAttributes) {
 
   auto spans = span_catcher->GetSpans();
   EXPECT_THAT(spans,
-              Contains(AllOf(SpanNamed("publish"),
+              Contains(AllOf(SpanNamed("test-topic publish"),
                              SpanHasAttributes(OTelAttribute<std::string>(
                                  sc::kThreadId, _)))));
   EXPECT_THAT(spans, Contains(AllOf(
-
-                         SpanNamed("publish"),
+                         SpanNamed("test-topic publish"),
                          SpanHasAttributes(OTelAttribute<std::string>(
                              sc::kCodeFunction, "BatchSink::AsyncPublish")))));
+  EXPECT_THAT(
+      spans, Contains(AllOf(
+                 SpanNamed("test-topic publish"),
+                 SpanHasAttributes(OTelAttribute<std::string>(
+                     /*sc::kMessagingOperationType=*/"messaging.operation.type",
+                     "publish")))));
   EXPECT_THAT(spans,
-              Contains(AllOf(SpanNamed("publish"),
+              Contains(AllOf(SpanNamed("test-topic publish"),
                              SpanHasAttributes(OTelAttribute<std::string>(
-                                 sc::kMessagingOperation, "publish")))));
+                                 sc::kMessagingSystem, "gcp_pubsub")))));
+  EXPECT_THAT(
+      spans, Contains(AllOf(SpanNamed("test-topic publish"),
+                            SpanHasAttributes(OTelAttribute<std::string>(
+                                "gcp.project_id", TestTopic().project_id())))));
+  EXPECT_THAT(spans,
+              Contains(AllOf(SpanNamed("test-topic publish"),
+                             SpanHasAttributes(OTelAttribute<std::string>(
+                                 "server.address", kDefaultEndpoint)))));
+  EXPECT_THAT(
+      spans, Contains(AllOf(
+                 SpanNamed("test-topic publish"),
+                 SpanHasAttributes(OTelAttribute<std::string>(
+                     sc::kMessagingDestinationName, TestTopic().topic_id())))));
 }
 
+#if OPENTELEMETRY_VERSION_MAJOR >= 2 || \
+    (OPENTELEMETRY_VERSION_MAJOR == 1 && OPENTELEMETRY_VERSION_MINOR >= 13)
+TEST(TracingBatchSink, PublishSpanIsRoot) {
+  auto span_catcher = InstallSpanCatcher();
+  auto message_span = MakeSpan("test span");
+  auto scope = opentelemetry::trace::Scope(message_span);
+  auto mock = std::make_unique<pubsub_testing::MockBatchSink>();
+  EXPECT_CALL(*mock, AddMessage(_));
+  EXPECT_CALL(*mock, AsyncPublish)
+      .WillOnce([](google::pubsub::v1::PublishRequest const& request) {
+        EXPECT_THAT(request, IsProtoEqual(MakeRequest(1)));
+        return make_ready_future(make_status_or(MakeResponse(request)));
+      });
+  auto batch_sink = MakeTestBatchSink(std::move(mock));
+  auto initial_spans = {message_span};
+  AddMessages(initial_spans, batch_sink);
+
+  auto response = batch_sink->AsyncPublish(MakeRequest(1)).get();
+  EXPECT_THAT(response, IsOk());
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(spans,
+              Contains(AllOf(SpanNamed("test-topic publish"), SpanIsRoot())));
+}
+#endif
+
 TEST(TracingBatchSink, AsyncPublishOnlyIncludeSampledLink) {
   namespace sc = ::opentelemetry::trace::SemanticConventions;
   // Create span before the span catcher so it is not sampled.
@@ -231,16 +277,13 @@ TEST(TracingBatchSink, AsyncPublishOnlyIncludeSampledLink) {
   EXPECT_THAT(response, IsOk());
 
   auto spans = span_catcher->GetSpans();
-  EXPECT_THAT(
-      spans,
-      Contains(AllOf(
-          SpanHasInstrumentationScope(), SpanKindIsProducer(),
-          SpanNamed("publish"),
-          SpanHasAttributes(
-              OTelAttribute<std::int64_t>(sc::kMessagingBatchMessageCount, 2)),
-          SpanLinksAre(AllOf(LinkHasSpanContext(message_span->GetContext()),
-                             SpanLinkAttributesAre(OTelAttribute<int64_t>(
-                                 "messaging.gcp_pubsub.message.link", 0)))))));
+  EXPECT_THAT(spans,
+              Contains(AllOf(SpanHasInstrumentationScope(), SpanKindIsClient(),
+                             SpanNamed("test-topic publish"),
+                             SpanHasAttributes(OTelAttribute<std::int64_t>(
+                                 sc::kMessagingBatchMessageCount, 2)),
+                             SpanLinksAre(AllOf(LinkHasSpanContext(
+                                 message_span->GetContext()))))));
 }
 
 TEST(TracingBatchSink, AsyncPublishSmallBatch) {
@@ -265,16 +308,12 @@ TEST(TracingBatchSink, AsyncPublishSmallBatch) {
   EXPECT_THAT(
       spans,
       Contains(AllOf(
-          SpanHasInstrumentationScope(), SpanKindIsProducer(),
-          SpanNamed("publish"),
+          SpanHasInstrumentationScope(), SpanKindIsClient(),
+          SpanNamed("test-topic publish"),
           SpanHasAttributes(
               OTelAttribute<std::int64_t>(sc::kMessagingBatchMessageCount, 2)),
-          SpanHasLinks(AllOf(LinkHasSpanContext(message_span1->GetContext()),
-                             SpanLinkAttributesAre(OTelAttribute<int64_t>(
-                                 "messaging.gcp_pubsub.message.link", 0))),
-                       AllOf(LinkHasSpanContext(message_span2->GetContext()),
-                             SpanLinkAttributesAre(OTelAttribute<int64_t>(
-                                 "messaging.gcp_pubsub.message.link", 1)))))));
+          SpanHasLinks(LinkHasSpanContext(message_span1->GetContext()),
+                       LinkHasSpanContext(message_span2->GetContext())))));
 }
 
 TEST(TracingBatchSink, AsyncPublishBatchWithOtelLimit) {
@@ -295,8 +334,8 @@ TEST(TracingBatchSink, AsyncPublishBatchWithOtelLimit) {
   auto spans = span_catcher->GetSpans();
   EXPECT_THAT(
       spans,
-      Contains(AllOf(SpanHasInstrumentationScope(), SpanKindIsProducer(),
-                     SpanNamed("publish"),
+      Contains(AllOf(SpanHasInstrumentationScope(), SpanKindIsClient(),
+                     SpanNamed("test-topic publish"),
                      SpanHasAttributes(OTelAttribute<std::int64_t>(
                          sc::kMessagingBatchMessageCount, kDefaultMaxLinks)),
                      SpanLinksSizeIs(kDefaultMaxLinks))));
@@ -309,7 +348,7 @@ TEST(TracingBatchSink, AsyncPublishLargeBatch) {
   EXPECT_CALL(*mock, AddMessage(_)).Times(batch_size);
   EXPECT_CALL(*mock, AsyncPublish)
       .WillOnce([](google::pubsub::v1::PublishRequest const& request) {
-        EXPECT_THAT(request, IsProtoEqual(MakeRequest(batch_size)));
+        EXPECT_THAT(request, IsProtoEqual(MakeRequest(kDefaultMaxLinks + 1)));
         return make_ready_future(make_status_or(MakeResponse(request)));
       });
   auto span_catcher = InstallSpanCatcher();
@@ -321,7 +360,7 @@ TEST(TracingBatchSink, AsyncPublishLargeBatch) {
 
   auto spans = span_catcher->GetSpans();
   EXPECT_THAT(spans, Contains(AllOf(
-                         SpanNamed("publish"),
+                         SpanNamed("test-topic publish"),
                          SpanHasAttributes(OTelAttribute<std::int64_t>(
                              sc::kMessagingBatchMessageCount, batch_size)))));
   EXPECT_THAT(spans, Contains(AllOf(SpanNamed("publish #0"), SpanKindIsClient(),
@@ -337,12 +376,13 @@ TEST(TracingBatchSink, AsyncPublishBatchWithCustomLimit) {
   auto mock = std::make_unique<pubsub_testing::MockBatchSink>();
   EXPECT_CALL(*mock, AddMessage(_)).Times(kBatchSize);
   EXPECT_CALL(*mock, AsyncPublish)
-      .WillOnce([](google::pubsub::v1::PublishRequest const& request) {
-        EXPECT_THAT(request, IsProtoEqual(MakeRequest(kBatchSize)));
+      .WillOnce([batch_size = kBatchSize](
+                    google::pubsub::v1::PublishRequest const& request) {
+        EXPECT_THAT(request, IsProtoEqual(MakeRequest(batch_size)));
         return make_ready_future(make_status_or(MakeResponse(request)));
       });
   auto batch_sink =
-      MakeTracingBatchSink(std::move(mock), MakeTestOptions(kMaxLinks));
+      MakeTestBatchSink(std::move(mock), MakeTestOptions(kMaxLinks));
 
   auto span_catcher = InstallSpanCatcher();
   AddMessages(CreateSpans(kBatchSize), batch_sink);
@@ -351,8 +391,8 @@ TEST(TracingBatchSink, AsyncPublishBatchWithCustomLimit) {
 
   auto spans = span_catcher->GetSpans();
   EXPECT_THAT(spans, Contains(AllOf(
-                         SpanHasInstrumentationScope(), SpanKindIsProducer(),
-                         SpanNamed("publish"),
+                         SpanHasInstrumentationScope(), SpanKindIsClient(),
+                         SpanNamed("test-topic publish"),
                          SpanHasAttributes(OTelAttribute<std::int64_t>(
                              sc::kMessagingBatchMessageCount, kBatchSize)))));
   EXPECT_THAT(spans, Contains(AllOf(SpanNamed("publish #0"), SpanKindIsClient(),
@@ -443,16 +483,13 @@ TEST(TracingBatchSink, Scope) {
   EXPECT_THAT(response, IsOk());
 
   auto spans = span_catcher->GetSpans();
-  EXPECT_THAT(
-      spans,
-      Contains(AllOf(
-          SpanHasInstrumentationScope(), SpanKindIsProducer(),
-          SpanNamed("publish"),
-          SpanHasAttributes(
-              OTelAttribute<std::int64_t>(sc::kMessagingBatchMessageCount, 1)),
-          SpanHasLinks(AllOf(LinkHasSpanContext(message_span->GetContext()),
-                             SpanLinkAttributesAre(OTelAttribute<int64_t>(
-                                 "messaging.gcp_pubsub.message.link", 0)))))));
+  EXPECT_THAT(spans,
+              Contains(AllOf(SpanHasInstrumentationScope(), SpanKindIsClient(),
+                             SpanNamed("test-topic publish"),
+                             SpanHasAttributes(OTelAttribute<std::int64_t>(
+                                 sc::kMessagingBatchMessageCount, 1)),
+                             SpanHasLinks(AllOf(LinkHasSpanContext(
+                                 message_span->GetContext()))))));
 }
 
 TEST(TracingBatchSink, ResumePublish) {
@@ -483,9 +520,9 @@ TEST(TracingBatchSink, AsyncPublishAddsLink) {
 
   EndSpans(message_spans);
 
-  EXPECT_THAT(span_catcher->GetSpans(),
-              Contains(AllOf(SpanNamed("test span 0"),
-                             SpanHasLinks(AllOf(LinkHasSpanContext(_)),
+  EXPECT_THAT(
+      span_catcher->GetSpans(),
+      Contains(AllOf(SpanNamed("test span 0"), SpanLinksSizeIs(1),
                      SpanHasEvents(EventNamed("gl-cpp.publish_start")))));
 }
 #else
