@@ -21,14 +21,12 @@
 #include "google/cloud/pubsub/internal/defaults.h"
 #include "google/cloud/pubsub/internal/flow_controlled_publisher_connection.h"
 #include "google/cloud/pubsub/internal/flow_controlled_publisher_tracing_connection.h"
-#include "google/cloud/pubsub/internal/message_batch.h"
-#include "google/cloud/pubsub/internal/noop_message_batch.h"
 #include "google/cloud/pubsub/internal/ordering_key_publisher_connection.h"
 #include "google/cloud/pubsub/internal/publisher_stub_factory.h"
 #include "google/cloud/pubsub/internal/publisher_tracing_connection.h"
 #include "google/cloud/pubsub/internal/rejects_with_ordering_key.h"
 #include "google/cloud/pubsub/internal/sequential_batch_sink.h"
-#include "google/cloud/pubsub/internal/tracing_message_batch.h"
+#include "google/cloud/pubsub/internal/tracing_batch_sink.h"
 #include "google/cloud/pubsub/options.h"
 #include "google/cloud/credentials.h"
 #include "google/cloud/internal/non_constructible.h"
@@ -45,18 +43,16 @@ namespace {
 std::shared_ptr<pubsub::PublisherConnection> ConnectionFromDecoratedStub(
     pubsub::Topic topic, Options opts,
     std::shared_ptr<BackgroundThreads> background,
-    std::shared_ptr<pubsub_internal::PublisherStub> stub,
-    std::shared_ptr<pubsub_internal::MessageBatch> message_batch) {
+    std::shared_ptr<pubsub_internal::PublisherStub> stub) {
   auto make_connection = [&]() -> std::shared_ptr<pubsub::PublisherConnection> {
     auto cq = background->cq();
     std::shared_ptr<pubsub_internal::BatchSink> sink =
         pubsub_internal::DefaultBatchSink::Create(stub, cq, opts);
     if (google::cloud::internal::TracingEnabled(opts)) {
-      message_batch = MakeTracingMessageBatch(std::move(message_batch), opts);
+      sink = MakeTracingBatchSink(topic, std::move(sink), opts);
     }
     if (opts.get<pubsub::MessageOrderingOption>()) {
-      auto factory = [topic, opts, sink, cq,
-                      message_batch](std::string const& key) {
+      auto factory = [topic, opts, sink, cq](std::string const& key) {
         auto used_sink = sink;
         if (!key.empty()) {
           // Only wrap the sink if there is an ordering key.
@@ -64,16 +60,14 @@ std::shared_ptr<pubsub::PublisherConnection> ConnectionFromDecoratedStub(
               std::move(used_sink));
         }
         return pubsub_internal::BatchingPublisherConnection::Create(
-            topic, opts, key, std::move(used_sink), cq,
-            std::move(message_batch));
+            topic, opts, key, std::move(used_sink), cq);
       };
       return pubsub_internal::OrderingKeyPublisherConnection::Create(
           std::move(factory));
     }
     return pubsub_internal::RejectsWithOrderingKey::Create(
         pubsub_internal::BatchingPublisherConnection::Create(
-            topic, opts, {}, std::move(sink), std::move(cq),
-            std::move(message_batch)));
+            topic, opts, {}, std::move(sink), std::move(cq)));
   };
   auto tracing_enabled = google::cloud::internal::TracingEnabled(opts);
   auto connection = make_connection();
@@ -129,9 +123,8 @@ std::shared_ptr<PublisherConnection> MakePublisherConnection(Topic topic,
   auto background = internal::MakeBackgroundThreadsFactory(opts)();
   auto stub =
       pubsub_internal::MakeRoundRobinPublisherStub(background->cq(), opts);
-  return ConnectionFromDecoratedStub(
-      std::move(topic), std::move(opts), std::move(background), std::move(stub),
-      std::make_shared<pubsub_internal::NoOpMessageBatch>());
+  return ConnectionFromDecoratedStub(std::move(topic), std::move(opts),
+                                     std::move(background), std::move(stub));
 }
 
 std::shared_ptr<PublisherConnection> MakePublisherConnection(
@@ -154,14 +147,13 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
 std::shared_ptr<pubsub::PublisherConnection> MakeTestPublisherConnection(
     pubsub::Topic topic, Options opts,
-    std::vector<std::shared_ptr<PublisherStub>> stubs,
-    std::shared_ptr<MessageBatch> message_batch) {
+    std::vector<std::shared_ptr<PublisherStub>> stubs) {
   auto background = internal::MakeBackgroundThreadsFactory(opts)();
   auto stub = pubsub_internal::MakeTestPublisherStub(background->cq(), opts,
                                                      std::move(stubs));
-  return pubsub::ConnectionFromDecoratedStub(
-      std::move(topic), std::move(opts), std::move(background), std::move(stub),
-      std::move(message_batch));
+  return pubsub::ConnectionFromDecoratedStub(std::move(topic), std::move(opts),
+                                             std::move(background),
+                                             std::move(stub));
 }
 
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
