@@ -88,68 +88,6 @@ TEST(PubSubSamplesCommon, SubscriberCommand) {
   EXPECT_EQ(1, call_count);
 }
 
-TEST(PubSubSamplesCommon, TopicAdminCommand) {
-  using ::google::cloud::testing_util::Usage;
-
-  // Pretend we are using the emulator to avoid loading the default
-  // credentials from $HOME, which do not exist when running with Bazel.
-  google::cloud::testing_util::ScopedEnvironment emulator(
-      "PUBSUB_EMULATOR_HOST", "localhost:8085");
-  int call_count = 0;
-  auto command = [&call_count](pubsub::TopicAdminClient const&,
-                               std::vector<std::string> const& argv) {
-    ++call_count;
-    ASSERT_EQ(2, argv.size());
-    EXPECT_EQ("a", argv[0]);
-    EXPECT_EQ("b", argv[1]);
-  };
-  auto const actual =
-      CreateTopicAdminCommand("command-name", {"foo", "bar"}, command);
-  EXPECT_EQ("command-name", actual.first);
-  EXPECT_THROW(
-      try { actual.second({}); } catch (Usage const& ex) {
-        EXPECT_THAT(ex.what(), HasSubstr("command-name"));
-        EXPECT_THAT(ex.what(), HasSubstr("foo"));
-        EXPECT_THAT(ex.what(), HasSubstr("bar"));
-        throw;
-      },
-      Usage);
-
-  ASSERT_NO_FATAL_FAILURE(actual.second({"a", "b"}));
-  EXPECT_EQ(1, call_count);
-}
-
-TEST(PubSubSamplesCommon, SubscriptionAdminCommand) {
-  using ::google::cloud::testing_util::Usage;
-
-  // Pretend we are using the emulator to avoid loading the default
-  // credentials from $HOME, which do not exist when running with Bazel.
-  google::cloud::testing_util::ScopedEnvironment emulator(
-      "PUBSUB_EMULATOR_HOST", "localhost:8085");
-  int call_count = 0;
-  auto command = [&call_count](pubsub::SubscriptionAdminClient const&,
-                               std::vector<std::string> const& argv) {
-    ++call_count;
-    ASSERT_EQ(2, argv.size());
-    EXPECT_EQ("a", argv[0]);
-    EXPECT_EQ("b", argv[1]);
-  };
-  auto const actual =
-      CreateSubscriptionAdminCommand("command-name", {"foo", "bar"}, command);
-  EXPECT_EQ("command-name", actual.first);
-  EXPECT_THROW(
-      try { actual.second({}); } catch (Usage const& ex) {
-        EXPECT_THAT(ex.what(), HasSubstr("command-name"));
-        EXPECT_THAT(ex.what(), HasSubstr("foo"));
-        EXPECT_THAT(ex.what(), HasSubstr("bar"));
-        throw;
-      },
-      Usage);
-
-  ASSERT_NO_FATAL_FAILURE(actual.second({"a", "b"}));
-  EXPECT_EQ(1, call_count);
-}
-
 TEST(PubSubSamplesCommon, SchemaServiceCommand) {
   using ::google::cloud::testing_util::Usage;
 
