@@ -51,6 +51,16 @@ TEST(SubscriptionBuilder, MakeOidcTokenWithAudience) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
+TEST(SubscriptionBuilder, MakeNoWrapper) {
+  auto const actual = PushConfigBuilder::MakeNoWrapper(true);
+  google::pubsub::v1::PushConfig::NoWrapper expected;
+  std::string const text = R"pb(
+    write_metadata: true
+  )pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 TEST(SubscriptionBuilder, Empty) {
   auto const actual = PushConfigBuilder().BuildModifyPushConfig(
       Subscription("test-project", "test-subscription"));
@@ -301,6 +311,79 @@ TEST(SubscriptionBuilder, SetPushConfigSetAuthentication) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
+TEST(SubscriptionBuilder, CreateWithBigQueryFull) {
+  auto const actual =
+      SubscriptionBuilder{}
+          .set_bigquery_config(
+              BigQueryConfigBuilder()
+                  .set_table("test-bq-project", "test-bq-data", "test-bq-table")
+                  .set_write_metadata(true)
+                  .set_use_topic_schema(true)
+                  .set_drop_unknown_fields(true))
+          .BuildCreateRequest(
+              Topic("test-project", "test-topic"),
+              Subscription("test-project", "test-subscription"));
+  google::pubsub::v1::Subscription expected;
+  std::string const text = R"pb(
+    topic: "projects/test-project/topics/test-topic"
+    name: "projects/test-project/subscriptions/test-subscription"
+    bigquery_config {
+      table: "test-bq-project:test-bq-data.test-bq-table"
+      write_metadata: true
+      use_topic_schema: true
+      drop_unknown_fields: true
+    })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(SubscriptionBuilder, UpdateWithBigQueryFull) {
+  auto const actual =
+      SubscriptionBuilder{}
+          .set_bigquery_config(
+              BigQueryConfigBuilder()
+                  .set_table("test-bq-project", "test-bq-data", "test-bq-table")
+                  .set_write_metadata(true)
+                  .set_use_topic_schema(true)
+                  .set_drop_unknown_fields(true))
+          .BuildUpdateRequest(
+              Subscription("test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+      bigquery_config {
+        table: "test-bq-project:test-bq-data.test-bq-table"
+        write_metadata: true
+        use_topic_schema: true
+        drop_unknown_fields: true
+      }
+    }
+    update_mask {
+      paths: "bigquery_config.drop_unknown_fields"
+      paths: "bigquery_config.table"
+      paths: "bigquery_config.use_topic_schema"
+      paths: "bigquery_config.write_metadata"
+    })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(SubscriptionBuilder, UpdateWithBigQueryEmpty) {
+  auto const actual = SubscriptionBuilder{}
+                          .set_bigquery_config(BigQueryConfigBuilder())
+                          .BuildUpdateRequest(Subscription(
+                              "test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+    }
+    update_mask { paths: "bigquery_config" })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 TEST(SubscriptionBuilder, SetAckDeadline) {
   auto const actual = SubscriptionBuilder{}
                           .set_ack_deadline(std::chrono::seconds(600))
@@ -317,6 +400,91 @@ TEST(SubscriptionBuilder, SetAckDeadline) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
+TEST(SubscriptionBuilder, CreateWithCloudStorageFull) {
+  auto const actual =
+      SubscriptionBuilder{}
+          .set_cloud_storage_config(
+              CloudStorageConfigBuilder()
+                  .set_bucket("test-bucket")
+                  .set_filename_prefix("test-filename-prefix")
+                  .set_filename_suffix("test-filename-suffix")
+                  .set_avro_config(
+                      CloudStorageConfigBuilder::MakeAvroConfig(true))
+                  .set_max_duration(std::chrono::minutes(1))
+                  .set_max_bytes(2000))
+          .BuildCreateRequest(
+              Topic("test-project", "test-topic"),
+              Subscription("test-project", "test-subscription"));
+  google::pubsub::v1::Subscription expected;
+  std::string const text = R"pb(
+    topic: "projects/test-project/topics/test-topic"
+    name: "projects/test-project/subscriptions/test-subscription"
+    cloud_storage_config {
+      bucket: "test-bucket"
+      filename_prefix: "test-filename-prefix"
+      filename_suffix: "test-filename-suffix"
+      avro_config { write_metadata: true }
+      max_duration { seconds: 60 }
+      max_bytes: 2000
+    })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(SubscriptionBuilder, UpdateWithCloudStorageFull) {
+  auto const actual =
+      SubscriptionBuilder{}
+          .set_cloud_storage_config(
+              CloudStorageConfigBuilder()
+                  .set_bucket("test-bucket")
+                  .set_filename_prefix("test-filename-prefix")
+                  .set_filename_suffix("test-filename-suffix")
+                  .set_avro_config(
+                      CloudStorageConfigBuilder::MakeAvroConfig(true))
+                  .set_max_duration(std::chrono::minutes(1))
+                  .set_max_bytes(2000))
+          .BuildUpdateRequest(
+              Subscription("test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+      cloud_storage_config {
+        bucket: "test-bucket"
+        filename_prefix: "test-filename-prefix"
+        filename_suffix: "test-filename-suffix"
+        avro_config { write_metadata: true }
+        max_duration { seconds: 60 }
+        max_bytes: 2000
+      }
+    }
+    update_mask {
+      paths: "cloud_storage_config.avro_config"
+      paths: "cloud_storage_config.bucket"
+      paths: "cloud_storage_config.filename_prefix"
+      paths: "cloud_storage_config.filename_suffix"
+      paths: "cloud_storage_config.max_bytes"
+      paths: "cloud_storage_config.max_duration"
+    })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(SubscriptionBuilder, UpdateWithCloudStorageEmpty) {
+  auto const actual = SubscriptionBuilder{}
+                          .set_cloud_storage_config(CloudStorageConfigBuilder())
+                          .BuildUpdateRequest(Subscription(
+                              "test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+    }
+    update_mask { paths: "cloud_storage_config" })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 TEST(SubscriptionBuilder, SetRetainAckedMessages) {
   auto const actual =
       SubscriptionBuilder{}.set_retain_acked_messages(true).BuildUpdateRequest(
@@ -493,6 +661,43 @@ TEST(SubscriptionBuilder, SetDeadLetterPolicy) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
+TEST(SubscriptionBuilder, SetRetryPolicy) {
+  auto const actual =
+      SubscriptionBuilder{}
+          .set_retry_policy(SubscriptionBuilder::MakeRetryPolicy(
+              std::chrono::seconds(10), std::chrono::seconds(450)))
+          .BuildUpdateRequest(
+              Subscription("test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+      retry_policy {
+        minimum_backoff { seconds: 10 }
+        maximum_backoff { seconds: 450 }
+      }
+    }
+    update_mask { paths: "retry_policy" })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(SubscriptionBuilder, EnableExactlyOnceDelivery) {
+  auto const actual = SubscriptionBuilder{}
+                          .enable_exactly_once_delivery(true)
+                          .BuildUpdateRequest(Subscription(
+                              "test-project", "test-subscription"));
+  google::pubsub::v1::UpdateSubscriptionRequest expected;
+  std::string const text = R"pb(
+    subscription {
+      name: "projects/test-project/subscriptions/test-subscription"
+      enable_exactly_once_delivery: true
+    }
+    update_mask { paths: "enable_exactly_once_delivery" })pb";
+  ASSERT_TRUE(TextFormat::ParseFromString(text, &expected));
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 template <typename Duration>
 void CheckMakeExpirationPolicy(Duration d,
                                std::string const& expected_as_text) {
