@@ -32,37 +32,39 @@ class MockSubscriberStub : public pubsub_internal::SubscriberStub {
   ~MockSubscriberStub() override = default;
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Subscription>, CreateSubscription,
-              (grpc::ClientContext&, google::pubsub::v1::Subscription const&),
+              (grpc::ClientContext&, Options const&,
+               google::pubsub::v1::Subscription const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Subscription>, GetSubscription,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::GetSubscriptionRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Subscription>, UpdateSubscription,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::UpdateSubscriptionRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::ListSubscriptionsResponse>,
               ListSubscriptions,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::ListSubscriptionsRequest const&),
               (override));
 
   MOCK_METHOD(Status, DeleteSubscription,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::DeleteSubscriptionRequest const& request),
               (override));
 
   MOCK_METHOD(Status, ModifyPushConfig,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::ModifyPushConfigRequest const& request),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::PullResponse>, Pull,
-              (grpc::ClientContext&, google::pubsub::v1::PullRequest const&),
+              (grpc::ClientContext&, Options const&,
+               google::pubsub::v1::PullRequest const&),
               (override));
 
   using StreamingPullStream = google::cloud::AsyncStreamingReadWriteRpc<
@@ -71,49 +73,69 @@ class MockSubscriberStub : public pubsub_internal::SubscriberStub {
 
   MOCK_METHOD(std::unique_ptr<StreamingPullStream>, AsyncStreamingPull,
               (google::cloud::CompletionQueue const&,
-               std::shared_ptr<grpc::ClientContext>),
+               std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions),
               (override));
 
   MOCK_METHOD(future<Status>, AsyncAcknowledge,
               (google::cloud::CompletionQueue&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::pubsub::v1::AcknowledgeRequest const&),
               (override));
 
   MOCK_METHOD(future<Status>, AsyncModifyAckDeadline,
               (google::cloud::CompletionQueue&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::pubsub::v1::ModifyAckDeadlineRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Snapshot>, CreateSnapshot,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::CreateSnapshotRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Snapshot>, GetSnapshot,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::GetSnapshotRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::ListSnapshotsResponse>,
               ListSnapshots,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::ListSnapshotsRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::Snapshot>, UpdateSnapshot,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::UpdateSnapshotRequest const&),
               (override));
 
   MOCK_METHOD(Status, DeleteSnapshot,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::pubsub::v1::DeleteSnapshotRequest const&),
               (override));
 
   MOCK_METHOD(StatusOr<google::pubsub::v1::SeekResponse>, Seek,
-              (grpc::ClientContext&, google::pubsub::v1::SeekRequest const&),
+              (grpc::ClientContext&, Options const&,
+               google::pubsub::v1::SeekRequest const&),
+              (override));
+
+  MOCK_METHOD(StatusOr<google::iam::v1::Policy>, SetIamPolicy,
+              (grpc::ClientContext&, Options const&,
+               google::iam::v1::SetIamPolicyRequest const&),
+              (override));
+
+  MOCK_METHOD(StatusOr<google::iam::v1::Policy>, GetIamPolicy,
+              (grpc::ClientContext&, Options const&,
+               google::iam::v1::GetIamPolicyRequest const&),
+              (override));
+
+  MOCK_METHOD(StatusOr<google::iam::v1::TestIamPermissionsResponse>,
+              TestIamPermissions,
+              (grpc::ClientContext&, Options const&,
+               google::iam::v1::TestIamPermissionsRequest const&),
               (override));
 };
 
