@@ -32,6 +32,9 @@ Options MakeTestOptions(Options opts) {
         std::make_shared<pubsub::ExponentialBackoffPolicy>(
             std::chrono::microseconds(1), std::chrono::microseconds(1), 2.0));
   }
+  if (!opts.has<pubsub::MaxOtelLinkCountOption>()) {
+    opts.set<pubsub::MaxOtelLinkCountOption>(128);
+  }
   return opts;
 }
 
