@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// TODO(#12987): Remove this file after the deprecation period expires
+#include "google/cloud/internal/disable_deprecation_warnings.inc"
 #include "google/cloud/pubsub/topic_admin_client.h"
 #include "google/cloud/pubsub/mocks/mock_topic_admin_connection.h"
 #include "google/cloud/testing_util/status_matchers.h"
