@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/pubsublite/internal/cloud_zone.h"
+#include "google/cloud/internal/make_status.h"
 
 namespace google {
 namespace cloud {
@@ -22,7 +23,8 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 StatusOr<CloudZone> MakeCloudZone(std::string const& zone) {
   std::vector<std::string> splits = absl::StrSplit(zone, '-');
   if (splits.size() != 3 || splits[2].length() != 1) {
-    return Status{StatusCode::kInvalidArgument, "Invalid zone name"};
+    return internal::InvalidArgumentError("Invalid zone name",
+                                          GCP_ERROR_INFO());
   }
   return CloudZone{CloudRegion{absl::StrCat(splits[0], "-", splits[1])},
                    splits[2][0]};
