@@ -18,6 +18,7 @@
 #include "google/cloud/pubsublite/testing/mock_publisher.h"
 #include "google/cloud/pubsublite/testing/mock_routing_policy.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
+#include "google/cloud/testing_util/status_matchers.h"
 #include "absl/memory/memory.h"
 #include <limits>
 
@@ -29,10 +30,11 @@ namespace {
 
 using ::google::cloud::pubsublite::MessageMetadata;
 using ::google::cloud::pubsublite_mocks::MockAdminServiceConnection;
-
 using ::google::cloud::testing_util::IsProtoEqual;
+using ::google::cloud::testing_util::StatusIs;
 using ::testing::_;
 using ::testing::ByMove;
+using ::testing::HasSubstr;
 using ::testing::InSequence;
 using ::testing::Mock;
 using ::testing::MockFunction;
@@ -126,9 +128,9 @@ TEST_F(MultipartitionPublisherNoneInitializedTest, PublishAfterShutdown) {
 
   shutdown.get();
 
-  EXPECT_EQ(multipartition_publisher_->Publish(PubSubMessage{}).get().status(),
-            (Status{StatusCode::kAborted, "`Shutdown` called"}));
-
+  EXPECT_THAT(
+      multipartition_publisher_->Publish(PubSubMessage{}).get().status(),
+      StatusIs(StatusCode::kAborted, "`Shutdown` called"));
   EXPECT_EQ(start.get(), Status());
 }
 
@@ -142,10 +144,10 @@ TEST_F(MultipartitionPublisherNoneInitializedTest,
           Return(ByMove(ReadyTopicPartitionsFuture(kOutOfBoundsPartition))));
   auto start = multipartition_publisher_->Start();
 
-  EXPECT_EQ(start.get(), Status(StatusCode::kInternal,
-                                "Returned partition count is too big: " +
-                                    std::to_string(kOutOfBoundsPartition)));
-
+  EXPECT_THAT(start.get(),
+              StatusIs(StatusCode::kInternal,
+                       HasSubstr("Returned partition count is too big: " +
+                                 std::to_string(kOutOfBoundsPartition))));
   EXPECT_CALL(alarm_token_, Destroy);
   multipartition_publisher_->Shutdown().get();
 }
@@ -165,16 +167,18 @@ TEST_F(MultipartitionPublisherNoneInitializedTest,
   future<StatusOr<MessageMetadata>> message0 =
       multipartition_publisher_->Publish(m0);
 
-  Status expected_status =
-      Status(StatusCode::kInternal, "Returned partition count is too big: " +
-                                        std::to_string(kOutOfBoundsPartition));
-
-  EXPECT_EQ(start.get(), expected_status);
+  EXPECT_THAT(start.get(),
+              StatusIs(StatusCode::kInternal,
+                       HasSubstr("Returned partition count is too big: " +
+                                 std::to_string(kOutOfBoundsPartition))));
 
   EXPECT_CALL(alarm_token_, Destroy);
   multipartition_publisher_->Shutdown().get();
 
-  EXPECT_EQ(message0.get().status(), expected_status);
+  EXPECT_THAT(message0.get().status(),
+              StatusIs(StatusCode::kInternal,
+                       HasSubstr("Returned partition count is too big: " +
+                                 std::to_string(kOutOfBoundsPartition))));
 }
 
 TEST_F(MultipartitionPublisherNoneInitializedTest,
@@ -204,10 +208,10 @@ TEST_F(MultipartitionPublisherNoneInitializedTest,
   num_partitions.set_value(
       ExamplePartitionsResponse(1));  // shouldn't do anything
 
-  EXPECT_EQ(message0.get().status(),
-            (Status{StatusCode::kAborted, "`Shutdown` called"}));
-  EXPECT_EQ(message1.get().status(),
-            (Status{StatusCode::kAborted, "`Shutdown` called"}));
+  EXPECT_THAT(message0.get().status(),
+              StatusIs(StatusCode::kAborted, HasSubstr("`Shutdown` called")));
+  EXPECT_THAT(message1.get().status(),
+              StatusIs(StatusCode::kAborted, HasSubstr("`Shutdown` called")));
 
   EXPECT_EQ(start.get(), Status());
 }
