@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/resourcemanager/projects_client.h"
+//! [all]
+#include "google/cloud/resourcemanager/v3/projects_client.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,17 +22,18 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace resourcemanager = ::google::cloud::resourcemanager;
+  namespace resourcemanager = ::google::cloud::resourcemanager_v3;
   auto client = resourcemanager::ProjectsClient(
       resourcemanager::MakeProjectsConnection());
 
   for (auto p : client.ListProjects("folders/" + std::string(argv[1]))) {
-    if (!p) throw std::runtime_error(p.status().message());
+    if (!p) throw std::move(p).status();
     std::cout << p->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
