@@ -12,27 +12,29 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/secretmanager/secret_manager_client.h"
+//! [all]
+#include "google/cloud/secretmanager/v1/secret_manager_client.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
-    std::cerr << "Usage: " << argv[0] << " project-id \n";
+    std::cerr << "Usage: " << argv[0] << " project-id\n";
     return 1;
   }
 
-  namespace secretmanager = ::google::cloud::secretmanager;
+  namespace secretmanager = ::google::cloud::secretmanager_v1;
   auto client = secretmanager::SecretManagerServiceClient(
       secretmanager::MakeSecretManagerServiceConnection());
 
   auto const parent = std::string("projects/") + argv[1];
-  for (auto const& secret : client.ListSecrets(parent)) {
-    std::cout << secret.value().DebugString() << "\n";
+  for (auto secret : client.ListSecrets(parent)) {
+    if (!secret) throw std::move(secret).status();
+    std::cout << secret->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
