@@ -12,10 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/serviceusage/service_usage_client.h"
+//! [all]
+#include "google/cloud/serviceusage/v1/service_usage_client.h"
 #include "google/cloud/project.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -23,7 +23,7 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace serviceusage = ::google::cloud::serviceusage;
+  namespace serviceusage = ::google::cloud::serviceusage_v1;
   auto client = serviceusage::ServiceUsageClient(
       serviceusage::MakeServiceUsageConnection());
 
@@ -32,12 +32,13 @@ int main(int argc, char* argv[]) try {
   request.set_parent(project.FullName());
   request.set_filter("state:ENABLED");
   for (auto s : client.ListServices(request)) {
-    if (!s) throw std::runtime_error(s.status().message());
+    if (!s) throw std::move(s).status();
     std::cout << s->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
