@@ -288,8 +288,8 @@ TEST_F(BackupExtraIntegrationTest, CreateBackupWithVersionTime) {
 
 /// @test Verify creating a backup with an expired version_time fails.
 TEST_F(BackupExtraIntegrationTest, CreateBackupWithExpiredVersionTime) {
-  auto instance_id =
-      spanner_testing::PickRandomInstance(generator_, ProjectId());
+  auto instance_id = spanner_testing::PickRandomInstance(
+      generator_, ProjectId(), "NOT labels.edition:enterprise");
   ASSERT_THAT(instance_id, IsOk());
   Instance in(ProjectId(), *instance_id);
   Database db(in, spanner_testing::RandomDatabaseName(generator_));
@@ -333,8 +333,8 @@ TEST_F(BackupExtraIntegrationTest, CreateBackupWithExpiredVersionTime) {
 
 /// @test Verify creating a backup with a future version_time fails.
 TEST_F(BackupExtraIntegrationTest, CreateBackupWithFutureVersionTime) {
-  auto instance_id =
-      spanner_testing::PickRandomInstance(generator_, ProjectId());
+  auto instance_id = spanner_testing::PickRandomInstance(
+      generator_, ProjectId(), "NOT labels.edition:enterprise");
   ASSERT_THAT(instance_id, IsOk());
   Instance in(ProjectId(), *instance_id);
   Database db(in, spanner_testing::RandomDatabaseName(generator_));
@@ -378,7 +378,7 @@ TEST_F(BackupExtraIntegrationTest, CreateBackupWithFutureVersionTime) {
 
 /// @test Tests backup/restore with Customer Managed Encryption Key
 TEST_F(BackupExtraIntegrationTest, BackupRestoreWithCMEK) {
-  if (!RunSlowBackupTests() || Emulator()) GTEST_SKIP();
+  if (!RunSlowBackupTests()) GTEST_SKIP();
 
   auto instance_id = spanner_testing::PickRandomInstance(
       generator_, ProjectId(),
@@ -389,6 +389,10 @@ TEST_F(BackupExtraIntegrationTest, BackupRestoreWithCMEK) {
   Instance in(ProjectId(), *instance_id);
 
   auto location = spanner_testing::InstanceLocation(in);
+  if (Emulator()) {
+    EXPECT_THAT(location, StatusIs(StatusCode::kUnavailable));
+    return;
+  }
   ASSERT_STATUS_OK(location);
   KmsKeyName encryption_key(in.project_id(), *location, kKeyRing, kKeyName);
 
