@@ -167,8 +167,10 @@ TEST_F(DatabaseAdminClientTest, DatabaseBasicCRUD) {
   EXPECT_EQ(database->name(), list_db->name());
   EXPECT_EQ(database->database_dialect(), list_db->database_dialect());
 
-  if (!emulator_) {
-    auto current_policy = client_.GetIamPolicy(database_.FullName());
+  auto current_policy = client_.GetIamPolicy(database_.FullName());
+  if (emulator_) {
+    EXPECT_THAT(current_policy, StatusIs(StatusCode::kUnimplemented));
+  } else {
     ASSERT_STATUS_OK(current_policy);
     EXPECT_EQ(0, current_policy->bindings_size());
 
@@ -227,10 +229,8 @@ TEST_F(DatabaseAdminClientTest, DatabaseBasicCRUD) {
   EXPECT_EQ(0, get_ddl_result->statements_size());
 
   std::vector<std::string> statements;
-  if (!emulator_) {  // version_retention_period
-    statements.push_back("ALTER DATABASE `" + database_.database_id() +
-                         "` SET OPTIONS (version_retention_period='7d')");
-  }
+  statements.push_back("ALTER DATABASE `" + database_.database_id() +
+                       "` SET OPTIONS (version_retention_period='7d')");
   statements.emplace_back(R"""(
         CREATE TABLE Singers (
           SingerId      INT64 NOT NULL,
@@ -242,17 +242,22 @@ TEST_F(DatabaseAdminClientTest, DatabaseBasicCRUD) {
       )""");
   auto metadata =
       client_.UpdateDatabaseDdl(database_.FullName(), statements).get();
-  ASSERT_STATUS_OK(metadata);
-  EXPECT_THAT(metadata->database(), EndsWith(database_.database_id()));
-  EXPECT_EQ(statements.size(), metadata->statements_size());
-  EXPECT_EQ(statements.size(), metadata->commit_timestamps_size());
-  if (metadata->statements_size() >= 1) {
-    EXPECT_THAT(metadata->statements(), Contains(HasSubstr("CREATE TABLE")));
+  if (emulator_) {  // version_retention_period
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kInvalidArgument));
+  } else {
+    ASSERT_STATUS_OK(metadata);
+    EXPECT_THAT(metadata->database(), EndsWith(database_.database_id()));
+    EXPECT_EQ(statements.size(), metadata->statements_size());
+    EXPECT_EQ(statements.size(), metadata->commit_timestamps_size());
+    if (metadata->statements_size() >= 1) {
+      EXPECT_THAT(metadata->statements(), Contains(HasSubstr("CREATE TABLE")));
+    }
+    if (metadata->statements_size() >= 2) {
+      EXPECT_THAT(metadata->statements(),
+                  Contains(HasSubstr("ALTER DATABASE")));
+    }
+    EXPECT_FALSE(metadata->throttled());
   }
-  if (metadata->statements_size() >= 2) {
-    EXPECT_THAT(metadata->statements(), Contains(HasSubstr("ALTER DATABASE")));
-  }
-  EXPECT_FALSE(metadata->throttled());
 
   // Verify that a new role can be created and returned.
   statements.clear();
@@ -261,7 +266,7 @@ TEST_F(DatabaseAdminClientTest, DatabaseBasicCRUD) {
       )""");
   metadata = client_.UpdateDatabaseDdl(database_.FullName(), statements).get();
   if (emulator_) {
-    EXPECT_THAT(metadata, StatusIs(StatusCode::kInvalidArgument));
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kInternal));
   } else {
     EXPECT_THAT(metadata, IsOk());
     std::vector<std::string> roles;
@@ -328,11 +333,7 @@ TEST_F(DatabaseAdminClientTest, VersionRetentionPeriodCreate) {
   ASSERT_THAT(list_db, IsOk());
   EXPECT_EQ(database->name(), list_db->name());
   EXPECT_EQ(database->database_dialect(), list_db->database_dialect());
-  if (emulator_) {  // version_retention_period
-    EXPECT_EQ("", list_db->version_retention_period());
-  } else {
-    EXPECT_EQ("7d", list_db->version_retention_period());
-  }
+  EXPECT_EQ("7d", list_db->version_retention_period());
 
   auto drop = client_.DropDatabase(database_.FullName());
   EXPECT_THAT(drop, IsOk());
@@ -417,6 +418,8 @@ TEST_F(DatabaseAdminClientTest, VersionRetentionPeriodUpdate) {
   auto ddl = client_.GetDatabaseDdl(database_.FullName());
   ASSERT_THAT(ddl, IsOk());
   if (emulator_) {  // version_retention_period
+    EXPECT_THAT(ddl->statements(),
+                Not(Contains(ContainsRegex("version_retention_period *= *"))));
   } else {
     EXPECT_THAT(ddl->statements(),
                 Contains(ContainsRegex("version_retention_period *= *'7d'")));
@@ -475,7 +478,6 @@ TEST_F(DatabaseAdminClientTest, VersionRetentionPeriodUpdateFailure) {
 
 /// @test Verify we can create a database with an encryption key.
 TEST_F(DatabaseAdminClientTest, CreateWithEncryptionKey) {
-  if (emulator_) GTEST_SKIP() << "emulator does not support CMEK";
   KmsKeyName encryption_key(instance_.project_id(), location_, kKeyRing,
                             kKeyName);
   google::spanner::admin::database::v1::CreateDatabaseRequest creq;
@@ -486,35 +488,39 @@ TEST_F(DatabaseAdminClientTest, CreateWithEncryptionKey) {
   auto database = client_.CreateDatabase(creq).get();
   ASSERT_STATUS_OK(database);
   EXPECT_EQ(database->name(), database_.FullName());
-  EXPECT_TRUE(database->has_encryption_config());
-  if (database->has_encryption_config()) {
-    EXPECT_EQ(database->encryption_config().kms_key_name(),
-              encryption_key.FullName());
-  }
+  if (emulator_) {
+    EXPECT_FALSE(database->has_encryption_config());
+  } else {
+    EXPECT_TRUE(database->has_encryption_config());
+    if (database->has_encryption_config()) {
+      EXPECT_EQ(database->encryption_config().kms_key_name(),
+                encryption_key.FullName());
+    }
 
-  auto get_result = client_.GetDatabase(database_.FullName());
-  ASSERT_STATUS_OK(get_result);
-  EXPECT_EQ(database->name(), get_result->name());
-  EXPECT_TRUE(get_result->has_encryption_config());
-  if (get_result->has_encryption_config()) {
-    EXPECT_EQ(get_result->encryption_config().kms_key_name(),
-              encryption_key.FullName());
-  }
+    auto get_result = client_.GetDatabase(database_.FullName());
+    ASSERT_STATUS_OK(get_result);
+    EXPECT_EQ(database->name(), get_result->name());
+    EXPECT_TRUE(get_result->has_encryption_config());
+    if (get_result->has_encryption_config()) {
+      EXPECT_EQ(get_result->encryption_config().kms_key_name(),
+                encryption_key.FullName());
+    }
 
-  // Verify that encryption config is returned via ListDatabases().
-  auto list_db = [&] {
-    for (auto const& db :
-         client_.ListDatabases(database_.instance().FullName())) {
-      if (db && db->name() == database_.FullName()) return db;
+    // Verify that encryption config is returned via ListDatabases().
+    auto list_db = [&] {
+      for (auto const& db :
+           client_.ListDatabases(database_.instance().FullName())) {
+        if (db && db->name() == database_.FullName()) return db;
+      }
+      return StatusOr<google::spanner::admin::database::v1::Database>{
+          Status{StatusCode::kNotFound, "disappeared"}};
+    }();
+    ASSERT_THAT(list_db, IsOk());
+    EXPECT_TRUE(list_db->has_encryption_config());
+    if (list_db->has_encryption_config()) {
+      EXPECT_EQ(list_db->encryption_config().kms_key_name(),
+                encryption_key.FullName());
     }
-    return StatusOr<google::spanner::admin::database::v1::Database>{
-        Status{StatusCode::kNotFound, "disappeared"}};
-  }();
-  ASSERT_THAT(list_db, IsOk());
-  EXPECT_TRUE(list_db->has_encryption_config());
-  if (list_db->has_encryption_config()) {
-    EXPECT_EQ(list_db->encryption_config().kms_key_name(),
-              encryption_key.FullName());
   }
 
   EXPECT_STATUS_OK(client_.DropDatabase(database_.FullName()));
@@ -523,7 +529,6 @@ TEST_F(DatabaseAdminClientTest, CreateWithEncryptionKey) {
 /// @test Verify creating a database fails if a nonexistent encryption key is
 /// supplied.
 TEST_F(DatabaseAdminClientTest, CreateWithNonexistentEncryptionKey) {
-  if (emulator_) GTEST_SKIP() << "emulator does not support CMEK";
   KmsKeyName nonexistent_encryption_key(instance_.project_id(), location_,
                                         kKeyRing, "ceci-n-est-pas-une-cle");
   google::spanner::admin::database::v1::CreateDatabaseRequest creq;
@@ -533,8 +538,14 @@ TEST_F(DatabaseAdminClientTest, CreateWithNonexistentEncryptionKey) {
   creq.mutable_encryption_config()->set_kms_key_name(
       nonexistent_encryption_key.FullName());
   auto database = client_.CreateDatabase(creq).get();
-  EXPECT_THAT(database, StatusIs(StatusCode::kFailedPrecondition,
-                                 HasSubstr("KMS Key provided is not usable")));
+  if (emulator_) {
+    ASSERT_STATUS_OK(database);
+    EXPECT_STATUS_OK(client_.DropDatabase(database->name()));
+  } else {
+    EXPECT_THAT(database,
+                StatusIs(StatusCode::kFailedPrecondition,
+                         HasSubstr("KMS Key provided is not usable")));
+  }
 }
 
 /// @test Verify basic operations for PostgreSQL-type databases.
@@ -546,18 +557,17 @@ TEST_F(DatabaseAdminClientTest, DatabasePostgreSQLBasics) {
   creq.set_database_dialect(
       google::spanner::admin::database::v1::DatabaseDialect::POSTGRESQL);
   auto database = client_.CreateDatabase(creq).get();
-  if (emulator_) {
-    // This will let us know when the emulator starts supporting PostgreSQL
-    // syntax to quote identifiers.
-    EXPECT_THAT(database,
-                StatusIs(StatusCode::kInvalidArgument,
-                         HasSubstr("Error parsing Spanner DDL statement")));
-    GTEST_SKIP() << "emulator does not support PostgreSQL";
-  }
   ASSERT_STATUS_OK(database);
   EXPECT_THAT(database->name(), EndsWith(database_.database_id()));
-  EXPECT_EQ(database->database_dialect(),
-            google::spanner::admin::database::v1::DatabaseDialect::POSTGRESQL);
+  if (emulator_) {
+    EXPECT_EQ(database->database_dialect(),
+              google::spanner::admin::database::v1::DatabaseDialect::
+                  DATABASE_DIALECT_UNSPECIFIED);
+  } else {
+    EXPECT_EQ(
+        database->database_dialect(),
+        google::spanner::admin::database::v1::DatabaseDialect::POSTGRESQL);
+  }
 
   // Verify that GetDatabase() returns the correct dialect.
   auto get = client_.GetDatabase(database->name());
@@ -644,6 +654,36 @@ TEST_F(DatabaseAdminClientTest, DropDatabaseProtection) {
   EXPECT_STATUS_OK(drop_unprotected);
 }
 
+TEST_F(DatabaseAdminClientTest, LROStartAwait) {
+  EXPECT_FALSE(DatabaseExists()) << "Database " << database_
+                                 << " already exists, this is unexpected as "
+                                    "the database id is selected at random.";
+
+  // Start an LRO.
+  auto operation = client_.CreateDatabase(
+      NoAwaitTag{}, database_.instance().FullName(),
+      absl::StrCat("CREATE DATABASE `", database_.database_id(), "`"));
+  ASSERT_STATUS_OK(operation);
+
+  // Verify that an error is returned if there is a mismatch between the RPC
+  // that returned the operation and the RPC in which is it used.
+  auto fail = client_.UpdateDatabase(*operation).get();
+  EXPECT_THAT(fail, StatusIs(StatusCode::kInvalidArgument));
+
+  // Wait for the LRO to complete.
+  auto database = client_.CreateDatabase(*operation).get();
+  ASSERT_STATUS_OK(database);
+  EXPECT_THAT(database->name(), EndsWith(database_.database_id()));
+
+  // Verify the operation completed successfully.
+  EXPECT_TRUE(DatabaseExists()) << "Database " << database_;
+
+  // Clean up the test artifacts.
+  auto drop_status = client_.DropDatabase(database_.FullName());
+  EXPECT_STATUS_OK(drop_status);
+  EXPECT_FALSE(DatabaseExists()) << "Database " << database_;
+}
+
 }  // namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace spanner
