@@ -16,6 +16,7 @@
 #include "google/cloud/internal/build_info.h"
 #include "google/cloud/internal/compiler_info.h"
 #include "google/cloud/internal/getenv.h"
+#include "google/cloud/internal/make_status.h"
 #include "absl/strings/match.h"
 #include <functional>
 #include <sstream>
@@ -114,8 +115,8 @@ google::cloud::StatusOr<Config> ParseArgs(std::vector<std::string> args) {
   };
 
   auto invalid_argument = [](std::string msg) {
-    return google::cloud::Status(google::cloud::StatusCode::kInvalidArgument,
-                                 std::move(msg));
+    return google::cloud::internal::InvalidArgumentError(std::move(msg),
+                                                         GCP_ERROR_INFO());
   };
 
   for (auto i = std::next(args.begin()); i != args.end(); ++i) {
