@@ -21,11 +21,15 @@
 #include "google/cloud/spanner/transaction.h"
 #include "google/cloud/backoff_policy.h"
 #include "google/cloud/credentials.h"
+#include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/getenv.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/retry_loop.h"
 #include "google/cloud/log.h"
+#include "google/cloud/status.h"
 #include "absl/types/optional.h"
 #include <grpcpp/grpcpp.h>
+#include <chrono>
 #include <thread>
 
 namespace google {
@@ -245,7 +249,8 @@ StatusOr<CommitResult> Client::Commit(
       // Treat this like mutator() returned a bad Status.
       Status status = error.status();
       if (status.ok()) {
-        status = Status(StatusCode::kUnknown, "OK Status thrown from mutator");
+        status = internal::UnknownError("OK Status thrown from mutator",
+                                        GCP_ERROR_INFO());
       }
       mutations = status;
     } catch (...) {
@@ -293,7 +298,14 @@ StatusOr<CommitResult> Client::Commit(
       // so that we have a slightly better chance of avoiding another abort.
       txn = MakeReadWriteTransaction(txn, txn_opts);
     }
-    std::this_thread::sleep_for(backoff_policy->OnCompletion());
+    std::chrono::nanoseconds delay = backoff_policy->OnCompletion();
+    if (internal::CurrentOptions().get<EnableServerRetriesOption>()) {
+      if (auto retry_info = internal::GetRetryInfo(status)) {
+        // Heed the `RetryInfo` from the service.
+        delay = retry_info->retry_delay();
+      }
+    }
+    std::this_thread::sleep_for(delay);
   }
 }
 
