@@ -87,10 +87,6 @@ class ClientIntegrationTest : public spanner_testing::DatabaseIntegrationTest {
     spanner_testing::DatabaseIntegrationTest::TearDownTestSuite();
   }
 
-  static bool EmulatorUnimplemented(Status const& status) {
-    return UsingEmulator() && status.code() == StatusCode::kUnimplemented;
-  }
-
   static std::unique_ptr<Client> client_;
 };
 
@@ -107,7 +103,6 @@ class PgClientIntegrationTest
   }
 
   void SetUp() override {
-    if (UsingEmulator()) return;
     auto commit_result = client_->Commit(
         Mutations{MakeDeleteMutation("Singers", KeySet::All())},
         Options{}.set<GrpcCompressionAlgorithmOption>(GRPC_COMPRESS_DEFLATE));
@@ -479,7 +474,9 @@ TEST_F(ClientIntegrationTest, CommitAtLeastOnceBatched) {
   auto commit_results = client_->CommitAtLeastOnce(std::move(mutation_groups));
   std::map<std::size_t, StatusOr<Timestamp>> results;
   for (auto& commit_result : commit_results) {
-    if (results.empty() && EmulatorUnimplemented(commit_result.status())) {
+    if (UsingEmulator()) {
+      EXPECT_THAT(commit_result, StatusIs(StatusCode::kUnimplemented));
+      EXPECT_THAT(results, IsEmpty());
       GTEST_SKIP();
     }
     ASSERT_THAT(commit_result, IsOk());
@@ -814,6 +811,29 @@ TEST_F(ClientIntegrationTest, ExecuteQueryExactStalenessDuration) {
   });
 }
 
+/// @test Test that a directed read within a read-write transaction fails.
+TEST_F(ClientIntegrationTest, DirectedReadWithinReadWriteTransaction) {
+  auto& client = *client_;
+  auto commit =
+      client_->Commit([&client](Transaction const& txn) -> StatusOr<Mutations> {
+        auto rows =
+            client.Read(txn, "Singers", KeySet::All(),
+                        {"SingerId", "FirstName", "LastName"},
+                        Options{}.set<DirectedReadOption>(IncludeReplicas(
+                            {ReplicaSelection(ReplicaType::kReadOnly)},
+                            /*auto_failover_disabled=*/true)));
+        using RowType = std::tuple<std::int64_t, std::string, std::string>;
+        for (auto& row : StreamOf<RowType>(rows)) {
+          if (!row) return row.status();
+        }
+        return Mutations{};
+      });
+  EXPECT_THAT(commit, StatusIs(UsingEmulator() ? StatusCode::kFailedPrecondition
+                                               : StatusCode::kInvalidArgument,
+                               HasSubstr("Directed reads can only be performed "
+                                         "in a read-only transaction")));
+}
+
 StatusOr<std::vector<std::vector<Value>>> AddSingerDataToTable(Client client) {
   std::vector<std::vector<Value>> expected_rows;
   auto commit = client.Commit(
@@ -1084,10 +1104,8 @@ TEST_F(ClientIntegrationTest, AnalyzeSql) {
 
   // This returns a ExecutionPlan without executing the query.
   auto plan = client_->AnalyzeSql(std::move(txn), std::move(sql));
-  if (!EmulatorUnimplemented(plan.status())) {
-    ASSERT_STATUS_OK(plan);
-    EXPECT_GT(plan->plan_nodes_size(), 0);
-  }
+  ASSERT_STATUS_OK(plan);
+  EXPECT_GT(plan->plan_nodes_size(), 0);
 }
 
 TEST_F(ClientIntegrationTest, ProfileQuery) {
@@ -1105,7 +1123,9 @@ TEST_F(ClientIntegrationTest, ProfileQuery) {
   EXPECT_GT(stats->size(), 0);
 
   auto plan = rows.ExecutionPlan();
-  if (!UsingEmulator() || plan) {
+  if (UsingEmulator()) {
+    EXPECT_FALSE(plan);
+  } else {
     ASSERT_TRUE(plan);
     EXPECT_GT(plan->plan_nodes_size(), 0);
   }
@@ -1133,10 +1153,8 @@ TEST_F(ClientIntegrationTest, ProfileDml) {
   EXPECT_GT(stats->size(), 0);
 
   auto plan = profile_result.ExecutionPlan();
-  if (!UsingEmulator() || plan) {
-    ASSERT_TRUE(plan);
-    EXPECT_GT(plan->plan_nodes_size(), 0);
-  }
+  ASSERT_TRUE(plan);
+  EXPECT_GT(plan->plan_nodes_size(), 0);
 }
 
 /// @test Verify database_dialect is returned in information schema.
@@ -1148,11 +1166,7 @@ TEST_F(ClientIntegrationTest, DatabaseDialect) {
       )"""));
   using RowType = std::tuple<std::string>;
   for (auto& row : StreamOf<RowType>(rows)) {
-    if (UsingEmulator()) {
-      EXPECT_THAT(row, AnyOf(IsOk(), StatusIs(StatusCode::kInvalidArgument)));
-    } else {
-      EXPECT_THAT(row, IsOk());
-    }
+    EXPECT_THAT(row, IsOk());
     if (!row) break;
     EXPECT_EQ("GOOGLE_STANDARD_SQL", std::get<0>(*row));
   }
@@ -1167,11 +1181,7 @@ TEST_F(PgClientIntegrationTest, DatabaseDialect) {
       )"""));
   using RowType = std::tuple<std::string>;
   for (auto& row : StreamOf<RowType>(rows)) {
-    if (UsingEmulator()) {
-      EXPECT_THAT(row, AnyOf(IsOk(), StatusIs(StatusCode::kNotFound)));
-    } else {
-      EXPECT_THAT(row, IsOk());
-    }
+    EXPECT_THAT(row, IsOk());
     if (!row) break;
     EXPECT_EQ("POSTGRESQL", std::get<0>(*row));
   }
@@ -1179,7 +1189,6 @@ TEST_F(PgClientIntegrationTest, DatabaseDialect) {
 
 /// @test Verify use of database role to read data.
 TEST_F(ClientIntegrationTest, FineGrainedAccessControl) {
-  if (UsingEmulator()) GTEST_SKIP();
   ASSERT_NO_FATAL_FAILURE(InsertTwoSingers());
 
   spanner_admin::DatabaseAdminClient admin_client(
@@ -1192,6 +1201,10 @@ TEST_F(ClientIntegrationTest, FineGrainedAccessControl) {
   auto metadata =
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
+  if (UsingEmulator()) {
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kInternal));
+    GTEST_SKIP();
+  }
   ASSERT_STATUS_OK(metadata);
 
   // Connect to the database using the Reader role.
@@ -1218,7 +1231,6 @@ TEST_F(ClientIntegrationTest, FineGrainedAccessControl) {
 
 /// @test Verify use of database role to read data.
 TEST_F(PgClientIntegrationTest, FineGrainedAccessControl) {
-  if (UsingEmulator()) GTEST_SKIP();
   ASSERT_NO_FATAL_FAILURE(InsertTwoSingers());
 
   spanner_admin::DatabaseAdminClient admin_client(
@@ -1231,6 +1243,10 @@ TEST_F(PgClientIntegrationTest, FineGrainedAccessControl) {
   auto metadata =
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
+  if (UsingEmulator()) {
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kFailedPrecondition));
+    GTEST_SKIP();
+  }
   ASSERT_STATUS_OK(metadata);
 
   // Connect to the database using the Reader role.
@@ -1257,8 +1273,6 @@ TEST_F(PgClientIntegrationTest, FineGrainedAccessControl) {
 
 /// @test Verify "FOREIGN KEY" "ON DELETE CASCADE".
 TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
-  if (UsingEmulator()) GTEST_SKIP();
-
   spanner_admin::DatabaseAdminClient admin_client(
       spanner_admin::MakeDatabaseAdminConnection());
 
@@ -1315,10 +1329,8 @@ TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
       "ShoppingCarts", {"CartId", "CustomerId", "CustomerName"},  //
       2, 2, "FKCustomer")});
   EXPECT_THAT(bad_key, StatusIs(StatusCode::kFailedPrecondition,
-                                AllOf(HasSubstr("Foreign key constraint"),
-                                      HasSubstr("FKShoppingCartsCustomerId"),
-                                      HasSubstr("ShoppingCarts"),
-                                      HasSubstr("Customers(CustomerId)"))));
+                                AllOf(HasSubstr("FKShoppingCartsCustomerId"),
+                                      HasSubstr("ShoppingCarts"))));
 
   // Delete a row in the referenced table. All rows referencing that key
   // from the referencing table will also be deleted.
@@ -1341,10 +1353,8 @@ TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
                 MakeDeleteMutation("Customers", KeySet().AddKey(MakeKey(1)))});
   EXPECT_THAT(conflict_commit,
               StatusIs(StatusCode::kFailedPrecondition,
-                       AllOf(HasSubstr("Cannot write a value"),
-                             HasSubstr("Customers.CustomerId"),
-                             HasSubstr("and delete it"),
-                             HasSubstr("in the same transaction"))));
+                       AllOf(HasSubstr("Cannot write"), HasSubstr("and delete"),
+                             HasSubstr("same transaction"))));
 
   // Conflicting operation: reference foreign key in referencing table
   // and delete key from referenced table in the same mutation.
@@ -1362,12 +1372,17 @@ TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
                                    {"CartId", "CustomerId", "CustomerName"},  //
                                    1, 2, "FKCustomer2"),
                 MakeDeleteMutation("Customers", KeySet().AddKey(MakeKey(1)))});
-  EXPECT_THAT(reconflict_commit,
-              StatusIs(StatusCode::kFailedPrecondition,
-                       AllOf(HasSubstr("Cannot modify a row"),
-                             HasSubstr("ShoppingCarts"),
-                             HasSubstr("referential action is deleting it"),
-                             HasSubstr("in the same transaction"))));
+  if (UsingEmulator()) {
+    EXPECT_THAT(reconflict_commit, StatusIs(StatusCode::kAlreadyExists,
+                                            HasSubstr("ShoppingCarts")));
+  } else {
+    EXPECT_THAT(reconflict_commit,
+                StatusIs(StatusCode::kFailedPrecondition,
+                         AllOf(HasSubstr("Cannot modify a row"),
+                               HasSubstr("ShoppingCarts"),
+                               HasSubstr("referential action is deleting it"),
+                               HasSubstr("in the same transaction"))));
+  }
 
   // Query INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS and validate DELETE_RULE.
   auto delete_rules = client_->ExecuteQuery(SqlStatement(R"""(
@@ -1379,7 +1394,8 @@ TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
   for (auto& delete_rule : StreamOf<RowType>(delete_rules)) {
     EXPECT_THAT(delete_rule, IsOk());
     if (!delete_rule) break;
-    EXPECT_EQ("CASCADE", std::get<0>(*delete_rule));
+    EXPECT_EQ(UsingEmulator() ? "NO ACTION" : "CASCADE",
+              std::get<0>(*delete_rule));
   }
 
   // Drop ON DELETE CASCADE constraint.
@@ -1396,8 +1412,6 @@ TEST_F(ClientIntegrationTest, ForeignKeyDeleteCascade) {
 
 /// @test Verify "FOREIGN KEY" "ON DELETE CASCADE".
 TEST_F(PgClientIntegrationTest, ForeignKeyDeleteCascade) {
-  if (UsingEmulator()) GTEST_SKIP();
-
   spanner_admin::DatabaseAdminClient admin_client(
       spanner_admin::MakeDatabaseAdminConnection());
 
@@ -1454,10 +1468,8 @@ TEST_F(PgClientIntegrationTest, ForeignKeyDeleteCascade) {
       "ShoppingCarts", {"CartId", "CustomerId", "CustomerName"},  //
       2, 2, "FKCustomer")});
   EXPECT_THAT(bad_key, StatusIs(StatusCode::kFailedPrecondition,
-                                AllOf(HasSubstr("Foreign key constraint"),
-                                      HasSubstr("fkshoppingcartscustomerid"),
-                                      HasSubstr("shoppingcarts"),
-                                      HasSubstr("customers(customerid)"))));
+                                AllOf(HasSubstr("fkshoppingcartscustomerid"),
+                                      HasSubstr("shoppingcarts"))));
 
   // Delete a row in the referenced table. All rows referencing that key
   // from the referencing table will also be deleted.
@@ -1480,10 +1492,8 @@ TEST_F(PgClientIntegrationTest, ForeignKeyDeleteCascade) {
                 MakeDeleteMutation("Customers", KeySet().AddKey(MakeKey(1)))});
   EXPECT_THAT(conflict_commit,
               StatusIs(StatusCode::kFailedPrecondition,
-                       AllOf(HasSubstr("Cannot write a value"),
-                             HasSubstr("customers.customerid"),
-                             HasSubstr("and delete it"),
-                             HasSubstr("in the same transaction"))));
+                       AllOf(HasSubstr("Cannot write"), HasSubstr("and delete"),
+                             HasSubstr("same transaction"))));
 
   // Conflicting operation: reference foreign key in referencing table
   // and delete key from referenced table in the same mutation.
@@ -1501,12 +1511,17 @@ TEST_F(PgClientIntegrationTest, ForeignKeyDeleteCascade) {
                                    {"CartId", "CustomerId", "CustomerName"},  //
                                    1, 2, "FKCustomer2"),
                 MakeDeleteMutation("Customers", KeySet().AddKey(MakeKey(1)))});
-  EXPECT_THAT(reconflict_commit,
-              StatusIs(StatusCode::kFailedPrecondition,
-                       AllOf(HasSubstr("Cannot modify a row"),
-                             HasSubstr("shoppingcarts"),
-                             HasSubstr("referential action is deleting it"),
-                             HasSubstr("in the same transaction"))));
+  if (UsingEmulator()) {
+    EXPECT_THAT(reconflict_commit, StatusIs(StatusCode::kAlreadyExists,
+                                            HasSubstr("shoppingcarts")));
+  } else {
+    EXPECT_THAT(reconflict_commit,
+                StatusIs(StatusCode::kFailedPrecondition,
+                         AllOf(HasSubstr("Cannot modify a row"),
+                               HasSubstr("shoppingcarts"),
+                               HasSubstr("referential action is deleting it"),
+                               HasSubstr("in the same transaction"))));
+  }
 
   // Query INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS and validate DELETE_RULE.
   auto delete_rules = client_->ExecuteQuery(SqlStatement(R"""(
@@ -1518,7 +1533,8 @@ TEST_F(PgClientIntegrationTest, ForeignKeyDeleteCascade) {
   for (auto& delete_rule : StreamOf<RowType>(delete_rules)) {
     EXPECT_THAT(delete_rule, IsOk());
     if (!delete_rule) break;
-    EXPECT_EQ("CASCADE", std::get<0>(*delete_rule));
+    EXPECT_EQ(UsingEmulator() ? "NO ACTION" : "CASCADE",
+              std::get<0>(*delete_rule));
   }
 
   // Drop ON DELETE CASCADE constraint.
@@ -1581,11 +1597,7 @@ TEST_F(ClientIntegrationTest, SupportedOptimizerVersions) {
       )"""));
   using RowType = std::tuple<std::int64_t, absl::CivilDay>;
   for (auto& row : StreamOf<RowType>(rows)) {
-    if (UsingEmulator()) {
-      EXPECT_THAT(row, StatusIs(StatusCode::kInvalidArgument));
-    } else {
-      EXPECT_THAT(row, IsOk());
-    }
+    EXPECT_THAT(row, IsOk());
     if (!row) break;
     EXPECT_GT(std::get<0>(*row), 0);
     EXPECT_GE(std::get<1>(*row), absl::CivilDay(1998, 9, 4));
@@ -1613,27 +1625,6 @@ TEST_F(ClientIntegrationTest, SpannerStatistics) {
   }
 }
 
-/// @test Verify the use of unified credentials.
-TEST_F(ClientIntegrationTest, UnifiedCredentials) {
-  auto options =
-      Options{}.set<UnifiedCredentialsOption>(MakeGoogleDefaultCredentials());
-  if (UsingEmulator()) {
-    options = Options{}
-                  .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-                  .set<internal::UseInsecureChannelOption>(true);
-  }
-
-  // Reconnect to the database using the new credentials.
-  auto client = Client(MakeConnection(GetDatabase(), options));
-
-  auto commit_result = client.Commit(Mutations{
-      InsertMutationBuilder("Singers", {"SingerId", "FirstName", "LastName"})
-          .EmplaceRow(1, "test-fname-1", "test-lname-1")
-          .EmplaceRow(2, "test-fname-2", "test-lname-2")
-          .Build()});
-  EXPECT_STATUS_OK(commit_result);
-}
-
 /// @test Verify backwards compatibility for MakeConnection() arguments.
 TEST_F(ClientIntegrationTest, MakeConnectionOverloads) {
   MakeConnection(GetDatabase(), ConnectionOptions());
