@@ -20,7 +20,10 @@
 #include "google/cloud/spanner/timestamp.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include "absl/time/time.h"
+#include <google/cloud/spanner/testing/singer.pb.h>
 #include <gmock/gmock.h>
+#include <cstdint>
+#include <string>
 #include <vector>
 
 namespace google {
@@ -32,7 +35,6 @@ namespace {
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::StatusIs;
-using ::testing::AllOf;
 using ::testing::AnyOf;
 using ::testing::ElementsAre;
 using ::testing::HasSubstr;
@@ -43,6 +45,16 @@ absl::Time MakeTime(std::time_t sec, int nanos) {
   return absl::FromTimeT(sec) + absl::Nanoseconds(nanos);
 }
 
+testing::SingerInfo MakeSinger(std::int64_t singer_id, std::string birth_date,
+                               std::string nationality, testing::Genre genre) {
+  testing::SingerInfo singer;
+  singer.set_singer_id(singer_id);
+  singer.set_birth_date(std::move(birth_date));
+  singer.set_nationality(std::move(nationality));
+  singer.set_genre(genre);
+  return singer;
+}
+
 // A helper function used in the test fixtures below. This function writes the
 // given data to the DataTypes table, then it reads all the data back and
 // returns it to the caller.
@@ -80,14 +92,7 @@ class DataTypeIntegrationTestTmpl : public T {
     auto commit_result = client_->Commit(Mutations{
         MakeDeleteMutation("DataTypes", KeySet::All()),
     });
-    if (T::UsingEmulator()) {
-      // PgDataTypeIntegrationTest::SetUpTestSuite() will fail until
-      // the emulator supports PostgreSQL syntax to quote identifiers.
-      ASSERT_THAT(commit_result,
-                  AnyOf(IsOk(), StatusIs(StatusCode::kNotFound)));
-    } else {
-      ASSERT_THAT(commit_result, IsOk());
-    }
+    ASSERT_THAT(commit_result, IsOk());
   }
 
   static void TearDownTestSuite() {
@@ -162,6 +167,100 @@ TEST_F(DataTypeIntegrationTest, WriteReadFloat64NaN) {
                           [](double d) { return std::isnan(d); }, true))));
 }
 
+TEST_F(DataTypeIntegrationTest, WriteReadFloat32) {
+  std::vector<float> const data = {
+      -std::numeric_limits<float>::infinity(),
+      std::numeric_limits<float>::lowest(),
+      std::numeric_limits<float>::min(),
+      -123.456F,
+      -123,
+      -42.42F,
+      -42,
+      -1.5,
+      -1,
+      -0.5,
+      0,
+      0.5,
+      1,
+      1.5,
+      42,
+      42.42F,
+      123,
+      123.456F,
+      std::numeric_limits<float>::max(),
+      std::numeric_limits<float>::infinity(),
+  };
+  auto result = WriteReadData(*client_, data, "Float32Value");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    ASSERT_STATUS_OK(result);
+    EXPECT_THAT(*result, UnorderedElementsAreArray(data));
+  }
+}
+
+TEST_F(PgDataTypeIntegrationTest, WriteReadFloat32) {
+  std::vector<float> const data = {
+      -std::numeric_limits<float>::infinity(),
+      std::numeric_limits<float>::lowest(),
+      std::numeric_limits<float>::min(),
+      -123.456F,
+      -123,
+      -42.42F,
+      -42,
+      -1.5,
+      -1,
+      -0.5,
+      0,
+      0.5,
+      1,
+      1.5,
+      42,
+      42.42F,
+      123,
+      123.456F,
+      std::numeric_limits<float>::max(),
+      std::numeric_limits<float>::infinity(),
+  };
+  auto result = WriteReadData(*client_, data, "Float32Value");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    ASSERT_STATUS_OK(result);
+    EXPECT_THAT(*result, UnorderedElementsAreArray(data));
+  }
+}
+
+TEST_F(DataTypeIntegrationTest, WriteReadFloat32NaN) {
+  // Since NaN is not equal to anything, including itself, we need to handle
+  // NaN separately from other Float32 values.
+  std::vector<float> const data = {
+      std::numeric_limits<float>::quiet_NaN(),
+  };
+  auto result = WriteReadData(*client_, data, "Float32Value");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(ElementsAre(ResultOf(
+                            [](float d) { return std::isnan(d); }, true))));
+  }
+}
+
+TEST_F(PgDataTypeIntegrationTest, WriteReadFloat32NaN) {
+  // Since NaN is not equal to anything, including itself, we need to handle
+  // NaN separately from other Float32 values.
+  std::vector<float> const data = {
+      std::numeric_limits<float>::quiet_NaN(),
+  };
+  auto result = WriteReadData(*client_, data, "Float32Value");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(ElementsAre(ResultOf(
+                            [](float d) { return std::isnan(d); }, true))));
+  }
+}
+
 TEST_F(DataTypeIntegrationTest, WriteReadString) {
   std::vector<std::string> const data = {
       "",
@@ -239,8 +338,6 @@ TEST_F(DataTypeIntegrationTest, WriteReadJson) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, WriteReadJson) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   std::vector<JsonB> const data = {
       JsonB(),                     //
       JsonB(R"("Hello world!")"),  //
@@ -271,8 +368,6 @@ TEST_F(DataTypeIntegrationTest, WriteReadNumeric) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, WriteReadNumeric) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   auto limit = std::string(131072, '9') + "." + std::string(16383, '9');
   auto min = MakePgNumeric("-" + limit);
   ASSERT_STATUS_OK(min);
@@ -292,6 +387,34 @@ TEST_F(PgDataTypeIntegrationTest, WriteReadNumeric) {
   EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
 }
 
+TEST_F(DataTypeIntegrationTest, WriteReadProtoEnum) {
+  std::vector<ProtoEnum<testing::Genre>> const data = {
+      testing::Genre::POP,
+      testing::Genre::JAZZ,
+      testing::Genre::FOLK,
+      testing::Genre::ROCK,
+  };
+  auto result = WriteReadData(*client_, data, "SingerGenre");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
+  }
+}
+
+TEST_F(DataTypeIntegrationTest, WriteReadProtoMessage) {
+  std::vector<ProtoMessage<testing::SingerInfo>> const data = {
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK),
+      MakeSinger(2123139547, "1942-06-18", "British", testing::Genre::POP),
+  };
+  auto result = WriteReadData(*client_, data, "SingerInfo");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
+  }
+}
+
 TEST_F(DataTypeIntegrationTest, WriteReadArrayBool) {
   std::vector<std::vector<bool>> const data = {
       std::vector<bool>{},
@@ -324,6 +447,20 @@ TEST_F(DataTypeIntegrationTest, WriteReadArrayFloat64) {
   EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
 }
 
+TEST_F(DataTypeIntegrationTest, WriteReadArrayFloat32) {
+  std::vector<std::vector<float>> const data = {
+      std::vector<float>{},
+      std::vector<float>{-0.5},
+      std::vector<float>{-0.5, 0.5, 1.5},
+  };
+  auto result = WriteReadData(*client_, data, "ArrayFloat32Value");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
+  }
+}
+
 TEST_F(DataTypeIntegrationTest, WriteReadArrayString) {
   std::vector<std::vector<std::string>> const data = {
       std::vector<std::string>{},
@@ -388,8 +525,6 @@ TEST_F(DataTypeIntegrationTest, WriteReadArrayJson) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, WriteReadArrayJson) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   std::vector<std::vector<JsonB>> const data = {
       std::vector<JsonB>{},
       std::vector<JsonB>{JsonB()},
@@ -419,8 +554,6 @@ TEST_F(DataTypeIntegrationTest, WriteReadArrayNumeric) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, WriteReadArrayNumeric) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   std::vector<std::vector<PgNumeric>> const data = {
       std::vector<PgNumeric>{},
       std::vector<PgNumeric>{PgNumeric()},
@@ -434,6 +567,40 @@ TEST_F(PgDataTypeIntegrationTest, WriteReadArrayNumeric) {
   EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
 }
 
+TEST_F(DataTypeIntegrationTest, WriteReadArrayProtoEnum) {
+  std::vector<std::vector<ProtoEnum<testing::Genre>>> const data = {
+      std::vector<ProtoEnum<testing::Genre>>{},
+      std::vector<ProtoEnum<testing::Genre>>{
+          testing::Genre::POP,
+          testing::Genre::JAZZ,
+          testing::Genre::FOLK,
+          testing::Genre::ROCK,
+      },
+  };
+  auto result = WriteReadData(*client_, data, "ArraySingerGenre");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
+  }
+}
+
+TEST_F(DataTypeIntegrationTest, WriteReadArrayProtoMessage) {
+  std::vector<std::vector<ProtoMessage<testing::SingerInfo>>> const data = {
+      std::vector<ProtoMessage<testing::SingerInfo>>{},
+      std::vector<ProtoMessage<testing::SingerInfo>>{
+          MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK),
+          MakeSinger(2123139547, "1942-06-18", "British", testing::Genre::POP),
+      },
+  };
+  auto result = WriteReadData(*client_, data, "ArraySingerInfo");
+  if (UsingEmulator()) {
+    EXPECT_THAT(result, StatusIs(StatusCode::kNotFound));
+  } else {
+    EXPECT_THAT(result, IsOkAndHolds(UnorderedElementsAreArray(data)));
+  }
+}
+
 TEST_F(DataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
   spanner_admin::DatabaseAdminClient admin_client(
       spanner_admin::MakeDatabaseAdminConnection());
@@ -447,12 +614,9 @@ TEST_F(DataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
   auto metadata =
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
-  EXPECT_THAT(metadata,
-              StatusIs(StatusCode::kFailedPrecondition,
-                       AnyOf(AllOf(HasSubstr("Index DataTypesByJsonValue"),
-                                   HasSubstr("unsupported type JSON")),
-                             AllOf(HasSubstr("index DataTypesByJsonValue"),
-                                   HasSubstr("Cannot reference JSON")))));
+  EXPECT_THAT(metadata, StatusIs(StatusCode::kFailedPrecondition,
+                                 AnyOf(HasSubstr("unsupported type JSON"),
+                                       HasSubstr("Cannot reference JSON"))));
 
   // Verify that a JSON column cannot be used as a primary key.
   statements.clear();
@@ -465,13 +629,10 @@ TEST_F(DataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
   EXPECT_THAT(metadata, StatusIs(StatusCode::kInvalidArgument,
-                                 AllOf(HasSubstr("has type JSON"),
-                                       HasSubstr("part of the primary key"))));
+                                 HasSubstr("has type JSON")));
 }
 
 TEST_F(PgDataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   spanner_admin::DatabaseAdminClient admin_client(
       spanner_admin::MakeDatabaseAdminConnection());
 
@@ -484,10 +645,13 @@ TEST_F(PgDataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
   auto metadata =
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
-  EXPECT_THAT(metadata,
-              StatusIs(StatusCode::kFailedPrecondition,
-                       AllOf(HasSubstr("Index datatypesbyjsonvalue"),
-                             HasSubstr("unsupported type PG.JSONB"))));
+  if (UsingEmulator()) {
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kFailedPrecondition,
+                                   HasSubstr("Cannot reference PG.JSONB")));
+  } else {
+    EXPECT_THAT(metadata, StatusIs(StatusCode::kFailedPrecondition,
+                                   HasSubstr("unsupported type PG.JSONB")));
+  }
 
   // Verify that a JSONB column cannot be used as a primary key.
   statements.clear();
@@ -501,13 +665,10 @@ TEST_F(PgDataTypeIntegrationTest, JsonIndexAndPrimaryKey) {
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
   EXPECT_THAT(metadata, StatusIs(StatusCode::kInvalidArgument,
-                                 AllOf(HasSubstr("has type PG.JSONB"),
-                                       HasSubstr("part of the primary key"))));
+                                 HasSubstr("has type PG.JSONB")));
 }
 
 TEST_F(PgDataTypeIntegrationTest, InsertAndQueryWithJson) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   auto& client = *client_;
   auto commit_result =
       client.Commit([&client](Transaction const& txn) -> StatusOr<Mutations> {
@@ -552,8 +713,6 @@ TEST_F(DataTypeIntegrationTest, InsertAndQueryWithNumericKey) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, NumericPrimaryKey) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support PostgreSQL";
-
   spanner_admin::DatabaseAdminClient admin_client(
       spanner_admin::MakeDatabaseAdminConnection());
 
@@ -569,13 +728,30 @@ TEST_F(PgDataTypeIntegrationTest, NumericPrimaryKey) {
       admin_client.UpdateDatabaseDdl(GetDatabase().FullName(), statements)
           .get();
   EXPECT_THAT(metadata, StatusIs(StatusCode::kInvalidArgument,
-                                 AllOf(HasSubstr("has type PG.NUMERIC"),
-                                       HasSubstr("part of the primary key"))));
+                                 HasSubstr("has type PG.NUMERIC")));
+}
+
+TEST_F(DataTypeIntegrationTest, InsertAndQueryWithProtoEnumKey) {
+  auto& client = *client_;
+  auto const key = testing::Genre::POP;
+
+  auto commit_result = client.Commit(
+      Mutations{InsertOrUpdateMutationBuilder("ProtoEnumKey", {"Key"})
+                    .EmplaceRow(key)
+                    .Build()});
+  if (UsingEmulator()) {
+    EXPECT_THAT(commit_result, StatusIs(StatusCode::kNotFound));
+  } else {
+    ASSERT_STATUS_OK(commit_result);
+    auto rows = client.Read("ProtoEnumKey", KeySet::All(), {"Key"});
+    using RowType = std::tuple<ProtoEnum<testing::Genre>>;
+    auto row = GetSingularRow(StreamOf<RowType>(rows));
+    ASSERT_STATUS_OK(row);
+    EXPECT_EQ(std::get<0>(*std::move(row)), key);
+  }
 }
 
 TEST_F(DataTypeIntegrationTest, DmlReturning) {
-  if (UsingEmulator()) GTEST_SKIP() << "emulator does not support THEN RETURN";
-
   auto& client = *client_;
   using RowType = std::tuple<std::string, std::int64_t>;
 
@@ -591,7 +767,7 @@ TEST_F(DataTypeIntegrationTest, DmlReturning) {
               THEN RETURN Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         insert_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) insert_actual.push_back(*std::move(row));
@@ -612,7 +788,7 @@ TEST_F(DataTypeIntegrationTest, DmlReturning) {
               THEN RETURN Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         update_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) update_actual.push_back(*std::move(row));
@@ -633,7 +809,7 @@ TEST_F(DataTypeIntegrationTest, DmlReturning) {
               THEN RETURN Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         delete_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) delete_actual.push_back(*std::move(row));
@@ -647,10 +823,6 @@ TEST_F(DataTypeIntegrationTest, DmlReturning) {
 }
 
 TEST_F(PgDataTypeIntegrationTest, DmlReturning) {
-  if (UsingEmulator()) {
-    GTEST_SKIP() << "emulator does not support PostgreSQL or RETURNING";
-  }
-
   auto& client = *client_;
   using RowType = std::tuple<std::string, std::int64_t>;
 
@@ -666,7 +838,7 @@ TEST_F(PgDataTypeIntegrationTest, DmlReturning) {
               RETURNING Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         insert_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) insert_actual.push_back(*std::move(row));
@@ -687,7 +859,7 @@ TEST_F(PgDataTypeIntegrationTest, DmlReturning) {
               RETURNING Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         update_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) update_actual.push_back(*std::move(row));
@@ -708,7 +880,7 @@ TEST_F(PgDataTypeIntegrationTest, DmlReturning) {
               RETURNING Id, Int64Value
         )""");
         auto rows = client.ExecuteQuery(txn, std::move(sql));
-        EXPECT_EQ(rows.RowsModified(), 4);
+        EXPECT_EQ(rows.RowsModified(), UsingEmulator() ? 0 : 4);
         delete_actual.clear();  // may be a re-run
         for (auto& row : StreamOf<RowType>(rows)) {
           if (row) delete_actual.push_back(*std::move(row));
