@@ -26,6 +26,7 @@
 #include "google/cloud/common_options.h"
 #include "google/cloud/grpc_error_delegate.h"
 #include "google/cloud/internal/algorithm.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/resumable_streaming_read_rpc.h"
 #include "google/cloud/internal/retry_loop.h"
 #include "google/cloud/internal/streaming_read_rpc.h"
@@ -96,17 +97,27 @@ class DirectedReadVisitor {
   std::function<google::spanner::v1::DirectedReadOptions*()> factory_;
 };
 
+inline std::shared_ptr<spanner::RetryPolicy> const& RetryPolicyPrototype(
+    Options const& options) {
+  return options.get<spanner::SpannerRetryPolicyOption>();
+}
+
+inline std::shared_ptr<spanner::BackoffPolicy> const& BackoffPolicyPrototype(
+    Options const& options) {
+  return options.get<spanner::SpannerBackoffPolicyOption>();
+}
+
 inline std::shared_ptr<spanner::RetryPolicy> const& RetryPolicyPrototype() {
-  return internal::CurrentOptions().get<spanner::SpannerRetryPolicyOption>();
+  return RetryPolicyPrototype(internal::CurrentOptions());
 }
 
 inline std::shared_ptr<spanner::BackoffPolicy> const& BackoffPolicyPrototype() {
-  return internal::CurrentOptions().get<spanner::SpannerBackoffPolicyOption>();
+  return BackoffPolicyPrototype((internal::CurrentOptions()));
 }
 
 inline bool RpcStreamTracingEnabled() {
   return internal::Contains(
-      internal::CurrentOptions().get<TracingComponentsOption>(), "rpc-streams");
+      internal::CurrentOptions().get<LoggingComponentsOption>(), "rpc-streams");
 }
 
 inline TracingOptions const& RpcTracingOptions() {
@@ -157,6 +168,10 @@ google::spanner::v1::TransactionOptions PartitionedDmlTransactionOptions() {
   google::spanner::v1::TransactionOptions options;
   *options.mutable_partitioned_dml() =
       google::spanner::v1::TransactionOptions_PartitionedDml();
+  auto const& current = internal::CurrentOptions();
+  if (current.get<spanner::ExcludeTransactionFromChangeStreamsOption>()) {
+    options.set_exclude_txn_from_change_streams(true);
+  }
   return options;
 }
 
@@ -194,9 +209,10 @@ spanner::Timestamp MakeTimestamp(google::protobuf::Timestamp const& proto) {
 // a malformed response that does not contain a `Transaction` should invalidate
 // the transaction with and also return this status.
 Status MissingTransactionStatus(std::string const& operation) {
-  return Status(StatusCode::kInternal,
-                "Begin transaction requested but no transaction returned (in " +
-                    operation + ")");
+  return internal::InternalError(
+      "Begin transaction requested but no transaction returned (in " +
+          operation + ")",
+      GCP_ERROR_INFO());
 }
 
 class StatusOnlyResultSetSource : public spanner::ResultSourceInterface {
@@ -495,16 +511,17 @@ StatusOr<google::spanner::v1::Transaction> ConnectionImpl::BeginTransaction(
   begin.mutable_request_options()->set_transaction_tag(ctx.tag);
 
   auto stub = session_pool_->GetStub(*session);
+  auto const& current = internal::CurrentOptions();
   auto response = RetryLoop(
-      RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
-      Idempotency::kIdempotent,
+      RetryPolicyPrototype(current)->clone(),
+      BackoffPolicyPrototype(current)->clone(), Idempotency::kIdempotent,
       [&stub, route_to_leader = ctx.route_to_leader](
-          grpc::ClientContext& context,
+          grpc::ClientContext& context, Options const& options,
           google::spanner::v1::BeginTransactionRequest const& request) {
         if (route_to_leader) RouteToLeader(context);
-        return stub->BeginTransaction(context, request);
+        return stub->BeginTransaction(context, options, request);
       },
-      begin, func);
+      current, begin, func);
   if (!response) {
     auto status = std::move(response).status();
     if (IsSessionNotFound(status)) session->set_bad();
@@ -640,16 +657,17 @@ StatusOr<std::vector<spanner::ReadPartition>> ConnectionImpl::PartitionReadImpl(
   *request.mutable_partition_options() = ToProto(partition_options);
 
   auto stub = session_pool_->GetStub(*session);
+  auto const& current = internal::CurrentOptions();
   for (;;) {
     auto response = RetryLoop(
         RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
         Idempotency::kIdempotent,
-        [&stub](grpc::ClientContext& context,
+        [&stub](grpc::ClientContext& context, Options const& options,
                 google::spanner::v1::PartitionReadRequest const& request) {
           RouteToLeader(context);  // always for PartitionRead()
-          return stub->PartitionRead(context, request);
+          return stub->PartitionRead(context, options, request);
         },
-        request, __func__);
+        current, request, __func__);
     if (s->has_begin()) {
       if (response.ok()) {
         if (!response->has_transaction()) {
@@ -868,24 +886,25 @@ StatusOr<ResultType> ConnectionImpl::CommonDmlImpl(
   // through the lifetime of the lambda. Note that the local variables are a
   // reference to avoid increasing refcounts twice, but the capture is by value.
   auto stub = session_pool_->GetStub(*session);
-  auto const& retry_policy_prototype = RetryPolicyPrototype();
-  auto const& backoff_policy_prototype = BackoffPolicyPrototype();
+  auto current = google::cloud::internal::SaveCurrentOptions();
+  auto const& retry_policy_prototype = RetryPolicyPrototype(*current);
+  auto const& backoff_policy_prototype = BackoffPolicyPrototype(*current);
 
   auto retry_resume_fn =
       [function_name, stub, retry_policy_prototype, backoff_policy_prototype,
-       session, route_to_leader = ctx.route_to_leader](
-          google::spanner::v1::ExecuteSqlRequest& request) mutable
+       session, route_to_leader = ctx.route_to_leader,
+       current](google::spanner::v1::ExecuteSqlRequest& request) mutable
       -> StatusOr<std::unique_ptr<ResultSourceInterface>> {
     StatusOr<google::spanner::v1::ResultSet> response = RetryLoop(
         retry_policy_prototype->clone(), backoff_policy_prototype->clone(),
         Idempotency::kIdempotent,
         [stub, route_to_leader](
-            grpc::ClientContext& context,
+            grpc::ClientContext& context, Options const& options,
             google::spanner::v1::ExecuteSqlRequest const& request) {
           if (route_to_leader) RouteToLeader(context);
-          return stub->ExecuteSql(context, request);
+          return stub->ExecuteSql(context, options, request);
         },
-        request, function_name);
+        *current, request, function_name);
     if (!response) {
       auto status = std::move(response).status();
       if (IsSessionNotFound(status)) session->set_bad();
@@ -955,16 +974,17 @@ ConnectionImpl::PartitionQueryImpl(
   *request.mutable_partition_options() = ToProto(params.partition_options);
 
   auto stub = session_pool_->GetStub(*session);
+  auto const& current = internal::CurrentOptions();
   for (;;) {
     auto response = RetryLoop(
         RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
         Idempotency::kIdempotent,
-        [&stub](grpc::ClientContext& context,
+        [&stub](grpc::ClientContext& context, Options const& options,
                 google::spanner::v1::PartitionQueryRequest const& request) {
           RouteToLeader(context);  // always for PartitionQuery()
-          return stub->PartitionQuery(context, request);
+          return stub->PartitionQuery(context, options, request);
         },
-        request, __func__);
+        current, request, __func__);
     if (s->has_begin()) {
       if (response.ok()) {
         if (!response->has_transaction()) {
@@ -1029,17 +1049,18 @@ StatusOr<spanner::BatchDmlResult> ConnectionImpl::ExecuteBatchDmlImpl(
   request.mutable_request_options()->set_request_tag(request_tag);
   request.mutable_request_options()->set_transaction_tag(ctx.tag);
 
+  auto const& current = internal::CurrentOptions();
   auto stub = session_pool_->GetStub(*session);
   for (;;) {
     auto response = RetryLoop(
         RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
         Idempotency::kIdempotent,
-        [&stub](grpc::ClientContext& context,
+        [&stub](grpc::ClientContext& context, Options const& options,
                 google::spanner::v1::ExecuteBatchDmlRequest const& request) {
           RouteToLeader(context);  // always for ExecuteBatchDml()
-          return stub->ExecuteBatchDml(context, request);
+          return stub->ExecuteBatchDml(context, options, request);
         },
-        request, __func__);
+        current, request, __func__);
     if (s->has_begin()) {
       if (response.ok() && response->result_sets_size() > 0) {
         if (!response->result_sets(0).metadata().has_transaction()) {
@@ -1139,6 +1160,11 @@ StatusOr<spanner::CommitResult> ConnectionImpl::CommitImpl(
   request.set_return_commit_stats(params.options.return_stats());
   request.mutable_request_options()->set_priority(
       ProtoRequestPriority(params.options.request_priority()));
+  if (params.options.max_commit_delay().has_value()) {
+    *request.mutable_max_commit_delay() =
+        google::protobuf::util::TimeUtil::MillisecondsToDuration(
+            params.options.max_commit_delay()->count());
+  }
 
   // params.options.transaction_tag() was either already used to set
   // ctx.tag (for a library-generated transaction), or it is ignored
@@ -1166,19 +1192,21 @@ StatusOr<spanner::CommitResult> ConnectionImpl::CommitImpl(
       break;
     }
     default:
-      return Status(StatusCode::kInternal, "TransactionSelector state error");
+      return internal::InternalError("TransactionSelector state error",
+                                     GCP_ERROR_INFO());
   }
 
+  auto const& current = internal::CurrentOptions();
   auto stub = session_pool_->GetStub(*session);
   auto response = RetryLoop(
-      RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
-      Idempotency::kIdempotent,
-      [&stub](grpc::ClientContext& context,
+      RetryPolicyPrototype(current)->clone(),
+      BackoffPolicyPrototype(current)->clone(), Idempotency::kIdempotent,
+      [&stub](grpc::ClientContext& context, Options const& options,
               google::spanner::v1::CommitRequest const& request) {
         RouteToLeader(context);  // always for Commit()
-        return stub->Commit(context, request);
+        return stub->Commit(context, options, request);
       },
-      request, __func__);
+      current, request, __func__);
   if (!response) {
     auto status = std::move(response).status();
     if (IsSessionNotFound(status)) session->set_bad();
@@ -1201,8 +1229,8 @@ Status ConnectionImpl::RollbackImpl(
     return s.status();
   }
   if (s->has_single_use()) {
-    return Status(StatusCode::kInvalidArgument,
-                  "Cannot rollback a single-use transaction");
+    return internal::InvalidArgumentError(
+        "Cannot rollback a single-use transaction", GCP_ERROR_INFO());
   }
 
   auto prepare_status = PrepareSession(session);
@@ -1224,15 +1252,16 @@ Status ConnectionImpl::RollbackImpl(
   request.set_session(session->session_name());
   request.set_transaction_id(s->id());
   auto stub = session_pool_->GetStub(*session);
+  auto const& current = internal::CurrentOptions();
   auto status = RetryLoop(
-      RetryPolicyPrototype()->clone(), BackoffPolicyPrototype()->clone(),
-      Idempotency::kIdempotent,
-      [&stub](grpc::ClientContext& context,
+      RetryPolicyPrototype(current)->clone(),
+      BackoffPolicyPrototype(current)->clone(), Idempotency::kIdempotent,
+      [&stub](grpc::ClientContext& context, Options const& options,
               google::spanner::v1::RollbackRequest const& request) {
         RouteToLeader(context);  // always for Rollback()
-        return stub->Rollback(context, request);
+        return stub->Rollback(context, options, request);
       },
-      request, __func__);
+      current, request, __func__);
   if (IsSessionNotFound(status)) session->set_bad();
   return status;
 }
@@ -1266,6 +1295,10 @@ spanner::BatchedCommitResultStream ConnectionImpl::BatchWriteImpl(
       *group->add_mutations() = std::move(m).as_proto();
     }
   }
+  auto const& current = internal::CurrentOptions();
+  if (current.get<spanner::ExcludeTransactionFromChangeStreamsOption>()) {
+    request.set_exclude_txn_from_change_streams(true);
+  }
 
   auto stub = session_pool_->GetStub(*session);
   auto factory = [stub = std::move(stub)](
