@@ -19,7 +19,6 @@
 #include "google/cloud/internal/api_client_header.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include "google/cloud/testing_util/validate_metadata.h"
-#include "absl/memory/memory.h"
 #include <gmock/gmock.h>
 #include <grpcpp/grpcpp.h>
 
@@ -29,70 +28,73 @@ namespace spanner_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
-using ::google::cloud::testing_util::IsContextMDValid;
-namespace gcsa = ::google::spanner::admin::database::v1;
+namespace gsad = ::google::spanner::admin::database;
+
+using ::google::cloud::testing_util::ValidateMetadataFixture;
 
 class DatabaseAdminMetadataTest : public ::testing::Test {
  protected:
   void SetUp() override {
     mock_ = std::make_shared<spanner_testing::MockDatabaseAdminStub>();
     DatabaseAdminMetadata stub(mock_);
-    expected_api_client_header_ = google::cloud::internal::ApiClientHeader();
   }
 
   void TearDown() override {}
 
+  void IsContextMDValid(grpc::ClientContext& context, std::string const& method,
+                        google::protobuf::Message const& request) {
+    return validate_metadata_fixture_.IsContextMDValid(
+        context, method, request,
+        google::cloud::internal::HandCraftedLibClientHeader());
+  }
+
   static Status TransientError() {
     return Status(StatusCode::kUnavailable, "try-again");
   }
 
   std::shared_ptr<spanner_testing::MockDatabaseAdminStub> mock_;
-  std::string expected_api_client_header_;
+  ValidateMetadataFixture validate_metadata_fixture_;
 };
 
 TEST_F(DatabaseAdminMetadataTest, CreateDatabase) {
   EXPECT_CALL(*mock_, AsyncCreateDatabase)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       gcsa::CreateDatabaseRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(*context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "CreateDatabase",
-                             expected_api_client_header_));
+      .WillOnce([this](CompletionQueue&, auto context,
+                       gsad::v1::CreateDatabaseRequest const& request) {
+        IsContextMDValid(*context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "CreateDatabase",
+                         request);
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(TransientError()));
       });
 
   DatabaseAdminMetadata stub(mock_);
   CompletionQueue cq;
-  gcsa::CreateDatabaseRequest request;
+  gsad::v1::CreateDatabaseRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
           .FullName());
   auto response = stub.AsyncCreateDatabase(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), response.get().status());
 }
 
 TEST_F(DatabaseAdminMetadataTest, UpdateDatabase) {
   EXPECT_CALL(*mock_, AsyncUpdateDatabaseDdl)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       gcsa::UpdateDatabaseDdlRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(*context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "UpdateDatabaseDdl",
-                             expected_api_client_header_));
+      .WillOnce([this](CompletionQueue&, auto context,
+                       gsad::v1::UpdateDatabaseDdlRequest const& request) {
+        IsContextMDValid(*context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "UpdateDatabaseDdl",
+                         request);
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(TransientError()));
       });
 
   DatabaseAdminMetadata stub(mock_);
   CompletionQueue cq;
-  gcsa::UpdateDatabaseDdlRequest request;
+  gsad::v1::UpdateDatabaseDdlRequest request;
   request.set_database(
       google::cloud::spanner::Database(
           google::cloud::spanner::Instance(
@@ -100,25 +102,24 @@ TEST_F(DatabaseAdminMetadataTest, UpdateDatabase) {
           "test-database")
           .FullName());
   auto response = stub.AsyncUpdateDatabaseDdl(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), response.get().status());
 }
 
 TEST_F(DatabaseAdminMetadataTest, DropDatabase) {
   EXPECT_CALL(*mock_, DropDatabase)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::DropDatabaseRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "DropDatabase",
-                             expected_api_client_header_));
+                       gsad::v1::DropDatabaseRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "DropDatabase",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::DropDatabaseRequest request;
+  gsad::v1::DropDatabaseRequest request;
   request.set_database(
       google::cloud::spanner::Database(
           google::cloud::spanner::Instance(
@@ -132,18 +133,17 @@ TEST_F(DatabaseAdminMetadataTest, DropDatabase) {
 TEST_F(DatabaseAdminMetadataTest, ListDatabases) {
   EXPECT_CALL(*mock_, ListDatabases)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::ListDatabasesRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "ListDatabases",
-                             expected_api_client_header_));
+                       gsad::v1::ListDatabasesRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "ListDatabases",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::ListDatabasesRequest request;
+  gsad::v1::ListDatabasesRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
@@ -154,39 +154,36 @@ TEST_F(DatabaseAdminMetadataTest, ListDatabases) {
 
 TEST_F(DatabaseAdminMetadataTest, RestoreDatabase) {
   EXPECT_CALL(*mock_, AsyncRestoreDatabase)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       gcsa::RestoreDatabaseRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(*context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "RestoreDatabase",
-                             expected_api_client_header_));
+      .WillOnce([this](CompletionQueue&, auto context,
+                       gsad::v1::RestoreDatabaseRequest const& request) {
+        IsContextMDValid(*context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "RestoreDatabase",
+                         request);
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(TransientError()));
       });
 
   DatabaseAdminMetadata stub(mock_);
   CompletionQueue cq;
-  gcsa::RestoreDatabaseRequest request;
+  gsad::v1::RestoreDatabaseRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
           .FullName());
   auto response = stub.AsyncRestoreDatabase(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), response.get().status());
 }
 
 TEST_F(DatabaseAdminMetadataTest, GetIamPolicy) {
   EXPECT_CALL(*mock_, GetIamPolicy)
       .WillOnce([this](grpc::ClientContext& context,
-                       google::iam::v1::GetIamPolicyRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "GetIamPolicy",
-                             expected_api_client_header_));
+                       google::iam::v1::GetIamPolicyRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "GetIamPolicy",
+                         request);
         return TransientError();
       });
 
@@ -206,12 +203,11 @@ TEST_F(DatabaseAdminMetadataTest, GetIamPolicy) {
 TEST_F(DatabaseAdminMetadataTest, SetIamPolicy) {
   EXPECT_CALL(*mock_, SetIamPolicy)
       .WillOnce([this](grpc::ClientContext& context,
-                       google::iam::v1::SetIamPolicyRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "SetIamPolicy",
-                             expected_api_client_header_));
+                       google::iam::v1::SetIamPolicyRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "SetIamPolicy",
+                         request);
         return TransientError();
       });
 
@@ -236,15 +232,15 @@ TEST_F(DatabaseAdminMetadataTest, SetIamPolicy) {
 
 TEST_F(DatabaseAdminMetadataTest, TestIamPermissions) {
   EXPECT_CALL(*mock_, TestIamPermissions)
-      .WillOnce([this](grpc::ClientContext& context,
-                       google::iam::v1::TestIamPermissionsRequest const&) {
-        EXPECT_STATUS_OK(
+      .WillOnce(
+          [this](grpc::ClientContext& context,
+                 google::iam::v1::TestIamPermissionsRequest const& request) {
             IsContextMDValid(context,
                              "google.spanner.admin.database.v1.DatabaseAdmin."
                              "TestIamPermissions",
-                             expected_api_client_header_));
-        return TransientError();
-      });
+                             request);
+            return TransientError();
+          });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
@@ -261,45 +257,42 @@ TEST_F(DatabaseAdminMetadataTest, TestIamPermissions) {
 
 TEST_F(DatabaseAdminMetadataTest, CreateBackup) {
   EXPECT_CALL(*mock_, AsyncCreateBackup)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       gcsa::CreateBackupRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(*context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "CreateBackup",
-                             expected_api_client_header_));
+      .WillOnce([this](CompletionQueue&, auto context,
+                       gsad::v1::CreateBackupRequest const& request) {
+        IsContextMDValid(*context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "CreateBackup",
+                         request);
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(TransientError()));
       });
 
   DatabaseAdminMetadata stub(mock_);
   CompletionQueue cq;
-  gcsa::CreateBackupRequest request;
+  gsad::v1::CreateBackupRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
           .FullName());
   auto response = stub.AsyncCreateBackup(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), response.get().status());
 }
 
 TEST_F(DatabaseAdminMetadataTest, GetBackup) {
   EXPECT_CALL(*mock_, GetBackup)
-      .WillOnce(
-          [this](grpc::ClientContext& context, gcsa::GetBackupRequest const&) {
-            EXPECT_STATUS_OK(IsContextMDValid(
-                context,
-                "google.spanner.admin.database.v1.DatabaseAdmin."
-                "GetBackup",
-                expected_api_client_header_));
-            return TransientError();
-          });
+      .WillOnce([this](grpc::ClientContext& context,
+                       gsad::v1::GetBackupRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "GetBackup",
+                         request);
+        return TransientError();
+      });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::GetBackupRequest request;
+  gsad::v1::GetBackupRequest request;
   request.set_name(
       google::cloud::spanner::Backup(
           google::cloud::spanner::Instance(
@@ -313,18 +306,17 @@ TEST_F(DatabaseAdminMetadataTest, GetBackup) {
 TEST_F(DatabaseAdminMetadataTest, DeleteBackup) {
   EXPECT_CALL(*mock_, DeleteBackup)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::DeleteBackupRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "DeleteBackup",
-                             expected_api_client_header_));
+                       gsad::v1::DeleteBackupRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "DeleteBackup",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::DeleteBackupRequest request;
+  gsad::v1::DeleteBackupRequest request;
   request.set_name(
       google::cloud::spanner::Backup(
           google::cloud::spanner::Instance(
@@ -338,18 +330,17 @@ TEST_F(DatabaseAdminMetadataTest, DeleteBackup) {
 TEST_F(DatabaseAdminMetadataTest, ListBackups) {
   EXPECT_CALL(*mock_, ListBackups)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::ListBackupsRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "ListBackups",
-                             expected_api_client_header_));
+                       gsad::v1::ListBackupsRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "ListBackups",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::ListBackupsRequest request;
+  gsad::v1::ListBackupsRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
@@ -361,18 +352,17 @@ TEST_F(DatabaseAdminMetadataTest, ListBackups) {
 TEST_F(DatabaseAdminMetadataTest, UpdateBackup) {
   EXPECT_CALL(*mock_, UpdateBackup)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::UpdateBackupRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "UpdateBackup",
-                             expected_api_client_header_));
+                       gsad::v1::UpdateBackupRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "UpdateBackup",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::UpdateBackupRequest request;
+  gsad::v1::UpdateBackupRequest request;
   request.mutable_backup()->set_name(
       google::cloud::spanner::Backup(
           google::cloud::spanner::Instance(
@@ -386,18 +376,17 @@ TEST_F(DatabaseAdminMetadataTest, UpdateBackup) {
 TEST_F(DatabaseAdminMetadataTest, ListBackupOperations) {
   EXPECT_CALL(*mock_, ListBackupOperations)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::ListBackupOperationsRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "ListBackupOperations",
-                             expected_api_client_header_));
+                       gsad::v1::ListBackupOperationsRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "ListBackupOperations",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::ListBackupOperationsRequest request;
+  gsad::v1::ListBackupOperationsRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
@@ -409,18 +398,17 @@ TEST_F(DatabaseAdminMetadataTest, ListBackupOperations) {
 TEST_F(DatabaseAdminMetadataTest, ListDatabaseOperations) {
   EXPECT_CALL(*mock_, ListDatabaseOperations)
       .WillOnce([this](grpc::ClientContext& context,
-                       gcsa::ListDatabaseOperationsRequest const&) {
-        EXPECT_STATUS_OK(
-            IsContextMDValid(context,
-                             "google.spanner.admin.database.v1.DatabaseAdmin."
-                             "ListDatabaseOperations",
-                             expected_api_client_header_));
+                       gsad::v1::ListDatabaseOperationsRequest const& request) {
+        IsContextMDValid(context,
+                         "google.spanner.admin.database.v1.DatabaseAdmin."
+                         "ListDatabaseOperations",
+                         request);
         return TransientError();
       });
 
   DatabaseAdminMetadata stub(mock_);
   grpc::ClientContext context;
-  gcsa::ListDatabaseOperationsRequest request;
+  gsad::v1::ListDatabaseOperationsRequest request;
   request.set_parent(
       google::cloud::spanner::Instance(
           google::cloud::Project("test-project-id"), "test-instance-id")
@@ -431,12 +419,11 @@ TEST_F(DatabaseAdminMetadataTest, ListDatabaseOperations) {
 
 TEST_F(DatabaseAdminMetadataTest, GetOperation) {
   EXPECT_CALL(*mock_, AsyncGetOperation)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       google::longrunning::GetOperationRequest const&) {
-        EXPECT_STATUS_OK(IsContextMDValid(
-            *context, "google.longrunning.Operations.GetOperation",
-            expected_api_client_header_));
+      .WillOnce([this](
+                    CompletionQueue&, auto context,
+                    google::longrunning::GetOperationRequest const& request) {
+        IsContextMDValid(*context, "google.longrunning.Operations.GetOperation",
+                         request);
         return make_ready_future(
             StatusOr<google::longrunning::Operation>(TransientError()));
       });
@@ -446,27 +433,27 @@ TEST_F(DatabaseAdminMetadataTest, GetOperation) {
   google::longrunning::GetOperationRequest request;
   request.set_name("operations/fake-operation-name");
   auto response = stub.AsyncGetOperation(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), response.get().status());
 }
 
 TEST_F(DatabaseAdminMetadataTest, CancelOperation) {
   EXPECT_CALL(*mock_, AsyncCancelOperation)
-      .WillOnce([this](CompletionQueue&,
-                       std::unique_ptr<grpc::ClientContext> context,
-                       google::longrunning::CancelOperationRequest const&) {
-        EXPECT_STATUS_OK(IsContextMDValid(
-            *context, "google.longrunning.Operations.CancelOperation",
-            expected_api_client_header_));
-        return make_ready_future(TransientError());
-      });
+      .WillOnce(
+          [this](CompletionQueue&, auto context,
+                 google::longrunning::CancelOperationRequest const& request) {
+            IsContextMDValid(*context,
+                             "google.longrunning.Operations.CancelOperation",
+                             request);
+            return make_ready_future(TransientError());
+          });
 
   DatabaseAdminMetadata stub(mock_);
   CompletionQueue cq;
   google::longrunning::CancelOperationRequest request;
   request.set_name("operations/fake-operation-name");
   auto status = stub.AsyncCancelOperation(
-      cq, absl::make_unique<grpc::ClientContext>(), request);
+      cq, std::make_shared<grpc::ClientContext>(), request);
   EXPECT_EQ(TransientError(), status.get());
 }
 
