@@ -36,8 +36,7 @@ void SetBasicDefaults(Options& opts) {
       std::move(opts), "GOOGLE_CLOUD_CPP_SPANNER_DEFAULT_ENDPOINT",
       "SPANNER_EMULATOR_HOST", "GOOGLE_CLOUD_CPP_SPANNER_DEFAULT_AUTHORITY",
       "spanner.googleapis.com");
-  opts =
-      internal::PopulateGrpcOptions(std::move(opts), "SPANNER_EMULATOR_HOST");
+  opts = internal::PopulateGrpcOptions(std::move(opts));
   if (!opts.has<GrpcNumChannelsOption>()) {
     opts.set<GrpcNumChannelsOption>(4);
   }
@@ -127,6 +126,15 @@ Options DefaultOptions(Options opts) {
 Options DefaultAdminOptions(Options opts) {
   SetBasicDefaults(opts);
 
+  // Manually default `GrpcCredentialOption`, because the legacy admin stubs do
+  // not support GUAC (aka `UnifiedCredentialsOption`).
+  auto e = internal::GetEnv("SPANNER_EMULATOR_HOST");
+  if (e && !e->empty()) {
+    opts.set<GrpcCredentialOption>(grpc::InsecureChannelCredentials());
+  } else if (!opts.has<GrpcCredentialOption>()) {
+    opts.set<GrpcCredentialOption>(grpc::GoogleDefaultCredentials());
+  }
+
   if (!opts.has<spanner::SpannerRetryPolicyOption>()) {
     opts.set<spanner::SpannerRetryPolicyOption>(
         std::make_shared<google::cloud::spanner::LimitedTimeRetryPolicy>(
