@@ -19,6 +19,7 @@
 #include "google/cloud/spanner/options.h"
 #include "google/cloud/completion_queue.h"
 #include "google/cloud/internal/async_retry_loop.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/retry_loop.h"
 #include "google/cloud/log.h"
 #include "google/cloud/options.h"
@@ -77,6 +78,7 @@ SessionPool::SessionPool(spanner::Database db,
 
 void SessionPool::Initialize() {
   internal::OptionsSpan span(opts_);
+  CreateMultiplexedSession();
   auto const min_sessions = opts_.get<spanner::SessionPoolMinSessionsOption>();
   if (min_sessions > 0) {
     std::unique_lock<std::mutex> lk(mu_);
@@ -97,6 +99,18 @@ SessionPool::~SessionPool() {
   // `weak_ptr` to `this` they hold. Any in-progress or subsequent `lock()`
   // will now return `nullptr`, in which case no work is done.
   current_timer_.cancel();
+
+  // Send fire-and-forget `AsyncDeleteSession()` calls for all sessions.
+  if (HasValidMultiplexedSession(std::unique_lock<std::mutex>(mu_))) {
+    AsyncDeleteSession(cq_, GetStub(*multiplexed_session_),
+                       multiplexed_session_->session_name())
+        .then([](auto result) { auto status = result.get(); });
+  }
+  for (auto const& session : sessions_) {
+    if (session->is_bad()) continue;
+    AsyncDeleteSession(cq_, GetStub(*session), session->session_name())
+        .then([](auto result) { auto status = result.get(); });
+  }
 }
 
 void SessionPool::ScheduleBackgroundWork(std::chrono::seconds relative_time) {
@@ -122,8 +136,9 @@ void SessionPool::DoBackgroundWork() {
 // Ensure the pool size conforms to what was specified in the `SessionOptions`,
 // creating or deleting sessions as necessary.
 void SessionPool::MaintainPoolSize() {
-  std::unique_lock<std::mutex> lk(mu_);
+  CreateMultiplexedSession();
   auto const min_sessions = opts_.get<spanner::SessionPoolMinSessionsOption>();
+  std::unique_lock<std::mutex> lk(mu_);
   if (create_calls_in_progress_ == 0 && total_sessions_ < min_sessions) {
     Grow(lk, total_sessions_ - min_sessions, WaitForSessionAllocation::kNoWait);
   }
@@ -189,6 +204,51 @@ void SessionPool::Erase(std::string const& session_name) {
   }
 }
 
+Status SessionPool::CreateMultiplexedSession() {
+  std::unique_lock<std::mutex> lk(mu_);
+  if (!HasValidMultiplexedSession(lk)) {
+    auto stub = GetStub(std::move(lk));
+    auto name = CreateMultiplexedSession(std::move(stub));
+    if (!name) return name.status();
+    auto session = std::make_shared<Session>(*std::move(name),
+                                             /*channel=*/nullptr, clock_);
+    std::unique_lock<std::mutex> lk(mu_);
+    multiplexed_session_ = std::move(session);
+  }
+  return Status{};
+}
+
+StatusOr<std::string> SessionPool::CreateMultiplexedSession(
+    std::shared_ptr<SpannerStub> stub) const {
+  google::spanner::v1::CreateSessionRequest request;
+  request.set_database(db_.FullName());
+  auto* session = request.mutable_session();
+  auto const& labels = opts_.get<spanner::SessionPoolLabelsOption>();
+  if (!labels.empty()) {
+    session->mutable_labels()->insert(labels.begin(), labels.end());
+  }
+  auto const& role = opts_.get<spanner::SessionCreatorRoleOption>();
+  if (!role.empty()) session->set_creator_role(role);
+  session->set_multiplexed(true);
+
+  auto response = RetryLoop(
+      retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
+      google::cloud::Idempotency::kIdempotent,
+      [&stub](grpc::ClientContext& context, Options const& options,
+              google::spanner::v1::CreateSessionRequest const& request) {
+        RouteToLeader(context);  // always for CreateSession()
+        return stub->CreateSession(context, options, request);
+      },
+      opts_, request, __func__);
+  if (!response) return std::move(response).status();
+  return response->name();
+}
+
+bool SessionPool::HasValidMultiplexedSession(
+    std::unique_lock<std::mutex> const&) const {
+  return multiplexed_session_ && !multiplexed_session_->is_bad();
+}
+
 /*
  * Grow the session pool by creating up to `sessions_to_create` sessions and
  * adding them to the pool.  Note that `lk` may be released and reacquired in
@@ -218,7 +278,8 @@ StatusOr<std::vector<SessionPool::CreateCount>>
 SessionPool::ComputeCreateCounts(int sessions_to_create) {
   if (total_sessions_ == max_pool_size_) {
     // Can't grow the pool since we're already at max size.
-    return Status(StatusCode::kResourceExhausted, "session pool exhausted");
+    return internal::ResourceExhaustedError("session pool exhausted",
+                                            GCP_ERROR_INFO());
   }
 
   // Compute how many Sessions to create on each Channel, trying to keep the
@@ -288,11 +349,32 @@ Status SessionPool::CreateSessions(
 }
 
 StatusOr<SessionHolder> SessionPool::Allocate(bool dissociate_from_pool) {
+  return Allocate(std::unique_lock<std::mutex>(mu_), dissociate_from_pool);
+}
+
+StatusOr<SessionHolder> SessionPool::Multiplexed() {
+  std::unique_lock<std::mutex> lk(mu_);
+  // If we don't have a multiplexed session (yet), use a regular one.
+  if (!HasValidMultiplexedSession(lk)) return Allocate(std::move(lk), false);
+  return multiplexed_session_;
+}
+
+std::shared_ptr<SpannerStub> SessionPool::GetStub(Session const& session) {
+  auto const& channel = session.channel();
+  if (channel) return channel->stub;
+
+  // Multiplexed sessions, or sessions that were created for partitioned
+  // Reads/Queries, do not have their own channel/stub, so return a stub
+  // to use by round-robining between the channels.
+  return GetStub(std::unique_lock<std::mutex>(mu_));
+}
+
+StatusOr<SessionHolder> SessionPool::Allocate(std::unique_lock<std::mutex> lk,
+                                              bool dissociate_from_pool) {
   // We choose to ignore the internal::CurrentOptions() here as it is
   // non-deterministic when RPCs to create sessions are actually made.
   // It is clearer if we just stick with the construction-time Options.
   internal::OptionsSpan span(opts_);
-  std::unique_lock<std::mutex> lk(mu_);
   for (;;) {
     if (!sessions_.empty()) {
       // return the most recently used session.
@@ -313,7 +395,8 @@ StatusOr<SessionHolder> SessionPool::Allocate(bool dissociate_from_pool) {
     if (total_sessions_ >= max_pool_size_) {
       if (opts_.get<spanner::SessionPoolActionOnExhaustionOption>() ==
           spanner::ActionOnExhaustion::kFail) {
-        return Status(StatusCode::kResourceExhausted, "session pool exhausted");
+        return internal::ResourceExhaustedError("session pool exhausted",
+                                                GCP_ERROR_INFO());
       }
       Wait(lk, [this] {
         return !sessions_.empty() || total_sessions_ < max_pool_size_;
@@ -345,16 +428,8 @@ StatusOr<SessionHolder> SessionPool::Allocate(bool dissociate_from_pool) {
   }
 }
 
-std::shared_ptr<SpannerStub> SessionPool::GetStub(Session const& session) {
-  auto const& channel = session.channel();
-  if (channel) {
-    return channel->stub;
-  }
-
-  // Sessions that were created for partitioned Reads/Queries do not have
-  // their own channel/stub; return a stub to use by round-robining between
-  // the channels.
-  std::unique_lock<std::mutex> lk(mu_);
+std::shared_ptr<SpannerStub> SessionPool::GetStub(
+    std::unique_lock<std::mutex>) {
   auto stub = (*next_dissociated_stub_channel_)->stub;
   if (++next_dissociated_stub_channel_ == channels_.end()) {
     next_dissociated_stub_channel_ = channels_.begin();
@@ -398,15 +473,16 @@ Status SessionPool::CreateSessionsSync(
   }
   request.set_session_count(std::int32_t{num_sessions});
   auto const& stub = channel->stub;
+  auto const& current = internal::CurrentOptions();
   auto response = RetryLoop(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       google::cloud::Idempotency::kIdempotent,
-      [&stub](grpc::ClientContext& context,
+      [&stub](grpc::ClientContext& context, Options const& options,
               google::spanner::v1::BatchCreateSessionsRequest const& request) {
         RouteToLeader(context);  // always for BatchCreateSessions()
-        return stub->BatchCreateSessions(context, request);
+        return stub->BatchCreateSessions(context, options, request);
       },
-      request, __func__);
+      current, request, __func__);
   return HandleBatchCreateSessionsDone(channel, std::move(response));
 }
 
@@ -462,11 +538,13 @@ SessionPool::AsyncBatchCreateSessions(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
       [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             internal::ImmutableOptions options,
              google::spanner::v1::BatchCreateSessionsRequest const& request) {
         RouteToLeader(*context);  // always for BatchCreateSessions()
-        return stub->AsyncBatchCreateSessions(cq, std::move(context), request);
+        return stub->AsyncBatchCreateSessions(cq, std::move(context),
+                                              std::move(options), request);
       },
-      std::move(request), __func__);
+      internal::SaveCurrentOptions(), std::move(request), __func__);
 }
 
 future<Status> SessionPool::AsyncDeleteSession(
@@ -478,10 +556,12 @@ future<Status> SessionPool::AsyncDeleteSession(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
       [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             google::cloud::internal::ImmutableOptions options,
              google::spanner::v1::DeleteSessionRequest const& request) {
-        return stub->AsyncDeleteSession(cq, std::move(context), request);
+        return stub->AsyncDeleteSession(cq, std::move(context),
+                                        std::move(options), request);
       },
-      std::move(request), __func__);
+      internal::SaveCurrentOptions(), std::move(request), __func__);
 }
 
 /// Refresh the session `session_name` by executing a `SELECT 1` query on it.
@@ -499,11 +579,13 @@ SessionPool::AsyncRefreshSession(CompletionQueue& cq,
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
       [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             google::cloud::internal::ImmutableOptions options,
              google::spanner::v1::ExecuteSqlRequest const& request) {
         // Read-only transaction, so no route-to-leader.
-        return stub->AsyncExecuteSql(cq, std::move(context), request);
+        return stub->AsyncExecuteSql(cq, std::move(context), std::move(options),
+                                     request);
       },
-      std::move(request), __func__);
+      internal::SaveCurrentOptions(), std::move(request), __func__);
 }
 
 Status SessionPool::HandleBatchCreateSessionsDone(
