@@ -73,12 +73,17 @@ class SessionPoolTest : public testing::Test {
   testing_util::ValidateMetadataFixture validate_metadata_fixture_;
 };
 
-// Matches a BatchCreateSessionsRequest with the specified `database_name`.
-MATCHER_P(DatabaseIs, database_name,
-          "BatchCreateSessionsRequest has expected database name") {
+// Matches a CreateSessionRequest or BatchCreateSessionsRequest with the
+// specified `database_name`.
+MATCHER_P(DatabaseIs, database_name, "request has expected database name") {
   return arg.database() == database_name;
 }
 
+// Matches a multiplexed CreateSessionRequest.
+MATCHER(IsMultiplexed, "is a multiplexed CreateSessionRequest") {
+  return arg.session().multiplexed();
+}
+
 // Matches a BatchCreateSessionsRequest with the specified `labels`.
 MATCHER_P(LabelsAre, labels, "BatchCreateSessionsRequest has expected labels") {
   auto const& arg_labels = arg.session_template().labels();
@@ -97,12 +102,30 @@ MATCHER_P(SessionCountIs, session_count,
   return arg.session_count() == session_count;
 }
 
+// Matches a DeleteSessionRequest with the specified `name`.
+MATCHER_P(SessionNameIs, name,
+          "DeleteSessionRequest has expected session name") {
+  return arg.name() == name;
+}
+
 google::protobuf::Timestamp Now() {
   auto now = spanner::MakeTimestamp(absl::Now()).value();
   return now.get<google::protobuf::Timestamp>().value();
 }
 
-// Create a response with the given `sessions`
+// Create a session with the given `name`.
+google::spanner::v1::Session MakeMultiplexedSession(std::string name,
+                                                    std::string role = "") {
+  google::spanner::v1::Session session;
+  session.set_name(std::move(name));
+  *session.mutable_create_time() = Now();
+  *session.mutable_approximate_last_use_time() = Now();
+  if (!role.empty()) session.set_creator_role(std::move(role));
+  session.set_multiplexed(true);
+  return session;
+}
+
+// Create a response with the given `sessions`.
 google::spanner::v1::BatchCreateSessionsResponse MakeSessionsResponse(
     std::vector<std::string> sessions, std::string role = "") {
   google::spanner::v1::BatchCreateSessionsResponse response;
@@ -130,17 +153,70 @@ std::shared_ptr<SessionPool> MakeTestSessionPool(
                          std::move(opts));
 }
 
+TEST_F(SessionPoolTest, Multiplexed) {
+  auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
+  auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
+  EXPECT_CALL(*mock, BatchCreateSessions)
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
+  auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
+  auto session = pool->Multiplexed();
+  ASSERT_STATUS_OK(session);
+  EXPECT_EQ((*session)->session_name(), "multiplexed");
+}
+
+TEST_F(SessionPoolTest, MultiplexedFallback) {
+  auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
+  auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
+  EXPECT_CALL(*mock, BatchCreateSessions)
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
+  auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
+  auto session = pool->Multiplexed();
+  ASSERT_STATUS_OK(session);
+  EXPECT_EQ((*session)->session_name(), "session1");
+}
+
 TEST_F(SessionPoolTest, AllocateRouteToLeader) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
-                                                  SessionCountIs(42))))
-      .WillOnce([this](grpc::ClientContext& context,
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
+                       google::spanner::v1::CreateSessionRequest const&) {
+        EXPECT_THAT(GetMetadata(context),
+                    Contains(Pair(kRouteToLeader, "true")));
+        return MakeMultiplexedSession("multiplexed");
+      });
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(
+                  _, _, AllOf(DatabaseIs(db.FullName()), SessionCountIs(42))))
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::spanner::v1::BatchCreateSessionsRequest const&) {
         EXPECT_THAT(GetMetadata(context),
                     Contains(Pair(kRouteToLeader, "true")));
         return MakeSessionsResponse({"session1"});
       });
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool =
@@ -157,15 +233,30 @@ TEST_F(SessionPoolTest, AllocateRouteToLeader) {
 TEST_F(SessionPoolTest, AllocateNoRouteToLeader) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
-                                                  SessionCountIs(42))))
-      .WillOnce([this](grpc::ClientContext& context,
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
+                       google::spanner::v1::CreateSessionRequest const&) {
+        EXPECT_THAT(GetMetadata(context),
+                    AnyOf(Contains(Pair(kRouteToLeader, "false")),
+                          Not(Contains(Pair(kRouteToLeader, _)))));
+        return MakeMultiplexedSession("multiplexed");
+      });
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(
+                  _, _, AllOf(DatabaseIs(db.FullName()), SessionCountIs(42))))
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::spanner::v1::BatchCreateSessionsRequest const&) {
         EXPECT_THAT(GetMetadata(context),
                     AnyOf(Contains(Pair(kRouteToLeader, "false")),
                           Not(Contains(Pair(kRouteToLeader, _)))));
         return MakeSessionsResponse({"session1"});
       });
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool =
@@ -182,12 +273,24 @@ TEST_F(SessionPoolTest, AllocateNoRouteToLeader) {
 TEST_F(SessionPoolTest, ReleaseBadSession) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
-                                                  SessionCountIs(1))))
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(
+                  _, _, AllOf(DatabaseIs(db.FullName()), SessionCountIs(1))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
-  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
-                                                  SessionCountIs(2))))
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(
+                  _, _, AllOf(DatabaseIs(db.FullName()), SessionCountIs(2))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session2"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .Times(0);
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session2")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -214,9 +317,13 @@ TEST_F(SessionPoolTest, ReleaseBadSession) {
 TEST_F(SessionPoolTest, CreateError) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(Status(StatusCode::kInternal, "init failure"))))
       .WillOnce(Return(ByMove(Status(StatusCode::kInternal, "some failure"))));
+  EXPECT_CALL(*mock, AsyncDeleteSession).Times(0);
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
@@ -228,8 +335,14 @@ TEST_F(SessionPoolTest, CreateError) {
 TEST_F(SessionPoolTest, ReuseSession) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
@@ -246,9 +359,17 @@ TEST_F(SessionPoolTest, ReuseSession) {
 TEST_F(SessionPoolTest, Lifo) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session2"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session2")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
@@ -278,8 +399,18 @@ TEST_F(SessionPoolTest, MinSessionsEagerAllocation) {
   int const min_sessions = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions)))
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
+  EXPECT_CALL(*mock, BatchCreateSessions(_, _, SessionCountIs(min_sessions)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3", "s2", "s1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -292,12 +423,31 @@ TEST_F(SessionPoolTest, MinSessionsMultipleAllocations) {
   int const min_sessions = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   // The constructor will make this call.
-  EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions)))
+  EXPECT_CALL(*mock, BatchCreateSessions(_, _, SessionCountIs(min_sessions)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3", "s2", "s1"}))));
   // When we run out of sessions it will make this call.
-  EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions + 1)))
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(_, _, SessionCountIs(min_sessions + 1)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s7", "s6", "s5", "s4"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s4")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s5")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s6")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s7")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -319,10 +469,20 @@ TEST_F(SessionPoolTest, MaxSessionsFailOnExhaustion) {
   int const max_sessions_per_channel = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -350,8 +510,14 @@ TEST_F(SessionPoolTest, MaxSessionsBlockUntilRelease) {
   int const max_sessions_per_channel = 1;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -381,8 +547,24 @@ TEST_F(SessionPoolTest, Labels) {
   auto db = spanner::Database("project", "instance", "database");
   std::map<std::string, std::string> labels = {
       {"k1", "v1"}, {"k2", "v2"}, {"k3", "v3"}};
-  EXPECT_CALL(*mock, BatchCreateSessions(_, LabelsAre(labels)))
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce(
+          [labels](grpc::ClientContext&, Options const&,
+                   google::spanner::v1::CreateSessionRequest const& request) {
+            auto const& request_labels = request.session().labels();
+            EXPECT_EQ((std::map<std::string, std::string>(
+                          request_labels.begin(), request_labels.end())),
+                      labels);
+            return MakeMultiplexedSession("multiplexed");
+          });
+  EXPECT_CALL(*mock, BatchCreateSessions(_, _, LabelsAre(labels)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -397,9 +579,23 @@ TEST_F(SessionPoolTest, CreatorRole) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   std::string const role = "public";
-  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
-                                                  CreatorRoleIs(role))))
+  EXPECT_CALL(
+      *mock,
+      CreateSession(_, _, AllOf(DatabaseIs(db.FullName()), IsMultiplexed())))
+      .WillOnce(
+          [role](grpc::ClientContext&, Options const&,
+                 google::spanner::v1::CreateSessionRequest const& request) {
+            EXPECT_EQ(request.session().creator_role(), role);
+            return MakeMultiplexedSession("multiplexed");
+          });
+  EXPECT_CALL(*mock,
+              BatchCreateSessions(
+                  _, _, AllOf(DatabaseIs(db.FullName()), CreatorRoleIs(role))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}, role))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("session1")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
@@ -414,14 +610,32 @@ TEST_F(SessionPoolTest, MultipleChannels) {
   auto mock1 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto mock2 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock1, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock1, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c1s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c1s2"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c1s3"}))));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock2, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c2s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c2s2"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c2s3"}))));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(db, {mock1, mock2}, threads.cq());
@@ -442,12 +656,39 @@ TEST_F(SessionPoolTest, MultipleChannelsPreAllocation) {
   auto mock2 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto mock3 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock1, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock1, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c1s1", "c1s2", "c1s3"}))));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock1, AsyncDeleteSession(_, _, _, SessionNameIs("c1s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock2, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c2s1", "c2s2", "c2s3"}))));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock2, AsyncDeleteSession(_, _, _, SessionNameIs("c2s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock3, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   EXPECT_CALL(*mock3, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c3s1", "c3s2", "c3s3"}))));
+  EXPECT_CALL(*mock3, AsyncDeleteSession(_, _, _, SessionNameIs("c3s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock3, AsyncDeleteSession(_, _, _, SessionNameIs("c3s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock3, AsyncDeleteSession(_, _, _, SessionNameIs("c3s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   // note that min_sessions will effectively be reduced to 9
@@ -477,6 +718,9 @@ TEST_F(SessionPoolTest, MultipleChannelsPreAllocation) {
 TEST_F(SessionPoolTest, GetStubForStublessSession) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, CreateSession)
+      .WillRepeatedly(
+          Return(ByMove(Status(StatusCode::kInternal, "init failure"))));
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   auto pool = MakeTestSessionPool(
       db, {mock}, threads.cq(),
@@ -488,9 +732,17 @@ TEST_F(SessionPoolTest, GetStubForStublessSession) {
 
 TEST_F(SessionPoolTest, SessionRefresh) {
   auto mock = std::make_shared<StrictMock<spanner_testing::MockSpannerStub>>();
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s2")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   google::spanner::v1::ResultSet result;
   auto constexpr kResultSetText = R"pb(
@@ -504,7 +756,7 @@ TEST_F(SessionPoolTest, SessionRefresh) {
 
   EXPECT_CALL(*mock, AsyncExecuteSql)
       .WillOnce(
-          [&result](CompletionQueue&, auto,
+          [&result](CompletionQueue&, auto, auto,
                     google::spanner::v1::ExecuteSqlRequest const& request) {
             EXPECT_EQ("s2", request.session());
             return make_ready_future(make_status_or(std::move(result)));
@@ -547,17 +799,32 @@ TEST_F(SessionPoolTest, SessionRefresh) {
   auto s2 = pool->Allocate();
   ASSERT_STATUS_OK(s2);
   EXPECT_EQ("s2", (*s2)->session_name());
+
+  // Cancel all pending operations, satisfying any remaining futures. When
+  // compiling with exceptions disabled the destructors eventually invoke
+  // `std::abort()`. On real programs, shutting down the completion queue
+  // will have the same effect.
+  impl->SimulateCompletion(false);
 }
 
 TEST_F(SessionPoolTest, SessionRefreshNotFound) {
   auto mock = std::make_shared<StrictMock<spanner_testing::MockSpannerStub>>();
+  EXPECT_CALL(*mock, CreateSession)
+      .WillOnce(Return(ByMove(MakeMultiplexedSession({"multiplexed"}))));
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3"}))));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("multiplexed")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s1")))
+      .WillOnce(Return(make_ready_future(Status{})));
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s2"))).Times(0);
+  EXPECT_CALL(*mock, AsyncDeleteSession(_, _, _, SessionNameIs("s3")))
+      .WillOnce(Return(make_ready_future(Status{})));
 
   EXPECT_CALL(*mock, AsyncExecuteSql)
-      .WillOnce([](CompletionQueue&, auto,
+      .WillOnce([](CompletionQueue&, auto, auto,
                    google::spanner::v1::ExecuteSqlRequest const& request) {
         EXPECT_EQ("s2", request.session());
         // The "SELECT 1" refresh returns "Session not found".
@@ -604,6 +871,12 @@ TEST_F(SessionPoolTest, SessionRefreshNotFound) {
   auto s3 = pool->Allocate();
   ASSERT_STATUS_OK(s3);
   EXPECT_EQ("s3", (*s3)->session_name());
+
+  // Cancel all pending operations, satisfying any remaining futures. When
+  // compiling with exceptions disabled the destructors eventually invoke
+  // `std::abort()`. In non-test programs, the completion queue does this
+  // automatically as part of its shutdown.
+  impl->SimulateCompletion(false);
 }
 
 }  // namespace
