@@ -21,8 +21,12 @@
 #include "google/cloud/spanner/json.h"
 #include "google/cloud/spanner/numeric.h"
 #include "google/cloud/spanner/oid.h"
+#include "google/cloud/spanner/proto_enum.h"
+#include "google/cloud/spanner/proto_message.h"
 #include "google/cloud/spanner/timestamp.h"
 #include "google/cloud/spanner/version.h"
+#include "google/cloud/internal/base64_transforms.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/throw_delegate.h"
 #include "google/cloud/optional.h"
 #include "google/cloud/status_or.h"
@@ -62,6 +66,7 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
  * ------------ | ------------
  * BOOL         | `bool`
  * INT64        | `std::int64_t`
+ * FLOAT32      | `float`
  * FLOAT64      | `double`
  * STRING       | `std::string`
  * BYTES        | `google::cloud::spanner::Bytes`
@@ -72,6 +77,8 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
  * OID(PG)      | `google::cloud::spanner::PgOid`
  * TIMESTAMP    | `google::cloud::spanner::Timestamp`
  * DATE         | `absl::CivilDay`
+ * ENUM         | `google::cloud::spanner::ProtoEnum<E>`
+ * PROTO        | `google::cloud::spanner::ProtoMessage<M>`
  * ARRAY        | `std::vector<T>`  // [1]
  * STRUCT       | `std::tuple<Ts...>`
  *
@@ -189,6 +196,8 @@ class Value {
   /// @copydoc Value(bool)
   explicit Value(std::int64_t v) : Value(PrivateConstructor{}, v) {}
   /// @copydoc Value(bool)
+  explicit Value(float v) : Value(PrivateConstructor{}, v) {}
+  /// @copydoc Value(bool)
   explicit Value(double v) : Value(PrivateConstructor{}, v) {}
   /// @copydoc Value(bool)
   explicit Value(std::string v) : Value(PrivateConstructor{}, std::move(v)) {}
@@ -212,6 +221,13 @@ class Value {
   /// @copydoc Value(bool)
   explicit Value(absl::CivilDay v)
       : Value(PrivateConstructor{}, std::move(v)) {}
+  /// @copydoc Value(bool)
+  template <typename E>
+  explicit Value(ProtoEnum<E> v) : Value(PrivateConstructor{}, std::move(v)) {}
+  /// @copydoc Value(bool)
+  template <typename M>
+  explicit Value(ProtoMessage<M> v)
+      : Value(PrivateConstructor{}, std::move(v)) {}
 
   /**
    * Constructs an instance from common C++ literal types that closely, though
@@ -306,10 +322,10 @@ class Value {
   template <typename T>
   StatusOr<T> get() const& {
     if (!TypeProtoIs(T{}, type_))
-      return Status(StatusCode::kUnknown, "wrong type");
+      return internal::UnknownError("wrong type", GCP_ERROR_INFO());
     if (value_.kind_case() == google::protobuf::Value::kNullValue) {
       if (IsOptional<T>::value) return T{};
-      return Status(StatusCode::kUnknown, "null value");
+      return internal::UnknownError("null value", GCP_ERROR_INFO());
     }
     return GetValue(T{}, value_, type_);
   }
@@ -318,10 +334,10 @@ class Value {
   template <typename T>
   StatusOr<T> get() && {
     if (!TypeProtoIs(T{}, type_))
-      return Status(StatusCode::kUnknown, "wrong type");
+      return internal::UnknownError("wrong type", GCP_ERROR_INFO());
     if (value_.kind_case() == google::protobuf::Value::kNullValue) {
       if (IsOptional<T>::value) return T{};
-      return Status(StatusCode::kUnknown, "null value");
+      return internal::UnknownError("null value", GCP_ERROR_INFO());
     }
     auto tag = T{};  // Works around an odd msvc issue
     return GetValue(std::move(tag), std::move(value_), type_);
@@ -366,6 +382,7 @@ class Value {
   // by the given `Type` proto.
   static bool TypeProtoIs(bool, google::spanner::v1::Type const&);
   static bool TypeProtoIs(std::int64_t, google::spanner::v1::Type const&);
+  static bool TypeProtoIs(float, google::spanner::v1::Type const&);
   static bool TypeProtoIs(double, google::spanner::v1::Type const&);
   static bool TypeProtoIs(Timestamp, google::spanner::v1::Type const&);
   static bool TypeProtoIs(CommitTimestamp, google::spanner::v1::Type const&);
@@ -377,6 +394,18 @@ class Value {
   static bool TypeProtoIs(Numeric const&, google::spanner::v1::Type const&);
   static bool TypeProtoIs(PgNumeric const&, google::spanner::v1::Type const&);
   static bool TypeProtoIs(PgOid const&, google::spanner::v1::Type const&);
+  template <typename E>
+  static bool TypeProtoIs(ProtoEnum<E> const&,
+                          google::spanner::v1::Type const& type) {
+    return type.code() == google::spanner::v1::TypeCode::ENUM &&
+           type.proto_type_fqn() == ProtoEnum<E>::TypeName();
+  }
+  template <typename M>
+  static bool TypeProtoIs(ProtoMessage<M> const&,
+                          google::spanner::v1::Type const& type) {
+    return type.code() == google::spanner::v1::TypeCode::PROTO &&
+           type.proto_type_fqn() == ProtoMessage<M>::TypeName();
+  }
   template <typename T>
   static bool TypeProtoIs(absl::optional<T>,
                           google::spanner::v1::Type const& type) {
@@ -419,6 +448,7 @@ class Value {
   // argument type is the tag, the argument value is ignored.
   static google::spanner::v1::Type MakeTypeProto(bool);
   static google::spanner::v1::Type MakeTypeProto(std::int64_t);
+  static google::spanner::v1::Type MakeTypeProto(float);
   static google::spanner::v1::Type MakeTypeProto(double);
   static google::spanner::v1::Type MakeTypeProto(std::string const&);
   static google::spanner::v1::Type MakeTypeProto(Bytes const&);
@@ -430,6 +460,20 @@ class Value {
   static google::spanner::v1::Type MakeTypeProto(Timestamp);
   static google::spanner::v1::Type MakeTypeProto(CommitTimestamp);
   static google::spanner::v1::Type MakeTypeProto(absl::CivilDay);
+  template <typename E>
+  static google::spanner::v1::Type MakeTypeProto(ProtoEnum<E>) {
+    google::spanner::v1::Type t;
+    t.set_code(google::spanner::v1::TypeCode::ENUM);
+    t.set_proto_type_fqn(ProtoEnum<E>::TypeName());
+    return t;
+  }
+  template <typename M>
+  static google::spanner::v1::Type MakeTypeProto(ProtoMessage<M>) {
+    google::spanner::v1::Type t;
+    t.set_code(google::spanner::v1::TypeCode::PROTO);
+    t.set_proto_type_fqn(ProtoMessage<M>::TypeName());
+    return t;
+  }
   static google::spanner::v1::Type MakeTypeProto(int);
   static google::spanner::v1::Type MakeTypeProto(char const*);
   template <typename T>
@@ -483,6 +527,7 @@ class Value {
   // https://github.com/googleapis/googleapis/blob/master/google/spanner/v1/type.proto
   static google::protobuf::Value MakeValueProto(bool b);
   static google::protobuf::Value MakeValueProto(std::int64_t i);
+  static google::protobuf::Value MakeValueProto(float f);
   static google::protobuf::Value MakeValueProto(double d);
   static google::protobuf::Value MakeValueProto(std::string s);
   static google::protobuf::Value MakeValueProto(Bytes b);
@@ -494,6 +539,16 @@ class Value {
   static google::protobuf::Value MakeValueProto(Timestamp ts);
   static google::protobuf::Value MakeValueProto(CommitTimestamp ts);
   static google::protobuf::Value MakeValueProto(absl::CivilDay d);
+  template <typename E>
+  static google::protobuf::Value MakeValueProto(ProtoEnum<E> e) {
+    return MakeValueProto(std::int64_t{E{e}});
+  }
+  template <typename M>
+  static google::protobuf::Value MakeValueProto(ProtoMessage<M> m) {
+    internal::Base64Encoder encoder;
+    for (auto c : std::string{m}) encoder.PushBack(c);
+    return MakeValueProto(std::move(encoder).FlushAndPad());
+  }
   static google::protobuf::Value MakeValueProto(int i);
   static google::protobuf::Value MakeValueProto(char const* s);
   template <typename T>
@@ -542,6 +597,8 @@ class Value {
   static StatusOr<std::int64_t> GetValue(std::int64_t,
                                          google::protobuf::Value const&,
                                          google::spanner::v1::Type const&);
+  static StatusOr<float> GetValue(float, google::protobuf::Value const&,
+                                  google::spanner::v1::Type const&);
   static StatusOr<double> GetValue(double, google::protobuf::Value const&,
                                    google::spanner::v1::Type const&);
   static StatusOr<std::string> GetValue(std::string const&,
@@ -572,6 +629,32 @@ class Value {
   static StatusOr<absl::CivilDay> GetValue(absl::CivilDay,
                                            google::protobuf::Value const&,
                                            google::spanner::v1::Type const&);
+  template <typename E>
+  static StatusOr<ProtoEnum<E>> GetValue(ProtoEnum<E>,
+                                         google::protobuf::Value const& pv,
+                                         google::spanner::v1::Type const& pt) {
+    if (pv.kind_case() != google::protobuf::Value::kStringValue) {
+      return internal::UnknownError("missing ENUM", GCP_ERROR_INFO());
+    }
+    auto value = GetValue(std::int64_t{}, pv, pt);
+    if (!value) return std::move(value).status();
+    if (static_cast<typename std::underlying_type_t<E>>(*value) != *value) {
+      return Status(StatusCode::kUnknown,
+                    "Value out of range: " + pv.string_value());
+    }
+    return ProtoEnum<E>(static_cast<E>(*std::move(value)));
+  }
+  template <typename M>
+  static StatusOr<ProtoMessage<M>> GetValue(ProtoMessage<M>,
+                                            google::protobuf::Value const& pv,
+                                            google::spanner::v1::Type const&) {
+    if (pv.kind_case() != google::protobuf::Value::kStringValue) {
+      return internal::UnknownError("missing PROTO", GCP_ERROR_INFO());
+    }
+    auto bytes = internal::Base64DecodeToBytes(pv.string_value());
+    if (!bytes) return std::move(bytes).status();
+    return ProtoMessage<M>(std::string(bytes->begin(), bytes->end()));
+  }
   template <typename T, typename V>
   static StatusOr<absl::optional<T>> GetValue(
       absl::optional<T> const&, V&& pv, google::spanner::v1::Type const& pt) {
@@ -586,7 +669,7 @@ class Value {
   static StatusOr<std::vector<T>> GetValue(
       std::vector<T> const&, V&& pv, google::spanner::v1::Type const& pt) {
     if (pv.kind_case() != google::protobuf::Value::kListValue) {
-      return Status(StatusCode::kUnknown, "missing ARRAY");
+      return internal::UnknownError("missing ARRAY", GCP_ERROR_INFO());
     }
     std::vector<T> v;
     for (int i = 0; i < pv.list_value().values().size(); ++i) {
@@ -602,7 +685,7 @@ class Value {
   static StatusOr<std::tuple<Ts...>> GetValue(
       std::tuple<Ts...> const&, V&& pv, google::spanner::v1::Type const& pt) {
     if (pv.kind_case() != google::protobuf::Value::kListValue) {
-      return Status(StatusCode::kUnknown, "missing STRUCT");
+      return internal::UnknownError("missing STRUCT", GCP_ERROR_INFO());
     }
     std::tuple<Ts...> tup;
     Status status;  // OK
