@@ -13,13 +13,17 @@
 // limitations under the License.
 
 #include "google/cloud/spanner/value.h"
+#include "google/cloud/internal/base64_transforms.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include "absl/time/time.h"
 #include "absl/types/optional.h"
+#include <google/cloud/spanner/testing/singer.pb.h>
 #include <google/protobuf/text_format.h>
 #include <gmock/gmock.h>
 #include <cmath>
+#include <cstdint>
+#include <iomanip>
 #include <ios>
 #include <limits>
 #include <string>
@@ -38,6 +42,7 @@ using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::HasSubstr;
+using ::testing::IsEmpty;
 using ::testing::Not;
 
 absl::Time MakeTime(std::int64_t sec, int nanos) {
@@ -65,6 +70,16 @@ std::vector<Timestamp> TestTimes() {
   return times;
 }
 
+testing::SingerInfo MakeSinger(std::int64_t singer_id, std::string birth_date,
+                               std::string nationality, testing::Genre genre) {
+  testing::SingerInfo singer;
+  singer.set_singer_id(singer_id);
+  singer.set_birth_date(std::move(birth_date));
+  singer.set_nationality(std::move(nationality));
+  singer.set_genre(genre);
+  return singer;
+}
+
 template <typename T>
 void TestBasicSemantics(T init) {
   Value const default_ctor{};
@@ -132,6 +147,7 @@ TEST(Value, BasicSemantics) {
 
   // Note: We skip testing the NaN case here because NaN always compares not
   // equal, even with itself. So NaN is handled in a separate test.
+  static_assert(std::numeric_limits<double>::has_infinity, "");
   auto const inf = std::numeric_limits<double>::infinity();
   for (auto x : {-inf, -1.0, -0.5, 0.0, 0.5, 1.0, inf}) {
     SCOPED_TRACE("Testing: double " + std::to_string(x));
@@ -142,6 +158,19 @@ TEST(Value, BasicSemantics) {
     TestBasicSemantics(v);
   }
 
+  // Note: We skip testing the NaN case here because NaN always compares not
+  // equal, even with itself. So NaN is handled in a separate test.
+  static_assert(std::numeric_limits<float>::has_infinity, "");
+  auto const inff = std::numeric_limits<float>::infinity();
+  for (auto x : {-inff, -1.0F, -0.5F, 0.0F, 0.5F, 1.0F, inff}) {
+    SCOPED_TRACE("Testing: float " + std::to_string(x));
+    TestBasicSemantics(x);
+    TestBasicSemantics(std::vector<float>(5, x));
+    std::vector<absl::optional<float>> v(5, x);
+    v.resize(10);
+    TestBasicSemantics(v);
+  }
+
   for (auto const& x :
        std::vector<std::string>{"", "f", "foo", "12345678901234567"}) {
     SCOPED_TRACE("Testing: std::string " + std::string(x));
@@ -234,6 +263,29 @@ TEST(Value, BasicSemantics) {
   std::vector<absl::optional<CommitTimestamp>> v(5, x);
   v.resize(10);
   TestBasicSemantics(v);
+
+  for (auto x : {testing::Genre::POP, testing::Genre::JAZZ,
+                 testing::Genre::FOLK, testing::Genre::ROCK}) {
+    SCOPED_TRACE("Testing: ProtoEnum<testing::Genre> " +
+                 testing::Genre_Name(x));
+    TestBasicSemantics(ProtoEnum<testing::Genre>(x));
+    TestBasicSemantics(std::vector<ProtoEnum<testing::Genre>>(5, x));
+    std::vector<absl::optional<ProtoEnum<testing::Genre>>> v(5, x);
+    v.resize(10);
+    TestBasicSemantics(v);
+  }
+
+  for (auto const& x :
+       {MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK),
+        MakeSinger(2123139547, "1942-06-18", "British", testing::Genre::POP)}) {
+    SCOPED_TRACE("Testing: ProtoMessage<testing::SingerInfo> { " +
+                 x.ShortDebugString() + " }");
+    TestBasicSemantics(ProtoMessage<testing::SingerInfo>(x));
+    TestBasicSemantics(std::vector<ProtoMessage<testing::SingerInfo>>(5, x));
+    std::vector<absl::optional<ProtoMessage<testing::SingerInfo>>> v(5, x);
+    v.resize(10);
+    TestBasicSemantics(v);
+  }
 }
 
 TEST(Value, Equality) {
@@ -249,6 +301,11 @@ TEST(Value, Equality) {
       {Value(MakePgNumeric(0).value()), Value(MakePgNumeric(1).value())},
       {Value(PgOid(200)), Value(PgOid(999))},
       {Value(absl::CivilDay(1970, 1, 1)), Value(absl::CivilDay(2020, 3, 15))},
+      {Value(ProtoEnum<testing::Genre>(testing::Genre::POP)),
+       Value(ProtoEnum<testing::Genre>(testing::Genre::JAZZ))},
+      {Value(ProtoMessage<testing::SingerInfo>()),
+       Value(ProtoMessage<testing::SingerInfo>(
+           MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK)))},
       {Value(std::vector<double>{1.2, 3.4}),
        Value(std::vector<double>{4.5, 6.7})},
       {Value(std::make_tuple(false, 123, "foo")),
@@ -425,6 +482,9 @@ TEST(Value, ConstructionFromLiterals) {
   Value v_int64(42);
   EXPECT_EQ(42, *v_int64.get<std::int64_t>());
 
+  Value v_float64(1.5);
+  EXPECT_EQ(1.5, *v_float64.get<double>());
+
   Value v_string("hello");
   EXPECT_EQ("hello", *v_string.get<std::string>());
 
@@ -476,6 +536,7 @@ TEST(Value, MixingTypes) {
 TEST(Value, SpannerArray) {
   using ArrayInt64 = std::vector<std::int64_t>;
   using ArrayDouble = std::vector<double>;
+  using ArrayFloat = std::vector<float>;
 
   ArrayInt64 const empty = {};
   Value const ve(empty);
@@ -499,6 +560,14 @@ TEST(Value, SpannerArray) {
   ASSERT_STATUS_OK(vd.get<ArrayDouble>());
   EXPECT_EQ(ad, *vd.get<ArrayDouble>());
 
+  ArrayFloat const af = {1.0, 2.0, 3.0};
+  Value const vf(af);
+  EXPECT_EQ(vf, vf);
+  EXPECT_NE(vi, vf);
+  EXPECT_THAT(vf.get<ArrayInt64>(), Not(IsOk()));
+  ASSERT_STATUS_OK(vf.get<ArrayFloat>());
+  EXPECT_EQ(af, *vf.get<ArrayFloat>());
+
   Value const null_vi = MakeNullValue<ArrayInt64>();
   EXPECT_EQ(null_vi, null_vi);
   EXPECT_NE(null_vi, vi);
@@ -513,6 +582,14 @@ TEST(Value, SpannerArray) {
   EXPECT_NE(null_vd, vi);
   EXPECT_THAT(null_vd.get<ArrayDouble>(), Not(IsOk()));
   EXPECT_THAT(null_vd.get<ArrayInt64>(), Not(IsOk()));
+
+  Value const null_vf = MakeNullValue<ArrayFloat>();
+  EXPECT_EQ(null_vf, null_vf);
+  EXPECT_NE(null_vf, null_vi);
+  EXPECT_NE(null_vf, vf);
+  EXPECT_NE(null_vf, vi);
+  EXPECT_THAT(null_vf.get<ArrayFloat>(), Not(IsOk()));
+  EXPECT_THAT(null_vf.get<ArrayInt64>(), Not(IsOk()));
 }
 
 TEST(Value, SpannerStruct) {
@@ -687,6 +764,39 @@ TEST(Value, ProtoConversionFloat64) {
   EXPECT_EQ("NaN", p.second.string_value());
 }
 
+TEST(Value, ProtoConversionFloat32) {
+  for (auto x : {-1.0F, -0.5F, 0.0F, 0.5F, 1.0F}) {
+    Value const v(x);
+    auto const p = spanner_internal::ToProto(v);
+    EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+    EXPECT_EQ(google::spanner::v1::TypeCode::FLOAT32, p.first.code());
+    EXPECT_EQ(x, p.second.number_value());
+  }
+
+  // Tests special cases
+  auto const inf = std::numeric_limits<float>::infinity();
+  Value v(inf);
+  auto p = spanner_internal::ToProto(v);
+  EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+  EXPECT_EQ(google::spanner::v1::TypeCode::FLOAT32, p.first.code());
+  EXPECT_EQ("Infinity", p.second.string_value());
+
+  v = Value(-inf);
+  p = spanner_internal::ToProto(v);
+  EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+  EXPECT_EQ(google::spanner::v1::TypeCode::FLOAT32, p.first.code());
+  EXPECT_EQ("-Infinity", p.second.string_value());
+
+  auto const nan = std::nanf("NaN");
+  v = Value(nan);
+  p = spanner_internal::ToProto(v);
+  // Note: Unlike IEEE 754, Spanner NaN values are considered equal
+  // (for easy sorting), so spanner::Value behaves the same way.
+  EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+  EXPECT_EQ(google::spanner::v1::TypeCode::FLOAT32, p.first.code());
+  EXPECT_EQ("NaN", p.second.string_value());
+}
+
 TEST(Value, ProtoConversionString) {
   for (auto const& x :
        std::vector<std::string>{"", "f", "foo", "12345678901234567890"}) {
@@ -812,6 +922,34 @@ TEST(Value, ProtoConversionDate) {
   }
 }
 
+TEST(Value, ProtoConversionProtoEnum) {
+  for (auto e : {testing::Genre::POP, testing::Genre::JAZZ,
+                 testing::Genre::FOLK, testing::Genre::ROCK}) {
+    Value const v{ProtoEnum<testing::Genre>(e)};
+    auto const p = spanner_internal::ToProto(v);
+    EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+    EXPECT_EQ(google::spanner::v1::TypeCode::ENUM, p.first.code());
+    EXPECT_EQ("google.cloud.spanner.testing.Genre", p.first.proto_type_fqn());
+    EXPECT_EQ(std::to_string(e), p.second.string_value());
+  }
+}
+
+TEST(Value, ProtoConversionProtoMessage) {
+  auto m = ProtoMessage<testing::SingerInfo>(
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK));
+
+  Value const v{m};
+  auto const p = spanner_internal::ToProto(v);
+  EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+  EXPECT_EQ(google::spanner::v1::TypeCode::PROTO, p.first.code());
+  EXPECT_EQ("google.cloud.spanner.testing.SingerInfo",
+            p.first.proto_type_fqn());
+
+  auto bytes = internal::Base64DecodeToBytes(p.second.string_value());
+  ASSERT_STATUS_OK(bytes);
+  EXPECT_EQ(std::string{m}, std::string(bytes->begin(), bytes->end()));
+}
+
 TEST(Value, ProtoConversionArray) {
   std::vector<std::int64_t> data{1, 2, 3};
   Value const v(data);
@@ -861,6 +999,12 @@ void SetProtoKind(Value& v, double x) {
   v = spanner_internal::FromProto(p.first, p.second);
 }
 
+void SetProtoKind(Value& v, float x) {
+  auto p = spanner_internal::ToProto(v);
+  p.second.set_number_value(x);
+  v = spanner_internal::FromProto(p.first, p.second);
+}
+
 void SetProtoKind(Value& v, char const* x) {
   auto p = spanner_internal::ToProto(v);
   p.second.set_string_value(x);
@@ -890,6 +1034,9 @@ TEST(Value, GetBadBool) {
   SetProtoKind(v, 0.0);
   EXPECT_THAT(v.get<bool>(), Not(IsOk()));
 
+  SetProtoKind(v, 0.0F);
+  EXPECT_THAT(v.get<bool>(), Not(IsOk()));
+
   SetProtoKind(v, "hello");
   EXPECT_THAT(v.get<bool>(), Not(IsOk()));
 }
@@ -909,6 +1056,21 @@ TEST(Value, GetBadDouble) {
   EXPECT_THAT(v.get<double>(), Not(IsOk()));
 }
 
+TEST(Value, GetBadFloat) {
+  Value v(0.0F);
+  ClearProtoKind(v);
+  EXPECT_THAT(v.get<float>(), Not(IsOk()));
+
+  SetProtoKind(v, google::protobuf::NULL_VALUE);
+  EXPECT_THAT(v.get<float>(), Not(IsOk()));
+
+  SetProtoKind(v, true);
+  EXPECT_THAT(v.get<float>(), Not(IsOk()));
+
+  SetProtoKind(v, "bad string");
+  EXPECT_THAT(v.get<float>(), Not(IsOk()));
+}
+
 TEST(Value, GetBadString) {
   Value v("hello");
   ClearProtoKind(v);
@@ -983,6 +1145,9 @@ TEST(Value, GetBadNumeric) {
   SetProtoKind(v, 0.0);
   EXPECT_THAT(v.get<std::string>(), Not(IsOk()));
 
+  SetProtoKind(v, 0.0F);
+  EXPECT_THAT(v.get<std::string>(), Not(IsOk()));
+
   SetProtoKind(v, "");
   EXPECT_THAT(v.get<std::int64_t>(), Not(IsOk()));
 
@@ -1086,6 +1251,63 @@ TEST(Value, GetBadDate) {
   EXPECT_THAT(v.get<absl::CivilDay>(), Not(IsOk()));
 }
 
+TEST(Value, GetBadProtoEnum) {
+  Value v(ProtoEnum<testing::Genre>{});
+  ClearProtoKind(v);
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+
+  SetProtoKind(v, google::protobuf::NULL_VALUE);
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+
+  SetProtoKind(v, "");
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+
+  SetProtoKind(v, "blah");
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+
+  SetProtoKind(v, "1blah");
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+
+  SetProtoKind(v, "2147483648");
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+}
+
+TEST(Value, GetBadProtoEnumFQN) {
+  Value v(ProtoEnum<testing::Genre>{});
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), IsOk());
+
+  auto p = spanner_internal::ToProto(v);
+  ASSERT_THAT(p.first.proto_type_fqn(), Not(IsEmpty()));
+  p.first.mutable_proto_type_fqn()->pop_back();
+  v = spanner_internal::FromProto(p.first, p.second);
+  EXPECT_THAT(v.get<ProtoEnum<testing::Genre>>(), Not(IsOk()));
+}
+
+TEST(Value, GetBadProtoMessage) {
+  Value v(ProtoMessage<testing::SingerInfo>(
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK)));
+  ClearProtoKind(v);
+  EXPECT_THAT(v.get<ProtoMessage<testing::SingerInfo>>(), Not(IsOk()));
+
+  SetProtoKind(v, google::protobuf::NULL_VALUE);
+  EXPECT_THAT(v.get<ProtoMessage<testing::SingerInfo>>(), Not(IsOk()));
+
+  SetProtoKind(v, "invalid-base64-because-of-hyphens");
+  EXPECT_THAT(v.get<ProtoMessage<testing::SingerInfo>>(), Not(IsOk()));
+}
+
+TEST(Value, GetBadProtoMessageFQN) {
+  Value v(ProtoMessage<testing::SingerInfo>(
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK)));
+  EXPECT_THAT(v.get<ProtoMessage<testing::SingerInfo>>(), IsOk());
+
+  auto p = spanner_internal::ToProto(v);
+  ASSERT_THAT(p.first.proto_type_fqn(), Not(IsEmpty()));
+  p.first.mutable_proto_type_fqn()->pop_back();
+  v = spanner_internal::FromProto(p.first, p.second);
+  EXPECT_THAT(v.get<ProtoMessage<testing::SingerInfo>>(), Not(IsOk()));
+}
+
 TEST(Value, GetBadOptional) {
   Value v(absl::optional<double>{});
   ClearProtoKind(v);
@@ -1169,6 +1391,9 @@ TEST(Value, OutputStream) {
   };
 
   auto const inf = std::numeric_limits<double>::infinity();
+  auto const inff = std::numeric_limits<float>::infinity();
+  auto const singer =
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK);
 
   struct TestCase {
     Value value;
@@ -1187,6 +1412,10 @@ TEST(Value, OutputStream) {
       {Value(42.0), "42.00", float4},
       {Value(inf), "inf", normal},
       {Value(-inf), "-inf", normal},
+      {Value(42.0F), "42", normal},
+      {Value(42.0F), "42.00", float4},
+      {Value(inff), "inf", normal},
+      {Value(-inff), "-inf", normal},
       {Value(""), "", normal},
       {Value("foo"), "foo", normal},
       {Value("NULL"), "NULL", normal},
@@ -1200,6 +1429,12 @@ TEST(Value, OutputStream) {
       {Value(PgOid(1234567890)), "1234567890", normal},
       {Value(absl::CivilDay()), "1970-01-01", normal},
       {Value(Timestamp()), "1970-01-01T00:00:00Z", normal},
+      {Value(ProtoEnum<testing::Genre>(testing::Genre::POP)),
+       "google.cloud.spanner.testing.POP", normal},
+      {Value(ProtoMessage<testing::SingerInfo>(singer)),
+       R"(google.cloud.spanner.testing.SingerInfo { singer_id: 1)"
+       R"( birth_date: "1817-05-25" nationality: "French" genre: FOLK })",
+       normal},
 
       // Tests string quoting: No quotes for scalars; quotes within aggregates
       {Value(""), "", normal},
@@ -1218,6 +1453,7 @@ TEST(Value, OutputStream) {
       {MakeNullValue<bool>(), "NULL", normal},
       {MakeNullValue<std::int64_t>(), "NULL", normal},
       {MakeNullValue<double>(), "NULL", normal},
+      {MakeNullValue<float>(), "NULL", normal},
       {MakeNullValue<std::string>(), "NULL", normal},
       {MakeNullValue<Bytes>(), "NULL", normal},
       {MakeNullValue<Json>(), "NULL", normal},
@@ -1226,6 +1462,8 @@ TEST(Value, OutputStream) {
       {MakeNullValue<PgOid>(), "NULL", normal},
       {MakeNullValue<absl::CivilDay>(), "NULL", normal},
       {MakeNullValue<Timestamp>(), "NULL", normal},
+      {MakeNullValue<ProtoEnum<testing::Genre>>(), "NULL", normal},
+      {MakeNullValue<ProtoMessage<testing::SingerInfo>>(), "NULL", normal},
 
       // Tests arrays
       {Value(std::vector<bool>{false, true}), "[0, 1]", normal},
@@ -1234,6 +1472,8 @@ TEST(Value, OutputStream) {
       {Value(std::vector<std::int64_t>{10, 11}), "[a, b]", hex},
       {Value(std::vector<double>{1.0, 2.0}), "[1, 2]", normal},
       {Value(std::vector<double>{1.0, 2.0}), "[1.000, 2.000]", float4},
+      {Value(std::vector<float>{1.0F, 2.0F}), "[1, 2]", normal},
+      {Value(std::vector<float>{1.0F, 2.0F}), "[1.000, 2.000]", float4},
       {Value(std::vector<std::string>{"a", "b"}), R"(["a", "b"])", normal},
       {Value(std::vector<Bytes>{2}), R"([B"", B""])", normal},
       {Value(std::vector<Json>{2}), R"([null, null])", normal},
@@ -1242,6 +1482,14 @@ TEST(Value, OutputStream) {
       {Value(std::vector<absl::CivilDay>{2}), "[1970-01-01, 1970-01-01]",
        normal},
       {Value(std::vector<Timestamp>{1}), "[1970-01-01T00:00:00Z]", normal},
+      {Value(std::vector<ProtoEnum<testing::Genre>>{testing::JAZZ,
+                                                    testing::FOLK}),
+       "[google.cloud.spanner.testing.JAZZ, google.cloud.spanner.testing.FOLK]",
+       normal},
+      {Value(std::vector<ProtoMessage<testing::SingerInfo>>{singer}),
+       R"([google.cloud.spanner.testing.SingerInfo { singer_id: 1)"
+       R"( birth_date: "1817-05-25" nationality: "French" genre: FOLK }])",
+       normal},
 
       // Tests arrays with null elements
       {Value(std::vector<absl::optional<double>>{1, {}, 2}), "[1, NULL, 2]",
@@ -1251,6 +1499,7 @@ TEST(Value, OutputStream) {
       {MakeNullValue<std::vector<bool>>(), "NULL", normal},
       {MakeNullValue<std::vector<std::int64_t>>(), "NULL", normal},
       {MakeNullValue<std::vector<double>>(), "NULL", normal},
+      {MakeNullValue<std::vector<float>>(), "NULL", normal},
       {MakeNullValue<std::vector<std::string>>(), "NULL", normal},
       {MakeNullValue<std::vector<Bytes>>(), "NULL", normal},
       {MakeNullValue<std::vector<Json>>(), "NULL", normal},
@@ -1258,6 +1507,9 @@ TEST(Value, OutputStream) {
       {MakeNullValue<std::vector<Numeric>>(), "NULL", normal},
       {MakeNullValue<std::vector<absl::CivilDay>>(), "NULL", normal},
       {MakeNullValue<std::vector<Timestamp>>(), "NULL", normal},
+      {MakeNullValue<std::vector<ProtoEnum<testing::Genre>>>(), "NULL", normal},
+      {MakeNullValue<std::vector<ProtoMessage<testing::SingerInfo>>>(), "NULL",
+       normal},
 
       // Tests structs
       {Value(std::make_tuple(true, 123)), "(1, 123)", normal},
@@ -1295,7 +1547,7 @@ TEST(Value, OutputStream) {
       // Tests null structs
       {MakeNullValue<std::tuple<bool>>(), "NULL", normal},
       {MakeNullValue<std::tuple<bool, std::int64_t>>(), "NULL", normal},
-      {MakeNullValue<std::tuple<bool, std::string>>(), "NULL", normal},
+      {MakeNullValue<std::tuple<float, std::string>>(), "NULL", normal},
       {MakeNullValue<std::tuple<double, Bytes, Timestamp>>(), "NULL", normal},
       {MakeNullValue<std::tuple<Numeric, absl::CivilDay>>(), "NULL", normal},
       {MakeNullValue<std::tuple<Json, std::vector<bool>>>(), "NULL", normal},
@@ -1313,6 +1565,12 @@ TEST(Value, OutputStream) {
   std::stringstream ss;
   ss << Value(std::nan(""));
   EXPECT_THAT(ss.str(), HasSubstr("nan"));
+
+  // `float std::nanf("")` is a special case because the output conversion
+  // is implementation defined. So, we just look for a "nan" substring.
+  std::stringstream ssf;
+  ssf << Value(std::nanf(""));
+  EXPECT_THAT(ssf.str(), HasSubstr("nan"));
 }
 
 // Ensures that the following expressions produce the same output.
@@ -1346,6 +1604,13 @@ TEST(Value, OutputStreamMatchesT) {
   StreamMatchesValueStream(std::numeric_limits<double>::infinity());
   StreamMatchesValueStream(-std::numeric_limits<double>::infinity());
 
+  // float
+  StreamMatchesValueStream(0.0F);
+  StreamMatchesValueStream(3.14F);
+  StreamMatchesValueStream(std::nanf("NaN"));
+  StreamMatchesValueStream(std::numeric_limits<float>::infinity());
+  StreamMatchesValueStream(-std::numeric_limits<float>::infinity());
+
   // std::string
   StreamMatchesValueStream("");
   StreamMatchesValueStream("foo");
@@ -1392,6 +1657,17 @@ TEST(Value, OutputStreamMatchesT) {
   StreamMatchesValueStream(Timestamp());
   StreamMatchesValueStream(MakeTimestamp(MakeTime(1, 1)).value());
 
+  // ProtoEnum
+  StreamMatchesValueStream(ProtoEnum<testing::Genre>());
+  StreamMatchesValueStream(ProtoEnum<testing::Genre>(testing::ROCK));
+  StreamMatchesValueStream(
+      ProtoEnum<testing::Genre>(static_cast<testing::Genre>(42)));
+
+  // ProtoMessage
+  StreamMatchesValueStream(ProtoMessage<testing::SingerInfo>());
+  StreamMatchesValueStream(ProtoMessage<testing::SingerInfo>(
+      MakeSinger(1, "1817-05-25", "French", testing::Genre::FOLK)));
+
   // std::vector<T>
   // Not included, because a raw vector cannot be streamed.
 
