@@ -29,16 +29,21 @@
 #include <atomic>
 #include <future>
 #include <iomanip>
+#include <iostream>
+#include <iterator>
+#include <map>
+#include <mutex>
 #include <random>
 #include <sstream>
+#include <string>
 #include <thread>
+#include <utility>
 #include <vector>
 
 namespace {
 using ::google::cloud::testing_util::FormatSize;
 using ::google::cloud::testing_util::Timer;
 namespace gcs = ::google::cloud::storage;
-namespace gcs_ex = ::google::cloud::storage_experimental;
 namespace gcs_bm = ::google::cloud::storage_benchmarks;
 using gcs_bm::AggregateDownloadThroughputOptions;
 using gcs_bm::FormatBandwidthGbPerSecond;
@@ -122,11 +127,7 @@ class Iteration {
 gcs::Client MakeClient(AggregateDownloadThroughputOptions const& options) {
   auto opts = options.client_options;
 #if GOOGLE_CLOUD_CPP_STORAGE_HAVE_GRPC
-  namespace gcs_ex = ::google::cloud::storage_experimental;
-  if (options.api == "GRPC") {
-    return gcs_ex::DefaultGrpcClient(
-        std::move(opts).set<gcs_ex::GrpcPluginOption>("media"));
-  }
+  if (options.api == "GRPC") return gcs::MakeGrpcClient();
 #endif  // GOOGLE_CLOUD_CPP_STORAGE_HAVE_GRPC
   return gcs::Client(std::move(opts));
 }
@@ -325,8 +326,8 @@ google::cloud::StatusOr<AggregateDownloadThroughputOptions> SelfTest(
     if (!value.empty()) continue;
     std::ostringstream os;
     os << "The environment variable " << var << " is not set or empty";
-    return google::cloud::Status(google::cloud::StatusCode::kUnknown,
-                                 std::move(os).str());
+    return google::cloud::internal::UnknownError(std::move(os).str(),
+                                                 GCP_ERROR_INFO());
   }
   auto bucket_name =
       GetEnv("GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME").value();
@@ -381,8 +382,6 @@ void PrintResults(AggregateDownloadThroughputOptions const& options,
     return v;
   };
   auto const labels = clean_csv_field(options.labels);
-  auto const grpc_plugin_config =
-      clean_csv_field(options.client_options.get<gcs_ex::GrpcPluginOption>());
   auto const* client_per_thread = options.client_per_thread ? "true" : "false";
   // Print the results after each iteration. Makes it possible to interrupt
   // the benchmark in the middle and still get some data.
