@@ -15,8 +15,12 @@
 #include "google/cloud/storage/benchmarks/aggregate_upload_throughput_options.h"
 #include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
+#include <iostream>
 #include <iterator>
 #include <sstream>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -25,51 +29,52 @@ namespace {
 
 namespace gcs = ::google::cloud::storage;
 namespace gcs_ex = ::google::cloud::storage_experimental;
+using ::google::cloud::internal::ErrorInfoBuilder;
 using ::google::cloud::testing_util::OptionDescriptor;
 
 StatusOr<AggregateUploadThroughputOptions> ValidateOptions(
     std::string const& usage, AggregateUploadThroughputOptions options) {
-  auto make_status = [](std::ostringstream& os) {
-    auto const code = google::cloud::StatusCode::kInvalidArgument;
-    return google::cloud::Status{code, std::move(os).str()};
+  auto make_status = [](std::ostringstream& os, ErrorInfoBuilder eib) {
+    return google::cloud::internal::InvalidArgumentError(std::move(os).str(),
+                                                         std::move(eib));
   };
 
   if (options.bucket_name.empty()) {
     std::ostringstream os;
     os << "Missing --bucket option\n" << usage << "\n";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   if (options.object_count <= 0) {
     std::ostringstream os;
     os << "Invalid number of objects (" << options.object_count
        << "), check your --object-count option\n";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   if (options.minimum_object_size > options.maximum_object_size) {
     std::ostringstream os;
     os << "Invalid object size range [" << options.minimum_object_size << ","
        << options.maximum_object_size << "], check your --minimum-object-size"
        << " and --maximum-object-size options";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   if (options.thread_count <= 0) {
     std::ostringstream os;
     os << "Invalid number of threads (" << options.thread_count
        << "), check your --thread-count option\n";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   if (options.iteration_count <= 0) {
     std::ostringstream os;
     os << "Invalid number of iterations (" << options.iteration_count
        << "), check your --iteration-count option\n";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   if (options.client_options.get<GrpcNumChannelsOption>() < 0) {
     std::ostringstream os;
     os << "Invalid number of gRPC channels ("
        << options.client_options.get<GrpcNumChannelsOption>()
        << "), check your --grpc-channel-count option\n";
-    return make_status(os);
+    return make_status(os, GCP_ERROR_INFO());
   }
   return options;
 }
@@ -124,7 +129,7 @@ ParseAggregateUploadThroughputOptions(std::vector<std::string> const& argv,
        "use a different storage::Client object in each thread",
        [&options](std::string const& val) {
          options.client_per_thread =
-             testing_util::ParseBoolean(val).value_or("true");
+             testing_util::ParseBoolean(val).value_or(true);
        }},
       {"--grpc-channel-count", "controls the number of gRPC channels",
        [&options](std::string const& val) {
@@ -193,7 +198,8 @@ ParseAggregateUploadThroughputOptions(std::vector<std::string> const& argv,
        << absl::StrJoin(std::next(unparsed.begin()), unparsed.end(), ", ")
        << "\n"
        << usage << "\n";
-    return Status{StatusCode::kInvalidArgument, std::move(os).str()};
+    return google::cloud::internal::InvalidArgumentError(std::move(os).str(),
+                                                         GCP_ERROR_INFO());
   }
 
   return ValidateOptions(usage, std::move(options));
