@@ -26,8 +26,10 @@
 #include <cstdint>
 #include <iomanip>
 #include <iostream>
+#include <set>
 #include <sstream>
 #include <string>
+#include <utility>
 
 namespace google {
 namespace cloud {
