@@ -16,9 +16,16 @@
 #include "google/cloud/storage/options.h"
 #include "google/cloud/common_options.h"
 #include "google/cloud/grpc_options.h"
+#include "google/cloud/internal/make_status.h"
 #include "absl/strings/str_split.h"
 #include "absl/time/time.h"
+#include <algorithm>
+#include <iostream>
+#include <set>
 #include <sstream>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -36,19 +43,22 @@ Status ValidateQuantizedRange(std::string const& name,
   if (!minimum.has_value() || !maximum.has_value()) {
     std::ostringstream os;
     os << "One of the range limits for " << name << " is missing";
-    return google::cloud::Status{StatusCode::kInvalidArgument, os.str()};
+    return google::cloud::internal::InvalidArgumentError(os.str(),
+                                                         GCP_ERROR_INFO());
   }
   if (*minimum > *maximum || *minimum < 0 || *maximum < 0) {
     std::ostringstream os;
     os << "Invalid range for " << name << " [" << *minimum << ',' << *maximum
        << "]";
-    return google::cloud::Status{StatusCode::kInvalidArgument, os.str()};
+    return google::cloud::internal::InvalidArgumentError(os.str(),
+                                                         GCP_ERROR_INFO());
   }
   if (quantum <= 0 || (quantum > *minimum && *minimum != 0)) {
     std::ostringstream os;
     os << "Invalid quantum for " << name << " (" << quantum
        << "), it should be in the (0," << *minimum << "] range";
-    return google::cloud::Status{StatusCode::kInvalidArgument, os.str()};
+    return google::cloud::internal::InvalidArgumentError(os.str(),
+                                                         GCP_ERROR_INFO());
   }
 
   return Status{};
@@ -103,13 +113,13 @@ using ::google::cloud::testing_util::OptionDescriptor;
 google::cloud::StatusOr<ThroughputOptions> ValidateParsedOptions(
     std::string const& usage, ThroughputOptions options) {
   auto make_status = [](std::ostringstream& os) {
-    auto const code = google::cloud::StatusCode::kInvalidArgument;
-    return google::cloud::Status{code, std::move(os).str()};
+    return google::cloud::internal::InvalidArgumentError(std::move(os).str(),
+                                                         GCP_ERROR_INFO());
   };
 
-  if (options.region.empty()) {
+  if (options.bucket.empty()) {
     std::ostringstream os;
-    os << "Missing value for --region option\n" << usage << "\n";
+    os << "Missing value for --bucket option\n" << usage << "\n";
     return make_status(os);
   }
 
@@ -240,14 +250,10 @@ google::cloud::StatusOr<ThroughputOptions> ParseThroughputOptions(
        [&wants_help](std::string const&) { wants_help = true; }},
       {"--description", "print benchmark description",
        [&wants_description](std::string const&) { wants_description = true; }},
-      {"--project-id", "use the given project id for the benchmark",
-       [&options](std::string const& val) { options.project_id = val; }},
+      {"--bucket", "use the given bucket for the benchmark",
+       [&options](std::string const& val) { options.bucket = val; }},
       {"--labels", "user-defined labels to tag the results",
        [&options](std::string const& val) { options.labels = val; }},
-      {"--region", "use the given region for the benchmark",
-       [&options](std::string const& val) { options.region = val; }},
-      {"--bucket-prefix", "use this prefix when creating the bucket",
-       [&options](std::string const& val) { options.bucket_prefix = val; }},
       {"--thread-count", "set the number of threads in the benchmark",
        [&options](std::string const& val) {
          options.thread_count = std::stoi(val);
@@ -479,23 +485,15 @@ google::cloud::StatusOr<ThroughputOptions> ParseThroughputOptions(
     return options;
   }
 
-  if (unparsed.size() > 2) {
+  if (unparsed.size() >= 2) {
     std::ostringstream os;
     os << "Unknown arguments or options:\n";
     for (auto const& a : unparsed) {
       os << "  " << a << "\n";
     }
     os << usage << "\n";
-    return Status{StatusCode::kInvalidArgument, os.str()};
-  }
-  if (unparsed.size() == 2) {
-    if (absl::StartsWith(unparsed[1], "--")) {
-      std::ostringstream os;
-      os << "Unknown option or invalid region name: " << unparsed[1];
-      os << usage << "\n";
-      return Status{StatusCode::kInvalidArgument, os.str()};
-    }
-    options.region = unparsed[1];
+    return google::cloud::internal::InvalidArgumentError(os.str(),
+                                                         GCP_ERROR_INFO());
   }
   return ValidateParsedOptions(usage, options);
 }
