@@ -13,19 +13,23 @@
 // limitations under the License.
 
 #include "google/cloud/storage/client.h"
+#include "google/cloud/storage/internal/base64.h"
 #include "google/cloud/storage/internal/connection_factory.h"
-#include "google/cloud/storage/internal/openssl_util.h"
 #include "google/cloud/storage/oauth2/service_account_credentials.h"
+#include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/internal/curl_handle.h"
 #include "google/cloud/internal/curl_options.h"
 #include "google/cloud/internal/filesystem.h"
 #include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/opentelemetry.h"
 #include "google/cloud/log.h"
+#include "absl/strings/str_split.h"
 #include <fstream>
 #include <memory>
+#include <string>
 #include <thread>
 #include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -132,7 +136,8 @@ StatusOr<ObjectMetadata> Client::UploadFileSimple(
     os << __func__ << "(" << request << ", " << file_name
        << "): UploadFromOffset (" << upload_offset
        << ") is bigger than the size of file source (" << file_size << ")";
-    return Status(StatusCode::kInvalidArgument, std::move(os).str());
+    return google::cloud::internal::InvalidArgumentError(std::move(os).str(),
+                                                         GCP_ERROR_INFO());
   }
   auto upload_size = (std::min)(
       request.GetOption<UploadLimit>().value_or(file_size - upload_offset),
@@ -143,7 +148,8 @@ StatusOr<ObjectMetadata> Client::UploadFileSimple(
     std::ostringstream os;
     os << __func__ << "(" << request << ", " << file_name
        << "): cannot open upload file source";
-    return Status(StatusCode::kNotFound, std::move(os).str());
+    return google::cloud::internal::NotFoundError(std::move(os).str(),
+                                                  GCP_ERROR_INFO());
   }
 
   std::string payload(static_cast<std::size_t>(upload_size), char{});
@@ -156,7 +162,8 @@ StatusOr<ObjectMetadata> Client::UploadFileSimple(
     os << __func__ << "(" << request << ", " << file_name << "): Actual read ("
        << is.gcount() << ") is smaller than upload_size (" << payload.size()
        << ")";
-    return Status(StatusCode::kInternal, std::move(os).str());
+    return google::cloud::internal::InternalError(std::move(os).str(),
+                                                  GCP_ERROR_INFO());
   }
   is.close();
   request.set_payload(payload);
@@ -186,14 +193,16 @@ integrity checks using the DisableMD5Hash() and DisableCrc32cChecksum() options.
     std::error_code size_err;
     auto file_size = google::cloud::internal::file_size(file_name, size_err);
     if (size_err) {
-      return Status(StatusCode::kNotFound, size_err.message());
+      return google::cloud::internal::NotFoundError(size_err.message(),
+                                                    GCP_ERROR_INFO());
     }
     if (file_size < upload_offset) {
       std::ostringstream os;
       os << __func__ << "(" << request << ", " << file_name
          << "): UploadFromOffset (" << upload_offset
          << ") is bigger than the size of file source (" << file_size << ")";
-      return Status(StatusCode::kInvalidArgument, std::move(os).str());
+      return google::cloud::internal::InvalidArgumentError(std::move(os).str(),
+                                                           GCP_ERROR_INFO());
     }
 
     auto upload_size = (std::min)(
@@ -206,7 +215,8 @@ integrity checks using the DisableMD5Hash() and DisableCrc32cChecksum() options.
     std::ostringstream os;
     os << __func__ << "(" << request << ", " << file_name
        << "): cannot open upload file source";
-    return Status(StatusCode::kNotFound, std::move(os).str());
+    return google::cloud::internal::NotFoundError(std::move(os).str(),
+                                                  GCP_ERROR_INFO());
   }
   // We set its offset before passing it to `UploadStreamResumable` so we don't
   // need to compute `UploadFromOffset` again.
@@ -230,10 +240,11 @@ StatusOr<ObjectMetadata> Client::UploadStreamResumable(
   // If `committed_size == upload_limit`, we will upload an empty string and
   // finalize the upload.
   if (committed_size > upload_limit) {
-    return Status(StatusCode::kOutOfRange,
-                  "UploadLimit (" + std::to_string(upload_limit) +
-                      ") is not bigger than the uploaded size (" +
-                      std::to_string(committed_size) + ") on GCS server");
+    return google::cloud::internal::OutOfRangeError(
+        absl::StrCat("UploadLimit (", upload_limit,
+                     ") is not bigger than the uploaded size (", committed_size,
+                     ") on GCS server"),
+        GCP_ERROR_INFO());
   }
   source.seekg(committed_size, std::ios::cur);
 
@@ -277,43 +288,40 @@ StatusOr<ObjectMetadata> Client::UploadStreamResumable(
     auto const actual_committed_size = upload->committed_size.value_or(0);
     if (actual_committed_size != expected) {
       // Defensive programming: unless there is a bug, this should be dead code.
-      return Status(
-          StatusCode::kInternal,
-          "Mismatch in committed size expected=" + std::to_string(expected) +
-              " got=" + std::to_string(actual_committed_size) +
+      return google::cloud::internal::InternalError(
+          absl::StrCat(
+              "Mismatch in committed size expected=", expected,
+              " got=", actual_committed_size,
               ". This is a bug, please report it at "
-              "https://github.com/googleapis/google-cloud-cpp/issues/new");
+              "https://github.com/googleapis/google-cloud-cpp/issues/new"),
+          GCP_ERROR_INFO());
     }
 
     // We only update `committed_size` when uploading is successful.
     committed_size = expected;
   }
-  return Status(StatusCode::kInternal,
-                "Upload did not complete but source is exhausted");
+  return google::cloud::internal::InternalError(
+      "Upload did not complete but source is exhausted", GCP_ERROR_INFO());
 }
 
 Status Client::DownloadFileImpl(internal::ReadObjectRangeRequest const& request,
                                 std::string const& file_name) {
-  auto report_error = [&request, file_name](char const* func, char const* what,
-                                            Status const& status) {
-    std::ostringstream msg;
-    msg << func << "(" << request << ", " << file_name << "): " << what
-        << " - status.message=" << status.message();
-    return Status(status.code(), std::move(msg).str());
+  auto const* func = __func__;
+  auto msg = [&request, &file_name, func](char const* what) {
+    std::ostringstream os;
+    os << func << "(" << request << ", " << file_name << "): " << what;
+    return std::move(os).str();
   };
 
   auto stream = ReadObjectImpl(request);
-  if (stream.bad()) {
-    return report_error(__func__, "cannot open download source object",
-                        stream.status());
-  }
+  if (stream.bad()) return stream.status();
 
   // Open the destination file, and immediate raise an exception on failure.
   std::ofstream os(file_name, std::ios::binary);
   if (!os.is_open()) {
-    return report_error(
-        __func__, "cannot open download destination file",
-        Status(StatusCode::kInvalidArgument, "ofstream::open()"));
+    return google::cloud::internal::InvalidArgumentError(
+        msg("cannot open download destination file - ofstream::open()"),
+        GCP_ERROR_INFO());
   }
 
   auto const& current = google::cloud::internal::CurrentOptions();
@@ -325,13 +333,11 @@ Status Client::DownloadFileImpl(internal::ReadObjectRangeRequest const& request,
   } while (os.good() && stream.good());
   os.close();
   if (!os.good()) {
-    return report_error(__func__, "cannot close download destination file",
-                        Status(StatusCode::kUnknown, "ofstream::close()"));
-  }
-  if (stream.bad()) {
-    return report_error(__func__, "error reading download source object",
-                        stream.status());
+    return google::cloud::internal::UnknownError(
+        msg("cannot close download destination file - ofstream::close()"),
+        GCP_ERROR_INFO());
   }
+  if (stream.bad()) return stream.status();
   return Status();
 }
 
@@ -386,7 +392,7 @@ StatusOr<std::string> Client::SignUrlV2(
   std::string signature = curl.MakeEscapedString(encoded).get();
 
   std::ostringstream os;
-  os << "https://storage.googleapis.com/" << request.bucket_name();
+  os << Endpoint() << '/' << request.bucket_name();
   if (!request.object_name().empty()) {
     os << '/' << curl.MakeEscapedString(request.object_name()).get();
   }
@@ -478,6 +484,21 @@ std::string CreateRandomPrefixName(std::string const& prefix) {
                                                   "abcdefghijklmnopqrstuvwxyz");
 }
 
+std::string Client::Endpoint() const {
+  return connection_->options().get<RestEndpointOption>();
+}
+
+// This can be optimized to not have a lot of string copies.
+// But the code is rarely used and not in any critical path.
+std::string Client::EndpointAuthority() const {
+  auto endpoint = Endpoint();
+  auto endpoint_authority = absl::string_view(endpoint);
+  if (!absl::ConsumePrefix(&endpoint_authority, "https://")) {
+    absl::ConsumePrefix(&endpoint_authority, "http://");
+  }
+  return std::string(endpoint_authority);
+}
+
 namespace internal {
 
 Client ClientImplDetails::CreateWithDecorations(
