@@ -25,6 +25,9 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <chrono>
+#include <memory>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -165,7 +168,7 @@ TEST_F(ClientTest, DefaultDecoratorsRestClient) {
   auto tested =
       Client(Options{}
                  .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-                 .set<TracingComponentsOption>({}));
+                 .set<LoggingComponentsOption>({}));
 
   auto const impl = ClientImplDetails::GetConnection(tested);
   ASSERT_THAT(impl, NotNull());
@@ -185,7 +188,7 @@ TEST_F(ClientTest, LoggingDecoratorsRestClient) {
   auto tested =
       Client(Options{}
                  .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-                 .set<TracingComponentsOption>({"raw-client"}));
+                 .set<LoggingComponentsOption>({"raw-client"}));
 
   auto const impl = ClientImplDetails::GetConnection(tested);
   ASSERT_THAT(impl, NotNull());
@@ -206,7 +209,7 @@ TEST_F(ClientTest, OTelEnableTracing) {
   // environment there may not be other credentials configured.
   auto options = Options{}
                      .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-                     .set<TracingComponentsOption>({"raw-client"});
+                     .set<LoggingComponentsOption>({"raw-client"});
 
   auto tested = Client(EnableTracing(std::move(options)));
   auto const impl = ClientImplDetails::GetConnection(tested);
@@ -226,7 +229,7 @@ TEST_F(ClientTest, OTelDisableTracing) {
   // environment there may not be other credentials configured.
   auto options = Options{}
                      .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-                     .set<TracingComponentsOption>({"raw-client"});
+                     .set<LoggingComponentsOption>({"raw-client"});
 
   auto tested = Client(DisableTracing(std::move(options)));
   auto const impl = ClientImplDetails::GetConnection(tested);
