@@ -16,8 +16,11 @@
 #include "google/cloud/storage/internal/object_requests.h"
 #include "google/cloud/storage/testing/mock_client.h"
 #include <gmock/gmock.h>
+#include <algorithm>
 #include <iostream>
+#include <memory>
 #include <string>
+#include <utility>
 
 namespace {
 
