@@ -18,8 +18,12 @@
 #include "google/cloud/internal/getenv.h"
 #include <iostream>
 #include <map>
+#include <random>
+#include <stdexcept>
 #include <string>
 #include <thread>
+#include <utility>
+#include <vector>
 
 namespace {
 std::string StartResumableUpload(google::cloud::storage::Client client,
