@@ -20,8 +20,12 @@
 #include <iostream>
 #include <iterator>
 #include <map>
+#include <random>
+#include <stdexcept>
 #include <string>
 #include <thread>
+#include <utility>
+#include <vector>
 
 namespace {
 
@@ -99,6 +103,29 @@ void ListObjectsAndPrefixes(google::cloud::storage::Client client,
   (std::move(client), argv.at(0), argv.at(1));
 }
 
+void ListObjectsAndFolders(google::cloud::storage::Client client,
+                           std::vector<std::string> const& argv) {
+  //! [list-objects-and-folders]
+  namespace gcs = ::google::cloud::storage;
+  [](gcs::Client client, std::string const& bucket_name,
+     std::string const& bucket_prefix) {
+    for (auto&& item : client.ListObjectsAndPrefixes(
+             bucket_name, gcs::Prefix(bucket_prefix), gcs::Delimiter("/"),
+             gcs::IncludeFoldersAsPrefixes(true))) {
+      if (!item) throw std::move(item).status();
+      auto result = *std::move(item);
+      if (absl::holds_alternative<gcs::ObjectMetadata>(result)) {
+        std::cout << "object_name="
+                  << absl::get<gcs::ObjectMetadata>(result).name() << "\n";
+      } else if (absl::holds_alternative<std::string>(result)) {
+        std::cout << "prefix     =" << absl::get<std::string>(result) << "\n";
+      }
+    }
+  }
+  //! [list-objects-and-folders]
+  (std::move(client), argv.at(0), argv.at(1));
+}
+
 void InsertObject(google::cloud::storage::Client client,
                   std::vector<std::string> const& argv) {
   //! [insert object]
@@ -664,6 +691,11 @@ void RunAll(std::vector<std::string> const& argv) {
   std::cout << "\nRunning ListObjectsAndPrefixes() example" << std::endl;
   ListObjectsAndPrefixes(client, {bucket_name, bucket_prefix});
 
+  if (!examples::UsingEmulator()) {
+    std::cout << "\nRunning ListObjectsAndFolders() example" << std::endl;
+    ListObjectsAndFolders(client, {bucket_name, bucket_prefix});
+  }
+
   // Cleanup the objects so the bucket can be deleted
   client.DeleteObject(bucket_name, bucket_prefix + "/foo/bar");
   client.DeleteObject(bucket_name, bucket_prefix + "/qux/bar");
@@ -770,6 +802,8 @@ int main(int argc, char* argv[]) {
       make_entry("list-versioned-objects", {}, ListVersionedObjects),
       make_entry("list-objects-and-prefixes", {"<prefix>"},
                  ListObjectsAndPrefixes),
+      make_entry("list-objects-and-folders", {"<prefix>"},
+                 ListObjectsAndFolders),
       make_entry("insert-object",
                  {"<object-name>", "<object-contents (string)>"}, InsertObject),
       make_entry("insert-object-strict-idempotency",
