@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "google/cloud/storage/idempotency_policy.h"
+#include <memory>
 
 namespace google {
 namespace cloud {
@@ -103,6 +104,10 @@ bool AlwaysRetryIdempotencyPolicy::IsIdempotent(
     internal::RewriteObjectRequest const&) const {
   return true;
 }
+bool AlwaysRetryIdempotencyPolicy::IsIdempotent(
+    internal::RestoreObjectRequest const&) const {
+  return true;
+}
 bool AlwaysRetryIdempotencyPolicy::IsIdempotent(
     internal::ResumableUploadRequest const&) const {
   return true;
@@ -360,6 +365,11 @@ bool StrictIdempotencyPolicy::IsIdempotent(
   return request.HasOption<IfGenerationMatch>();
 }
 
+bool StrictIdempotencyPolicy::IsIdempotent(
+    internal::RestoreObjectRequest const& request) const {
+  return request.HasOption<IfGenerationMatch>();
+}
+
 bool StrictIdempotencyPolicy::IsIdempotent(
     internal::ResumableUploadRequest const& request) const {
   // Only the pre-conditions on the destination matter. If they are not set, it
