@@ -13,6 +13,8 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/access_control_common_parser.h"
+#include "google/cloud/storage/internal/metadata_parser.h"
+#include <utility>
 // This file contains the implementation for deprecated functions, we need to
 // disable the warnings.
 #include "google/cloud/internal/disable_deprecation_warnings.inc"
@@ -24,9 +26,7 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 Status AccessControlCommonParser::FromJson(AccessControlCommon& result,
                                            nlohmann::json const& json) {
-  if (!json.is_object()) {
-    return Status(StatusCode::kInvalidArgument, __func__);
-  }
+  if (!json.is_object()) return NotJsonObject(json, GCP_ERROR_INFO());
   result.bucket_ = json.value("bucket", "");
   result.domain_ = json.value("domain", "");
   result.email_ = json.value("email", "");
