@@ -14,6 +14,9 @@
 
 #include "google/cloud/storage/internal/access_control_common.h"
 #include <gmock/gmock.h>
+// This file contains tests for deprecated functions, we need to disable the
+// warnings.
+#include "google/cloud/internal/disable_deprecation_warnings.inc"
 
 namespace google {
 namespace cloud {
@@ -22,7 +25,7 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 namespace {
 /// @test Verify the well-known values defined in AccessControlCommon.
-TEST(ccessControlCommonTest, WellKnownValues) {
+TEST(AccessControlCommonTest, WellKnownValues) {
   EXPECT_EQ("OWNER", AccessControlCommon::ROLE_OWNER());
   EXPECT_EQ("READER", AccessControlCommon::ROLE_READER());
 
