@@ -19,7 +19,9 @@
 #include "google/cloud/storage/internal/hash_function_impl.h"
 #include "google/cloud/storage/options.h"
 #include "google/cloud/storage/testing/canonical_errors.h"
+#include "google/cloud/storage/testing/mock_hash_function.h"
 #include "google/cloud/testing_util/async_sequencer.h"
+#include "google/cloud/testing_util/is_proto_equal.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 
@@ -28,14 +30,19 @@ namespace cloud {
 namespace storage_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
+using ::google::cloud::storage::testing::MockHashFunction;
 using ::google::cloud::storage::testing::canonical_errors::PermanentError;
 using ::google::cloud::storage_experimental::WritePayload;
 using ::google::cloud::testing_util::AsyncSequencer;
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::IsOkAndHolds;
+using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::_;
 using ::testing::An;
+using ::testing::Pair;
+using ::testing::Return;
+using ::testing::UnorderedElementsAre;
 using ::testing::VariantWith;
 
 using Request = google::storage::v2::BidiWriteObjectRequest;
@@ -43,18 +50,6 @@ using Response = google::storage::v2::BidiWriteObjectResponse;
 using MockStream =
     ::google::cloud::mocks::MockAsyncStreamingReadWriteRpc<Request, Response>;
 
-class MockHash : public storage::internal::HashFunction {
- public:
-  MOCK_METHOD(std::string, Name, (), (const, override));
-  MOCK_METHOD(void, Update, (absl::string_view), (override));
-  MOCK_METHOD(Status, Update, (std::int64_t, absl::string_view), (override));
-  MOCK_METHOD(Status, Update, (std::int64_t, absl::string_view, std::uint32_t),
-              (override));
-  MOCK_METHOD(Status, Update, (std::int64_t, absl::Cord const&, std::uint32_t),
-              (override));
-  MOCK_METHOD(storage::internal::HashValues, Finish, (), (override));
-};
-
 auto TestOptions() {
   return google::cloud::internal::MakeImmutableOptions(
       Options{}.set<storage::RestEndpointOption>(
@@ -70,18 +65,20 @@ auto MakeTestResponse() {
 }
 
 auto MakeTestObject() {
-  return storage::ObjectMetadata{}
-      .set_size(2048)
-      .set_bucket("test-bucket")
-      .set_name("test-object")
-      .set_self_link(
-          "https://test-only.p.googleapis.com/storage/v1/b/test-bucket/o/"
-          "test-object")
-      .set_media_link(
-          "https://test-only.p.googleapis.com/download/storage/v1/b/"
-          "test-bucket/o/test-object?generation=0&alt=media")
-      .set_id("test-bucket/test-object/0")
-      .set_kind("storage#object");
+  auto object = google::storage::v2::Object{};
+  object.set_size(2048);
+  object.set_bucket("projects/_/buckets/test-bucket");
+  object.set_name("test-object");
+  return object;
+}
+
+auto MakeRequest() {
+  auto request = Request{};
+  request.set_upload_id("test-upload-id");
+  // We use this field as a canary to verify the request fields are preserved.
+  request.mutable_common_object_request_params()->set_encryption_algorithm(
+      "test-only-algo");
+  return request;
 }
 
 TEST(AsyncWriterConnectionTest, Basic) {
@@ -90,14 +87,23 @@ TEST(AsyncWriterConnectionTest, Basic) {
   EXPECT_CALL(*mock, Finish).WillOnce([] {
     return make_ready_future(Status{});
   });
-  auto hash = std::make_shared<MockHash>();
+  EXPECT_CALL(*mock, GetRequestMetadata)
+      .WillOnce(Return(RpcMetadata{{{"hk0", "v0"}, {"hk1", "v1"}},
+                                   {{"tk0", "v0"}, {"tk1", "v1"}}}));
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(0);
   EXPECT_CALL(*hash, Finish).Times(0);
 
-  AsyncWriterConnectionImpl tested(TestOptions(), std::move(mock),
-                                   "test-upload-id", hash, 1024);
+  AsyncWriterConnectionImpl tested(TestOptions(), MakeRequest(),
+                                   std::move(mock), hash, 1024);
   EXPECT_EQ(tested.UploadId(), "test-upload-id");
   EXPECT_THAT(tested.PersistedState(), VariantWith<std::int64_t>(1024));
+
+  auto const metadata = tested.GetRequestMetadata();
+  EXPECT_THAT(metadata.headers,
+              UnorderedElementsAre(Pair("hk0", "v0"), Pair("hk1", "v1")));
+  EXPECT_THAT(metadata.trailers,
+              UnorderedElementsAre(Pair("tk0", "v0"), Pair("tk1", "v1")));
 }
 
 TEST(AsyncWriterConnectionTest, ResumeFinalized) {
@@ -106,15 +112,15 @@ TEST(AsyncWriterConnectionTest, ResumeFinalized) {
   EXPECT_CALL(*mock, Finish).WillOnce([] {
     return make_ready_future(Status{});
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(0);
   EXPECT_CALL(*hash, Finish).Times(0);
 
-  AsyncWriterConnectionImpl tested(TestOptions(), std::move(mock),
-                                   "test-upload-id", hash, MakeTestObject());
+  AsyncWriterConnectionImpl tested(TestOptions(), MakeRequest(),
+                                   std::move(mock), hash, MakeTestObject());
   EXPECT_EQ(tested.UploadId(), "test-upload-id");
-  EXPECT_THAT(tested.PersistedState(),
-              VariantWith<storage::ObjectMetadata>(MakeTestObject()));
+  EXPECT_THAT(tested.PersistedState(), VariantWith<google::storage::v2::Object>(
+                                           IsProtoEqual(MakeTestObject())));
 }
 
 TEST(AsyncWriterConnectionTest, Cancel) {
@@ -123,12 +129,12 @@ TEST(AsyncWriterConnectionTest, Cancel) {
   EXPECT_CALL(*mock, Finish).WillOnce([] {
     return make_ready_future(Status{});
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(0);
   EXPECT_CALL(*hash, Finish).Times(0);
 
-  AsyncWriterConnectionImpl tested(TestOptions(), std::move(mock),
-                                   "test-upload-id", hash, 1024);
+  AsyncWriterConnectionImpl tested(TestOptions(), MakeRequest(),
+                                   std::move(mock), hash, 1024);
   tested.Cancel();
   EXPECT_EQ(tested.UploadId(), "test-upload-id");
   EXPECT_THAT(tested.PersistedState(), VariantWith<std::int64_t>(1024));
@@ -149,6 +155,8 @@ TEST(AsyncWriterConnectionTest, WriteSimple) {
       .WillRepeatedly([&](Request const& request, grpc::WriteOptions wopt) {
         EXPECT_FALSE(request.finish_write());
         EXPECT_EQ(request.write_offset(), offset);
+        EXPECT_EQ(request.common_object_request_params().encryption_algorithm(),
+                  "test-only-algo");
         EXPECT_FALSE(wopt.is_last_message());
         offset += kChunk;
         return sequencer.PushBack("Write");
@@ -159,13 +167,13 @@ TEST(AsyncWriterConnectionTest, WriteSimple) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _))
       .Times(kWriteCount * kChunkCount);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, offset);
+      TestOptions(), MakeRequest(), std::move(mock), hash, offset);
 
   for (int i = 0; i != kWriteCount; ++i) {
     auto response =
@@ -200,12 +208,12 @@ TEST(AsyncWriterConnectionTest, WriteError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 0);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 0);
 
   auto response = tested->Write(WritePayload(std::string(kChunk, 'A')));
   auto next = sequencer.PopFrontWithName();
@@ -233,12 +241,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedWriteFailsWithoutError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 0);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 0);
 
   auto response = tested->Write(WritePayload(std::string(kChunk, 'A')));
   auto next = sequencer.PopFrontWithName();
@@ -258,6 +266,8 @@ TEST(AsyncWriterConnectionTest, FinalizeEmpty) {
       .WillOnce([&](Request const& request, grpc::WriteOptions wopt) {
         EXPECT_TRUE(request.finish_write());
         EXPECT_TRUE(wopt.is_last_message());
+        EXPECT_EQ(request.common_object_request_params().encryption_algorithm(),
+                  "test-only-algo");
         return sequencer.PushBack("Write");
       });
   EXPECT_CALL(*mock, Read).WillOnce([&]() {
@@ -272,12 +282,12 @@ TEST(AsyncWriterConnectionTest, FinalizeEmpty) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -286,7 +296,8 @@ TEST(AsyncWriterConnectionTest, FinalizeEmpty) {
   ASSERT_THAT(next.second, "Read");
   next.first.set_value(true);
   auto object = response.get();
-  EXPECT_THAT(object, IsOkAndHolds(MakeTestObject())) << "=" << *object;
+  EXPECT_THAT(object, IsOkAndHolds(IsProtoEqual(MakeTestObject())))
+      << "=" << object->DebugString();
 
   tested = {};
   next = sequencer.PopFrontWithName();
@@ -307,12 +318,12 @@ TEST(AsyncWriterConnectionTest, FinalizeFails) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -336,12 +347,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedFinalizeFailsWithoutError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -371,12 +382,12 @@ TEST(AsyncWriterConnectionTest, QueryFinalFails) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -409,12 +420,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedQueryFinalFailsWithoutError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -447,12 +458,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedQueryFinalMissingResource) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 0);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 0);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -477,6 +488,8 @@ TEST(AsyncWriterConnectionTest, FlushEmpty) {
         EXPECT_TRUE(request.flush());
         EXPECT_TRUE(request.state_lookup());
         EXPECT_FALSE(wopt.is_last_message());
+        EXPECT_EQ(request.common_object_request_params().encryption_algorithm(),
+                  "test-only-algo");
         return sequencer.PushBack("Write");
       });
   EXPECT_CALL(*mock, Read).WillOnce([&]() {
@@ -493,12 +506,12 @@ TEST(AsyncWriterConnectionTest, FlushEmpty) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto flush = tested->Flush(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -530,12 +543,12 @@ TEST(AsyncWriterConnectionTest, FlushFails) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -559,12 +572,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedFlushFailsWithoutError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(1);
   EXPECT_CALL(*hash, Finish).Times(1);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto response = tested->Finalize(WritePayload{});
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Write");
@@ -589,12 +602,12 @@ TEST(AsyncWriterConnectionTest, QueryFails) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(0);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto query = tested->Query();
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Read");
@@ -619,12 +632,12 @@ TEST(AsyncWriterConnectionTest, UnexpectedQueryFailsWithoutError) {
       return PermanentError();
     });
   });
-  auto hash = std::make_shared<MockHash>();
+  auto hash = std::make_shared<MockHashFunction>();
   EXPECT_CALL(*hash, Update(_, An<absl::Cord const&>(), _)).Times(0);
   EXPECT_CALL(*hash, Finish).Times(0);
 
   auto tested = std::make_unique<AsyncWriterConnectionImpl>(
-      TestOptions(), std::move(mock), "test-upload-id", hash, 1024);
+      TestOptions(), MakeRequest(), std::move(mock), hash, 1024);
   auto query = tested->Query();
   auto next = sequencer.PopFrontWithName();
   ASSERT_THAT(next.second, "Read");
