@@ -19,9 +19,12 @@
 #include "absl/strings/match.h"
 #include <chrono>
 #include <functional>
+#include <memory>
 #include <sstream>
+#include <string>
 #include <thread>
 #include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -56,7 +59,8 @@ Status ValidateCommittedSize(UploadChunkRequest const& request,
     os << " session_id=" << request.upload_session_url();
     os << ", result=" << response;
     os << ", request=" << request;
-    return Status(StatusCode::kInternal, os.str());
+    return google::cloud::internal::InternalError(std::move(os).str(),
+                                                  GCP_ERROR_INFO());
   }
   if (*response.committed_size > expected_committed_size) {
     std::stringstream os;
@@ -83,7 +87,8 @@ Status ValidateCommittedSize(UploadChunkRequest const& request,
     os << " session_id=" << request.upload_session_url();
     os << ", result=" << response;
     os << ", request=" << request;
-    return Status(StatusCode::kInternal, os.str());
+    return google::cloud::internal::InternalError(std::move(os).str(),
+                                                  GCP_ERROR_INFO());
   }
   return {};
 }
@@ -94,8 +99,8 @@ bool UploadChunkOnFailure(RetryPolicy& retry_policy, Status const& status) {
   // TODO(#9273) - use ErrorInfo when it becomes available
   if (status.code() == StatusCode::kAborted &&
       absl::StartsWith(status.message(), "Concurrent requests received.")) {
-    return retry_policy.OnFailure(Status(
-        StatusCode::kUnavailable, "TODO(#9273) - workaround service problems"));
+    return retry_policy.OnFailure(google::cloud::internal::UnavailableError(
+        "TODO(#9273) - workaround service problems", status.error_info()));
   }
   return retry_policy.OnFailure(status);
 }
@@ -113,7 +118,8 @@ Status MissingCommittedSize(int error_count, int upload_count, int reset_count,
   os << "All requests (" << upload_count << ") have succeeded, but they lacked"
      << " a committed_size value. This requires querying the write status."
      << " The client library performed " << reset_count << " such queries.";
-  return Status{StatusCode::kDeadlineExceeded, std::move(os).str()};
+  return google::cloud::internal::DeadlineExceededError(std::move(os).str(),
+                                                        GCP_ERROR_INFO());
 }
 
 Status PartialWriteStatus(int error_count, int upload_count,
@@ -126,7 +132,8 @@ Status PartialWriteStatus(int error_count, int upload_count,
      << " not completed the full write. The expected committed size is "
      << expected_committed_size << " and the current committed size is "
      << committed_size;
-  return Status{StatusCode::kDeadlineExceeded, std::move(os).str()};
+  return google::cloud::internal::DeadlineExceededError(std::move(os).str(),
+                                                        GCP_ERROR_INFO());
 }
 
 auto constexpr kIdempotencyTokenHeader = "x-goog-gcs-idempotency-token";
@@ -361,36 +368,39 @@ StatusOr<ObjectMetadata> StorageConnectionImpl::GetObjectMetadata(
       google::cloud::internal::CurrentOptions(), request, __func__);
 }
 
-StatusOr<std::unique_ptr<ObjectReadSource>>
-StorageConnectionImpl::ReadObjectNotWrapped(
-    ReadObjectRangeRequest const& request, RetryPolicy& retry_policy,
-    BackoffPolicy& backoff_policy) {
-  auto const idempotency = current_idempotency_policy().IsIdempotent(request)
-                               ? Idempotency::kIdempotent
-                               : Idempotency::kNonIdempotent;
-  return RestRetryLoop(
-      retry_policy, backoff_policy, idempotency,
-      [token = MakeIdempotencyToken(), this](
-          rest_internal::RestContext& context, Options const& options,
-          auto const& request) {
-        context.AddHeader(kIdempotencyTokenHeader, token);
-        return stub_->ReadObject(context, options, request);
-      },
-      google::cloud::internal::CurrentOptions(), request, __func__);
-}
-
 StatusOr<std::unique_ptr<ObjectReadSource>> StorageConnectionImpl::ReadObject(
     ReadObjectRangeRequest const& request) {
-  auto retry_policy = current_retry_policy();
-  auto backoff_policy = current_backoff_policy();
-  auto child = ReadObjectNotWrapped(request, *retry_policy, *backoff_policy);
-  if (!child) {
-    return child;
-  }
+  auto current = google::cloud::internal::SaveCurrentOptions();
+
   auto self = shared_from_this();
-  return std::unique_ptr<ObjectReadSource>(new RetryObjectReadSource(
-      self, request, *std::move(child), std::move(retry_policy),
-      std::move(backoff_policy)));
+  auto const* where = __func__;
+  auto factory = [self = shared_from_this(), current, where](
+                     ReadObjectRangeRequest const& request,
+                     RetryPolicy& retry_policy, BackoffPolicy& backoff_policy) {
+    auto const idempotency =
+        current->get<IdempotencyPolicyOption>()->IsIdempotent(request)
+            ? Idempotency::kIdempotent
+            : Idempotency::kNonIdempotent;
+    return RestRetryLoop(
+        retry_policy, backoff_policy, idempotency,
+        [self, token = self->MakeIdempotencyToken()](
+            rest_internal::RestContext& context, Options const& options,
+            ReadObjectRangeRequest const& request) {
+          context.AddHeader(kIdempotencyTokenHeader, token);
+          return self->stub_->ReadObject(context, options, request);
+        },
+        *current, request, where);
+  };
+
+  auto retry_policy = current->get<RetryPolicyOption>()->clone();
+  auto backoff_policy = current->get<BackoffPolicyOption>()->clone();
+  auto child = factory(request, *retry_policy, *backoff_policy);
+  if (!child) return child;
+
+  return std::unique_ptr<ObjectReadSource>(
+      std::make_unique<RetryObjectReadSource>(
+          std::move(factory), std::move(current), request, *std::move(child),
+          std::move(retry_policy), std::move(backoff_policy)));
 }
 
 StatusOr<ListObjectsResponse> StorageConnectionImpl::ListObjects(
@@ -489,6 +499,23 @@ StatusOr<RewriteObjectResponse> StorageConnectionImpl::RewriteObject(
       google::cloud::internal::CurrentOptions(), request, __func__);
 }
 
+StatusOr<ObjectMetadata> StorageConnectionImpl::RestoreObject(
+    RestoreObjectRequest const& request) {
+  auto const idempotency = current_idempotency_policy().IsIdempotent(request)
+                               ? Idempotency::kIdempotent
+                               : Idempotency::kNonIdempotent;
+
+  return RestRetryLoop(
+      current_retry_policy(), current_backoff_policy(), idempotency,
+      [token = MakeIdempotencyToken(), this](
+          rest_internal::RestContext& context, Options const& options,
+          auto const& request) {
+        context.AddHeader(kIdempotencyTokenHeader, token);
+        return stub_->RestoreObject(context, options, request);
+      },
+      google::cloud::internal::CurrentOptions(), request, __func__);
+}
+
 StatusOr<CreateResumableUploadResponse>
 StorageConnectionImpl::CreateResumableUpload(
     ResumableUploadRequest const& request) {
@@ -598,9 +625,9 @@ StatusOr<QueryResumableUploadResponse> StorageConnectionImpl::UploadChunk(
       [](std::chrono::milliseconds d) { std::this_thread::sleep_for(d); };
   sleeper = google::cloud::internal::MakeTracedSleeper(
       current, std::move(sleeper), "Backoff");
-  auto last_status =
-      Status(StatusCode::kDeadlineExceeded,
-             "Retry policy exhausted before first attempt was made.");
+  auto last_status = google::cloud::internal::DeadlineExceededError(
+      "Retry policy exhausted before first attempt was made.",
+      GCP_ERROR_INFO());
 
   auto retry_policy = current_retry_policy();
   auto backoff_policy = current_backoff_policy();
@@ -686,7 +713,12 @@ StatusOr<QueryResumableUploadResponse> StorageConnectionImpl::UploadChunk(
         ValidateCommittedSize(request, *result, expected_committed_size);
     if (!validate.ok()) return validate;
 
-    committed_size = *result->committed_size;
+    if (committed_size != *result->committed_size) {
+      // A partial write is a sign of progress. We reset the backoff policy.
+      // Otherwise we would unnecessarily delay the next upload.
+      backoff_policy = current_backoff_policy();
+      committed_size = *result->committed_size;
+    }
 
     if (committed_size != expected_committed_size || request.last_chunk()) {
       // If we still have to send data, restart the loop. On the last chunk,
