@@ -13,11 +13,14 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/default_object_acl_requests.h"
+#include "google/cloud/storage/internal/metadata_parser.h"
 #include "google/cloud/storage/internal/object_access_control_parser.h"
 #include "google/cloud/storage/internal/object_acl_requests.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
 #include <nlohmann/json.hpp>
 #include <iostream>
+#include <string>
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -35,9 +38,7 @@ std::ostream& operator<<(std::ostream& os,
 StatusOr<ListDefaultObjectAclResponse>
 ListDefaultObjectAclResponse::FromHttpResponse(std::string const& payload) {
   auto json = nlohmann::json::parse(payload, nullptr, false);
-  if (!json.is_object()) {
-    return Status(StatusCode::kInvalidArgument, __func__);
-  }
+  if (!json.is_object()) return ExpectedJsonObject(payload, GCP_ERROR_INFO());
   ListDefaultObjectAclResponse result;
   for (auto const& kv : json["items"].items()) {
     auto parsed = internal::ObjectAccessControlParser::FromJson(kv.value());
