@@ -17,6 +17,8 @@
 #include <gmock/gmock.h>
 #include <algorithm>
 #include <cctype>
+#include <random>
+#include <string>
 
 namespace google {
 namespace cloud {
