@@ -13,16 +13,17 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/grpc/object_request_parser.h"
+#include "google/cloud/storage/internal/base64.h"
 #include "google/cloud/storage/internal/grpc/bucket_name.h"
 #include "google/cloud/storage/internal/grpc/object_access_control_parser.h"
 #include "google/cloud/storage/internal/grpc/object_metadata_parser.h"
 #include "google/cloud/storage/internal/object_access_control_parser.h"
-#include "google/cloud/storage/internal/openssl_util.h"
 #include "google/cloud/storage/internal/patch_builder_details.h"
+#include "google/cloud/internal/absl_str_cat_quiet.h"
 #include "google/cloud/internal/invoke_result.h"
 #include "google/cloud/internal/make_status.h"
 #include "google/cloud/internal/time_utils.h"
-#include "google/cloud/log.h"
+#include <iterator>
 
 namespace google {
 namespace cloud {
@@ -336,6 +337,9 @@ google::storage::v2::GetObjectRequest ToProto(
   result.set_generation(request.GetOption<storage::Generation>().value_or(0));
   auto projection = request.GetOption<storage::Projection>().value_or("");
   if (projection == "full") result.mutable_read_mask()->add_paths("*");
+  if (request.GetOption<storage::SoftDeleted>().value_or(false)) {
+    result.set_soft_deleted(true);
+  }
   return result;
 }
 
@@ -374,24 +378,29 @@ StatusOr<google::storage::v2::ReadObjectRequest> ToProto(
   if (request.HasOption<storage::Generation>()) {
     r.set_generation(request.GetOption<storage::Generation>().value());
   }
-  if (request.HasOption<storage::ReadRange>()) {
+  auto const offset = request.GetOption<storage::ReadFromOffset>().value_or(0);
+  if (!request.HasOption<storage::ReadRange>()) {
+    r.set_read_offset(offset);
+  } else {
     auto const range = request.GetOption<storage::ReadRange>().value();
-    r.set_read_offset(range.begin);
-    r.set_read_limit(range.end - range.begin);
+    auto const merged_offset = (std::max)(range.begin, offset);
+    auto const merged_limit = range.end - merged_offset;
+    if (merged_limit <= 0) {
+      return internal::InvalidArgumentError(
+          absl::StrCat("Invalid gRPC range offset=", merged_offset,
+                       ", limit=", merged_limit, ", from offset=", offset,
+                       " range=[", range.begin, ",", range.end, "]"),
+          GCP_ERROR_INFO());
+    }
+
+    r.set_read_offset(merged_offset);
+    r.set_read_limit(merged_limit);
   }
+
   if (request.HasOption<storage::ReadLast>()) {
     auto const offset = request.GetOption<storage::ReadLast>().value();
     r.set_read_offset(-offset);
   }
-  if (request.HasOption<storage::ReadFromOffset>()) {
-    auto const offset = request.GetOption<storage::ReadFromOffset>().value();
-    if (offset > r.read_offset()) {
-      if (r.read_limit() > 0) {
-        r.set_read_limit(offset - r.read_offset());
-      }
-      r.set_read_offset(offset);
-    }
-  }
   SetGenerationConditions(r, request);
   SetMetagenerationConditions(r, request);
 
@@ -526,11 +535,6 @@ StatusOr<google::storage::v2::UpdateObjectRequest> ToProto(
   return result;
 }
 
-StatusOr<google::storage::v2::WriteObjectRequest> ToProto(
-    storage_experimental::InsertObjectRequest const& request) {
-  return ToProtoImpl(request);
-}
-
 StatusOr<google::storage::v2::WriteObjectRequest> ToProto(
     storage::internal::InsertObjectMediaRequest const& request) {
   return ToProtoImpl(request);
@@ -538,7 +542,7 @@ StatusOr<google::storage::v2::WriteObjectRequest> ToProto(
 
 storage::internal::QueryResumableUploadResponse FromProto(
     google::storage::v2::WriteObjectResponse const& p, Options const& options,
-    google::cloud::internal::StreamingRpcMetadata metadata) {
+    google::cloud::RpcMetadata metadata) {
   storage::internal::QueryResumableUploadResponse response;
   if (p.has_persisted_size()) {
     response.committed_size = static_cast<std::uint64_t>(p.persisted_size());
@@ -546,7 +550,10 @@ storage::internal::QueryResumableUploadResponse FromProto(
   if (p.has_resource()) {
     response.payload = storage_internal::FromProto(p.resource(), options);
   }
-  response.request_metadata = std::move(metadata);
+  response.request_metadata = std::move(metadata.headers);
+  response.request_metadata.insert(
+      std::make_move_iterator(metadata.trailers.begin()),
+      std::make_move_iterator(metadata.trailers.end()));
   return response;
 }
 
@@ -569,12 +576,16 @@ google::storage::v2::ListObjectsRequest ToProto(
   result.set_include_trailing_delimiter(
       request.GetOption<storage::IncludeTrailingDelimiter>().value_or(false));
   result.set_prefix(request.GetOption<storage::Prefix>().value_or(""));
-  result.set_versions(request.GetOption<storage::Versions>().value_or(""));
+  result.set_versions(request.GetOption<storage::Versions>().value_or(false));
   result.set_lexicographic_start(
       request.GetOption<storage::StartOffset>().value_or(""));
   result.set_lexicographic_end(
       request.GetOption<storage::EndOffset>().value_or(""));
   result.set_match_glob(request.GetOption<storage::MatchGlob>().value_or(""));
+  result.set_soft_deleted(
+      request.GetOption<storage::SoftDeleted>().value_or(false));
+  result.set_include_folders_as_prefixes(
+      request.GetOption<storage::IncludeFoldersAsPrefixes>().value_or(false));
   return result;
 }
 
@@ -655,6 +666,22 @@ StatusOr<google::storage::v2::RewriteObjectRequest> ToProto(
   return result;
 }
 
+StatusOr<google::storage::v2::RestoreObjectRequest> ToProto(
+    storage::internal::RestoreObjectRequest const& request) {
+  google::storage::v2::RestoreObjectRequest result;
+  auto status = SetCommonObjectParameters(result, request);
+
+  result.set_bucket(GrpcBucketIdToName(request.bucket_name()));
+  result.set_object(request.object_name());
+  result.set_generation(request.generation());
+  SetGenerationConditions(result, request);
+  SetMetagenerationConditions(result, request);
+  result.set_copy_source_acl(
+      request.GetOption<storage::CopySourceAcl>().value_or(false));
+
+  return result;
+}
+
 storage::internal::RewriteObjectResponse FromProto(
     google::storage::v2::RewriteResponse const& response,
     Options const& options) {
