@@ -214,6 +214,20 @@ TEST(GrpcObjectRequestParser, DeleteObjectAllFields) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
+TEST(GrpcObjectRequestParser, GetObjectMetadata) {
+  google::storage::v2::GetObjectRequest expected;
+  EXPECT_TRUE(TextFormat::ParseFromString(
+      R"pb(
+        bucket: "projects/_/buckets/test-bucket" object: "test-object"
+      )pb",
+      &expected));
+
+  storage::internal::GetObjectMetadataRequest req("test-bucket", "test-object");
+
+  auto const actual = ToProto(req);
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 TEST(GrpcObjectRequestParser, GetObjectMetadataAllFields) {
   google::storage::v2::GetObjectRequest expected;
   EXPECT_TRUE(TextFormat::ParseFromString(
@@ -226,6 +240,7 @@ TEST(GrpcObjectRequestParser, GetObjectMetadataAllFields) {
         if_metageneration_match: 3
         if_metageneration_not_match: 4
         read_mask { paths: "*" }
+        soft_deleted: true
       )pb",
       &expected));
 
@@ -234,8 +249,7 @@ TEST(GrpcObjectRequestParser, GetObjectMetadataAllFields) {
       storage::Generation(7), storage::IfGenerationMatch(1),
       storage::IfGenerationNotMatch(2), storage::IfMetagenerationMatch(3),
       storage::IfMetagenerationNotMatch(4), storage::Projection("full"),
-      storage::UserProject("test-user-project"),
-      storage::UserProject("test-user-project"),
+      storage::SoftDeleted(true), storage::UserProject("test-user-project"),
       storage::QuotaUser("test-quota-user"), storage::UserIp("test-user-ip"));
 
   auto const actual = ToProto(req);
@@ -353,6 +367,45 @@ TEST(GrpcObjectRequestParser,
   EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument));
 }
 
+TEST(GrpcObjectRequestParser, ReadObjectRangeMergeWithOffset) {
+  auto expected = google::storage::v2::ReadObjectRequest{};
+  EXPECT_TRUE(TextFormat::ParseFromString(
+      R"pb(
+        bucket: "projects/_/buckets/test-bucket"
+        object: "test-object"
+        read_offset: 2000
+        read_limit: 1000
+      )pb",
+      &expected));
+  auto request =
+      storage::internal::ReadObjectRangeRequest("test-bucket", "test-object")
+          .set_multiple_options(storage::ReadRange(2000, 3000));
+  EXPECT_THAT(ToProto(request), IsOkAndHolds(IsProtoEqual(expected)));
+
+  request.set_multiple_options(storage::ReadFromOffset(1900));
+  EXPECT_THAT(ToProto(request), IsOkAndHolds(IsProtoEqual(expected)));
+
+  request.set_multiple_options(storage::ReadFromOffset(2100));
+  expected.set_read_offset(2100);
+  expected.set_read_limit(900);
+  EXPECT_THAT(ToProto(request), IsOkAndHolds(IsProtoEqual(expected)));
+
+  request.set_multiple_options(storage::ReadFromOffset(2300));
+  expected.set_read_offset(2300);
+  expected.set_read_limit(700);
+  EXPECT_THAT(ToProto(request), IsOkAndHolds(IsProtoEqual(expected)));
+}
+
+TEST(GrpcObjectRequestParser, ReadOffsetEmptyRange) {
+  auto request =
+      storage::internal::ReadObjectRangeRequest("test-bucket", "test-object")
+          .set_multiple_options(storage::ReadFromOffset(3000),
+                                storage::ReadRange(2000, 3000));
+
+  auto const actual = ToProto(request);
+  EXPECT_THAT(actual, StatusIs(StatusCode::kInvalidArgument));
+}
+
 TEST(GrpcObjectRequestParser, PatchObjectRequestAllOptions) {
   auto constexpr kTextProto = R"pb(
     predefined_acl: "projectPrivate"
@@ -752,88 +805,6 @@ TEST(GrpcObjectRequestParser, InsertObjectMediaRequestWithObjectMetadata) {
   EXPECT_THAT(actual, IsProtoEqual(expected));
 }
 
-TEST(GrpcObjectRequestParser, InsertObjectRequestSimple) {
-  storage_proto::WriteObjectRequest expected;
-  EXPECT_TRUE(TextFormat::ParseFromString(
-      R"pb(
-        write_object_spec: {
-          resource: {
-            bucket: "projects/_/buckets/test-bucket-name"
-            name: "test-object-name"
-          }
-        }
-      )pb",
-      &expected));
-
-  storage_experimental::InsertObjectRequest request("test-bucket-name",
-                                                    "test-object-name");
-  auto actual = ToProto(request).value();
-  EXPECT_THAT(actual, IsProtoEqual(expected));
-}
-
-TEST(GrpcObjectRequestParser, InsertObjectRequestAllOptions) {
-  auto constexpr kTextProto = R"pb(
-    write_object_spec {
-      resource: {
-        bucket: "projects/_/buckets/test-bucket-name"
-        name: "test-object-name"
-        content_type: "test-content-type"
-        content_encoding: "test-content-encoding"
-        # Should not be set, the proto file says these values should
-        # not be included in the upload
-        #     crc32c:
-        #     md5_hash:
-        kms_key: "test-kms-key-name"
-      }
-      predefined_acl: "private"
-      if_generation_match: 0
-      if_generation_not_match: 7
-      if_metageneration_match: 42
-      if_metageneration_not_match: 84
-    })pb";
-  storage_proto::WriteObjectRequest expected;
-  EXPECT_TRUE(TextFormat::ParseFromString(kTextProto, &expected));
-  *expected.mutable_common_object_request_params() =
-      ExpectedCommonObjectRequestParams();
-
-  auto constexpr kContents = "The quick brown fox jumps over the lazy dog";
-
-  storage_experimental::InsertObjectRequest request("test-bucket-name",
-                                                    "test-object-name");
-  request.set_multiple_options(
-      storage::ContentType("test-content-type"),
-      storage::ContentEncoding("test-content-encoding"),
-      storage::Crc32cChecksumValue(storage::ComputeCrc32cChecksum(kContents)),
-      storage::MD5HashValue(storage::ComputeMD5Hash(kContents)),
-      storage::PredefinedAcl("private"), storage::IfGenerationMatch(0),
-      storage::IfGenerationNotMatch(7), storage::IfMetagenerationMatch(42),
-      storage::IfMetagenerationNotMatch(84), storage::Projection::Full(),
-      storage::UserProject("test-user-project"),
-      storage::QuotaUser("test-quota-user"), storage::UserIp("test-user-ip"),
-      storage::EncryptionKey::FromBinaryKey("01234567"),
-      storage::KmsKeyName("test-kms-key-name"));
-
-  auto actual = ToProto(request).value();
-  EXPECT_THAT(actual, IsProtoEqual(expected));
-}
-
-TEST(GrpcObjectRequestParser, InsertObjectRequestWithObjectMetadata) {
-  storage_proto::WriteObjectRequest expected;
-  auto& resource = *expected.mutable_write_object_spec()->mutable_resource();
-  resource = ExpectedFullObjectMetadata();
-  resource.set_bucket("projects/_/buckets/test-bucket-name");
-  resource.set_name("test-object-name");
-  resource.set_storage_class("STANDARD");
-
-  storage_experimental::InsertObjectRequest request("test-bucket-name",
-                                                    "test-object-name");
-  request.set_multiple_options(storage::WithObjectMetadata(
-      FullObjectMetadata().set_storage_class("STANDARD")));
-
-  auto actual = ToProto(request).value();
-  EXPECT_THAT(actual, IsProtoEqual(expected));
-}
-
 TEST(GrpcObjectRequestParser, WriteObjectResponseSimple) {
   google::storage::v2::WriteObjectResponse input;
   ASSERT_TRUE(TextFormat::ParseFromString(
@@ -859,7 +830,8 @@ TEST(GrpcObjectRequestParser, WriteObjectResponseWithResource) {
       &input));
 
   auto const actual = FromProto(
-      input, Options{}, {{"header", "value"}, {"other-header", "other-value"}});
+      input, Options{},
+      RpcMetadata{{{"header", "value"}, {"other-header", "other-value"}}, {}});
   EXPECT_FALSE(actual.committed_size.has_value());
   ASSERT_TRUE(actual.payload.has_value());
   EXPECT_EQ(actual.payload->name(), "test-object-name");
@@ -870,6 +842,20 @@ TEST(GrpcObjectRequestParser, WriteObjectResponseWithResource) {
                                    Pair("other-header", "other-value")));
 }
 
+TEST(GrpcObjectRequestParser, ListObjectsRequest) {
+  google::storage::v2::ListObjectsRequest expected;
+  ASSERT_TRUE(TextFormat::ParseFromString(
+      R"pb(
+        parent: "projects/_/buckets/test-bucket"
+      )pb",
+      &expected));
+
+  storage::internal::ListObjectsRequest req("test-bucket");
+
+  auto const actual = ToProto(req);
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
 TEST(GrpcObjectRequestParser, ListObjectsRequestAllFields) {
   google::storage::v2::ListObjectsRequest expected;
   ASSERT_TRUE(TextFormat::ParseFromString(
@@ -884,6 +870,8 @@ TEST(GrpcObjectRequestParser, ListObjectsRequestAllFields) {
         lexicographic_start: "test/prefix/a"
         lexicographic_end: "test/prefix/abc"
         match_glob: "**/*.cc"
+        soft_deleted: true
+        include_folders_as_prefixes: true
       )pb",
       &expected));
 
@@ -894,6 +882,7 @@ TEST(GrpcObjectRequestParser, ListObjectsRequestAllFields) {
       storage::IncludeTrailingDelimiter(true), storage::Prefix("test/prefix"),
       storage::Versions(true), storage::StartOffset("test/prefix/a"),
       storage::EndOffset("test/prefix/abc"), storage::MatchGlob("**/*.cc"),
+      storage::SoftDeleted(true), storage::IncludeFoldersAsPrefixes(true),
       storage::UserProject("test-user-project"),
       storage::QuotaUser("test-quota-user"), storage::UserIp("test-user-ip"));
 
@@ -1057,6 +1046,50 @@ TEST(GrpcObjectRequestParser, RewriteObjectResponse) {
   EXPECT_EQ(actual.resource.name(), "object-name");
 }
 
+TEST(GrpcObjectRequestParser, RestoreObjectSimpleRequest) {
+  google::storage::v2::RestoreObjectRequest expected;
+  EXPECT_TRUE(
+      TextFormat::ParseFromString(R"pb(
+                                    bucket: "projects/_/buckets/test-bucket"
+                                    object: "test-object",
+                                    generation: 1234678,
+                                    copy_source_acl: false,
+                                  )pb",
+                                  &expected));
+
+  storage::internal::RestoreObjectRequest req("test-bucket", "test-object",
+                                              1234678);
+
+  auto actual = ToProto(req).value();
+  EXPECT_THAT(actual, IsProtoEqual(expected));
+}
+
+TEST(GrpcObjectRequestParser, RestoreObjectRequestAllFields) {
+  google::storage::v2::RestoreObjectRequest expected;
+  ASSERT_TRUE(TextFormat::ParseFromString(
+      R"pb(
+        bucket: "projects/_/buckets/test-bucket"
+        object: "test-object",
+        generation: 1234678,
+        if_generation_match: 1
+        if_generation_not_match: 2
+        if_metageneration_match: 3
+        if_metageneration_not_match: 4
+        copy_source_acl: true
+      )pb",
+      &expected));
+
+  storage::internal::RestoreObjectRequest req("test-bucket", "test-object",
+                                              1234678);
+  req.set_multiple_options(
+      storage::IfGenerationMatch(1), storage::IfGenerationNotMatch(2),
+      storage::IfMetagenerationMatch(3), storage::IfMetagenerationNotMatch(4),
+      storage::CopySourceAcl(true));
+  auto const actual = ToProto(req);
+  ASSERT_STATUS_OK(actual);
+  EXPECT_THAT(*actual, IsProtoEqual(expected));
+}
+
 TEST(GrpcObjectRequestParser, CopyObjectRequestAllOptions) {
   auto constexpr kTextProto = R"pb(
     destination_bucket: "projects/_/buckets/destination-bucket"
@@ -1333,11 +1366,10 @@ TEST(GrpcObjectRequestParser, MaybeFinalizeUploadChunkRequest) {
                         bool with_md5) -> std::shared_ptr<HashFunction> {
     if (with_crc32c && with_md5) {
       return std::make_shared<CompositeFunction>(
-          std::make_unique<Crc32cHashFunction>(),
-          std::make_unique<MD5HashFunction>());
+          std::make_unique<Crc32cHashFunction>(), MD5HashFunction::Create());
     }
     if (with_crc32c) return std::make_shared<Crc32cHashFunction>();
-    if (with_md5) return std::make_shared<MD5HashFunction>();
+    if (with_md5) return MD5HashFunction::Create();
     return std::make_shared<NullHashFunction>();
   };
 
