@@ -20,25 +20,20 @@
 #include "google/cloud/storage/internal/grpc/bucket_request_parser.h"
 #include "google/cloud/storage/internal/grpc/configure_client_context.h"
 #include "google/cloud/storage/internal/grpc/ctype_cord_workaround.h"
-#include "google/cloud/storage/internal/grpc/hmac_key_metadata_parser.h"
-#include "google/cloud/storage/internal/grpc/hmac_key_request_parser.h"
+#include "google/cloud/storage/internal/grpc/default_options.h"
 #include "google/cloud/storage/internal/grpc/notification_metadata_parser.h"
-#include "google/cloud/storage/internal/grpc/notification_request_parser.h"
 #include "google/cloud/storage/internal/grpc/object_access_control_parser.h"
 #include "google/cloud/storage/internal/grpc/object_metadata_parser.h"
 #include "google/cloud/storage/internal/grpc/object_read_source.h"
 #include "google/cloud/storage/internal/grpc/object_request_parser.h"
-#include "google/cloud/storage/internal/grpc/service_account_parser.h"
+#include "google/cloud/storage/internal/grpc/scale_stall_timeout.h"
 #include "google/cloud/storage/internal/grpc/sign_blob_request_parser.h"
 #include "google/cloud/storage/internal/grpc/split_write_object_data.h"
 #include "google/cloud/storage/internal/grpc/synthetic_self_link.h"
 #include "google/cloud/storage/internal/storage_stub_factory.h"
-#include "google/cloud/grpc_options.h"
 #include "google/cloud/internal/big_endian.h"
-#include "google/cloud/internal/getenv.h"
 #include "google/cloud/internal/invoke_result.h"
-#include "google/cloud/internal/populate_common_options.h"
-#include "google/cloud/log.h"
+#include "google/cloud/internal/make_status.h"
 #include "absl/strings/match.h"
 #include "absl/time/time.h"
 #include <grpcpp/grpcpp.h>
@@ -80,9 +75,9 @@ StatusOr<storage::BucketAccessControl> FindBucketAccessControl(
     if (acl.entity() != entity && acl.entity_alt() != entity) continue;
     return FromProto(acl, response->bucket_id(), bucket_self_link);
   }
-  return Status(
-      StatusCode::kNotFound,
-      "cannot find entity <" + entity + "> in bucket " + response->bucket_id());
+  return google::cloud::internal::NotFoundError(
+      "cannot find entity <" + entity + "> in bucket " + response->bucket_id(),
+      GCP_ERROR_INFO());
 }
 
 // Used in the implementation of `*ObjectAcl()`.
@@ -96,9 +91,10 @@ StatusOr<storage::ObjectAccessControl> FindObjectAccessControl(
     return FromProto(acl, bucket_id, response->name(), response->generation(),
                      object_self_link);
   }
-  return Status(StatusCode::kNotFound, "cannot find entity <" + entity +
-                                           "> in bucket/object " + bucket_id +
-                                           "/" + response->name());
+  return google::cloud::internal::NotFoundError(
+      "cannot find entity <" + entity + "> in bucket/object " + bucket_id +
+          "/" + response->name(),
+      GCP_ERROR_INFO());
 }
 
 // Used in the implementation of `*DefaultObjectAcl()`.
@@ -109,23 +105,16 @@ StatusOr<storage::ObjectAccessControl> FindDefaultObjectAccessControl(
     if (acl.entity() != entity && acl.entity_alt() != entity) continue;
     return FromProtoDefaultObjectAccessControl(acl, response->bucket_id());
   }
-  return Status(
-      StatusCode::kNotFound,
-      "cannot find entity <" + entity + "> in bucket " + response->bucket_id());
-}
-
-std::chrono::milliseconds ScaleStallTimeout(std::chrono::milliseconds timeout,
-                                            std::uint32_t size,
-                                            std::uint32_t quantum) {
-  if (timeout == std::chrono::milliseconds(0)) return timeout;
-  if (quantum <= size || size == 0) return timeout;
-  return timeout * quantum / size;
+  return google::cloud::internal::NotFoundError(
+      "cannot find entity <" + entity + "> in bucket " + response->bucket_id(),
+      GCP_ERROR_INFO());
 }
 
 Status TimeoutError(std::chrono::milliseconds timeout, std::string const& op) {
-  return Status(StatusCode::kDeadlineExceeded,
-                "timeout [" + absl::FormatDuration(absl::FromChrono(timeout)) +
-                    "] while waiting for " + op);
+  return google::cloud::internal::DeadlineExceededError(
+      "timeout [" + absl::FormatDuration(absl::FromChrono(timeout)) +
+          "] while waiting for " + op,
+      GCP_ERROR_INFO());
 }
 
 StatusOr<storage::internal::QueryResumableUploadResponse>
@@ -187,62 +176,6 @@ CloseWriteObjectStream(std::chrono::milliseconds timeout,
 
 using ::google::cloud::internal::MakeBackgroundThreadsFactory;
 
-int DefaultGrpcNumChannels(std::string const& endpoint) {
-  // When using DirectPath the gRPC library already does load balancing across
-  // multiple sockets, it makes little sense to perform additional load
-  // balancing in the client library.
-  if (absl::StartsWith(endpoint, "google-c2p:///") ||
-      absl::StartsWith(endpoint, "google-c2p-experimental:///")) {
-    return 1;
-  }
-  // When not using DirectPath, there are limits to the bandwidth per channel,
-  // we want to create more channels to avoid hitting said limits.  The value
-  // here is mostly a guess: we know 1 channel is too little for most
-  // applications, but the ideal number depends on the workload.  The
-  // application can always override this default, so it is not important to
-  // have it exactly right.
-  auto constexpr kMinimumChannels = 4;
-  auto const count = std::thread::hardware_concurrency();
-  return (std::max)(kMinimumChannels, static_cast<int>(count));
-}
-
-Options DefaultOptionsGrpc(Options options) {
-  using ::google::cloud::internal::GetEnv;
-  // Experiments show that gRPC gets better upload throughput when the upload
-  // buffer is at least 32MiB.
-  auto constexpr kDefaultGrpcUploadBufferSize = 32 * 1024 * 1024L;
-  options = google::cloud::internal::MergeOptions(
-      std::move(options), Options{}.set<storage::UploadBufferSizeOption>(
-                              kDefaultGrpcUploadBufferSize));
-  options =
-      storage::internal::DefaultOptionsWithCredentials(std::move(options));
-  if (!options.has<UnifiedCredentialsOption>() &&
-      !options.has<GrpcCredentialOption>()) {
-    options.set<UnifiedCredentialsOption>(
-        google::cloud::MakeGoogleDefaultCredentials(
-            google::cloud::internal::MakeAuthOptions(options)));
-  }
-  auto const testbench =
-      GetEnv("CLOUD_STORAGE_EXPERIMENTAL_GRPC_TESTBENCH_ENDPOINT");
-  if (testbench.has_value() && !testbench->empty()) {
-    options.set<EndpointOption>(*testbench);
-    // The emulator does not support HTTPS or authentication, use insecure
-    // (sometimes called "anonymous") credentials, which disable SSL.
-    options.set<UnifiedCredentialsOption>(MakeInsecureCredentials());
-  }
-
-  options = google::cloud::internal::MergeOptions(
-      std::move(options), Options{}
-                              .set<EndpointOption>("storage.googleapis.com")
-                              .set<AuthorityOption>("storage.googleapis.com"));
-  // We can only compute this once the endpoint is known, so take an additional
-  // step.
-  auto const num_channels =
-      DefaultGrpcNumChannels(options.get<EndpointOption>());
-  return google::cloud::internal::MergeOptions(
-      std::move(options), Options{}.set<GrpcNumChannelsOption>(num_channels));
-}
-
 GrpcStub::GrpcStub(Options opts)
     : options_(std::move(opts)),
       background_(MakeBackgroundThreadsFactory(options_)()),
@@ -268,7 +201,7 @@ StatusOr<storage::internal::ListBucketsResponse> GrpcStub::ListBuckets(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->ListBuckets(ctx, proto);
+  auto response = stub_->ListBuckets(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response);
 }
@@ -280,7 +213,7 @@ StatusOr<storage::BucketMetadata> GrpcStub::CreateBucket(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->CreateBucket(ctx, proto);
+  auto response = stub_->CreateBucket(ctx, options, proto);
   if (response) return FromProto(*response, options);
   // GCS returns kFailedPrecondition when the bucket already exists. I filed
   // a bug to change this to kAborted, for consistency with JSON.  In either
@@ -288,8 +221,8 @@ StatusOr<storage::BucketMetadata> GrpcStub::CreateBucket(
   // as we do for the JSON transport.
   auto const code = response.status().code();
   if (code == StatusCode::kFailedPrecondition || code == StatusCode::kAborted) {
-    return Status(StatusCode::kAlreadyExists, response.status().message(),
-                  response.status().error_info());
+    return google::cloud::internal::AlreadyExistsError(
+        response.status().message(), response.status().error_info());
   }
   return std::move(response).status();
 }
@@ -309,7 +242,7 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteBucket(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto status = stub_->DeleteBucket(ctx, proto);
+  auto status = stub_->DeleteBucket(ctx, options, proto);
   if (!status.ok()) return status;
   return storage::internal::EmptyResponse{};
 }
@@ -321,7 +254,7 @@ StatusOr<storage::BucketMetadata> GrpcStub::UpdateBucket(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->UpdateBucket(ctx, proto);
+  auto response = stub_->UpdateBucket(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -341,7 +274,7 @@ StatusOr<storage::NativeIamPolicy> GrpcStub::GetNativeBucketIamPolicy(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->GetIamPolicy(ctx, proto);
+  auto response = stub_->GetIamPolicy(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response);
 }
@@ -353,7 +286,7 @@ StatusOr<storage::NativeIamPolicy> GrpcStub::SetNativeBucketIamPolicy(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->SetIamPolicy(ctx, proto);
+  auto response = stub_->SetIamPolicy(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response);
 }
@@ -366,7 +299,7 @@ GrpcStub::TestBucketIamPermissions(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->TestIamPermissions(ctx, proto);
+  auto response = stub_->TestIamPermissions(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response);
 }
@@ -378,7 +311,7 @@ StatusOr<storage::BucketMetadata> GrpcStub::LockBucketRetentionPolicy(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->LockBucketRetentionPolicy(ctx, proto);
+  auto response = stub_->LockBucketRetentionPolicy(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -462,12 +395,12 @@ StatusOr<storage::ObjectMetadata> GrpcStub::CopyObject(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->RewriteObject(ctx, *proto);
+  auto response = stub_->RewriteObject(ctx, options, *proto);
   if (!response) return std::move(response).status();
   if (!response->done()) {
-    return Status(
-        StatusCode::kOutOfRange,
-        "Object too large, use RewriteObject() instead of CopyObject()");
+    return google::cloud::internal::OutOfRangeError(
+        "Object too large, use RewriteObject() instead of CopyObject()",
+        GCP_ERROR_INFO());
   }
   return FromProto(response->resource(), options);
 }
@@ -492,12 +425,15 @@ GrpcStub::ReadObject(rest_internal::RestContext& context,
   auto stream = stub_->ReadObject(std::move(ctx), options, *proto_request);
 
   // The default timer source is a no-op. It does not set a timer, and always
-  // returns an indication that the timer expired.  The GrpcObjectReadSource
-  // takes no action on expired timers.
+  // returns an indication that the timer was cancelled.  `GrpcObjectReadSource`
+  // takes no action on cancelled timers.
   GrpcObjectReadSource::TimerSource timer_source = [] {
     return make_ready_future(false);
   };
-  auto const timeout = options.get<storage::DownloadStallTimeoutOption>();
+  auto const timeout = ScaleStallTimeout(
+      options.get<storage::DownloadStallTimeoutOption>(),
+      options.get<storage::DownloadStallMinimumRateOption>(),
+      google::storage::v2::ServiceConstants::MAX_READ_CHUNK_BYTES);
   if (timeout != std::chrono::seconds(0)) {
     // Change to an active timer.
     timer_source = [timeout, cq = background_->cq()]() mutable {
@@ -518,7 +454,7 @@ StatusOr<storage::internal::ListObjectsResponse> GrpcStub::ListObjects(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->ListObjects(ctx, proto);
+  auto response = stub_->ListObjects(ctx, options, proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -530,7 +466,7 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteObject(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->DeleteObject(ctx, proto);
+  auto response = stub_->DeleteObject(ctx, options, proto);
   if (!response.ok()) return response;
   return storage::internal::EmptyResponse{};
 }
@@ -543,7 +479,7 @@ StatusOr<storage::ObjectMetadata> GrpcStub::UpdateObject(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->UpdateObject(ctx, *proto);
+  auto response = stub_->UpdateObject(ctx, options, *proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -564,7 +500,7 @@ StatusOr<storage::ObjectMetadata> GrpcStub::ComposeObject(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->ComposeObject(ctx, *proto);
+  auto response = stub_->ComposeObject(ctx, options, *proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -577,7 +513,20 @@ StatusOr<storage::internal::RewriteObjectResponse> GrpcStub::RewriteObject(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  auto response = stub_->RewriteObject(ctx, *proto);
+  auto response = stub_->RewriteObject(ctx, options, *proto);
+  if (!response) return std::move(response).status();
+  return FromProto(*response, options);
+}
+
+StatusOr<storage::ObjectMetadata> GrpcStub::RestoreObject(
+    rest_internal::RestContext& context, Options const& options,
+    storage::internal::RestoreObjectRequest const& request) {
+  auto proto = ToProto(request);
+  if (!proto) return std::move(proto).status();
+  grpc::ClientContext ctx;
+  ApplyQueryParameters(ctx, options, request);
+  AddIdempotencyToken(ctx, context);
+  auto response = stub_->RestoreObject(ctx, options, *proto);
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -596,7 +545,7 @@ GrpcStub::CreateResumableUpload(
   if (timeout.count() != 0) {
     ctx.set_deadline(std::chrono::system_clock::now() + timeout);
   }
-  auto response = stub_->StartResumableWrite(ctx, *proto_request);
+  auto response = stub_->StartResumableWrite(ctx, options, *proto_request);
   if (!response.ok()) return std::move(response).status();
 
   return storage::internal::CreateResumableUploadResponse{
@@ -614,7 +563,7 @@ GrpcStub::QueryResumableUpload(
   if (timeout.count() != 0) {
     ctx.set_deadline(std::chrono::system_clock::now() + timeout);
   }
-  auto response = stub_->QueryWriteStatus(ctx, ToProto(request));
+  auto response = stub_->QueryWriteStatus(ctx, options, ToProto(request));
   if (!response) return std::move(response).status();
   return FromProto(*response, options);
 }
@@ -629,7 +578,7 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteResumableUpload(
   if (timeout.count() != 0) {
     ctx.set_deadline(std::chrono::system_clock::now() + timeout);
   }
-  auto response = stub_->CancelResumableWrite(ctx, ToProto(request));
+  auto response = stub_->CancelResumableWrite(ctx, options, ToProto(request));
   if (!response) return std::move(response).status();
   return storage::internal::EmptyResponse{};
 }
@@ -764,10 +713,10 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteBucketAcl(
           return a.entity() == entity || a.entity_alt() == entity;
         });
     if (i == acl.end()) {
-      return Status(StatusCode::kNotFound,
-                    "the entity <" + request.entity() +
-                        "> is not present in the ACL for bucket " +
-                        request.bucket_name());
+      return google::cloud::internal::NotFoundError(
+          "the entity <" + request.entity() +
+              "> is not present in the ACL for bucket " + request.bucket_name(),
+          GCP_ERROR_INFO());
     }
     acl.erase(i, acl.end());
     return acl;
@@ -858,10 +807,10 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteObjectAcl(
           return a.entity() == entity || a.entity_alt() == entity;
         });
     if (i == acl.end()) {
-      return Status(StatusCode::kNotFound,
-                    "the entity <" + request.entity() +
-                        "> is not present in the ACL for object " +
-                        request.object_name());
+      return google::cloud::internal::NotFoundError(
+          "the entity <" + request.entity() +
+              "> is not present in the ACL for object " + request.object_name(),
+          GCP_ERROR_INFO());
     }
     acl.erase(i, acl.end());
     return acl;
@@ -964,10 +913,10 @@ StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteDefaultObjectAcl(
           return a.entity() == entity || a.entity_alt() == entity;
         });
     if (i == acl.end()) {
-      return Status(StatusCode::kNotFound,
-                    "the entity <" + request.entity() +
-                        "> is not present in the ACL for bucket " +
-                        request.bucket_name());
+      return google::cloud::internal::NotFoundError(
+          "the entity <" + request.entity() +
+              "> is not present in the ACL for bucket " + request.bucket_name(),
+          GCP_ERROR_INFO());
     }
     acl.erase(i, acl.end());
     return acl;
@@ -1019,75 +968,39 @@ StatusOr<storage::ObjectAccessControl> GrpcStub::PatchDefaultObjectAcl(
 }
 
 StatusOr<storage::ServiceAccount> GrpcStub::GetServiceAccount(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::GetProjectServiceAccountRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->GetServiceAccount(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::GetProjectServiceAccountRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::internal::ListHmacKeysResponse> GrpcStub::ListHmacKeys(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::ListHmacKeysRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->ListHmacKeys(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::ListHmacKeysRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::internal::CreateHmacKeyResponse> GrpcStub::CreateHmacKey(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::CreateHmacKeyRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->CreateHmacKey(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::CreateHmacKeyRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteHmacKey(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::DeleteHmacKeyRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->DeleteHmacKey(ctx, proto);
-  if (!response.ok()) return response;
-  return storage::internal::EmptyResponse{};
+    rest_internal::RestContext&, Options const&,
+    storage::internal::DeleteHmacKeyRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::HmacKeyMetadata> GrpcStub::GetHmacKey(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::GetHmacKeyRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->GetHmacKey(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::GetHmacKeyRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::HmacKeyMetadata> GrpcStub::UpdateHmacKey(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::UpdateHmacKeyRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->UpdateHmacKey(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::UpdateHmacKeyRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::internal::SignBlobResponse> GrpcStub::SignBlob(
@@ -1105,51 +1018,27 @@ StatusOr<storage::internal::SignBlobResponse> GrpcStub::SignBlob(
 
 StatusOr<storage::internal::ListNotificationsResponse>
 GrpcStub::ListNotifications(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::ListNotificationsRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->ListNotificationConfigs(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::ListNotificationsRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::NotificationMetadata> GrpcStub::CreateNotification(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::CreateNotificationRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->CreateNotificationConfig(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::CreateNotificationRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::NotificationMetadata> GrpcStub::GetNotification(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::GetNotificationRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->GetNotificationConfig(ctx, proto);
-  if (!response) return std::move(response).status();
-  return FromProto(*response);
+    rest_internal::RestContext&, Options const&,
+    storage::internal::GetNotificationRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 StatusOr<storage::internal::EmptyResponse> GrpcStub::DeleteNotification(
-    rest_internal::RestContext& context, Options const& options,
-    storage::internal::DeleteNotificationRequest const& request) {
-  auto proto = ToProto(request);
-  grpc::ClientContext ctx;
-  ApplyQueryParameters(ctx, options, request);
-  AddIdempotencyToken(ctx, context);
-  auto response = stub_->DeleteNotificationConfig(ctx, proto);
-  if (!response.ok()) return response;
-  return storage::internal::EmptyResponse{};
+    rest_internal::RestContext&, Options const&,
+    storage::internal::DeleteNotificationRequest const&) {
+  return Status(StatusCode::kUnimplemented, "");
 }
 
 std::vector<std::string> GrpcStub::InspectStackStructure() const {
@@ -1163,7 +1052,7 @@ StatusOr<google::storage::v2::Bucket> GrpcStub::GetBucketMetadataImpl(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  return stub_->GetBucket(ctx, proto);
+  return stub_->GetBucket(ctx, options, proto);
 }
 
 StatusOr<google::storage::v2::Bucket> GrpcStub::PatchBucketImpl(
@@ -1174,7 +1063,7 @@ StatusOr<google::storage::v2::Bucket> GrpcStub::PatchBucketImpl(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  return stub_->UpdateBucket(ctx, *proto);
+  return stub_->UpdateBucket(ctx, options, *proto);
 }
 
 StatusOr<google::storage::v2::Object> GrpcStub::GetObjectMetadataImpl(
@@ -1184,7 +1073,7 @@ StatusOr<google::storage::v2::Object> GrpcStub::GetObjectMetadataImpl(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  return stub_->GetObject(ctx, proto);
+  return stub_->GetObject(ctx, options, proto);
 }
 
 StatusOr<google::storage::v2::Object> GrpcStub::PatchObjectImpl(
@@ -1195,7 +1084,7 @@ StatusOr<google::storage::v2::Object> GrpcStub::PatchObjectImpl(
   grpc::ClientContext ctx;
   ApplyQueryParameters(ctx, options, request);
   AddIdempotencyToken(ctx, context);
-  return stub_->UpdateObject(ctx, *proto);
+  return stub_->UpdateObject(ctx, options, *proto);
 }
 
 StatusOr<google::storage::v2::Bucket> GrpcStub::ModifyBucketAccessControl(
@@ -1224,10 +1113,10 @@ StatusOr<google::storage::v2::Bucket> GrpcStub::ModifyBucketAccessControl(
   auto patch = PatchBucketImpl(context, options, patch_request);
   // Retry on failed preconditions
   if (patch.status().code() == StatusCode::kFailedPrecondition) {
-    return Status(
-        StatusCode::kUnavailable,
+    return google::cloud::internal::UnavailableError(
         "retrying BucketAccessControl change due to conflict, bucket=" +
-            request.bucket_name());
+            request.bucket_name(),
+        patch.status().error_info());
   }
   return patch;
 }
@@ -1259,10 +1148,10 @@ StatusOr<google::storage::v2::Object> GrpcStub::ModifyObjectAccessControl(
   auto patch = PatchObjectImpl(context, options, patch_request);
   // Retry on failed preconditions
   if (patch.status().code() == StatusCode::kFailedPrecondition) {
-    return Status(
-        StatusCode::kUnavailable,
+    return google::cloud::internal::UnavailableError(
         "retrying ObjectAccessControl change due to conflict, bucket=" +
-            request.bucket_name() + ", object=" + request.object_name());
+            request.bucket_name() + ", object=" + request.object_name(),
+        patch.status().error_info());
   }
   return patch;
 }
@@ -1291,10 +1180,10 @@ StatusOr<google::storage::v2::Bucket> GrpcStub::ModifyDefaultAccessControl(
   auto patch = PatchBucketImpl(context, options, patch_request);
   // Retry on failed preconditions
   if (patch.status().code() == StatusCode::kFailedPrecondition) {
-    return Status(
-        StatusCode::kUnavailable,
+    return google::cloud::internal::UnavailableError(
         "retrying BucketAccessControl change due to conflict, bucket=" +
-            request.bucket_name());
+            request.bucket_name(),
+        patch.status().error_info());
   }
   return patch;
 }
