@@ -14,13 +14,10 @@
 
 #include "google/cloud/storage/internal/grpc/stub.h"
 #include "google/cloud/storage/grpc_plugin.h"
-#include "google/cloud/storage/options.h"
 #include "google/cloud/storage/testing/mock_storage_stub.h"
 #include "google/cloud/credentials.h"
-#include "google/cloud/grpc_options.h"
 #include "google/cloud/options.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
-#include "google/cloud/testing_util/scoped_environment.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include "google/cloud/testing_util/validate_metadata.h"
 #include <google/protobuf/text_format.h>
@@ -40,7 +37,6 @@ using ::google::cloud::storage::QuotaUser;
 using ::google::cloud::storage::internal::CreateNullHashFunction;
 using ::google::cloud::storage::testing::MockInsertStream;
 using ::google::cloud::storage::testing::MockStorageStub;
-using ::google::cloud::testing_util::ScopedEnvironment;
 using ::google::cloud::testing_util::StatusIs;
 using ::google::cloud::testing_util::ValidateMetadataFixture;
 using ::testing::Contains;
@@ -81,73 +77,10 @@ std::unique_ptr<GrpcStub> CreateTestClient(
                                     TestOptions());
 }
 
-TEST(DefaultOptionsGrpc, DefaultOptionsGrpcChannelCount) {
-  using ::google::cloud::GrpcNumChannelsOption;
-  struct TestCase {
-    std::string endpoint;
-    int lower_bound;
-    int upper_bound;
-  } cases[] = {
-      {"storage.googleapis.com", 4, std::numeric_limits<int>::max()},
-      {"google-c2p:///storage.googleapis.com", 1, 1},
-      {"google-c2p-experimental:///storage.googleapis.com", 1, 1},
-  };
-
-  for (auto const& test : cases) {
-    SCOPED_TRACE("Testing with " + test.endpoint);
-    auto opts =
-        DefaultOptionsGrpc(TestOptions().set<EndpointOption>(test.endpoint));
-    auto const count = opts.get<GrpcNumChannelsOption>();
-    EXPECT_LE(test.lower_bound, count);
-    EXPECT_GE(test.upper_bound, count);
-
-    auto override = DefaultOptionsGrpc(TestOptions()
-                                           .set<EndpointOption>(test.endpoint)
-                                           .set<GrpcNumChannelsOption>(42));
-    EXPECT_EQ(42, override.get<GrpcNumChannelsOption>());
-  }
-}
-
-TEST(DefaultOptionsGrpc, DefaultOptionsGrpcEndpointNoEnv) {
-  auto expected = std::string("storage.googleapis.com");
-  auto alternatives = [](std::string const& value) {
-    return std::vector<absl::optional<std::string>>{absl::nullopt, value};
-  };
-
-  for (auto const& opt : alternatives("from-option")) {
-    SCOPED_TRACE("Testing with opt " + opt.value_or("<unset>"));
-    auto options = TestOptions();
-    if (opt.has_value()) {
-      expected = *opt;
-      options.set<EndpointOption>(*opt);
-    }
-    for (auto const& env : alternatives("from-env")) {
-      SCOPED_TRACE("Testing with env " + opt.value_or("<unset>"));
-      auto setenv = ScopedEnvironment(
-          "CLOUD_STORAGE_EXPERIMENTAL_GRPC_TESTBENCH_ENDPOINT", env);
-      if (env.has_value()) expected = *env;
-      auto actual = DefaultOptionsGrpc(options);
-      EXPECT_EQ(actual.get<EndpointOption>(), expected);
-    }
-  }
-}
-
-TEST(DefaultOptionsGrpc, DefaultOptionsUploadBuffer) {
-  auto const with_defaults =
-      DefaultOptionsGrpc(Options{}).get<storage::UploadBufferSizeOption>();
-  EXPECT_GE(with_defaults, 32 * 1024 * 1024L);
-
-  auto const with_override =
-      DefaultOptionsGrpc(
-          Options{}.set<storage::UploadBufferSizeOption>(256 * 1024))
-          .get<storage::UploadBufferSizeOption>();
-  EXPECT_EQ(with_override, 256 * 1024L);
-}
-
 TEST_F(GrpcClientTest, QueryResumableUpload) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, QueryWriteStatus)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::QueryWriteStatusRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata, UnorderedElementsAre(
@@ -171,7 +104,7 @@ TEST_F(GrpcClientTest, DeleteResumableUpload) {
   EXPECT_CALL(*mock, CancelResumableWrite)
       .WillOnce(
           [this](
-              grpc::ClientContext& context,
+              grpc::ClientContext& context, Options const&,
               google::storage::v2::CancelResumableWriteRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -225,7 +158,7 @@ TEST_F(GrpcClientTest, CreateBucket) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, CreateBucket)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::storage::v2::CreateBucketRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -275,7 +208,7 @@ TEST_F(GrpcClientTest, CreateBucketAlreadyExists) {
 TEST_F(GrpcClientTest, GetBucket) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, GetBucket)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::storage::v2::GetBucketRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -300,7 +233,7 @@ TEST_F(GrpcClientTest, DeleteBucket) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, DeleteBucket)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::storage::v2::DeleteBucketRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -324,7 +257,7 @@ TEST_F(GrpcClientTest, DeleteBucket) {
 TEST_F(GrpcClientTest, ListBuckets) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, ListBuckets)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::storage::v2::ListBucketsRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -349,7 +282,7 @@ TEST_F(GrpcClientTest, LockBucketRetentionPolicy) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, LockBucketRetentionPolicy)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::storage::v2::LockBucketRetentionPolicyRequest const&) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -374,7 +307,7 @@ TEST_F(GrpcClientTest, UpdateBucket) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, UpdateBucket)
       .WillOnce([this](
-                    grpc::ClientContext& context,
+                    grpc::ClientContext& context, Options const&,
                     google::storage::v2::UpdateBucketRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -400,7 +333,7 @@ TEST_F(GrpcClientTest, PatchBucket) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, UpdateBucket)
       .WillOnce([this](
-                    grpc::ClientContext& context,
+                    grpc::ClientContext& context, Options const&,
                     google::storage::v2::UpdateBucketRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -426,7 +359,7 @@ TEST_F(GrpcClientTest, PatchBucket) {
 TEST_F(GrpcClientTest, GetNativeBucketIamPolicy) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, GetIamPolicy)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::iam::v1::GetIamPolicyRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -450,7 +383,7 @@ TEST_F(GrpcClientTest, GetNativeBucketIamPolicy) {
 TEST_F(GrpcClientTest, SetNativeBucketIamPolicy) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, SetIamPolicy)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        google::iam::v1::SetIamPolicyRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -476,7 +409,7 @@ TEST_F(GrpcClientTest, TestBucketIamPermissions) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, TestIamPermissions)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::iam::v1::TestIamPermissionsRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -530,7 +463,7 @@ TEST_F(GrpcClientTest, InsertObjectMedia) {
 TEST_F(GrpcClientTest, CopyObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, RewriteObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::RewriteObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -561,7 +494,7 @@ TEST_F(GrpcClientTest, CopyObject) {
 TEST_F(GrpcClientTest, CopyObjectTooLarge) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, RewriteObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::RewriteObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -595,7 +528,7 @@ TEST_F(GrpcClientTest, CopyObjectTooLarge) {
 TEST_F(GrpcClientTest, GetObjectMetadata) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, GetObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::GetObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -646,7 +579,7 @@ TEST_F(GrpcClientTest, ReadObject) {
 TEST_F(GrpcClientTest, ListObjects) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, ListObjects)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::ListObjectsRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -671,7 +604,7 @@ TEST_F(GrpcClientTest, DeleteObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, DeleteObject)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::storage::v2::DeleteObjectRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -697,7 +630,7 @@ TEST_F(GrpcClientTest, UpdateObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, UpdateObject)
       .WillOnce(
-          [this](grpc::ClientContext& context,
+          [this](grpc::ClientContext& context, Options const&,
                  google::storage::v2::UpdateObjectRequest const& request) {
             auto metadata = GetMetadata(context);
             EXPECT_THAT(metadata,
@@ -728,7 +661,7 @@ TEST_F(GrpcClientTest, UpdateObject) {
 TEST_F(GrpcClientTest, PatchObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, UpdateObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::UpdateObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -756,7 +689,7 @@ TEST_F(GrpcClientTest, PatchObject) {
 TEST_F(GrpcClientTest, ComposeObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, ComposeObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::ComposeObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -783,7 +716,7 @@ TEST_F(GrpcClientTest, ComposeObject) {
 TEST_F(GrpcClientTest, RewriteObject) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, RewriteObject)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::RewriteObjectRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -811,10 +744,38 @@ TEST_F(GrpcClientTest, RewriteObject) {
   EXPECT_EQ(response.status(), PermanentError());
 }
 
+TEST_F(GrpcClientTest, RestoreObject) {
+  auto mock = std::make_shared<MockStorageStub>();
+  EXPECT_CALL(*mock, RestoreObject)
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
+                       v2::RestoreObjectRequest const& request) {
+        auto metadata = GetMetadata(context);
+        EXPECT_THAT(metadata,
+                    UnorderedElementsAre(
+                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
+                        Pair("x-goog-quota-user", "test-quota-user"),
+                        Pair("x-goog-fieldmask", "field1,field2")));
+        EXPECT_THAT(request.bucket(), "projects/_/buckets/test-bucket");
+        EXPECT_THAT(request.object(), "test-object");
+        EXPECT_THAT(request.generation(), 1234);
+        EXPECT_THAT(request.copy_source_acl(), false);
+        return PermanentError();
+      });
+  auto client = CreateTestClient(mock);
+  auto context = TestContext();
+  auto response = client->RestoreObject(
+      context, TestOptions(),
+      storage::internal::RestoreObjectRequest("test-bucket", "test-object",
+                                              1234)
+          .set_multiple_options(Fields("field1,field2"),
+                                QuotaUser("test-quota-user")));
+  EXPECT_EQ(response.status(), PermanentError());
+}
+
 TEST_F(GrpcClientTest, CreateResumableUpload) {
   auto mock = std::make_shared<MockStorageStub>();
   EXPECT_CALL(*mock, StartResumableWrite)
-      .WillOnce([this](grpc::ClientContext& context,
+      .WillOnce([this](grpc::ClientContext& context, Options const&,
                        v2::StartResumableWriteRequest const& request) {
         auto metadata = GetMetadata(context);
         EXPECT_THAT(metadata,
@@ -838,258 +799,6 @@ TEST_F(GrpcClientTest, CreateResumableUpload) {
   EXPECT_EQ(response.status(), PermanentError());
 }
 
-TEST_F(GrpcClientTest, GetServiceAccount) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, GetServiceAccount)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::GetServiceAccountRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->GetServiceAccount(
-      context, TestOptions(),
-      storage::internal::GetProjectServiceAccountRequest("test-project-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, CreateHmacKey) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, CreateHmacKey)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::CreateHmacKeyRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->CreateHmacKey(
-      context, TestOptions(),
-      storage::internal::CreateHmacKeyRequest("test-project-id",
-                                              "test-service-account-email")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, DeleteHmacKey) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, DeleteHmacKey)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::DeleteHmacKeyRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->DeleteHmacKey(
-      context, TestOptions(),
-      storage::internal::DeleteHmacKeyRequest("test-project-id",
-                                              "test-access-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, GetHmacKey) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, GetHmacKey)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::GetHmacKeyRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->GetHmacKey(
-      context, TestOptions(),
-      storage::internal::GetHmacKeyRequest("test-project-id", "test-access-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, ListHmacKeys) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, ListHmacKeys)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::ListHmacKeysRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->ListHmacKeys(
-      context, TestOptions(),
-      storage::internal::ListHmacKeysRequest("test-project-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, UpdateHmacKey) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, UpdateHmacKey)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::UpdateHmacKeyRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.hmac_key().project(), "projects/test-project-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->UpdateHmacKey(
-      context, TestOptions(),
-      storage::internal::UpdateHmacKeyRequest(
-          "test-project-id", "test-access-id",
-          storage::HmacKeyMetadata().set_state(
-              storage::HmacKeyMetadata::state_deleted()))
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, ListNotificationConfigs) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, ListNotificationConfigs)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::ListNotificationConfigsRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.parent(), "projects/_/buckets/test-bucket-name");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->ListNotifications(
-      context, TestOptions(),
-      storage::internal::ListNotificationsRequest("test-bucket-name")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, CreateNotificationConfig) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, CreateNotificationConfig)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::CreateNotificationConfigRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.parent(), "projects/_/buckets/test-bucket-name");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->CreateNotification(
-      context, TestOptions(),
-      storage::internal::CreateNotificationRequest(
-          "test-bucket-name", storage::NotificationMetadata())
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, GetNotificationConfig) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, GetNotificationConfig)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::GetNotificationConfigRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.name(),
-                    "projects/_/buckets/test-bucket-name/notificationConfigs/"
-                    "test-notification-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->GetNotification(
-      context, TestOptions(),
-      storage::internal::GetNotificationRequest("test-bucket-name",
-                                                "test-notification-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
-TEST_F(GrpcClientTest, DeleteNotificationConfig) {
-  auto mock = std::make_shared<MockStorageStub>();
-  EXPECT_CALL(*mock, DeleteNotificationConfig)
-      .WillOnce([this](grpc::ClientContext& context,
-                       v2::DeleteNotificationConfigRequest const& request) {
-        auto metadata = GetMetadata(context);
-        EXPECT_THAT(metadata,
-                    UnorderedElementsAre(
-                        Pair(kIdempotencyTokenHeader, "test-token-1234"),
-                        Pair("x-goog-quota-user", "test-quota-user"),
-                        Pair("x-goog-fieldmask", "field1,field2")));
-        EXPECT_THAT(request.name(),
-                    "projects/_/buckets/test-bucket-name/notificationConfigs/"
-                    "test-notification-id");
-        return PermanentError();
-      });
-  auto client = CreateTestClient(mock);
-  auto context = TestContext();
-  auto response = client->DeleteNotification(
-      context, TestOptions(),
-      storage::internal::DeleteNotificationRequest("test-bucket-name",
-                                                   "test-notification-id")
-          .set_multiple_options(Fields("field1,field2"),
-                                QuotaUser("test-quota-user")));
-  EXPECT_EQ(response.status(), PermanentError());
-}
-
 }  // namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace storage_internal
