@@ -13,9 +13,8 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/hash_function_impl.h"
+#include "google/cloud/storage/internal/base64.h"
 #include "google/cloud/storage/internal/crc32c.h"
-#include "google/cloud/storage/internal/object_requests.h"
-#include "google/cloud/storage/internal/openssl_util.h"
 #include "google/cloud/internal/big_endian.h"
 #include "google/cloud/internal/make_status.h"
 
@@ -27,32 +26,13 @@ namespace internal {
 namespace {
 
 using ::google::cloud::internal::InvalidArgumentError;
+using ::google::cloud::storage_internal::Crc32c;
 using ::google::cloud::storage_internal::ExtendCrc32c;
 
-using ContextPtr = std::unique_ptr<EVP_MD_CTX, MD5HashFunction::ContextDeleter>;
-
-ContextPtr CreateDigestCtx() {
-// The name of the function to create and delete EVP_MD_CTX objects changed
-// with OpenSSL 1.1.0.
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)  // Older than version 1.1.0.
-  return ContextPtr(EVP_MD_CTX_create());
-#else
-  return ContextPtr(EVP_MD_CTX_new());
-#endif
-}
-
-void DeleteDigestCtx(EVP_MD_CTX* context) {
-// The name of the function to free an EVP_MD_CTX changed in OpenSSL 1.1.0.
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L)  // Older than version 1.1.0.
-  EVP_MD_CTX_destroy(context);
-#else
-  EVP_MD_CTX_free(context);
-#endif  // OPENSSL_VERSION_NUMBER >= 0x10100000L
-}
-
 template <typename Buffer>
 bool AlreadyHashed(std::int64_t offset, Buffer const& buffer,
                    std::int64_t minimum_offset) {
+  // TODO(#14566) - maybe this is more forgiving than we want to be
   auto const end = offset + buffer.size();
   return static_cast<std::int64_t>(end) <= minimum_offset;
 }
@@ -116,18 +96,10 @@ HashValues CompositeFunction::Finish() {
   return Merge(a_->Finish(), b_->Finish());
 }
 
-MD5HashFunction::MD5HashFunction() : impl_(CreateDigestCtx()) {
-  EVP_DigestInit_ex(impl_.get(), EVP_md5(), nullptr);
-}
-
-void MD5HashFunction::Update(absl::string_view buffer) {
-  EVP_DigestUpdate(impl_.get(), buffer.data(), buffer.size());
-}
-
 Status MD5HashFunction::Update(std::int64_t offset, absl::string_view buffer) {
-  if (offset == minimum_offset_) {
+  if (offset == minimum_offset_ || minimum_offset_ == 0) {
     Update(buffer);
-    minimum_offset_ += buffer.size();
+    minimum_offset_ = offset + buffer.size();
     return {};
   }
   if (AlreadyHashed(offset, buffer, minimum_offset_)) return {};
@@ -141,11 +113,11 @@ Status MD5HashFunction::Update(std::int64_t offset, absl::string_view buffer,
 
 Status MD5HashFunction::Update(std::int64_t offset, absl::Cord const& buffer,
                                std::uint32_t /*buffer_crc*/) {
-  if (offset == minimum_offset_) {
+  if (offset == minimum_offset_ || minimum_offset_ == 0) {
     for (auto i = buffer.chunk_begin(); i != buffer.chunk_end(); ++i) {
       Update(*i);
     }
-    minimum_offset_ += buffer.size();
+    minimum_offset_ = offset + buffer.size();
     return {};
   }
   if (AlreadyHashed(offset, buffer, minimum_offset_)) return {};
@@ -154,33 +126,20 @@ Status MD5HashFunction::Update(std::int64_t offset, absl::Cord const& buffer,
 
 HashValues MD5HashFunction::Finish() {
   if (hashes_.has_value()) return *hashes_;
-  std::vector<std::uint8_t> hash(EVP_MD_size(EVP_md5()));
-  unsigned int len = 0;
-  // Note: EVP_DigestFinal_ex() consumes an `unsigned char*` for the output
-  // array.  On some platforms (read PowerPC and ARM), the default `char` is
-  // unsigned. In those platforms it is possible that
-  // `std::uint8_t != unsigned char` and the `reinterpret_cast<>` is needed. It
-  // should be safe in any case.
-  EVP_DigestFinal_ex(impl_.get(), reinterpret_cast<unsigned char*>(hash.data()),
-                     &len);
-  hash.resize(len);
+  Hash hash = FinishImpl();
   hashes_ = HashValues{/*.crc32c=*/{}, /*.md5=*/Base64Encode(hash)};
   return *hashes_;
 }
 
-void MD5HashFunction::ContextDeleter::operator()(EVP_MD_CTX* context) {
-  DeleteDigestCtx(context);
-}
-
 void Crc32cHashFunction::Update(absl::string_view buffer) {
   current_ = ExtendCrc32c(current_, buffer);
 }
 
 Status Crc32cHashFunction::Update(std::int64_t offset,
                                   absl::string_view buffer) {
-  if (offset == minimum_offset_) {
+  if (offset == minimum_offset_ || minimum_offset_ == 0) {
     Update(buffer);
-    minimum_offset_ += buffer.size();
+    minimum_offset_ = offset + buffer.size();
     return {};
   }
   if (AlreadyHashed(offset, buffer, minimum_offset_)) return {};
@@ -189,9 +148,9 @@ Status Crc32cHashFunction::Update(std::int64_t offset,
 
 Status Crc32cHashFunction::Update(std::int64_t offset, absl::string_view buffer,
                                   std::uint32_t buffer_crc) {
-  if (offset == minimum_offset_) {
+  if (offset == minimum_offset_ || minimum_offset_ == 0) {
     current_ = ExtendCrc32c(current_, buffer, buffer_crc);
-    minimum_offset_ += buffer.size();
+    minimum_offset_ = offset + buffer.size();
     return {};
   }
   if (AlreadyHashed(offset, buffer, minimum_offset_)) return {};
@@ -200,9 +159,9 @@ Status Crc32cHashFunction::Update(std::int64_t offset, absl::string_view buffer,
 
 Status Crc32cHashFunction::Update(std::int64_t offset, absl::Cord const& buffer,
                                   std::uint32_t buffer_crc) {
-  if (offset == minimum_offset_) {
+  if (offset == minimum_offset_ || minimum_offset_ == 0) {
     current_ = ExtendCrc32c(current_, buffer, buffer_crc);
-    minimum_offset_ += buffer.size();
+    minimum_offset_ = offset + buffer.size();
     return {};
   }
   if (AlreadyHashed(offset, buffer, minimum_offset_)) return {};
@@ -239,6 +198,43 @@ Status PrecomputedHashFunction::Update(std::int64_t /*offset*/,
 
 HashValues PrecomputedHashFunction::Finish() { return precomputed_hash_; }
 
+std::string Crc32cMessageHashFunction::Name() const {
+  return "crc32c-message(" + child_->Name() + ")";
+}
+
+void Crc32cMessageHashFunction::Update(absl::string_view buffer) {
+  return child_->Update(buffer);
+}
+
+Status Crc32cMessageHashFunction::Update(std::int64_t offset,
+                                         absl::string_view buffer) {
+  return child_->Update(offset, buffer);
+}
+
+Status Crc32cMessageHashFunction::Update(std::int64_t offset,
+                                         absl::string_view buffer,
+                                         std::uint32_t buffer_crc) {
+  if (Crc32c(buffer) != buffer_crc) {
+    // No need to update the child, this is an error that should stop any
+    // download.
+    return InvalidArgumentError("mismatched crc32c checksum", GCP_ERROR_INFO());
+  }
+  return child_->Update(offset, buffer, buffer_crc);
+}
+
+Status Crc32cMessageHashFunction::Update(std::int64_t offset,
+                                         absl::Cord const& buffer,
+                                         std::uint32_t buffer_crc) {
+  if (Crc32c(buffer) != buffer_crc) {
+    // No need to update the child, this is an error that should stop any
+    // download.
+    return InvalidArgumentError("mismatched crc32c checksum", GCP_ERROR_INFO());
+  }
+  return child_->Update(offset, buffer, buffer_crc);
+}
+
+HashValues Crc32cMessageHashFunction::Finish() { return child_->Finish(); }
+
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace storage
