@@ -16,6 +16,9 @@
 #include "google/cloud/storage/internal/hmac_key_metadata_parser.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <sstream>
+#include <string>
+#include <utility>
 
 namespace google {
 namespace cloud {
