@@ -16,6 +16,8 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <nlohmann/json.hpp>
+#include <sstream>
+#include <string>
 
 namespace google {
 namespace cloud {
