@@ -16,6 +16,7 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <memory>
+#include <string>
 
 namespace google {
 namespace cloud {
@@ -37,6 +38,8 @@ class MockMinimalIamCredentialsRest
  public:
   MOCK_METHOD(StatusOr<google::cloud::AccessToken>, GenerateAccessToken,
               (oauth2_internal::GenerateAccessTokenRequest const&), (override));
+  MOCK_METHOD(StatusOr<std::string>, universe_domain, (Options const& options),
+              (override, const));
 };
 
 TEST(ImpersonateServiceAccountCredentialsTest, Basic) {
