@@ -13,7 +13,9 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/make_jwt_assertion.h"
-#include "google/cloud/storage/internal/openssl_util.h"
+#include "google/cloud/storage/internal/base64.h"
+#include "google/cloud/internal/sign_using_sha256.h"
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -26,7 +28,8 @@ StatusOr<std::string> MakeJWTAssertionNoThrow(std::string const& header,
                                               std::string const& pem_contents) {
   auto const body =
       UrlsafeBase64Encode(header) + '.' + UrlsafeBase64Encode(payload);
-  auto pem_signature = internal::SignStringWithPem(body, pem_contents);
+  auto pem_signature =
+      google::cloud::internal::SignUsingSha256(body, pem_contents);
   if (!pem_signature) return std::move(pem_signature).status();
   return body + '.' + UrlsafeBase64Encode(*pem_signature);
 }
