@@ -16,6 +16,8 @@
 #include "google/cloud/storage/internal/object_access_control_parser.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <sstream>
+#include <string>
 
 namespace google {
 namespace cloud {
