@@ -18,12 +18,15 @@
 #include "google/cloud/storage/auto_finalize.h"
 #include "google/cloud/storage/download_options.h"
 #include "google/cloud/storage/hashing_options.h"
+#include "google/cloud/storage/include_folders_as_prefixes.h"
 #include "google/cloud/storage/internal/const_buffer.h"
 #include "google/cloud/storage/internal/generic_object_request.h"
 #include "google/cloud/storage/internal/hash_function.h"
 #include "google/cloud/storage/internal/hash_values.h"
 #include "google/cloud/storage/internal/http_response.h"
 #include "google/cloud/storage/object_metadata.h"
+#include "google/cloud/storage/override_unlocked_retention.h"
+#include "google/cloud/storage/soft_deleted.h"
 #include "google/cloud/storage/upload_options.h"
 #include "google/cloud/storage/version.h"
 #include "google/cloud/storage/well_known_parameters.h"
@@ -34,6 +37,7 @@
 #include <memory>
 #include <numeric>
 #include <string>
+#include <utility>
 #include <vector>
 
 namespace google {
@@ -46,8 +50,9 @@ namespace internal {
  */
 class ListObjectsRequest
     : public GenericRequest<ListObjectsRequest, MaxResults, Prefix, Delimiter,
-                            IncludeTrailingDelimiter, StartOffset, EndOffset,
-                            MatchGlob, Projection, UserProject, Versions> {
+                            IncludeFoldersAsPrefixes, IncludeTrailingDelimiter,
+                            StartOffset, EndOffset, MatchGlob, Projection,
+                            SoftDeleted, UserProject, Versions> {
  public:
   ListObjectsRequest() = default;
   explicit ListObjectsRequest(std::string bucket_name)
@@ -87,7 +92,7 @@ class GetObjectMetadataRequest
     : public GenericObjectRequest<
           GetObjectMetadataRequest, Generation, IfGenerationMatch,
           IfGenerationNotMatch, IfMetagenerationMatch, IfMetagenerationNotMatch,
-          Projection, UserProject> {
+          Projection, SoftDeleted, UserProject> {
  public:
   using GenericObjectRequest::GenericObjectRequest;
 };
@@ -238,7 +243,7 @@ class UpdateObjectRequest
     : public GenericObjectRequest<
           UpdateObjectRequest, Generation, EncryptionKey, IfGenerationMatch,
           IfGenerationNotMatch, IfMetagenerationMatch, IfMetagenerationNotMatch,
-          PredefinedAcl, Projection, UserProject> {
+          OverrideUnlockedRetention, PredefinedAcl, Projection, UserProject> {
  public:
   UpdateObjectRequest() = default;
   explicit UpdateObjectRequest(std::string bucket_name, std::string object_name,
@@ -291,7 +296,8 @@ class PatchObjectRequest
     : public GenericObjectRequest<
           PatchObjectRequest, Generation, IfGenerationMatch,
           IfGenerationNotMatch, IfMetagenerationMatch, IfMetagenerationNotMatch,
-          PredefinedAcl, EncryptionKey, Projection, UserProject,
+          OverrideUnlockedRetention, PredefinedAcl, EncryptionKey, Projection,
+          UserProject,
           // PredefinedDefaultObjectAcl has no effect in an `Objects: patch`
           // request.  We are keeping it here for backwards compatibility. It
           // was introduced in error (should have been PredefinedAcl), and it
@@ -372,6 +378,34 @@ struct RewriteObjectResponse {
 
 std::ostream& operator<<(std::ostream& os, RewriteObjectResponse const& r);
 
+/**
+ * Represents a request to the `Objects: restore` API
+ */
+class RestoreObjectRequest
+    : public GenericObjectRequest<
+          RestoreObjectRequest, Generation, CopySourceAcl, EncryptionKey,
+          IfGenerationMatch, IfGenerationNotMatch, IfMetagenerationMatch,
+          IfMetagenerationNotMatch, Projection, UserProject> {
+ public:
+  RestoreObjectRequest() = default;
+  RestoreObjectRequest(std::string bucket_name, std::string object_name,
+                       std::int64_t generation)
+      : bucket_name_(std::move(bucket_name)),
+        object_name_(std::move(object_name)),
+        generation_(std::move(generation)) {}
+
+  std::string const& bucket_name() const { return bucket_name_; }
+  std::string const& object_name() const { return object_name_; }
+  std::int64_t const& generation() const { return generation_; }
+
+ private:
+  std::string bucket_name_;
+  std::string object_name_;
+  std::int64_t generation_;
+};
+
+std::ostream& operator<<(std::ostream& os, RestoreObjectRequest const& r);
+
 /**
  * Represents a request to start a resumable upload in `Objects: insert`.
  *
