@@ -19,6 +19,10 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <nlohmann/json.hpp>
+#include <sstream>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -62,7 +66,8 @@ TEST(ObjectRequestsTest, ParseAclListFailure) {
 TEST(ObjectRequestsTest, List) {
   ListObjectsRequest request("my-bucket");
   EXPECT_EQ("my-bucket", request.bucket_name());
-  request.set_multiple_options(UserProject("my-project"), Prefix("foo/"));
+  request.set_multiple_options(UserProject("my-project"), Prefix("foo/"),
+                               SoftDeleted(true));
 
   std::ostringstream os;
   os << request;
@@ -70,6 +75,7 @@ TEST(ObjectRequestsTest, List) {
   EXPECT_THAT(actual, HasSubstr("my-bucket"));
   EXPECT_THAT(actual, HasSubstr("userProject=my-project"));
   EXPECT_THAT(actual, HasSubstr("prefix=foo/"));
+  EXPECT_THAT(actual, HasSubstr("softDeleted=true"));
 }
 
 TEST(ObjectRequestsTest, ParseListResponse) {
@@ -146,7 +152,8 @@ TEST(ObjectRequestsTest, ParseListResponseFailureInItems) {
 
 TEST(ObjectRequestsTest, Get) {
   GetObjectMetadataRequest request("my-bucket", "my-object");
-  request.set_multiple_options(Generation(1), IfMetagenerationMatch(3));
+  request.set_multiple_options(Generation(1), IfMetagenerationMatch(3),
+                               SoftDeleted(true));
   std::ostringstream os;
   os << request;
   auto str = os.str();
@@ -154,6 +161,7 @@ TEST(ObjectRequestsTest, Get) {
   EXPECT_THAT(str, HasSubstr("my-object"));
   EXPECT_THAT(str, HasSubstr("generation=1"));
   EXPECT_THAT(str, HasSubstr("ifMetagenerationMatch=3"));
+  EXPECT_THAT(str, HasSubstr("softDeleted=true"));
 }
 
 TEST(ObjectRequestsTest, InsertObjectMedia) {
@@ -510,6 +518,30 @@ TEST(ObjectRequestsTest, RewriteObjectResponseFailureInResource) {
   EXPECT_THAT(actual, Not(IsOk()));
 }
 
+TEST(ObjectRequestsTest, RestoreObject) {
+  RestoreObjectRequest request("test-bucket", "test-object", 1234);
+  EXPECT_EQ("test-bucket", request.bucket_name());
+  EXPECT_EQ("test-object", request.object_name());
+  EXPECT_EQ(1234, request.generation());
+  request.set_multiple_options(IfGenerationMatch(7), IfGenerationNotMatch(8),
+                               IfMetagenerationMatch(9),
+                               IfMetagenerationNotMatch(10),
+                               UserProject("my-project"), CopySourceAcl(true));
+
+  std::ostringstream os;
+  os << request;
+  std::string actual = os.str();
+  EXPECT_THAT(actual, HasSubstr("test-bucket"));
+  EXPECT_THAT(actual, HasSubstr("test-object"));
+  EXPECT_THAT(actual, HasSubstr("generation=1234"));
+  EXPECT_THAT(actual, HasSubstr("ifGenerationMatch=7"));
+  EXPECT_THAT(actual, HasSubstr("ifGenerationNotMatch=8"));
+  EXPECT_THAT(actual, HasSubstr("ifMetagenerationMatch=9"));
+  EXPECT_THAT(actual, HasSubstr("ifMetagenerationNotMatch=10"));
+  EXPECT_THAT(actual, HasSubstr("userProject=my-project"));
+  EXPECT_THAT(actual, HasSubstr("copySourceAcl=true"));
+}
+
 TEST(ObjectRequestsTest, ResumableUpload) {
   ResumableUploadRequest request("source-bucket", "source-object");
   EXPECT_EQ("source-bucket", request.bucket_name());
