@@ -143,16 +143,18 @@ RestStub::RestStub(
 Options RestStub::ResolveStorageAuthority(Options const& options) {
   auto endpoint = RestEndpoint(options);
   if (options.has<AuthorityOption>() ||
-      !absl::StrContains(endpoint, "googleapis.com"))
+      !absl::StrContains(endpoint, "googleapis.com")) {
     return options;
+  }
   return Options(options).set<AuthorityOption>("storage.googleapis.com");
 }
 
 Options RestStub::ResolveIamAuthority(Options const& options) {
   auto endpoint = IamEndpoint(options);
   if (options.has<AuthorityOption>() ||
-      !absl::StrContains(endpoint, "googleapis.com"))
+      !absl::StrContains(endpoint, "googleapis.com")) {
     return options;
+  }
   return Options(options).set<AuthorityOption>("iamcredentials.googleapis.com");
 }
 
@@ -190,8 +192,8 @@ StatusOr<BucketMetadata> RestStub::CreateBucket(
   // This seems to be the only case where kAlreadyExists is a better match
   // for 409 than kAborted.
   if (!response && response.status().code() == StatusCode::kAborted) {
-    return Status(StatusCode::kAlreadyExists, response.status().message(),
-                  response.status().error_info());
+    return google::cloud::internal::AlreadyExistsError(
+        response.status().message(), response.status().error_info());
   }
   return response;
 }
@@ -616,6 +618,24 @@ StatusOr<RewriteObjectResponse> RestStub::RewriteObject(
                                  {absl::MakeConstSpan(json_payload)}));
 }
 
+StatusOr<ObjectMetadata> RestStub::RestoreObject(
+    rest_internal::RestContext& context, Options const& options,
+    RestoreObjectRequest const& request) {
+  RestRequestBuilder builder(absl::StrCat(
+      "storage/", options.get<TargetApiVersionOption>(), "/b/",
+      request.bucket_name(), "/o/", UrlEncode(request.object_name()),
+      "/restore?generation=", request.generation()));
+  auto auth = AddAuthorizationHeader(options, builder);
+  if (!auth.ok()) return auth;
+  request.AddOptionsToHttpRequest(builder);
+  builder.AddHeader("Content-Type", "application/json");
+  std::string json_payload("{}");
+
+  return CheckedFromString<ObjectMetadataParser>(
+      storage_rest_client_->Post(context, std::move(builder).BuildRequest(),
+                                 {absl::MakeConstSpan(json_payload)}));
+}
+
 StatusOr<CreateResumableUploadResponse> RestStub::CreateResumableUpload(
     rest_internal::RestContext& context, Options const& options,
     ResumableUploadRequest const& request) {
