@@ -12,14 +12,19 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include "google/cloud/storage/internal/retry_object_read_source.h"
 #include "google/cloud/storage/internal/connection_impl.h"
 #include "google/cloud/storage/retry_policy.h"
 #include "google/cloud/storage/testing/canonical_errors.h"
 #include "google/cloud/storage/testing/mock_client.h"
 #include "google/cloud/storage/testing/mock_generic_stub.h"
 #include "google/cloud/testing_util/chrono_literals.h"
+#include "google/cloud/testing_util/opentelemetry_matchers.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <memory>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -32,6 +37,7 @@ using ::google::cloud::storage::testing::MockGenericStub;
 using ::google::cloud::storage::testing::MockObjectReadSource;
 using ::google::cloud::storage::testing::canonical_errors::PermanentError;
 using ::google::cloud::storage::testing::canonical_errors::TransientError;
+using ::google::cloud::testing_util::StatusIs;
 using ::testing::_;
 using ::testing::Contains;
 using ::testing::HasSubstr;
@@ -212,7 +218,7 @@ TEST(RetryObjectReadSourceTest, BackoffPolicyResetOnSuccess) {
 
   EXPECT_EQ(0, num_backoff_policy_called);
 
-  // We really do not care how many times the policy is closed before we make
+  // We really do not care how many times the policy is cloned before we make
   // the first `Read()` call.  We only care about how it increases.
   auto const initial_clone_count = backoff_policy_mock.NumClones();
   auto source = client->ReadObject(ReadObjectRangeRequest{});
@@ -222,21 +228,22 @@ TEST(RetryObjectReadSourceTest, BackoffPolicyResetOnSuccess) {
 
   // raw_source1 and raw_source2 fail, then a success
   ASSERT_STATUS_OK((*source)->Read(nullptr, 1024));
-  // Two retries, so the backoff policy was called twice.
-  EXPECT_EQ(2, num_backoff_policy_called);
-  // The backoff should have been cloned during the read.
+  // Two retries, the first one does not get a backoff, so there should be one
+  // call to OnCompletion.
+  EXPECT_EQ(1, num_backoff_policy_called);
+  // The backoff should have been cloned during the Read() call.
   EXPECT_EQ(initial_clone_count + 2, backoff_policy_mock.NumClones());
-  // The backoff policy was used twice in the first retry.
-  EXPECT_EQ(2, backoff_policy_mock.NumCallsFromLastClone());
+  // The backoff policy was used once in the first retry.
+  EXPECT_EQ(1, backoff_policy_mock.NumCallsFromLastClone());
 
   // raw_source3 fails, then a success
   ASSERT_STATUS_OK((*source)->Read(nullptr, 1024));
-  // This read caused a third retry.
-  EXPECT_EQ(3, num_backoff_policy_called);
+  // This read caused another retry, but no call to backoff.
+  EXPECT_EQ(1, num_backoff_policy_called);
   // The backoff should have been cloned during the read.
   EXPECT_EQ(initial_clone_count + 3, backoff_policy_mock.NumClones());
-  // The backoff policy was only once in the second retry.
-  EXPECT_EQ(1, backoff_policy_mock.NumCallsFromLastClone());
+  // The backoff policy was cloned once, but never used in the second retry.
+  EXPECT_EQ(0, backoff_policy_mock.NumCallsFromLastClone());
 }
 
 /// @test Check that retry policy is shared between reads and resetting session
@@ -269,6 +276,49 @@ TEST(RetryObjectReadSourceTest, RetryPolicyExhaustedOnResetSession) {
   EXPECT_THAT(res.status().message(), HasSubstr("Retry policy exhausted"));
 }
 
+/// @test Check that retry policy is shared between reads and resetting session
+TEST(RetryObjectReadSourceTest, ResumePolicyOrder) {
+  ::testing::MockFunction<void(std::chrono::milliseconds)> backoff;
+  ::testing::MockFunction<StatusOr<std::unique_ptr<ObjectReadSource>>(
+      ReadObjectRangeRequest const&, RetryPolicy&, BackoffPolicy&)>
+      factory;
+
+  auto make_partial = [] {
+    auto source = std::make_unique<MockObjectReadSource>();
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    return std::unique_ptr<ObjectReadSource>(std::move(source));
+  };
+
+  auto source = std::make_unique<MockObjectReadSource>();
+  {
+    ::testing::InSequence sequence;
+    EXPECT_CALL(*source, Read)
+        .WillOnce(Return(ReadSourceResult{static_cast<std::size_t>(512 * 1024),
+                                          HttpResponse{100, "", {}}}));
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    // No backoffs to resume after a (partially) successful request:
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+  }
+
+  auto options = BasicTestPolicies();
+  auto retry_policy = options.get<RetryPolicyOption>()->clone();
+  auto backoff_policy = options.get<BackoffPolicyOption>()->clone();
+
+  RetryObjectReadSource tested(
+      factory.AsStdFunction(),
+      google::cloud::internal::MakeImmutableOptions(std::move(options)),
+      ReadObjectRangeRequest{}, std::move(source), std::move(retry_policy),
+      std::move(backoff_policy), backoff.AsStdFunction());
+
+  ASSERT_STATUS_OK(tested.Read(nullptr, 1024));
+  auto response = tested.Read(nullptr, 1024);
+  EXPECT_THAT(response, StatusIs(TransientError().code()));
+}
+
 /// @test `ReadLast` behaviour after a transient failure
 TEST(RetryObjectReadSourceTest, TransientFailureWithReadLastOption) {
   auto mock = std::make_unique<MockGenericStub>();
@@ -439,6 +489,119 @@ TEST(RetryObjectReadSourceTest, DiscardDataForDecompressiveTranscoding) {
               Contains(Pair("x-test-only", "download 3 r1")));
 }
 
+#ifdef GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
+using ::google::cloud::testing_util::DisableTracing;
+using ::google::cloud::testing_util::EnableTracing;
+using ::google::cloud::testing_util::SpanNamed;
+using ::testing::ElementsAre;
+using ::testing::IsEmpty;
+
+TEST(RetryObjectReadSourceTest, TracingEnabled) {
+  auto span_catcher = testing_util::InstallSpanCatcher();
+
+  ::testing::MockFunction<void(std::chrono::milliseconds)> backoff;
+  ::testing::MockFunction<StatusOr<std::unique_ptr<ObjectReadSource>>(
+      ReadObjectRangeRequest const&, RetryPolicy&, BackoffPolicy&)>
+      factory;
+
+  auto make_partial = [] {
+    auto source = std::make_unique<MockObjectReadSource>();
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    return std::unique_ptr<ObjectReadSource>(std::move(source));
+  };
+
+  auto source = std::make_unique<MockObjectReadSource>();
+  {
+    ::testing::InSequence sequence;
+    EXPECT_CALL(*source, Read)
+        .WillOnce(Return(ReadSourceResult{static_cast<std::size_t>(512 * 1024),
+                                          HttpResponse{100, "", {}}}));
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    // No backoffs to resume after a (partially) successful request:
+    //    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+  }
+
+  // Override the default backoff policy, it results in very short periods which
+  // are skipped by the span wrappers.
+  using ms = std::chrono::milliseconds;
+  auto options = EnableTracing(BasicTestPolicies().set<BackoffPolicyOption>(
+      ExponentialBackoffPolicy(ms(2), ms(2), 2.0).clone()));
+  auto retry_policy = options.get<RetryPolicyOption>()->clone();
+  auto backoff_policy = options.get<BackoffPolicyOption>()->clone();
+
+  RetryObjectReadSource tested(
+      factory.AsStdFunction(),
+      google::cloud::internal::MakeImmutableOptions(std::move(options)),
+      ReadObjectRangeRequest{}, std::move(source), std::move(retry_policy),
+      std::move(backoff_policy), backoff.AsStdFunction());
+
+  ASSERT_STATUS_OK(tested.Read(nullptr, 1024));
+  auto response = tested.Read(nullptr, 1024);
+  EXPECT_THAT(response, StatusIs(TransientError().code()));
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(spans, ElementsAre(SpanNamed("Backoff"), SpanNamed("Backoff")));
+}
+
+TEST(RetryObjectReadSourceTest, TracingDisabled) {
+  auto span_catcher = testing_util::InstallSpanCatcher();
+
+  ::testing::MockFunction<void(std::chrono::milliseconds)> backoff;
+  ::testing::MockFunction<StatusOr<std::unique_ptr<ObjectReadSource>>(
+      ReadObjectRangeRequest const&, RetryPolicy&, BackoffPolicy&)>
+      factory;
+
+  auto make_partial = [] {
+    auto source = std::make_unique<MockObjectReadSource>();
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    return std::unique_ptr<ObjectReadSource>(std::move(source));
+  };
+
+  auto source = std::make_unique<MockObjectReadSource>();
+  {
+    ::testing::InSequence sequence;
+    EXPECT_CALL(*source, Read)
+        .WillOnce(Return(ReadSourceResult{static_cast<std::size_t>(512 * 1024),
+                                          HttpResponse{100, "", {}}}));
+    EXPECT_CALL(*source, Read).WillOnce(Return(TransientError()));
+    // No backoffs to resume after a (partially) successful request:
+    //    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+    EXPECT_CALL(backoff, Call).Times(1);
+    EXPECT_CALL(factory, Call).WillOnce(make_partial);
+  }
+
+  // Override the default backoff policy, it results in very short periods which
+  // are skipped by the span wrappers.
+  using ms = std::chrono::milliseconds;
+  auto options = DisableTracing(BasicTestPolicies().set<BackoffPolicyOption>(
+      ExponentialBackoffPolicy(ms(2), ms(2), 2.0).clone()));
+  auto retry_policy = options.get<RetryPolicyOption>()->clone();
+  auto backoff_policy = options.get<BackoffPolicyOption>()->clone();
+
+  RetryObjectReadSource tested(
+      factory.AsStdFunction(),
+      google::cloud::internal::MakeImmutableOptions(std::move(options)),
+      ReadObjectRangeRequest{}, std::move(source), std::move(retry_policy),
+      std::move(backoff_policy), backoff.AsStdFunction());
+
+  ASSERT_STATUS_OK(tested.Read(nullptr, 1024));
+  auto response = tested.Read(nullptr, 1024);
+  EXPECT_THAT(response, StatusIs(TransientError().code()));
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(spans, IsEmpty());
+}
+
+#endif  // GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
+
 }  // namespace
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
