@@ -17,6 +17,9 @@
 #include "google/cloud/internal/parse_rfc3339.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <sstream>
+#include <string>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -28,6 +31,7 @@ namespace {
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::ElementsAre;
 using ::testing::HasSubstr;
+using ::testing::StartsWith;
 
 TEST(V2SignedUrlRequests, Sign) {
   V2SignUrlRequest request("GET", "test-bucket", "test-object");
@@ -641,6 +645,14 @@ TEST(V4SignedUrlRequests, BucketBoundHostnameReset) {
   EXPECT_EQ(expected, actual);
 }
 
+TEST(V4SignedUrlRequests, CustomEndpoint) {
+  auto const custom_endpoint_authority = std::string{"storage.mydomain.com"};
+  V4SignUrlRequest request("GET", "test-bucket", "test-object",
+                           custom_endpoint_authority);
+  ASSERT_STATUS_OK(request.Validate());
+  EXPECT_THAT(request.Hostname(), StartsWith(custom_endpoint_authority));
+}
+
 TEST(DefaultCtorsWork, Trivial) {
   EXPECT_FALSE(ExpirationTime().has_value());
   EXPECT_FALSE(AddExtensionHeaderOption().has_value());
