@@ -26,6 +26,11 @@
 #include "google/cloud/internal/unified_grpc_credentials.h"
 #include "google/cloud/log.h"
 #include <grpcpp/grpcpp.h>
+#include <algorithm>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -49,6 +54,11 @@ std::shared_ptr<grpc::Channel> CreateGrpcChannel(
   // to send and/or receive those queries.
   args.SetInt(GRPC_ARG_DNS_ENABLE_SRV_QUERIES, 0);
 
+  // This is needed to filter GCS+gRPC metrics based on the authority field.
+  if (options.has<AuthorityOption>()) {
+    args.SetString(GRPC_ARG_DEFAULT_AUTHORITY, options.get<AuthorityOption>());
+  }
+
   return auth.CreateChannel(options.get<EndpointOption>(), std::move(args));
 }
 
@@ -78,12 +88,12 @@ CreateDecoratedStubs(google::cloud::CompletionQueue cq, Options const& options,
   stub = std::make_shared<StorageMetadata>(
       std::move(stub), std::multimap<std::string, std::string>{},
       internal::HandCraftedLibClientHeader());
-  if (google::cloud::internal::Contains(options.get<TracingComponentsOption>(),
+  if (google::cloud::internal::Contains(options.get<LoggingComponentsOption>(),
                                         "rpc")) {
     GCP_LOG(INFO) << "Enabled logging for gRPC calls";
     stub = std::make_shared<StorageLogging>(
         std::move(stub), options.get<GrpcTracingOptionsOption>(),
-        options.get<TracingComponentsOption>());
+        options.get<LoggingComponentsOption>());
   }
   if (internal::TracingEnabled(options)) {
     stub = MakeStorageTracingStub(std::move(stub));
