@@ -13,7 +13,12 @@
 // limitations under the License.
 
 #include "google/cloud/storage/internal/tracing_connection.h"
+#include "google/cloud/storage/internal/tracing_object_read_source.h"
 #include "google/cloud/internal/opentelemetry.h"
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -128,8 +133,11 @@ TracingConnection::ReadObject(
     storage::internal::ReadObjectRangeRequest const& request) {
   auto span = internal::MakeSpan("storage::Client::ReadObject");
   auto scope = opentelemetry::trace::Scope(span);
-  // TODO(#11393) - add a wrapper for ReadObjectSource.
-  return internal::EndSpan(*span, impl_->ReadObject(request));
+  auto reader = impl_->ReadObject(request);
+  if (!reader) return internal::EndSpan(*span, std::move(reader));
+  return std::unique_ptr<storage::internal::ObjectReadSource>(
+      std::make_unique<TracingObjectReadSource>(std::move(span),
+                                                *std::move(reader)));
 }
 
 StatusOr<storage::internal::ListObjectsResponse> TracingConnection::ListObjects(
@@ -176,6 +184,13 @@ TracingConnection::RewriteObject(
   return internal::EndSpan(*span, impl_->RewriteObject(request));
 }
 
+StatusOr<storage::ObjectMetadata> TracingConnection::RestoreObject(
+    storage::internal::RestoreObjectRequest const& request) {
+  auto span = internal::MakeSpan("storage::Client::RestoreObject");
+  auto scope = opentelemetry::trace::Scope(span);
+  return internal::EndSpan(*span, impl_->RestoreObject(request));
+}
+
 StatusOr<storage::internal::CreateResumableUploadResponse>
 TracingConnection::CreateResumableUpload(
     storage::internal::ResumableUploadRequest const& request) {
