@@ -14,6 +14,10 @@
 
 #include "google/cloud/storage/internal/tuple_filter.h"
 #include <gmock/gmock.h>
+#include <memory>
+#include <string>
+#include <tuple>
+#include <utility>
 
 namespace google {
 namespace cloud {
