@@ -14,6 +14,9 @@
 
 #include "google/cloud/storage/oauth2/authorized_user_credentials.h"
 #include <nlohmann/json.hpp>
+#include <memory>
+#include <string>
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -70,7 +73,7 @@ AuthorizedUserCredentials<storage::internal::CurlRequestBuilder,
     : AuthorizedUserCredentials(
           google::cloud::oauth2_internal::AuthorizedUserCredentialsInfo{
               info.client_id, info.client_secret, info.refresh_token,
-              info.token_uri},
+              info.token_uri, info.universe_domain},
           channel_options) {}
 
 AuthorizedUserCredentials<storage::internal::CurlRequestBuilder,
