@@ -23,6 +23,10 @@
 #include <gmock/gmock.h>
 #include <nlohmann/json.hpp>
 #include <cstring>
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
