@@ -14,17 +14,19 @@
 
 #include "google/cloud/storage/oauth2/service_account_credentials.h"
 #include "google/cloud/storage/client_options.h"
+#include "google/cloud/storage/internal/base64.h"
 #include "google/cloud/storage/internal/make_jwt_assertion.h"
-#include "google/cloud/storage/internal/openssl_util.h"
 #include "google/cloud/internal/absl_str_join_quiet.h"
 #include "google/cloud/internal/oauth2_cached_credentials.h"
 #include "google/cloud/internal/oauth2_service_account_credentials.h"
+#include "google/cloud/internal/parse_service_account_p12_file.h"
+#include "google/cloud/internal/sign_using_sha256.h"
 #include <nlohmann/json.hpp>
-#include <openssl/err.h>
-#include <openssl/pem.h>
-#include <openssl/pkcs12.h>
 #include <array>
 #include <cstdio>
+#include <memory>
+#include <string>
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -147,7 +149,8 @@ oauth2_internal::ServiceAccountCredentialsInfo MapServiceAccountCredentialsInfo(
   return {std::move(info.client_email), std::move(info.private_key_id),
           std::move(info.private_key),  std::move(info.token_uri),
           std::move(info.scopes),       std::move(info.subject),
-          enable_self_signed_jwt};
+          enable_self_signed_jwt,       /*.universe_domain=*/absl::nullopt,
+          /*.project_id=*/absl::nullopt};
 }
 
 }  // namespace internal
