@@ -13,7 +13,10 @@
 // limitations under the License.
 
 #include "google/cloud/storage/object_read_stream.h"
+#include "google/cloud/internal/make_status.h"
 #include "google/cloud/log.h"
+#include <memory>
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -23,7 +26,8 @@ namespace {
 std::unique_ptr<internal::ObjectReadStreambuf> MakeErrorStreambuf() {
   return std::make_unique<internal::ObjectReadStreambuf>(
       internal::ReadObjectRangeRequest("", ""),
-      Status(StatusCode::kUnimplemented, "null stream"));
+      google::cloud::internal::UnimplementedError("null stream",
+                                                  GCP_ERROR_INFO()));
 }
 }  // namespace
 
