@@ -16,7 +16,10 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <nlohmann/json.hpp>
+#include <string>
 #include <type_traits>
+#include <utility>
+#include <vector>
 
 namespace google {
 namespace cloud {
