@@ -19,6 +19,7 @@
 #include "google/cloud/storage/internal/storage_stub.h"
 #include "google/cloud/testing_util/mock_async_streaming_read_rpc.h"
 #include <gmock/gmock.h>
+#include <memory>
 
 namespace google {
 namespace cloud {
@@ -28,80 +29,80 @@ namespace testing {
 class MockStorageStub : public storage_internal::StorageStub {
  public:
   MOCK_METHOD(Status, DeleteBucket,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::DeleteBucketRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Bucket>, GetBucket,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::GetBucketRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Bucket>, CreateBucket,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::CreateBucketRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::ListBucketsResponse>, ListBuckets,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::ListBucketsRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Bucket>, LockBucketRetentionPolicy,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::LockBucketRetentionPolicyRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::iam::v1::Policy>, GetIamPolicy,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::iam::v1::GetIamPolicyRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::iam::v1::Policy>, SetIamPolicy,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::iam::v1::SetIamPolicyRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::iam::v1::TestIamPermissionsResponse>,
               TestIamPermissions,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::iam::v1::TestIamPermissionsRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Bucket>, UpdateBucket,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::UpdateBucketRequest const&),
               (override));
   MOCK_METHOD(Status, DeleteNotificationConfig,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::DeleteNotificationConfigRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::NotificationConfig>,
               GetNotificationConfig,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::GetNotificationConfigRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::NotificationConfig>,
               CreateNotificationConfig,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::CreateNotificationConfigRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::ListNotificationConfigsResponse>,
               ListNotificationConfigs,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::ListNotificationConfigsRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Object>, ComposeObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::ComposeObjectRequest const&),
               (override));
   MOCK_METHOD(Status, DeleteObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::DeleteObjectRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Object>, RestoreObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::RestoreObjectRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::CancelResumableWriteResponse>,
               CancelResumableWrite,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::CancelResumableWriteRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Object>, GetObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::GetObjectRequest const&),
               (override));
   MOCK_METHOD(std::unique_ptr<google::cloud::internal::StreamingReadRpc<
@@ -111,7 +112,7 @@ class MockStorageStub : public storage_internal::StorageStub {
                google::storage::v2::ReadObjectRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::Object>, UpdateObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::UpdateObjectRequest const&),
               (override));
   MOCK_METHOD((std::unique_ptr<google::cloud::internal::StreamingWriteRpc<
@@ -126,59 +127,62 @@ class MockStorageStub : public storage_internal::StorageStub {
                    google::storage::v2::BidiWriteObjectResponse>>),
               AsyncBidiWriteObject,
               (google::cloud::CompletionQueue const&,
-               std::shared_ptr<grpc::ClientContext>),
+               std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions),
               (override));
 
   MOCK_METHOD(StatusOr<google::storage::v2::ListObjectsResponse>, ListObjects,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::ListObjectsRequest const&));
   MOCK_METHOD(StatusOr<google::storage::v2::RewriteResponse>, RewriteObject,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::RewriteObjectRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::StartResumableWriteResponse>,
               StartResumableWrite,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::StartResumableWriteRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::QueryWriteStatusResponse>,
               QueryWriteStatus,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::QueryWriteStatusRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::ServiceAccount>, GetServiceAccount,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::GetServiceAccountRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::CreateHmacKeyResponse>,
               CreateHmacKey,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::CreateHmacKeyRequest const&),
               (override));
   MOCK_METHOD(Status, DeleteHmacKey,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::DeleteHmacKeyRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::HmacKeyMetadata>, GetHmacKey,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::GetHmacKeyRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::ListHmacKeysResponse>, ListHmacKeys,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::ListHmacKeysRequest const&),
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::HmacKeyMetadata>, UpdateHmacKey,
-              (grpc::ClientContext&,
+              (grpc::ClientContext&, Options const&,
                google::storage::v2::UpdateHmacKeyRequest const&),
               (override));
   MOCK_METHOD(future<StatusOr<google::storage::v2::Object>>, AsyncComposeObject,
               (google::cloud::CompletionQueue&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::storage::v2::ComposeObjectRequest const&),
               (override));
   MOCK_METHOD(future<Status>, AsyncDeleteObject,
               (google::cloud::CompletionQueue&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::storage::v2::DeleteObjectRequest const&),
               (override));
   MOCK_METHOD(std::unique_ptr<::google::cloud::internal::AsyncStreamingReadRpc<
@@ -186,26 +190,37 @@ class MockStorageStub : public storage_internal::StorageStub {
               AsyncReadObject,
               (google::cloud::CompletionQueue const&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::storage::v2::ReadObjectRequest const&),
               (override));
+  MOCK_METHOD(future<StatusOr<google::storage::v2::RewriteResponse>>,
+              AsyncRewriteObject,
+              (google::cloud::CompletionQueue&,
+               std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
+               google::storage::v2::RewriteObjectRequest const&),
+              (override));
   using AsyncWriteObjectReturnType =
       std::unique_ptr<::google::cloud::internal::AsyncStreamingWriteRpc<
           google::storage::v2::WriteObjectRequest,
           google::storage::v2::WriteObjectResponse>>;
   MOCK_METHOD(AsyncWriteObjectReturnType, AsyncWriteObject,
               (google::cloud::CompletionQueue const&,
-               std::shared_ptr<grpc::ClientContext>),
+               std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions),
               (override));
   MOCK_METHOD(
       future<StatusOr<google::storage::v2::StartResumableWriteResponse>>,
       AsyncStartResumableWrite,
       (google::cloud::CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
+       google::cloud::internal::ImmutableOptions,
        google::storage::v2::StartResumableWriteRequest const&),
       (override));
   MOCK_METHOD(future<StatusOr<google::storage::v2::QueryWriteStatusResponse>>,
               AsyncQueryWriteStatus,
               (google::cloud::CompletionQueue&,
                std::shared_ptr<grpc::ClientContext>,
+               google::cloud::internal::ImmutableOptions,
                google::storage::v2::QueryWriteStatusRequest const&),
               (override));
 };
@@ -221,8 +236,8 @@ class MockInsertStream : public google::cloud::internal::StreamingWriteRpc<
               (override));
   MOCK_METHOD(StatusOr<google::storage::v2::WriteObjectResponse>, Close, (),
               (override));
-  MOCK_METHOD(google::cloud::internal::StreamingRpcMetadata, GetRequestMetadata,
-              (), (const, override));
+  MOCK_METHOD(google::cloud::RpcMetadata, GetRequestMetadata, (),
+              (const, override));
 };
 
 class MockObjectMediaStream : public google::cloud::internal::StreamingReadRpc<
@@ -231,8 +246,8 @@ class MockObjectMediaStream : public google::cloud::internal::StreamingReadRpc<
   MOCK_METHOD(void, Cancel, (), (override));
   MOCK_METHOD((absl::variant<Status, google::storage::v2::ReadObjectResponse>),
               Read, (), (override));
-  MOCK_METHOD(google::cloud::internal::StreamingRpcMetadata, GetRequestMetadata,
-              (), (const, override));
+  MOCK_METHOD(google::cloud::RpcMetadata, GetRequestMetadata, (),
+              (const, override));
 };
 
 class MockAsyncInsertStream
@@ -249,8 +264,8 @@ class MockAsyncInsertStream
   MOCK_METHOD(future<bool>, WritesDone, (), (override));
   MOCK_METHOD(future<StatusOr<google::storage::v2::WriteObjectResponse>>,
               Finish, (), (override));
-  MOCK_METHOD(google::cloud::internal::StreamingRpcMetadata, GetRequestMetadata,
-              (), (const, override));
+  MOCK_METHOD(google::cloud::RpcMetadata, GetRequestMetadata, (),
+              (const, override));
 };
 
 using MockAsyncObjectMediaStream =
