@@ -19,6 +19,8 @@
 #include "google/cloud/internal/format_time_point.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <nlohmann/json.hpp>
+#include <memory>
+#include <string>
 
 namespace google {
 namespace cloud {
