@@ -20,6 +20,9 @@
 #include "google/cloud/testing_util/status_matchers.h"
 #include "absl/types/optional.h"
 #include <gmock/gmock.h>
+#include <sstream>
+#include <string>
+#include <vector>
 #ifndef _WIN32
 #include <dlfcn.h>
 #endif  // _WIN32
