@@ -14,11 +14,12 @@
 
 #include "google/cloud/storage/testing/storage_integration_test.h"
 #include "google/cloud/internal/getenv.h"
-#include "google/cloud/testing_util/scoped_environment.h"
 #include "google/cloud/testing_util/setenv.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
-#include <nlohmann/json.hpp>
+#include <sstream>
+#include <string>
+#include <utility>
 #include <vector>
 
 namespace google {
@@ -32,16 +33,11 @@ using ::testing::IsEmpty;
 using ::testing::Not;
 
 class GrpcIntegrationTest
-    : public google::cloud::storage::testing::StorageIntegrationTest,
-      public ::testing::WithParamInterface<std::string> {
+    : public google::cloud::storage::testing::StorageIntegrationTest {
  protected:
-  GrpcIntegrationTest()
-      : grpc_config_("GOOGLE_CLOUD_CPP_STORAGE_GRPC_CONFIG", {}) {}
+  GrpcIntegrationTest() = default;
 
   void SetUp() override {
-    std::string const grpc_config_value = GetParam();
-    google::cloud::testing_util::SetEnv("GOOGLE_CLOUD_CPP_STORAGE_GRPC_CONFIG",
-                                        grpc_config_value);
     project_id_ = GetEnv("GOOGLE_CLOUD_PROJECT").value_or("");
     ASSERT_THAT(project_id_, Not(IsEmpty()))
         << "GOOGLE_CLOUD_PROJECT is not set";
@@ -58,53 +54,46 @@ class GrpcIntegrationTest
  private:
   std::string project_id_;
   std::string bucket_name_;
-  testing_util::ScopedEnvironment grpc_config_;
 };
 
-TEST_P(GrpcIntegrationTest, ObjectCRUD) {
+TEST_F(GrpcIntegrationTest, ObjectCRUD) {
   auto bucket_client = MakeBucketIntegrationTestClient();
-  ASSERT_STATUS_OK(bucket_client);
-
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto bucket_name = MakeRandomBucketName();
   auto object_name = MakeRandomObjectName();
-  auto bucket_metadata = bucket_client->CreateBucketForProject(
+
+  auto bucket_metadata = bucket_client.CreateBucketForProject(
       bucket_name, project_id(), BucketMetadata());
   ASSERT_STATUS_OK(bucket_metadata);
 
   EXPECT_EQ(bucket_name, bucket_metadata->name());
 
-  auto object_metadata = client->InsertObject(
+  auto object_metadata = client.InsertObject(
       bucket_name, object_name, LoremIpsum(), IfGenerationMatch(0));
   ASSERT_STATUS_OK(object_metadata);
 
-  auto stream = client->ReadObject(bucket_name, object_name);
+  auto stream = client.ReadObject(bucket_name, object_name);
 
   std::string actual(std::istreambuf_iterator<char>{stream}, {});
   EXPECT_EQ(LoremIpsum(), actual);
   EXPECT_STATUS_OK(stream.status());
 
   // This is part of the test, not just a cleanup.
-  auto delete_object_status = client->DeleteObject(
+  auto delete_object_status = client.DeleteObject(
       bucket_name, object_name, Generation(object_metadata->generation()));
   EXPECT_STATUS_OK(delete_object_status);
 
-  auto delete_bucket_status = bucket_client->DeleteBucket(bucket_name);
+  auto delete_bucket_status = bucket_client.DeleteBucket(bucket_name);
   EXPECT_STATUS_OK(delete_bucket_status);
 }
 
-TEST_P(GrpcIntegrationTest, WriteResume) {
+TEST_F(GrpcIntegrationTest, WriteResume) {
   auto bucket_client = MakeBucketIntegrationTestClient();
-  ASSERT_STATUS_OK(bucket_client);
-
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto bucket_name = MakeRandomBucketName();
   auto object_name = MakeRandomObjectName();
-  auto bucket_metadata = bucket_client->CreateBucketForProject(
+
+  auto bucket_metadata = bucket_client.CreateBucketForProject(
       bucket_name, project_id(), BucketMetadata());
   ASSERT_STATUS_OK(bucket_metadata);
 
@@ -115,15 +104,15 @@ TEST_P(GrpcIntegrationTest, WriteResume) {
   std::string session_id;
   {
     auto old_os =
-        client->WriteObject(bucket_name, object_name, IfGenerationMatch(0),
-                            NewResumableUploadSession());
+        client.WriteObject(bucket_name, object_name, IfGenerationMatch(0),
+                           NewResumableUploadSession());
     ASSERT_TRUE(old_os.good()) << "status=" << old_os.metadata().status();
     session_id = old_os.resumable_session_id();
     std::move(old_os).Suspend();
   }
 
-  auto os = client->WriteObject(bucket_name, object_name,
-                                RestoreResumableUploadSession(session_id));
+  auto os = client.WriteObject(bucket_name, object_name,
+                               RestoreResumableUploadSession(session_id));
   ASSERT_TRUE(os.good()) << "status=" << os.metadata().status();
   EXPECT_EQ(session_id, os.resumable_session_id());
   os << LoremIpsum();
@@ -139,23 +128,20 @@ TEST_P(GrpcIntegrationTest, WriteResume) {
     EXPECT_EQ("resumable", meta.metadata("x_emulator_upload"));
   }
 
-  auto status = client->DeleteObject(bucket_name, object_name);
+  auto status = client.DeleteObject(bucket_name, object_name);
   EXPECT_STATUS_OK(status);
 
-  auto delete_bucket_status = bucket_client->DeleteBucket(bucket_name);
+  auto delete_bucket_status = bucket_client.DeleteBucket(bucket_name);
   EXPECT_STATUS_OK(delete_bucket_status);
 }
 
-TEST_P(GrpcIntegrationTest, InsertLarge) {
+TEST_F(GrpcIntegrationTest, InsertLarge) {
   auto bucket_client = MakeBucketIntegrationTestClient();
-  ASSERT_STATUS_OK(bucket_client);
-
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto bucket_name = MakeRandomBucketName();
   auto object_name = MakeRandomObjectName();
-  auto bucket_metadata = bucket_client->CreateBucketForProject(
+
+  auto bucket_metadata = bucket_client.CreateBucketForProject(
       bucket_name, project_id(), BucketMetadata());
   ASSERT_STATUS_OK(bucket_metadata);
   ScheduleForDelete(*bucket_metadata);
@@ -164,24 +150,21 @@ TEST_P(GrpcIntegrationTest, InsertLarge) {
   // multiple of 256 KiB.
   auto const desired_size = 8 * 1024 * 1024L + 253 * 1024 + 15;
   auto data = MakeRandomData(desired_size);
-  auto metadata = client->InsertObject(bucket_name, object_name, data,
-                                       IfGenerationMatch(0));
+  auto metadata =
+      client.InsertObject(bucket_name, object_name, data, IfGenerationMatch(0));
   ASSERT_STATUS_OK(metadata);
   ScheduleForDelete(*metadata);
 
   EXPECT_EQ(desired_size, metadata->size());
 }
 
-TEST_P(GrpcIntegrationTest, StreamLargeChunks) {
+TEST_F(GrpcIntegrationTest, StreamLargeChunks) {
   auto bucket_client = MakeBucketIntegrationTestClient();
-  ASSERT_STATUS_OK(bucket_client);
-
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto bucket_name = MakeRandomBucketName();
   auto object_name = MakeRandomObjectName();
-  auto bucket_metadata = bucket_client->CreateBucketForProject(
+
+  auto bucket_metadata = bucket_client.CreateBucketForProject(
       bucket_name, project_id(), BucketMetadata());
   ASSERT_STATUS_OK(bucket_metadata);
   ScheduleForDelete(*std::move(bucket_metadata));
@@ -190,7 +173,7 @@ TEST_P(GrpcIntegrationTest, StreamLargeChunks) {
   auto const desired_size = 8 * 1024 * 1024L;
   auto data = MakeRandomData(desired_size);
   auto stream =
-      client->WriteObject(bucket_name, object_name, IfGenerationMatch(0));
+      client.WriteObject(bucket_name, object_name, IfGenerationMatch(0));
   stream.write(data.data(), data.size());
   EXPECT_TRUE(stream.good());
   stream.write(data.data(), data.size());
@@ -203,51 +186,39 @@ TEST_P(GrpcIntegrationTest, StreamLargeChunks) {
   EXPECT_EQ(2 * desired_size, stream.metadata()->size());
 }
 
-TEST_P(GrpcIntegrationTest, QuotaUser) {
+TEST_F(GrpcIntegrationTest, QuotaUser) {
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto object_name = MakeRandomObjectName();
 
   auto metadata =
-      client->InsertObject(bucket_name(), object_name, LoremIpsum(),
-                           IfGenerationMatch(0), QuotaUser("test-only"));
+      client.InsertObject(bucket_name(), object_name, LoremIpsum(),
+                          IfGenerationMatch(0), QuotaUser("test-only"));
   ASSERT_STATUS_OK(metadata);
   ScheduleForDelete(*metadata);
 }
 
-TEST_P(GrpcIntegrationTest, FieldFilter) {
+TEST_F(GrpcIntegrationTest, FieldFilter) {
   if (UsingEmulator()) GTEST_SKIP();
   auto const* fields = UsingGrpc() ? "resource.bucket,resource.name,resource."
                                      "generation,resource.content_type"
                                    : "bucket,name,generation,contentType";
   auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
   auto object_name = MakeRandomObjectName();
 
-  auto metadata = client->InsertObject(
+  auto metadata = client.InsertObject(
       bucket_name(), object_name, LoremIpsum(), IfGenerationMatch(0),
       ContentType("text/plain"), ContentEncoding("utf-8"), Fields(fields));
   ASSERT_STATUS_OK(metadata);
   ScheduleForDelete(*metadata);
 
-  // If the Fields() filter works, then size() would be 0
-  if (!UsingEmulator()) {
-    EXPECT_EQ(metadata->size(), 0);
-    EXPECT_EQ(metadata->content_type(), "text/plain");
-    EXPECT_EQ(metadata->content_encoding(), "");
-  }
+  // If the Fields() filter works, then `size()` and `content_encoding()` should
+  // have default values (despite having non-defaults in production) but the
+  // `content_type()` should have the value set in production.
+  EXPECT_EQ(metadata->size(), 0);
+  EXPECT_EQ(metadata->content_encoding(), "");
+  EXPECT_EQ(metadata->content_type(), "text/plain");
 }
 
-#if GOOGLE_CLOUD_CPP_STORAGE_HAVE_GRPC
-INSTANTIATE_TEST_SUITE_P(GrpcIntegrationMediaTest, GrpcIntegrationTest,
-                         ::testing::Values("media"));
-#else
-INSTANTIATE_TEST_SUITE_P(GrpcIntegrationMediaTest, GrpcIntegrationTest,
-                         ::testing::Values("none"));
-#endif  // GOOGLE_CLOUD_CPP_STORAGE_HAVE_GRPC
-
 }  // namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace storage
