@@ -14,9 +14,9 @@
 
 #include "google/cloud/storage/testing/storage_integration_test.h"
 #include "google/cloud/internal/getenv.h"
-#include "google/cloud/testing_util/scoped_environment.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <string>
 #include <vector>
 
 namespace google {
@@ -26,7 +26,6 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
 using ::google::cloud::internal::GetEnv;
-using ::google::cloud::testing_util::ScopedEnvironment;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::_;
 using ::testing::AllOf;
@@ -41,56 +40,52 @@ class GrpcObjectMetadataIntegrationTest
     : public google::cloud::storage::testing::StorageIntegrationTest {};
 
 TEST_F(GrpcObjectMetadataIntegrationTest, ObjectMetadataCRUD) {
-  ScopedEnvironment grpc_config("GOOGLE_CLOUD_CPP_STORAGE_GRPC_CONFIG",
-                                "metadata");
   auto const bucket_name =
       GetEnv("GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME").value_or("");
   ASSERT_THAT(bucket_name, Not(IsEmpty()))
       << "GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME is not set";
 
-  auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient(/*use_grpc=*/true);
   auto object_name = MakeRandomObjectName();
   auto rewrite_name = MakeRandomObjectName();
   auto copy_name = MakeRandomObjectName();
   auto compose_name = MakeRandomObjectName();
 
   // Use the full projection to get consistent behavior out of gRPC and REST.
-  auto insert = client->InsertObject(bucket_name, object_name, LoremIpsum(),
-                                     IfGenerationMatch(0), Projection::Full());
+  auto insert = client.InsertObject(bucket_name, object_name, LoremIpsum(),
+                                    IfGenerationMatch(0), Projection::Full());
   ASSERT_STATUS_OK(insert);
   ScheduleForDelete(*insert);
 
   auto get =
-      client->GetObjectMetadata(bucket_name, object_name, Projection::Full());
+      client.GetObjectMetadata(bucket_name, object_name, Projection::Full());
   ASSERT_STATUS_OK(get);
   EXPECT_EQ(*insert, *get);
 
   std::vector<std::string> names;
-  for (auto const& object : client->ListObjects(bucket_name)) {
+  for (auto const& object : client.ListObjects(bucket_name)) {
     ASSERT_STATUS_OK(object);
     names.push_back(object->name());
   }
   EXPECT_THAT(names, Contains(object_name));
 
-  auto rewrite = client->RewriteObjectBlocking(bucket_name, object_name,
-                                               bucket_name, rewrite_name);
+  auto rewrite = client.RewriteObjectBlocking(bucket_name, object_name,
+                                              bucket_name, rewrite_name);
   ASSERT_STATUS_OK(rewrite);
   ScheduleForDelete(*rewrite);
 
   auto copy =
-      client->CopyObject(bucket_name, object_name, bucket_name, rewrite_name);
+      client.CopyObject(bucket_name, object_name, bucket_name, rewrite_name);
   ASSERT_STATUS_OK(copy);
   ScheduleForDelete(*copy);
 
-  auto patch = client->PatchObject(
+  auto patch = client.PatchObject(
       bucket_name, object_name,
       ObjectMetadataPatchBuilder{}.SetCacheControl("no-cache"));
   ASSERT_STATUS_OK(patch);
   EXPECT_EQ(patch->cache_control(), "no-cache");
 
-  auto compose = client->ComposeObject(
+  auto compose = client.ComposeObject(
       bucket_name,
       {
           ComposeSourceObject{object_name, absl::nullopt, absl::nullopt},
@@ -103,52 +98,48 @@ TEST_F(GrpcObjectMetadataIntegrationTest, ObjectMetadataCRUD) {
   auto custom = std::chrono::system_clock::now() + std::chrono::hours(24);
   auto desired_metadata = ObjectMetadata(*patch).set_custom_time(custom);
   auto updated =
-      client->UpdateObject(bucket_name, object_name, desired_metadata);
+      client.UpdateObject(bucket_name, object_name, desired_metadata);
   ASSERT_STATUS_OK(updated);
   ASSERT_TRUE(updated->has_custom_time());
   ASSERT_EQ(updated->custom_time(), custom);
 
-  auto del = client->DeleteObject(bucket_name, object_name);
+  auto del = client.DeleteObject(bucket_name, object_name);
   ASSERT_STATUS_OK(del);
 
-  get = client->GetObjectMetadata(bucket_name, object_name);
+  get = client.GetObjectMetadata(bucket_name, object_name);
   EXPECT_THAT(get, StatusIs(StatusCode::kNotFound));
 }
 
 TEST_F(GrpcObjectMetadataIntegrationTest, PatchMetadata) {
-  ScopedEnvironment grpc_config("GOOGLE_CLOUD_CPP_STORAGE_GRPC_CONFIG",
-                                "metadata");
   auto const bucket_name =
       GetEnv("GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME").value_or("");
   ASSERT_THAT(bucket_name, Not(IsEmpty()))
       << "GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME is not set";
 
-  auto client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient(/*use_grpc=*/true);
   auto object_name = MakeRandomObjectName();
 
   // Use the full projection to get consistent behavior out of gRPC and REST.
-  auto insert = client->InsertObject(bucket_name, object_name, LoremIpsum(),
-                                     IfGenerationMatch(0), Projection::Full());
+  auto insert = client.InsertObject(bucket_name, object_name, LoremIpsum(),
+                                    IfGenerationMatch(0), Projection::Full());
   ASSERT_STATUS_OK(insert);
   ScheduleForDelete(*insert);
 
-  auto patch = client->PatchObject(bucket_name, object_name,
-                                   ObjectMetadataPatchBuilder{}
-                                       .SetMetadata("test-key0", "v0")
-                                       .SetMetadata("test-key1", "v1")
-                                       .SetMetadata("test-key2", "v2"));
+  auto patch = client.PatchObject(bucket_name, object_name,
+                                  ObjectMetadataPatchBuilder{}
+                                      .SetMetadata("test-key0", "v0")
+                                      .SetMetadata("test-key1", "v1")
+                                      .SetMetadata("test-key2", "v2"));
   ASSERT_STATUS_OK(patch);
   EXPECT_THAT(patch->metadata(), AllOf(Contains(Pair("test-key0", "v0")),
                                        Contains(Pair("test-key1", "v1")),
                                        Contains(Pair("test-key2", "v2"))));
 
-  patch = client->PatchObject(bucket_name, object_name,
-                              ObjectMetadataPatchBuilder{}
-                                  .SetMetadata("test-key0", "new-v0")
-                                  .ResetMetadata("test-key1")
-                                  .SetMetadata("test-key3", "v3"));
+  patch = client.PatchObject(bucket_name, object_name,
+                             ObjectMetadataPatchBuilder{}
+                                 .SetMetadata("test-key0", "new-v0")
+                                 .ResetMetadata("test-key1")
+                                 .SetMetadata("test-key3", "v3"));
   ASSERT_STATUS_OK(patch);
   EXPECT_THAT(patch->metadata(), AllOf(Contains(Pair("test-key0", "new-v0")),
                                        Not(Contains(Pair("test-key1", _))),
