@@ -19,6 +19,7 @@
 #include "google/cloud/internal/rest_request.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <string>
 
 namespace google {
 namespace cloud {
@@ -67,12 +68,14 @@ class KeyFileIntegrationTest
 };
 
 TEST_P(KeyFileIntegrationTest, ObjectWriteSignAndReadDefaultAccount) {
+  if (UsingGrpc()) GTEST_SKIP();
+
   auto credentials =
       oauth2::CreateServiceAccountCredentialsFromFilePath(key_filename_);
   ASSERT_STATUS_OK(credentials);
 
-  Client client(Options{}.set<Oauth2CredentialsOption>(*credentials));
-
+  auto client = MakeIntegrationTestClient(
+      Options{}.set<Oauth2CredentialsOption>(*credentials));
   auto object_name = MakeRandomObjectName();
   std::string expected = LoremIpsum();
 
@@ -93,12 +96,14 @@ TEST_P(KeyFileIntegrationTest, ObjectWriteSignAndReadDefaultAccount) {
 }
 
 TEST_P(KeyFileIntegrationTest, ObjectWriteSignAndReadExplicitAccount) {
+  if (UsingGrpc()) GTEST_SKIP();
+
   auto credentials =
       oauth2::CreateServiceAccountCredentialsFromFilePath(key_filename_);
   ASSERT_STATUS_OK(credentials);
 
-  Client client(Options{}.set<Oauth2CredentialsOption>(*credentials));
-
+  auto client = MakeIntegrationTestClient(
+      Options{}.set<Oauth2CredentialsOption>(*credentials));
   auto object_name = MakeRandomObjectName();
   std::string expected = LoremIpsum();
 
