@@ -18,6 +18,8 @@
 #include "google/cloud/testing_util/scoped_log.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <string>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -27,12 +29,14 @@ namespace {
 
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::StatusIs;
+using ::testing::_;
+using ::testing::Contains;
 using ::testing::HasSubstr;
 using ::testing::Not;
+using ::testing::Pair;
 
 class ObjectChecksumIntegrationTest
-    : public google::cloud::storage::testing::StorageIntegrationTest,
-      public ::testing::WithParamInterface<std::string> {
+    : public google::cloud::storage::testing::StorageIntegrationTest {
  protected:
   void SetUp() override {
     bucket_name_ = google::cloud::internal::GetEnv(
@@ -45,12 +49,11 @@ class ObjectChecksumIntegrationTest
 };
 
 /// @test Verify that CRC32C checksums are enabled by default.
-TEST_P(ObjectChecksumIntegrationTest, InsertObjectDefault) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, InsertObjectDefault) {
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto meta = client->InsertObject(bucket_name_, object_name, LoremIpsum(),
-                                   DisableMD5Hash(true), IfGenerationMatch(0));
+  auto meta = client.InsertObject(bucket_name_, object_name, LoremIpsum(),
+                                  DisableMD5Hash(true), IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
@@ -61,13 +64,12 @@ TEST_P(ObjectChecksumIntegrationTest, InsertObjectDefault) {
 }
 
 /// @test Verify that `DisableCrc32cChecksum(true)` works as expected.
-TEST_P(ObjectChecksumIntegrationTest, InsertObjectExplicitDisable) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, InsertObjectExplicitDisable) {
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto meta = client->InsertObject(bucket_name_, object_name, LoremIpsum(),
-                                   DisableCrc32cChecksum(true),
-                                   DisableMD5Hash(true), IfGenerationMatch(0));
+  auto meta = client.InsertObject(bucket_name_, object_name, LoremIpsum(),
+                                  DisableCrc32cChecksum(true),
+                                  DisableMD5Hash(true), IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
@@ -78,13 +80,13 @@ TEST_P(ObjectChecksumIntegrationTest, InsertObjectExplicitDisable) {
 }
 
 /// @test Verify that `DisableCrc32cChecksum(false)` works as expected.
-TEST_P(ObjectChecksumIntegrationTest, InsertObjectExplicitEnable) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, InsertObjectExplicitEnable) {
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto meta = client->InsertObject(bucket_name_, object_name, LoremIpsum(),
-                                   DisableCrc32cChecksum(false),
-                                   DisableMD5Hash(true), IfGenerationMatch(0));
+
+  auto meta = client.InsertObject(bucket_name_, object_name, LoremIpsum(),
+                                  DisableCrc32cChecksum(false),
+                                  DisableMD5Hash(true), IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
@@ -94,11 +96,11 @@ TEST_P(ObjectChecksumIntegrationTest, InsertObjectExplicitEnable) {
   }
 }
 
-TEST_P(ObjectChecksumIntegrationTest, InsertObjectWithValueSuccess) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, InsertObjectWithValueSuccess) {
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto meta = client->InsertObject(
+
+  auto meta = client.InsertObject(
       bucket_name_, object_name, LoremIpsum(),
       Crc32cChecksumValue(ComputeCrc32cChecksum(LoremIpsum())),
       DisableMD5Hash(true), IfGenerationMatch(0));
@@ -111,11 +113,14 @@ TEST_P(ObjectChecksumIntegrationTest, InsertObjectWithValueSuccess) {
   }
 }
 
-TEST_P(ObjectChecksumIntegrationTest, InsertObjectWithValueFailure) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, InsertObjectWithValueFailure) {
+  // TODO(#14385) - the emulator does not support this feature for gRPC.
+  if (UsingEmulator() && UsingGrpc()) GTEST_SKIP();
+
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto failure = client->InsertObject(
+
+  auto failure = client.InsertObject(
       bucket_name_, object_name, LoremIpsum(), DisableMD5Hash(true),
       IfGenerationMatch(0), Crc32cChecksumValue(ComputeCrc32cChecksum("")));
   EXPECT_THAT(failure, Not(IsOk()));
@@ -123,11 +128,11 @@ TEST_P(ObjectChecksumIntegrationTest, InsertObjectWithValueFailure) {
 
 /// @test Verify that CRC32C checksums are computed by default in WriteObject().
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectDefault) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto os = client->WriteObject(bucket_name_, object_name, DisableMD5Hash(true),
-                                IfGenerationMatch(0));
+
+  auto os = client.WriteObject(bucket_name_, object_name, DisableMD5Hash(true),
+                               IfGenerationMatch(0));
   os << LoremIpsum();
   os.Close();
   auto meta = os.metadata();
@@ -140,21 +145,26 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectDefault) {
   EXPECT_THAT(os.computed_hash(),
               HasSubstr(ComputeCrc32cChecksum(LoremIpsum())));
   if (meta->has_metadata("x_emulator_upload")) {
-    // Streaming uploads over REST cannot include checksums
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_crc32c")) << *meta;
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_md5")) << *meta;
+    if (UsingGrpc()) {
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_crc32c", _)));
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
+    } else {
+      // Streaming uploads over REST cannot include checksums
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_crc32c", _)));
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
+    }
   }
 }
 
 /// @test Verify that CRC32C checksums can be explicitly disabled in
 /// WriteObject().
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectExplicitDisable) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto os = client->WriteObject(bucket_name_, object_name,
-                                DisableCrc32cChecksum(true),
-                                DisableMD5Hash(true), IfGenerationMatch(0));
+
+  auto os =
+      client.WriteObject(bucket_name_, object_name, DisableCrc32cChecksum(true),
+                         DisableMD5Hash(true), IfGenerationMatch(0));
   os << LoremIpsum();
   os.Close();
   auto meta = os.metadata();
@@ -166,21 +176,19 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectExplicitDisable) {
   EXPECT_THAT(os.computed_hash(),
               Not(HasSubstr(ComputeCrc32cChecksum(LoremIpsum()))));
   if (meta->has_metadata("x_emulator_upload")) {
-    // Streaming uploads over REST cannot include checksums
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_crc32c")) << *meta;
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_md5")) << *meta;
+    EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_crc32c", _)));
+    EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
   }
 }
 
 /// @test Verify that CRC32C checksums can be explicitly enabled in
 /// WriteObject().
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectExplicitEnable) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto os = client->WriteObject(bucket_name_, object_name,
-                                DisableCrc32cChecksum(false),
-                                DisableMD5Hash(true), IfGenerationMatch(0));
+  auto os = client.WriteObject(bucket_name_, object_name,
+                               DisableCrc32cChecksum(false),
+                               DisableMD5Hash(true), IfGenerationMatch(0));
   os << LoremIpsum();
   os.Close();
   auto meta = os.metadata();
@@ -192,18 +200,23 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectExplicitEnable) {
   EXPECT_THAT(os.computed_hash(),
               HasSubstr(ComputeCrc32cChecksum(LoremIpsum())));
   if (meta->has_metadata("x_emulator_upload")) {
-    // Streaming uploads over REST cannot include checksums
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_crc32c")) << *meta;
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_md5")) << *meta;
+    if (UsingGrpc()) {
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_crc32c", _)));
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
+    } else {
+      // Streaming uploads over REST cannot include checksums
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_crc32c", _)));
+      EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
+    }
   }
 }
 
 /// @test Verify that valid CRC32C checksums values work in WriteObject().
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectWithValueSuccess) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto os = client->WriteObject(
+
+  auto os = client.WriteObject(
       bucket_name_, object_name,
       Crc32cChecksumValue(ComputeCrc32cChecksum(LoremIpsum())),
       DisableMD5Hash(true), IfGenerationMatch(0));
@@ -214,19 +227,20 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectWithValueSuccess) {
   ScheduleForDelete(*meta);
 
   if (meta->has_metadata("x_emulator_upload")) {
-    // Streaming uploads over REST cannot include checksums unless provided by
-    // the application when the upload begins.
-    EXPECT_TRUE(meta->has_metadata("x_emulator_crc32c")) << *meta;
-    EXPECT_TRUE(meta->has_metadata("x_emulator_no_md5")) << *meta;
+    EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_crc32c", _)));
+    EXPECT_THAT(meta->metadata(), Contains(Pair("x_emulator_no_md5", _)));
   }
 }
 
 /// @test Verify that incorrect CRC32C checksums values work in WriteObject().
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectWithValueFailure) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  // TODO(#14385) - the emulator does not support this feature for gRPC.
+  if (UsingEmulator() && UsingGrpc()) GTEST_SKIP();
+
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto os = client->WriteObject(
+
+  auto os = client.WriteObject(
       bucket_name_, object_name, MD5HashValue(ComputeMD5Hash("")),
       DisableCrc32cChecksum(true), IfGenerationMatch(0));
   os << LoremIpsum();
@@ -239,15 +253,14 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectWithValueFailure) {
 /// receives bad data.
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectReceiveBadChecksum) {
   // This test is disabled when not using the emulator as it relies on the
-  // emulator to inject faults.
-  if (!UsingEmulator()) GTEST_SKIP();
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  // emulator to inject faults. The emulator does not support this type of fault
+  // injection for gRPC either.
+  if (!UsingEmulator() || UsingGrpc()) GTEST_SKIP();
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
 
   // Create a stream to upload an object.
-  ObjectWriteStream stream = client->WriteObject(
+  ObjectWriteStream stream = client.WriteObject(
       bucket_name_, object_name, DisableMD5Hash(true),
       CustomHeader("x-goog-emulator-instructions", "inject-upload-data-error"),
       IfGenerationMatch(0));
@@ -261,13 +274,13 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectReceiveBadChecksum) {
 
 /// @test Verify that CRC32C checksum mismatches are reported by default.
 TEST_F(ObjectChecksumIntegrationTest, WriteObjectUploadBadChecksum) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  // TODO(#14385) - the emulator does not support this feature for gRPC.
+  if (UsingEmulator() && UsingGrpc()) GTEST_SKIP();
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
 
   // Create a stream to upload an object.
-  ObjectWriteStream stream = client->WriteObject(
+  ObjectWriteStream stream = client.WriteObject(
       bucket_name_, object_name, Crc32cChecksumValue(ComputeCrc32cChecksum("")),
       DisableMD5Hash(true), IfGenerationMatch(0));
   stream << LoremIpsum() << "\n";
@@ -277,16 +290,19 @@ TEST_F(ObjectChecksumIntegrationTest, WriteObjectUploadBadChecksum) {
 }
 
 /// @test Verify that CRC32C checksums are computed by default on downloads.
-TEST_P(ObjectChecksumIntegrationTest, ReadObjectDefault) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+TEST_F(ObjectChecksumIntegrationTest, ReadObjectDefault) {
+  // TODO(#14385) - the emulator does not support this feature for gRPC.
+  if (UsingEmulator() && UsingGrpc()) GTEST_SKIP();
+
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  auto meta = client->InsertObject(bucket_name_, object_name, LoremIpsum(),
-                                   IfGenerationMatch(0));
+
+  auto meta = client.InsertObject(bucket_name_, object_name, LoremIpsum(),
+                                  IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
-  auto stream = client->ReadObject(bucket_name_, object_name);
+  auto stream = client.ReadObject(bucket_name_, object_name);
   auto const actual = std::string{std::istreambuf_iterator<char>{stream}, {}};
   ASSERT_FALSE(stream.IsOpen());
 
@@ -296,19 +312,20 @@ TEST_P(ObjectChecksumIntegrationTest, ReadObjectDefault) {
 
 /// @test Verify that CRC32C checksum mismatches are reported by default on
 /// downloads.
-TEST_P(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerGetc) {
+TEST_F(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerGetc) {
   // This test is disabled when not using the emulator as it relies on the
-  // emulator to inject faults.
-  if (!UsingEmulator()) GTEST_SKIP();
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  // emulator to inject faults. The emulator does not support this type of fault
+  // injection for gRPC either.
+  if (!UsingEmulator() || UsingGrpc()) GTEST_SKIP();
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-  StatusOr<ObjectMetadata> meta = client->InsertObject(
+
+  StatusOr<ObjectMetadata> meta = client.InsertObject(
       bucket_name_, object_name, LoremIpsum(), IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
-  auto stream = client->ReadObject(
+  auto stream = client.ReadObject(
       bucket_name_, object_name, DisableMD5Hash(true),
       CustomHeader("x-goog-emulator-instructions", "return-corrupted-data"));
 
@@ -334,20 +351,21 @@ TEST_P(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerGetc) {
 
 /// @test Verify that CRC32C checksum mismatches are reported by default on
 /// downloads.
-TEST_P(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerRead) {
+TEST_F(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerRead) {
   // This test is disabled when not using the emulator as it relies on the
-  // emulator to inject faults.
-  if (!UsingEmulator()) GTEST_SKIP();
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  // emulator to inject faults. The emulator does not support this type of fault
+  // injection for gRPC either.
+  if (!UsingEmulator() || UsingGrpc()) GTEST_SKIP();
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
+
   StatusOr<ObjectMetadata> meta =
-      client->InsertObject(bucket_name_, object_name, LoremIpsum(),
-                           IfGenerationMatch(0), Projection::Full());
+      client.InsertObject(bucket_name_, object_name, LoremIpsum(),
+                          IfGenerationMatch(0), Projection::Full());
   ASSERT_STATUS_OK(meta);
   ScheduleForDelete(*meta);
 
-  auto stream = client->ReadObject(
+  auto stream = client.ReadObject(
       bucket_name_, object_name, DisableMD5Hash(true),
       CustomHeader("x-goog-emulator-instructions", "return-corrupted-data"));
 
@@ -362,10 +380,6 @@ TEST_P(ObjectChecksumIntegrationTest, ReadObjectCorruptedByServerRead) {
   EXPECT_EQ(stream.received_hash(), meta->crc32c());
 }
 
-INSTANTIATE_TEST_SUITE_P(ObjectChecksumIntegrationTestJson,
-                         ObjectChecksumIntegrationTest,
-                         ::testing::Values("JSON"));
-
 }  // anonymous namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace storage
