@@ -22,7 +22,11 @@
 #include <cstdio>
 #include <fstream>
 #include <future>
+#include <iostream>
+#include <mutex>
+#include <string>
 #include <thread>
+#include <vector>
 
 namespace google {
 namespace cloud {
@@ -157,12 +161,11 @@ class ObjectFileMultiThreadedTest
 };
 
 TEST_F(ObjectFileMultiThreadedTest, Download) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto const names = CreateNames();
+
   std::cout << "Create test objects " << std::flush;
-  ASSERT_NO_FATAL_FAILURE(CreateObjects(*client, names));
+  ASSERT_NO_FATAL_FAILURE(CreateObjects(client, names));
   std::cout << " DONE\n";
 
   // Create multiple threads, each downloading a portion of the objects.
@@ -178,7 +181,7 @@ TEST_F(ObjectFileMultiThreadedTest, Download) {
         std::cout << '.' << std::flush;
       }
       auto status =
-          client->DownloadToFile(bucket_name_, name.object_name, name.filename);
+          client.DownloadToFile(bucket_name_, name.object_name, name.filename);
       if (!status.ok()) return status;  // stop on the first error
     }
     return Status();
@@ -200,7 +203,7 @@ TEST_F(ObjectFileMultiThreadedTest, Download) {
   }
 
   std::cout << "Delete test objects " << std::flush;
-  ASSERT_NO_FATAL_FAILURE(DeleteObjects(*client, names));
+  ASSERT_NO_FATAL_FAILURE(DeleteObjects(client, names));
   std::cout << " DONE\n";
 }
 
