@@ -17,7 +17,13 @@
 #include "google/cloud/internal/getenv.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
+#include <algorithm>
+#include <iostream>
+#include <random>
+#include <sstream>
+#include <string>
 #include <thread>
+#include <utility>
 
 namespace google {
 namespace cloud {
@@ -46,16 +52,14 @@ class ObjectResumableWriteIntegrationTest
 };
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteWithContentType) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
 
   // We will construct the expected response while streaming the data up.
   std::ostringstream expected;
 
   // Create the object, but only if it does not exist already.
-  auto os = client->WriteObject(
+  auto os = client.WriteObject(
       bucket_name_, object_name, IfGenerationMatch(0),
       WithObjectMetadata(ObjectMetadata().set_content_type("text/plain")));
   os.exceptions(std::ios_base::failbit);
@@ -75,9 +79,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteWithContentType) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteWithContentTypeFailure) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto bucket_name = MakeRandomBucketName();
   auto object_name = MakeRandomObjectName();
 
@@ -85,7 +87,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteWithContentTypeFailure) {
   std::ostringstream expected;
 
   // Create the object, but only if it does not exist already.
-  auto os = client->WriteObject(
+  auto os = client.WriteObject(
       bucket_name, object_name, IfGenerationMatch(0),
       WithObjectMetadata(ObjectMetadata().set_content_type("text/plain")));
   EXPECT_TRUE(os.bad());
@@ -94,17 +96,15 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteWithContentTypeFailure) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteWithUseResumable) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
 
   // We will construct the expected response while streaming the data up.
   std::ostringstream expected;
 
   // Create the object, but only if it does not exist already.
-  auto os = client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
-                                NewResumableUploadSession());
+  auto os = client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
+                               NewResumableUploadSession());
   os.exceptions(std::ios_base::failbit);
   os << LoremIpsum();
   EXPECT_FALSE(os.resumable_session_id().empty());
@@ -121,9 +121,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteWithUseResumable) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteResume) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
 
   // We will construct the expected response while streaming the data up.
@@ -133,15 +131,15 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResume) {
   std::string session_id;
   {
     auto old_os =
-        client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
-                            NewResumableUploadSession());
+        client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
+                           NewResumableUploadSession());
     ASSERT_TRUE(old_os.good()) << "status=" << old_os.metadata().status();
     session_id = old_os.resumable_session_id();
     std::move(old_os).Suspend();
   }
 
-  auto os = client->WriteObject(bucket_name_, object_name,
-                                RestoreResumableUploadSession(session_id));
+  auto os = client.WriteObject(bucket_name_, object_name,
+                               RestoreResumableUploadSession(session_id));
   ASSERT_TRUE(os.good()) << "status=" << os.metadata().status();
   EXPECT_EQ(session_id, os.resumable_session_id());
   os << LoremIpsum();
@@ -158,9 +156,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResume) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeWithPartial) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
   auto constexpr kUploadQuantum = 256 * 1024;
   auto const q0 = MakeRandomData(kUploadQuantum);
@@ -170,7 +166,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeWithPartial) {
   auto const session_id = [&]() {
     // Start the upload, add some data, and flush it.
     auto os =
-        client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
+        client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
     EXPECT_TRUE(os.good()) << "status=" << os.last_status();
     os.write(q0.data(), q0.size());
     os.flush();
@@ -182,8 +178,8 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeWithPartial) {
 
   auto expected_committed_size = static_cast<std::uint64_t>(q0.size());
   for (auto const& data : {q1, q2}) {
-    auto os = client->WriteObject(bucket_name_, object_name,
-                                  RestoreResumableUploadSession(session_id));
+    auto os = client.WriteObject(bucket_name_, object_name,
+                                 RestoreResumableUploadSession(session_id));
     ASSERT_TRUE(os.good()) << "status=" << os.last_status();
     EXPECT_EQ(os.resumable_session_id(), session_id);
     EXPECT_EQ(os.next_expected_byte(), expected_committed_size);
@@ -194,8 +190,8 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeWithPartial) {
     std::move(os).Suspend();
   }
 
-  auto os = client->WriteObject(bucket_name_, object_name,
-                                RestoreResumableUploadSession(session_id));
+  auto os = client.WriteObject(bucket_name_, object_name,
+                               RestoreResumableUploadSession(session_id));
   ASSERT_TRUE(os.good()) << "status=" << os.last_status();
   EXPECT_EQ(os.resumable_session_id(), session_id);
   EXPECT_EQ(os.next_expected_byte(), expected_committed_size);
@@ -206,27 +202,24 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeWithPartial) {
   EXPECT_EQ(object_name, meta.name());
   EXPECT_EQ(bucket_name_, meta.bucket());
 
-  auto stream = client->ReadObject(bucket_name_, object_name);
+  auto stream = client.ReadObject(bucket_name_, object_name);
   ASSERT_STATUS_OK(stream.status());
   auto const actual = std::string{std::istreambuf_iterator<char>{stream}, {}};
   EXPECT_EQ(q0 + q1 + q2, actual);
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteNotChunked) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
   auto constexpr kUploadQuantum = 256 * 1024;
   auto const payload =
-      std::string(internal::ClientImplDetails::GetConnection(*client)
+      std::string(internal::ClientImplDetails::GetConnection(client)
                       ->options()
                       .get<UploadBufferSizeOption>(),
                   '*');
   auto const header = MakeRandomData(kUploadQuantum / 2);
 
-  auto os =
-      client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
+  auto os = client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
   ASSERT_TRUE(os.good()) << "status=" << os.metadata().status();
   // Write a small header that is too small to be flushed...
   os.write(header.data(), header.size());
@@ -251,24 +244,21 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteNotChunked) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeFinalizedUpload) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
+  auto client = MakeIntegrationTestClient();
   auto object_name = MakeRandomObjectName();
-
   // Start a resumable upload and finalize the upload.
   std::string session_id;
   {
     auto old_os =
-        client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
-                            NewResumableUploadSession());
+        client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
+                           NewResumableUploadSession());
     ASSERT_TRUE(old_os.good()) << "status=" << old_os.metadata().status();
     session_id = old_os.resumable_session_id();
     old_os << LoremIpsum();
   }
 
-  auto os = client->WriteObject(bucket_name_, object_name,
-                                RestoreResumableUploadSession(session_id));
+  auto os = client.WriteObject(bucket_name_, object_name,
+                               RestoreResumableUploadSession(session_id));
   EXPECT_FALSE(os.IsOpen());
   EXPECT_EQ(session_id, os.resumable_session_id());
   ASSERT_STATUS_OK(os.metadata());
@@ -283,23 +273,22 @@ TEST_F(ObjectResumableWriteIntegrationTest, WriteResumeFinalizedUpload) {
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, StreamingWriteFailure) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
 
   auto object_name = MakeRandomObjectName();
 
   std::string expected = LoremIpsum();
 
   // Create the object, but only if it does not exist already.
-  StatusOr<ObjectMetadata> meta = client->InsertObject(
+  StatusOr<ObjectMetadata> meta = client.InsertObject(
       bucket_name_, object_name, expected, IfGenerationMatch(0));
   ASSERT_STATUS_OK(meta);
 
   EXPECT_EQ(object_name, meta->name());
   EXPECT_EQ(bucket_name_, meta->bucket());
 
-  auto os = client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
-                                NewResumableUploadSession());
+  auto os = client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
+                               NewResumableUploadSession());
   os << "Expected failure data:\n" << LoremIpsum();
 
   // This operation should fail because the object already exists.
@@ -319,23 +308,20 @@ TEST_F(ObjectResumableWriteIntegrationTest, StreamingWriteFailure) {
     EXPECT_EQ(os.metadata().status().error_info().reason(), "conditionNotMet");
   }
 
-  auto status = client->DeleteObject(bucket_name_, object_name);
+  auto status = client.DeleteObject(bucket_name_, object_name);
   EXPECT_STATUS_OK(status);
 }
 
 TEST_F(ObjectResumableWriteIntegrationTest, StreamingWriteSlow) {
-  std::chrono::seconds timeout(3);
-  auto retry_policy =
-      LimitedTimeRetryPolicy(/*maximum_duration=*/timeout).clone();
-  StatusOr<Client> client = MakeIntegrationTestClient(std::move(retry_policy));
-  ASSERT_STATUS_OK(client);
+  auto const timeout = std::chrono::seconds(3);
+  auto client = MakeIntegrationTestClient(Options{}.set<RetryPolicyOption>(
+      LimitedTimeRetryPolicy(/*maximum_duration=*/timeout).clone()));
 
   auto object_name = MakeRandomObjectName();
 
   auto data = MakeRandomData(1024 * 1024);
 
-  auto os =
-      client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
+  auto os = client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
   os.write(data.data(), data.size());
   EXPECT_FALSE(os.bad());
   std::cout << "Sleeping to force timeout ... " << std::flush;
@@ -356,7 +342,8 @@ TEST_F(ObjectResumableWriteIntegrationTest, WithXUploadContentLength) {
   auto constexpr kMiB = 1024 * 1024L;
   auto constexpr kChunkSize = 2 * kMiB;
 
-  Client client(Options{}.set<UploadBufferSizeOption>(kChunkSize));
+  auto client = MakeIntegrationTestClient(
+      Options{}.set<UploadBufferSizeOption>(kChunkSize));
 
   auto const chunk = MakeRandomData(kChunkSize);
 
@@ -389,8 +376,8 @@ TEST_F(ObjectResumableWriteIntegrationTest, WithXUploadContentLengthRandom) {
   auto constexpr kQuantum = 256 * 1024L;
   size_t constexpr kChunkSize = 2 * kQuantum;
 
-  Client client(Options{}.set<UploadBufferSizeOption>(kChunkSize));
-
+  auto client = MakeIntegrationTestClient(
+      Options{}.set<UploadBufferSizeOption>(kChunkSize));
   auto const chunk = MakeRandomData(kChunkSize);
 
   std::uniform_int_distribution<std::size_t> size_gen(kQuantum, 5 * kQuantum);
@@ -420,8 +407,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WithXUploadContentLengthRandom) {
 
 TEST_F(ObjectResumableWriteIntegrationTest, WithInvalidXUploadContentLength) {
   if (UsingEmulator() || UsingGrpc()) GTEST_SKIP();
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
+  auto client = MakeIntegrationTestClient();
 
   auto constexpr kChunkSize = 256 * 1024L;
   auto const chunk = MakeRandomData(kChunkSize);
@@ -430,7 +416,7 @@ TEST_F(ObjectResumableWriteIntegrationTest, WithInvalidXUploadContentLength) {
   auto const desired_size = 5 * kChunkSize;
   // Use an invalid value in the X-Upload-Content-Length header, the library
   // should return an error.
-  auto os = client->WriteObject(
+  auto os = client.WriteObject(
       bucket_name_, object_name, IfGenerationMatch(0),
       CustomHeader("X-Upload-Content-Length", std::to_string(3 * kChunkSize)));
   auto offset = 0L;
