@@ -13,11 +13,12 @@
 // limitations under the License.
 
 #include "google/cloud/storage/well_known_headers.h"
-#include "google/cloud/storage/internal/openssl_util.h"
+#include "google/cloud/storage/internal/base64.h"
 #include "google/cloud/internal/sha256_hash.h"
 #include <algorithm>
 #include <iomanip>
 #include <iostream>
+#include <string>
 
 namespace google {
 namespace cloud {
