@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/storagetransfer/storage_transfer_client.h"
+//! [all]
+#include "google/cloud/storagetransfer/v1/storage_transfer_client.h"
 #include <iostream>
-#include <stdexcept>
+#include <string>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,19 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace storagetransfer = ::google::cloud::storagetransfer;
+  namespace storagetransfer = ::google::cloud::storagetransfer_v1;
   auto client = storagetransfer::StorageTransferServiceClient(
       storagetransfer::MakeStorageTransferServiceConnection());
 
   ::google::storagetransfer::v1::ListTransferJobsRequest request;
-  request.set_filter("{\"projectId\": \"" + std::string{argv[1]} + "\"}");
+  request.set_filter(R"""({"projectId": ")""" + std::string{argv[1]} + "\"}");
   for (auto r : client.ListTransferJobs(request)) {
-    if (!r) throw std::runtime_error(r.status().message());
+    if (!r) throw std::move(r).status();
     std::cout << r->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
