@@ -12,26 +12,30 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/tasks/cloud_tasks_client.h"
+//! [all]
+#include "google/cloud/tasks/v2/cloud_tasks_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
-    std::cerr << "Usage: " << argv[0] << " project-id location\n";
+    std::cerr << "Usage: " << argv[0] << " project-id location-id\n";
     return 1;
   }
 
-  namespace tasks = ::google::cloud::tasks;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace tasks = ::google::cloud::tasks_v2;
   auto client = tasks::CloudTasksClient(tasks::MakeCloudTasksConnection());
-  auto const parent =
-      std::string("projects/") + argv[1] + "/locations/" + argv[2];
-  for (auto const& queue : client.ListQueues(parent)) {
-    std::cout << queue.value().DebugString() << "\n";
+
+  for (auto queue : client.ListQueues(location.FullName())) {
+    if (!queue) throw std::move(queue).status();
+    std::cout << queue->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
