@@ -12,29 +12,30 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/tpu/tpu_client.h"
-#include "google/cloud/project.h"
+//! [all]
+#include "google/cloud/tpu/v2/tpu_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
-  if (argc != 3) {
-    std::cerr << "Usage: " << argv[0] << " project-id location-id\n";
+  if (argc != 2) {
+    std::cerr << "Usage: " << argv[0] << " project-id\n";
     return 1;
   }
 
-  namespace tpu = ::google::cloud::tpu;
+  auto const location = google::cloud::Location(argv[1], "-");
+
+  namespace tpu = ::google::cloud::tpu_v2;
   auto client = tpu::TpuClient(tpu::MakeTpuConnection());
 
-  auto const project = google::cloud::Project(argv[1]);
-  auto const parent = project.FullName() + "/locations/" + argv[2];
-  for (auto n : client.ListNodes(parent)) {
-    if (!n) throw std::runtime_error(n.status().message());
+  for (auto n : client.ListNodes(location.FullName())) {
+    if (!n) throw std::move(n).status();
     std::cout << n->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
